-- Register Media ---------------------------------------------------------------------
local media = LibStub("LibSharedMedia-3.0")
media:Register("border", "Skinner", [[Interface\Addons\MayronUI\Media\Borders\Skinner.tga]])
media:Register("border", "Glow", [[Interface\Addons\MayronUI\Media\Borders\glow.tga]])
media:Register("background", "MayronUI", [[Interface\Addons\MayronUI\Media\ToolTip.tga]])
media:Register("background", "MUI_DataFields", [[Interface\Addons\MayronUI\Media\DataFields.tga]])
media:Register("statusbar", "MUI_StatusBar", [[Interface\AddOns\MayronUI\Media\StatusBar\MUI_StatusBar]])

---------------------------------------------------------------------------------------

--[[
	If you want to change settings for the UI then please use the 
	built in GUI ("/mui config") in-game and DO NOT ALTER THIS FILE!
	Unless of course you are an experienced Lua programmer.
	
	Thank you for reading,
	Mayron.
--]]

-- Globals: ------>

MayronUI = LibStub("AceAddon-3.0"):NewAddon("MayronUI", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
MayronUI:SetDefaultModuleState(false)

-- Modules:
MayronUI_BottomUI = MayronUI:NewModule("BottomUI");
MayronUI_SetupUI = MayronUI:NewModule("SetupUI");
MayronUI_ChatBox = MayronUI:NewModule("ChatBox");
MayronUI_CastBars = MayronUI:NewModule("CastBars");
MayronUI_MiniMap = MayronUI:NewModule("MiniMap");
MayronUI_SideBars = MayronUI:NewModule("SideBars");
MayronUI_ReskinUI = MayronUI:NewModule("ReskinUI");

function MayronUI:SetTrackerPosition()
	local mui = MayronUI.db.global.Tracker;
	if (MayronUI_SideBars:IsEnabled()) then
		ObjectiveTrackerFrame:ClearAllPoints();
		if (mui["Point"]) then
			ObjectiveTrackerFrame:SetPoint(mui["Point"], UIParent, mui["Relative Point"], mui["x"], mui["y"]);
		elseif (mui["x"]) then
			ObjectiveTrackerFrame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", mui["x"], mui["specialY"]);
			ObjectiveTrackerFrame:SetPoint("RIGHT", MUI_SideBars, "LEFT", mui["specialX"], 0);
		end
	else	
		if (not mui["x"] or not mui["y"] or not mui["Point"] or not mui["Relative Point"]) then
			mui["y"], mui["x"] = ObjectiveTrackerFrame:GetTop(), ObjectiveTrackerFrame:GetRight();	
			mui["Point"], mui["Relative Point"] = "TOPLEFT", "BOTTOMLEFT";
		end
		ObjectiveTrackerFrame:ClearAllPoints();
		ObjectiveTrackerFrame:SetPoint(mui["Point"], UIParent, mui["Relative Point"], mui["x"], mui["y"]);
	end
end

--hooksecurefunc("ChangeActionBarPage", )
hooksecurefunc("UpdateContainerFrameAnchors", MayronUI.SetTrackerPosition)

MUI_MEDIA = "Interface\\AddOns\\MayronUI\\Media\\";

if ( UnitClass("player") == "Priest" ) then
	MUI_PREFIX = "|cff359AFFMayronUI:|r"; 
	MUI_COLOR = {r = 0.9, g = 0.9, b = 0.9, a = 1};
else
	MUI_PREFIX = "|c"..RAID_CLASS_COLORS[select(2, UnitClass("player"))].colorStr.."MayronUI:|r"; 
	MUI_COLOR = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[(select(2, UnitClass("player")))];	
end

----------------->

local function print(...)
	DEFAULT_CHAT_FRAME:AddMessage(string.join(" ", MUI_PREFIX, tostringall(...)));
end

local defaults = {
	global = {
		Layouts = {},
		General = {},
		AddOns = {}, -- List of Addons installed already.
		Tracker = {},
		SavePoint = {},
	},
	profile = {		
		modules = {
			["BottomUI"] = false,			
			["ChatBox"] = false,
			["CastBars"] = false,
			["MiniMap"] = false,
			["SideBars"] = false,
			["ReskinUI"] = false,
			-- ["SetupUI"] missing due to OnInitialize function handling it.
		},
		install = true,		
		BottomUI = {
			["Unit Frames"] = {},
			["Action Bars"] = {},
			["Data Fields"] = {},
			["Unit Names"] = {},
			["Portrait Gradients"] = {},
			ModKey = "C",
			General = {},
		}, 
		ChatFrame = {
			["Top Left"] = {
				ModKeys = {"C", "S"}, -- 2nd and 3rd row
				functions = {"Character", "Spell Book", "Talents", "LFD", "Mounts and Pets", "PVP", "Encounter Journal", "World Map", "Achievements", "Switch"},
				--specialFunc = {}, -- Side/Layout button function				
			},
			["Top Right"] = {
				ModKeys = {"C", "S"}, -- 2nd and 3rd row
				functions = {" ", " ", " ", " ", " ", " ", " ", " ", " ", "Switch"},
			},
			["Bottom Left"] = {
				ModKeys = {"C", "S"}, -- 2nd and 3rd row
				functions = {"Character", "Spell Book", "Talents", "LFD", "Mounts and Pets", "PVP", "Encounter Journal", "World Map", "Achievements", "Switch"},
				TabBackground = true,
			},
			["Bottom Right"] = {
				ModKeys = {"C", "S"}, -- 2nd and 3rd row
				functions = {" ", " ", " ", " ", " ", " ", " ", " ", " ", "Switch"},
				TabBackground = true,
			},
		},
		ReskinUI = { -- fill table with frames to skin
			Appearance = {},
		}, 
		ChatBox = {}, -- Does not need to be on install. Uses while loop
		SideBars = {},
		MiniMap = {},
		CastBars = {},
	},
}
-- GUI Config:
local ModOptions = { -- For tbls to be dynamically generated.
	["MayronUI_BottomUI_Options"] = {		
		Artwork = {
			type = "group",
			name = "Artwork",
			desc = "Controls the appearance of the frames used to display the Artwork.",
			childGroups = "select",
			order = 2,
			get = function(info)
				local ftype, property = select(4, unpack(info))
				if ( property:find("Expand") ) then 
					property = "Special Height";
				end
				if ( property:find("Retract") ) then 
					property = "Height";
				end
				local defaults = {
					["Action Bars"] = {
						["Width"] = 751,
						["Height"] = 40,
						["Y Offset"] = -110,
						["Alpha"] = 1,
						["Special Height"] = 75,
					},
					["Unit Frames"] = {
						["Left Width"] = 324,
						["Right Width"] = 324,
						["Center Width"] = 102,
						["Height"] = 180,
						["Alpha"] = 0.8,
						["Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b},
					},
					["Data Fields"] = {
						["Width"] = 751,
						["Height"] = 20,
						["Alpha"] = 1,
						["Texture"] = "MUI_DataFields",
						["Color"] = {r= 0.4, g = 0.4, b = 0.4},
					},
					["Unit Names"] = {						
						["Width"] = 232,
						["Inner Offset"] = 23,
						["Height"] = 18,
						["Alpha"] = 0.8,
						["Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b},
					},
					["Portrait Gradients"] = {				
						["Height"] = 20,
						["Alpha"] = 0.4,
					},
				}
				local mui = MayronUI.db.profile.BottomUI[ftype][property];
				if (mui) then	
					if (type(mui) == "table") then
						return mui.r, mui.g, mui.b;
					else
						return mui;
					end
				else
					local value = defaults[ftype][property];
					if (type(value) == "table") then
						return value.r, value.g, value.b;
					else
						return value;
					end
				end
			end,
			set = function(info, value, ...)			
				if (...) then
					local oldValue = value;
					value = {};
					value.r, value.g, value.b, value.a = oldValue, ...;
				end
				local ftype, property = select(4, unpack(info))
				if ( (ftype == "Action Bars") and (property:find("Height")) ) then
					if ( property:find("Expand") ) then 
						property = "Special Height";
						if (value <= (MayronUI.db.profile.BottomUI["Action Bars"]["Height"] or 40) ) then return end
					else
						if (value >= (MayronUI.db.profile.BottomUI["Action Bars"]["Special Height"] or 75) ) then return end						
					end
				end				
				local dvalue =  ( (ftype == "Unit Frames") and { ["Left Width"] = 324, ["Right Width"] = 324, ["Center Width"] = 102, ["Height"] = 180, ["Alpha"] = 0.8} ) or
				( (ftype == "Action Bars") and { ["Width"] = 751, ["Height"] = 40, ["Special Height"] = 75, ["Alpha"] = 1} ) or
				( (ftype == "Data Fields") and { ["Width"] = 751, ["Height"] = 20, ["Alpha"] = 1, ["Texture"] = "MUI_DataFields", ["Color"] = {0.4, 0.4, 0.4}} ) or
				( (ftype == "Unit Names") and { ["Width"] = 232, ["Height"] = 18, ["Y Offset"] = -110, ["Inner Offset"] = 23, ["Alpha"] = 0.8} ) or
				( (ftype == "Portrait Gradients") and { ["Height"] = 20, ["Alpha"] = 0.4} );
				
				-- compare with default values:
				if (...) then
					if (value.r == MUI_COLOR.r and value.g == MUI_COLOR.g and value.b == MUI_COLOR.b) then value = nil end
				else
					if ( value == dvalue[property] ) then
						value = nil;
					end
				end
				MayronUI.db.profile.BottomUI[ftype][property] = value;
				MayronUI_BottomUI:UpdateSettings(ftype, property)
			end,
			args = {},
		},
		Animation = {
			type = "group",
			name = "Animation",
			desc = "Controls the behaviour of the Animations used in the BottomUI.",
			order = 3,
			args = {
				["Speed"] = {
					order = 1,
					type = "range",
					name = "Speed",
					min = 0.2,
					max = 1.4,
					desc = "Default is 0.8",
					step = 0.1,
					set = function(info, value)
						if (value == 0.8) then value = nil end
						MayronUI.db.profile.BottomUI["Animation Speed"] = value;
						MayronUI_BottomUI:SetAnimation(value)
					end,
					get = function()
						return MayronUI_BottomUI:GetAnimation();
					end,
				},			
			},
		},
		General = {
			type = "group",
			name = "General",
			desc = "Controls the general settings of the module.",
			childGroups = "select",
			order = 4,
			get = function(info)
				local addon = select(4, unpack(info))
				if (not MayronUI.db.profile.BottomUI.General) then return true end
				if (not MayronUI.db.profile.BottomUI.General[addon]) then return true end
				return false;
			end,
			set = function(info, value)
				local addon = select(4, unpack(info))
				if (not value) then value = true else value = nil end
				MayronUI.db.profile.BottomUI.General[addon] = value;	
				if (addon == "ShadowUF") then
					local currentProfile = ShadowUF.db:GetCurrentProfile();
					SUFUnittargettarget:ClearAllPoints();
					if (not MayronUI.db.profile.BottomUI.General["ShadowUF"]) then						
						SUFUnittargettarget:SetPoint("BOTTOM", "MUI_Bottom", "TOP", 0, 4);
						local SUF = ShadowedUFDB["profiles"][currentProfile]["positions"]["targettarget"];
						SUF["point"] = "BOTTOM"; SUF["anchorTo"] = "MUI_Bottom"; 
						SUF["relativePoint"] = "TOP"; SUF["x"] = 0; SUF["y"] = 4;						
					else
						SUFUnittargettarget:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 88);
						local SUF = ShadowedUFDB["profiles"][currentProfile]["positions"]["targettarget"];
						SUF["point"] = "BOTTOM"; SUF["anchorTo"] = "UIParent"; 
						SUF["relativePoint"] = "BOTTOM"; SUF["x"] = 0; SUF["y"] = 88;	
					end								
					ShadowUF.Layout:Reload();
				elseif (addon == "Grid") then					
					if (_G["GridLayoutFrame"] and (Grid.db:GetCurrentProfile() == "MayronUIH")) then
						GridLayoutFrame:ClearAllPoints();
						if (not MayronUI.db.profile.BottomUI.General["Grid"]) then								
							GridLayoutFrame:SetPoint("BOTTOMRIGHT", MUI_Bottom, "TOPRIGHT", 26, 88);						
						else
							GridLayoutFrame:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOM", 376, 148);
						end
						Grid:GetModule("GridLayout"):SavePosition();
					end
				end
			end,
			args = {
				header = {
					order = 0.5,
					type = "header",
					name = "Anchored AddOns",
				},
				desc1 = {
					order = 1,
					type = "description",
					name = "\nAllow MayronUI to automatically handle the positioning of AddOn frames to be anchored to the Bottom of the UI.\n",
					width = "full",
				},
				["ShadowUF"] = {
					order = 2,
					type = "toggle",
					desc = "Shadowed Unit Frames by default has been anchored to the BottomUI in order to move with the Expand / Retract Animation.",
					name = "ShadowUF",
					hidden = function() if (IsAddOnLoaded("ShadowedUnitFrames")) then return false else return true end end,
				},
				["Grid"] = {
					order = 3,
					type = "toggle",
					desc = "While Grid is to the \"MayronUIH\" Profile, by default has been anchored to the BottomUI in order to move with the Expand / Retract Animation.",
					name = "Grid",
					hidden = function() if (IsAddOnLoaded("Grid")) then return false else return true end end,
				},				
				sep1 = {
					order = 3.1,
					type = "description",
					name = "\n",
					width = "full",
				},
				desc2 = {
					order = 3.2,
					type = "header",
					name = "Expand / Retract Modifier Key",
					width = "full",
				},
				["ModKey"] = {
					order = 4,
					type = "select",
					name = "Modifier Key",
					desc = "Choose which Modifier Key should be held down while out of combat to show this button.",
					get = function()
						return MayronUI.db.profile.BottomUI.ModKey;
					end,
					set = function(info, value)
						MayronUI.db.profile.BottomUI.ModKey = value;								
					end,
					values = {
						none = "none",
						S = "SHIFT",
						C = "CTRL",
						A = "ALT",
						SC = "SHIFT + CTRL",
						SA = "SHIFT + ALT",
						CA = "CTRL + ALT",
						SCA = "SHIFT + CTRL + ALT",
					},
					style = "radio",
				}
			
			},
		},
	},
	
	["MayronUI_CastBars_Options"] = {
		CastBars = {
			type = "group",
			name = "Cast Bars",
			desc = "Enable / Disable Cast Bars and their elements.",
			order = 1,
			get = function(info)
				local barType = select(4, unpack(info))
				barType = string.gsub(barType, "_", "");
				if (MayronUI.db.profile.CastBars[barType].Enabled) then return true end
			end,
			set = function(info, value)	
				local barType = select(4, unpack(info))
				barType = string.gsub(barType, "_", "");
				if (not value) then value = nil end
				MayronUI.db.profile.CastBars[barType].Enabled = value;
				local bar;
				if (not _G["MUI_"..barType.."CastBar"]) then
					bar = MayronUI:CastBarSetup(barType) -- internal function to register events
				else
					bar = _G["MUI_"..barType.."CastBar"];
					MUI_CastBar_Load(bar, barType:lower());					
					MayronUI:CastBarSetup(barType)
				end
				if (not value) then 
					bar:UnregisterAllEvents()
					bar:Hide()
				end
				MayronUI:PositionCastBars(barType)
				MayronUI.LoadGUI()
			end,
			args = {
				header = {
					order = 0.5,
					type = "header",
					name = "Enabled Cast Bars",
				},
				_Player = {
					order = 1,
					type = "toggle",
					name = "Player",
				},
				_Target = {
					order = 2,
					type = "toggle",
					name = "Target",
				},
				_Focus = {
					order = 3,
					type = "toggle",
					name = "Focus",
				},
				_TargetTarget = {
					order = 4,
					type = "toggle",
					name = "Target of Target",
				},
				sep1 = {
					order = 4.1,
					type = "description",
					name = "\n",
					width = "full",
				},
				toggle = {
					type = "execute",
					order = 5,
					name = "Toggle Cast Bars",
					desc = "This will show all cast bars for easier editing.",
					func = function(info)
						local t = MayronUI.db.profile.CastBars.Appearance;
						for barType, _ in pairs(MayronUI.db.profile.CastBars) do							
							if (_G["MUI_"..barType.."CastBar"]) then
								local bar = _G["MUI_"..barType.."CastBar"];
								if (t.toggle) then
									bar:SetAlpha(0);
									bar:Hide();									
								else 
									bar:SetMinMaxValues(0, 100)
									bar:SetAlpha(1);
									bar:Show();
									bar:SetValue(100);
									if (_G[bar:GetName().."Text"].showText) then
										_G[bar:GetName().."Text"]:SetText(barType);
									end
									_G[bar:GetName().."Timer"]:SetText("95.5 / 100.0");
									--[[if (bar:GetName() == "MUI_PlayerCastBar" ) then
										MUI_PlayerCastBarLatency:Show(); 
										MUI_PlayerCastBarLatency:SetWidth(120);
										MUI_PlayerCastBarLatency:SetVertexColor(1,1,1,1);
										MUI_PlayerCastBarLatency:SetAlpha(1)
									end]]
								end
							end
						end
						if (not t.toggle) then t.toggle = true else t.toggle = nil end
					end,
				},
			},
		},
		Appearance = {
			type = "group",
			name = "Appearance",
			desc = "Controls the appearance of the Cast Bars.",
			order = 2,
			get = function(info)
				local str = select(4, unpack(info));
				if (MayronUI.db.profile.CastBars.Appearance[str] == nil) then 
					local name, property = string.split("_", str);
					local f = {
						-- default settings here:
						["Size"] = function() return 12 end,
						["Field"] = function() return true end,
						["Font"] = function() return "Friz Quadrata TT" end,
						["Type"] = function() if (name == "Border") then return "Skinner" else return "MUI_StatusBar" end end,
						["Color"] = function() 
							if (name == "Interruptable") then return 0, 0.45, 0.73, 0.6;								
							elseif (name == "Non-interruptable") then return 1, 0.47, 0, 0.6;
							elseif (name == "LatencyBar") then return 0, 0.2, 0.41, 0.8;
							elseif (name == "Background") then return 0, 0, 0, 0.5;
							elseif (name == "Border") then return 0, 0, 0, 1;
							elseif (name == "Success") then return 0, 1, 0, 1;
							elseif (name == "Failed") then return 1, 0, 0, 1;
							elseif (name == "Text") then return 1, 1, 1, 1;
							elseif (name == "Timer") then return 0.03, 0.78, 0.97, 1;							
							end
						end,
						["Outline"] = function() return 1 end,
						["EdgeSize"] = function() return 1 end,
						["Inset"] = function() return 0 end,						
						["X"] = function() if (name == "Timer") then return "-5" else return "5" end end,
						["Y"] = function() if (name == "Timer") then return "-5" else return "0" end end,
					}
					return f[property]();
				elseif (type(MayronUI.db.profile.CastBars.Appearance[str]) == "table") then
					return unpack(MayronUI.db.profile.CastBars.Appearance[str]);
				else
					return MayronUI.db.profile.CastBars.Appearance[str];
				end
			end,
			set = function(info, value, ...)
				local str = select(4, unpack(info));
				local name, property = string.split("_", str);					
				local f = {
					-- default settings here:
					["Size"] = function() return 12 end,
					["Field"] = function() return true end,
					["Font"] = function() return "Friz Quadrata TT" end,
					["Type"] = function() if (name == "Border") then return "Skinner" else return "MUI_StatusBar" end end,
					["Color"] = function() 
						if (name == "Interruptable") then return 0, 0.45, 0.73, 0.6;								
						elseif (name == "Non-interruptable") then return 1, 0.47, 0, 0.6;
						elseif (name == "LatencyBar") then return 0, 0.2, 0.41, 0.8;
						elseif (name == "Background") then return 0, 0, 0, 0.5;
						elseif (name == "Border") then return 0, 0, 0, 1;
						elseif (name == "Success") then return 0, 1, 0, 1;
						elseif (name == "Failed") then return 1, 0, 0, 1;
						elseif (name == "Text") then return 1, 1, 1, 1;
						elseif (name == "Timer") then return 0.03, 0.78, 0.97, 1;							
						end
					end,
					["Outline"] = function() return 1 end,
					["EdgeSize"] = function() return 1 end,
					["Inset"] = function() return 0 end,					
					["X"] = function() if (name == "Timer") then return "-5" else return "5" end end,
					["Y"] = function() if (name == "Timer") then return "-5" else return "0" end end,
				}
				local reset;
				if (...) then
					local r, g, b, a = value, ...;
					local defaults = {};
					defaults.r, defaults.g, defaults.b, defaults.a = f[property]();
					if (r == defaults.r and g == defaults.g and b == defaults.b and a == defaults.a) then reset = true end
					MayronUI.db.profile.CastBars.Appearance[str] = {r, g, b, a};
				else
					local default = f[property]();
					if (value == default) then reset = true end
					MayronUI.db.profile.CastBars.Appearance[str] = value;					
				end	
				MayronUI:CastBarSetup(nil, str);
				if (reset) then MayronUI.db.profile.CastBars.Appearance[str] = nil end				
			end,
			args = {
				headerColor = {
					order = 0.5,
					type = "header",
					name = "Colors",
				},
				["Interruptable_Color"] = {
					order = 1,
					type = "color",
					hasAlpha = true,
					name = "Interruptible",
					desc = "Alter the color of the cast bar for Interruptible abilities.",
				},
				["Non-interruptable_Color"] = {
					order = 2,
					type = "color",
					hasAlpha = true,
					name = "Non-interruptible",
					desc = "Alter the color of the cast bar for non-Interruptible abilities.",
				},
				["LatencyBar_Color"] = {
					order = 3,
					type = "color",
					hasAlpha = true,
					name = "Latency indicator",
					desc = "Alter the color of the Latency indicator.",
				},
				["Background_Color"] = {
					order = 3.5,
					type = "color",
					hasAlpha = true,
					name = "Background",
					desc = "Alter the background color.",
				},
				["Border_Color"] = {
					order = 4,
					type = "color",
					hasAlpha = true,
					name = "Border",
					desc = "Alter the border color.",
				},
				["Success_Color"] = {
					order = 5,
					type = "color",
					hasAlpha = true,
					name = "Successful Cast",
					desc = "Alter the ability was successfully casted color.",
				},
				["Failed_Color"] = {
					order = 6,
					type = "color",
					hasAlpha = true,
					name = "Failed Cast",
					desc = "Alter the ability failed to finish casting color.",
				},
				sep1 = {
					order = 6.1,
					type = "description",
					name = "\n",
					width = "full",
				},
				headerTexture = {
					order = 6.6,
					type = "header",
					name = "Textures",
				},
				["Border_Type"] = {
					order = 7,
					type = "select",
					dialogControl = "LSM30_Border",
					name = "Border Type",
					desc = "Alter the border style.",
					values = AceGUIWidgetLSMlists.border,
				},
				["StatusBar_Type"] = {
					order = 8,
					type = "select",
					dialogControl = "LSM30_Statusbar",
					name = "Status Bar Type",
					desc = "Alter the status bar style.",
					values = AceGUIWidgetLSMlists.statusbar,
				},
				sep2 = {
					order = 9.1,
					type = "description",
					name = "",
					width = "full",
				},
				["Border_EdgeSize"] = {
					order = 10,
					type = "range",
					name = "Edge Size",
					min = 1,
					max = 32,
					step = 1,
				},
				["Border_Inset"] = {
					order = 11,
					type = "range",
					name = "Inset",
					min = -30,
					max = 32,
					step = 1,
				},
				sep3 = {
					order = 11.1,
					type = "description",
					name = "\n",
					width = "full",
				},
				headerText1 = {
					order = 11.6,
					type = "header",
					name = "Ability Name",
				},
				["Text_Field"] = {
					order = 12,
					name = "Ability Name",
					desc = "Show/Hide the name of the Ability being casted.",
					type = "toggle",
				},
				["Text_Size"] = {
					order = 13,
					type = "range",
					name = "Size",
					min = 8,
					max = 18,
					step = 1,
				},
				["Text_Color"] = {
					order = 14,
					type = "color",
					hasAlpha  = true,
					name = "Text Color",
				},
				sep4 = {
					order = 14.1,
					type = "description",
					name = "",
					width = "full",
				},
				["Text_Font"] = {
					order = 15,
					type = "select",
					dialogControl = "LSM30_Font",
					name = "Font",
					values = AceGUIWidgetLSMlists.font,
				},
				["Text_Outline"] = {
					order = 16,
					type = "select",
					name = "Outline",
					values = {"None", "Monochrome", "Thin outline", "Thick outline"},
				},
				["Text_X"] = {
					order = 17,
					type = "input",
					name = "X Offset",
				},
				["Text_Y"] = {
					order = 18,
					type = "input",
					name = "Y Offset",
				},
				sep6 = {
					order = 18.1,
					type = "description",
					name = "\n",
					width = "full",
				},
				headerText2 = {
					order = 18.6,
					type = "header",
					name = "Timer Text",
				},
				["Timer_Field"] = {
					order = 19,
					name = "Timer Text",
					desc = "Show/Hide the Duration and/or Time Remaining of the Ability being casted.",
					type = "toggle",
				},
				["Duration_Field"] = {
					order = 20,
					name = "Duration",
					disabled = function() if (MayronUI.db.profile.CastBars.Appearance["Timer_Field"] == false) then return true end end,
					desc = "Show/Hide the Duration and/or Time Remaining of the Ability being casted.",
					type = "toggle",
				},
				["Remaining_Field"] = {
					order = 21,
					name = "Time Remaining",
					disabled = function() if (not MayronUI.db.profile.CastBars.Appearance["Timer_Field"] == false) then return true end end,
					desc = "Show/Hide the Duration and/or Time Remaining of the Ability being casted.",
					type = "toggle",
				},
				sep7 = {
					order = 21.1,
					type = "description",
					name = "",
					width = "full",
				},
				["Timer_Size"] = {
					order = 22,
					type = "range",
					name = "Size",
					min = 8,
					max = 18,
					step = 1,
				},
				["Timer_Color"] = {
					order = 23,
					type = "color",
					hasAlpha  = true,
					name = "Text Color",
				},
				sep8 = {
					order = 23.1,
					type = "description",
					name = "",
					width = "full",
				},
				["Timer_Font"] = {
					order = 24,
					type = "select",
					dialogControl = "LSM30_Font",
					name = "Font",
					values = AceGUIWidgetLSMlists.font,
				},
				["Timer_Outline"] = {
					order = 25,
					type = "select",
					name = "Outline",
					values = {"None", "Monochrome", "Thin outline", "Thick outline"},
				},
				["Timer_X"] = {
					order = 26,
					type = "input",
					name = "X Offset",
				},
				["Timer_Y"] = {
					order = 27,
					type = "input",
					name = "Y Offset",
				},
			},
		},
	},
	
	["MayronUI_ChatBox_Options"] = {
		ChatBoxes = {
			type = "group",
			name = "Chat Boxes",
			desc = "Control the functionality of buttons on each active Chat Box",
			order = 1,
			get = function(info)
				local name = select(4, unpack(info));
				name = string.gsub(name, "_", " ");
				return MayronUI.db.profile.ChatBox[name];
			end,
			set = function(info, value)
				local name = select(4, unpack(info));				
				name = string.gsub(name, "_", " ");
				local id = {["Top Left"] = 1, ["Top Right"] = 2, ["Bottom Left"] = 3, ["Bottom Right"] = 4};
				if (value) then
					MayronUI.db.profile.ChatBox[name] = id[name];
					MUI_ChatFrame_CornerSet();					
				else
					MayronUI.db.profile.ChatBox[name] = nil;
					_G["MUI_ChatFrame"..id[name]]:Hide()
				end
				local change;
				-- Comment this section out if you do not want Bazooka switching profiles automatically:
				if ( IsAddOnLoaded("Bazooka") ) then
					if ( (Bazooka.db:GetCurrentProfile() == "Default") and (not MayronUI.db.profile.ChatBox["Top Left"]) ) then
						if (MayronUI.db.profile.ChatBox["Bottom Left"]) then
							Bazooka.db:SetProfile("Bottom Left");
							
							-- Chat Frame settings
							FCF_SetLocked(ChatFrame1, 1);
							ChatFrame1:ClearAllPoints();
							ChatFrame1:SetPoint('BOTTOMLEFT',UIParent, 'BOTTOMLEFT', 34, 30);
							ChatFrame1:SetUserPlaced(true);
							FCF_SavePositionAndDimensions(ChatFrame1);

							-- Grid:
							if ( IsAddOnLoaded("Grid") ) then
								if (_G["GridLayoutFrame"]) then
									local current = Grid.db:GetCurrentProfile();
									Grid.db:SetProfile("Default");
									GridLayoutFrame:ClearAllPoints();
									GridLayoutFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 2, -2);
									Grid:GetModule("GridLayout"):SavePosition();
									Grid.db:SetProfile(current);
								end
								MayronUI.db.profile.BottomUI.ReAnchor = nil;
							end
							-- SUF:
							if ( IsAddOnLoaded("ShadowedUnitFrames") ) then
								if (_G["ShadowUF"] and ShadowUF.db) then
									local current = ShadowUF.db:GetCurrentProfile();
									ShadowUF.db:SetProfile("Default");
									local SUF = ShadowedUFDB["profiles"]["Default"]["positions"]["party"];
									SUF["point"] = "TOPLEFT"; SUF["anchorTo"] = "UIParent"; 
									SUF["relativePoint"] = "TOPLEFT"; SUF["x"] = 6; SUF["y"] = -4;									
									ShadowedUFDB["profiles"]["Default"]["units"]["party"]["attribPoint"] = "TOP";	
									ShadowUF:ProfilesChanged()
									ShadowUF.db:SetProfile(current);
								end
								MayronUI.db.profile.BottomUI.ReAnchor = nil;
							end
							
							change = true;							
						end
					elseif ( (Bazooka.db:GetCurrentProfile() == "Bottom Left") and (not MayronUI.db.profile.ChatBox["Bottom Left"]) ) then
						if (MayronUI.db.profile.ChatBox["Top Left"]) then
							Bazooka.db:SetProfile("Default");
							
							-- Chat Frame settings
							FCF_SetLocked(ChatFrame1, 1);
							ChatFrame1:ClearAllPoints();
							ChatFrame1:SetPoint('TOPLEFT',UIParent,'TOPLEFT', 34, -55);
							ChatFrame1:SetUserPlaced(true);
							FCF_SavePositionAndDimensions(ChatFrame1);							
							
							-- Grid:
							if ( IsAddOnLoaded("Grid") ) then
								if (_G["GridLayoutFrame"]) then
									local current = Grid.db:GetCurrentProfile();
									Grid.db:SetProfile("Default");
									GridLayoutFrame:ClearAllPoints();
									GridLayoutFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 2, 2);
									Grid:GetModule("GridLayout"):SavePosition();
									Grid.db:SetProfile(current);
								end
								MayronUI.db.profile.BottomUI.ReAnchor = nil;
							end
							-- SUF:
							if ( IsAddOnLoaded("ShadowedUnitFrames") ) then
								if (_G["ShadowUF"] and ShadowUF.db) then
									local current = ShadowUF.db:GetCurrentProfile();
									ShadowUF.db:SetProfile("Default");
									local SUF = ShadowedUFDB["profiles"]["Default"]["positions"]["party"];
									SUF["point"] = "BOTTOMLEFT"; SUF["anchorTo"] = "UIParent"; 
									SUF["relativePoint"] = "BOTTOMLEFT"; SUF["x"] = 6; SUF["y"] = 2;
									ShadowedUFDB["profiles"]["Default"]["units"]["party"]["attribPoint"] = "BOTTOM";
									ShadowUF:ProfilesChanged()
									ShadowUF.db:SetProfile(current);									
								end
								MayronUI.db.profile.BottomUI.ReAnchor = nil;
							end
						end
					end					
					if ( MayronUI.db.profile.ChatFrame.ReAnchor_Other ) then
						local cbAnchor, profile, points;
						if ( (MayronUI.db.profile.ChatBox["Top Left"]) and (not MayronUI.db.profile.ChatBox["Bottom Left"]) ) then
							cbAnchor = MUI_ChatFrame1;
							profile = "Default";
							points = {-30, 16};
						elseif ( (MayronUI.db.profile.ChatBox["Bottom Left"]) and (not MayronUI.db.profile.ChatBox["Top Left"]) ) then
							cbAnchor = MUI_ChatFrame3;
							profile = "Bottom Left";
							points = {-20, 26};
						end
						if (cbAnchor and profile and points) then
							for i = 2, 4 do
								local bar = _G["BazookaBar_"..i]
								if (bar) then
									bar:ClearAllPoints();
									if (i == 2) then
										bar:SetPoint("TOPLEFT", cbAnchor, "TOPLEFT", 4, points[1]);
									elseif (i == 3) then
										bar:SetPoint("TOPLEFT", BazookaBar_2, "BOTTOMLEFT", 0, -3);
									else
										bar:SetPoint("TOPLEFT", BazookaBar_3, "BOTTOMLEFT", 0, -3);
									end
									local left = bar:GetLeft();
									local bottom = bar:GetBottom();
									BazookaDB["profiles"][profile]["bars"][i]["point"] = "BOTTOMLEFT";
									BazookaDB["profiles"][profile]["bars"][i]["relPoint"] = "BOTTOMLEFT";
									BazookaDB["profiles"][profile]["bars"][i]["y"] = bottom;
									BazookaDB["profiles"][profile]["bars"][i]["x"] = left;
								end
							end
							for i = 7, 5, -1 do
								local bar = _G["BazookaBar_"..i]
								if (bar) then
									bar:ClearAllPoints();
									if (i == 7) then
										bar:SetPoint("BOTTOMLEFT", cbAnchor, "BOTTOMLEFT", 2, points[2]);
									elseif (i == 6) then
										bar:SetPoint("BOTTOMLEFT", BazookaBar_7, "TOPLEFT", 2, 3);
									else
										bar:SetPoint("BOTTOMLEFT", BazookaBar_6, "TOPLEFT", 0, 3);
									end
									local left = bar:GetLeft();
									local bottom = bar:GetBottom();
									BazookaDB["profiles"][profile]["bars"][i]["point"] = "BOTTOMLEFT";
									BazookaDB["profiles"][profile]["bars"][i]["relPoint"] = "BOTTOMLEFT";
									BazookaDB["profiles"][profile]["bars"][i]["y"] = bottom;
									BazookaDB["profiles"][profile]["bars"][i]["x"] = left;
								end
							end
						end
						if (change) then
							MayronUI.db.profile.ChatFrame.ReAnchor_Other = nil;
						end
					end
				end
				--> End of Section -----------------<					
				MayronUI.LoadGUI()
			end,
			args = {
				header = {
					order = 0.5,
					type = "header",
					name = "Enable Artwork",
				},
				["Top_Left"] = {
					order = 1,
					name = "Top Left",
					desc = "Enable the Top Left MayronUI chat box art.",
					type = "toggle",
				},
				["Top_Right"] = {
					order = 2,
					name = "Top Right",
					desc = "Enable the Top Right MayronUI chat box art.",
					type = "toggle",
				},
				["Bottom_Left"] = {
					order = 3,
					name = "Bottom Left",
					desc = "Enable the Bottom Left MayronUI chat box art.",
					type = "toggle",
				},
				["Bottom_Right"] = {
					order = 4,
					name = "Bottom Right",
					desc = "Enable the Bottom Right MayronUI chat box art.",
					type = "toggle",
				},
			},
		},
		General = {
			type = "group",
			name = "General",
			desc = "Contains General Chat Box settings.",
			order = 3,
			args = {
				Message = {
					order = 1,
					type = "description",
					name = "Leatrix Plus contains most of the General settings you will need to use.\n\nMake sure that you do not uninstall this AddOn!\n",
					width = "full",
				},
				["Toggle"] = {
					order = 2,
					type = "execute",
					name = "Leatrix Plus",
					func = function() 
						if (IsAddOnLoaded("Leatrix_Plus") and leatrix) then
							leatrix();
						else
							print("Missing AddOn: Leatrix Plus!");
						end
					end,
				},			
			},
		},
	},
	
	["MayronUI_MiniMap_Options"] = {
		Message = {
			order = 2.1,
			type = "description",
			name = "\n|cff00ccffRemember:|r Alt Key + Left Click on the Mini-Map to view a list of commands to Move and Resize it!\n\nMore options are coming soon!\n",
			width = "full",
		},
		["Scale"] = {
			order = 3,
			type = "range",
			desc = "Adjust the Scale of the Minimap.\n\nDefault is 1",
			name = "Scale",
			min = 0.5,
			max = 1.5,
			step = 0.1,
			get	= function(info)
				return MayronUI.db.profile.MiniMap.scale or 1;
			end,
			set = function(info, value)
				Minimap:SetScale(value)
				if (value == 1) then value = nil end
				MayronUI.db.profile.MiniMap.scale = value;
			end,
		},
	},

	["MayronUI_SideBars_Options"] = {
		Artwork = {
			type = "group",
			name = "Artwork",
			desc = "Controls the behaviour of the Animation used with the Side Bar.",
			childGroups = "select",
			order = 2,
			get = function(info)
				local property = select(4, unpack(info))
				local f = { ["1 Bar Width"] = 44, ["2 Bars Width"] = 80, ["Height"] = 486, ["ofsy"] = 64}
				local value = MayronUI.db.profile.SideBars[property] or f[property]
				if (property == "ofsy") then
					return tostring(value);
				else
					return value;
				end
			end,
			set = function(info, value)
				local property = select(4, unpack(info));
				local mui = MayronUI.db.profile.SideBars;
				local f = { ["1 Bar Width"] = 44, ["2 Bars Width"] = 80, ["Height"] = 486, ["ofsy"] = 64}
				if ( property:find("1") ) then 
					if ( (mui["2 Bars Width"] or f["2 Bars Width"]) <= value) then return end				
				end
				if ( property:find("2") ) then 
					if ( (mui["1 Bar Width"] or f["1 Bar Width"]) >= value) then return end				
				end
				value = tonumber(value);
				if (value == f[property]) then value = nil end
				mui[property] = value;
				MayronUI_SideBars:UpdateSettings();
			end,
			args = {
				["1 Bar Width"] = {
					order = 1,
					type = "range",
					desc = "Adjust the Width of the artwork when 1 Action bar is shown.\n\nDefault is 44",
					name = "1 Bar Width",
					min = 20,
					max = 200,
					step = 1,			
				},
				["2 Bars Width"] = {
					order = 2,
					type = "range",
					desc = "Adjust the Width of the artwork when 2 Action bars are shown.\n\nDefault is 80",
					name = "2 Bars Width",
					min = 40,
					max = 220,
					step = 1,			
				},
				["Height"] = {
					order = 3,
					type = "range",
					desc = "Adjust the Height of the artwork.\n\nDefault is 486",
					name = "Height",
					min = 340,
					max = 800,
					step = 1,
				},
				["ofsy"] = {
					order = 4,
					type = "input",
					desc = "Adjust the Y Offset of the artwork.\n\nDefault is 64",
					name = "Y Offset",
				},
			},
		},
		Animation = {
			type = "group",
			name = "Animation",
			desc = "Controls the appearance of the frame used to display the Artwork.",
			childGroups = "select",
			order = 3,
			args = {	
				["Speed"] = {
					order = 1,
					type = "range",
					desc = "Adjust the animation speed for how quick the artwork and action bars should expand and retract.\n\nDefault is 0.8",
					name = "Speed",
					min = 0.2,
					max = 1.4,
					step = 0.1,
					set = function(info, value)
						if (value == 0.8) then value = nil end
						MayronUI.db.profile.SideBars["Animation Speed"] = value;
						MayronUI_SideBars:SetAnimation(value)
					end,
					get = function()
						return MayronUI_SideBars:GetAnimation();
					end,
				},
			},
		},		
	},

	["MayronUI_ReskinUI_Options"] = {
		BlizzardFrames = {
			type = "group",
			name = "Blizzard Frames",
			desc = "Choose which Blizzard frames should be Re-skinned.",
			order = 2,
			args = {},
		},
		AddonFrames = {
			type = "group",
			name = "AddOn Frames",
			desc = "Choose which AddOn frames should be Re-skinned.",
			order = 3,
			args = {},
		},
		Appearance = {
			type = "group",
			name = "Appearance",			
			desc = "Customise the appearance of the Reskin.",
			order = 4,
			get = function(info)
				local str = select(4, unpack(info));				
				local loc = MayronUI.db.profile.ReskinUI.Appearance;
				if (loc[str] ~= nil) then
					if (type(loc[str]) == "table") then return unpack(loc[str]) end
					return loc[str];
				end
				local default = {
					["Background_Color"] = function() return 0.2, 0.2, 0.2, 0.9 end, 
					["Border_Color"] = function() return 0, 0, 0, 1 end,					  
					["DropDown_ClassColor"] = function() return true end,
					["Tab_ClassColor"] = function() return true end,		
					["Tab_Color"] = function() return MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, MUI_COLOR.a end,
					["DropDown_Color"] = function() return MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, MUI_COLOR.a end,
					["Border_Type"] = function() return "Skinner" end,
					["Background_Type"] = function() return "MayronUI" end,
					["Border_EdgeSize"] = function() return 1 end,
					["Border_Inset"] = function() return 0 end,
				}
				return default[str]();
			end,
			set = function(info, ...)
				local str = select(4, unpack(info));
				local default = {
					["Background_Color"] = function() return 0.2, 0.2, 0.2, 0.9 end, 
					["Border_Color"] = function() return 0, 0, 0, 1 end,					  
					["DropDown_ClassColor"] = function() return true end,
					["Tab_ClassColor"] = function() return true end,		
					["Tab_Color"] = function() return MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, MUI_COLOR.a end,
					["DropDown_Color"] = function() return MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, MUI_COLOR.a end,
					["Border_Type"] = function() return "Skinner" end,
					["Background_Type"] = function() return "MayronUI" end,
					["Border_EdgeSize"] = function() return 1 end,
					["Border_Inset"] = function() return 0 end,
				}
				local loc = MayronUI.db.profile.ReskinUI.Appearance;
				local newValue = {};
				for id, num in ipairs( {...} ) do
					newValue[id] = ( (type(select(1, ...)) == "number") and (math.floor((num * 10) + 0.5) / 10) ) or num;
				end
				default = { default[str]() };
				local same = true;
				for id, value in ipairs(default) do
					if (newValue[id] ~= default[id]) then
						if ( (#newValue) == 1 ) then
							newValue = unpack(newValue);
						end
						loc[str] = newValue; -- different value detected.
						same = nil;
						break
					end					
				end
				if (same) then loc[str] = nil end;
				if (loc.Toggle) then MayronUI_ReskinUI:TestFrame(toggle) end
			end,
			args = {
				Message = {
					order = 0.1,
					type = "description",
					name = "\n|cff00ccffWarning:|r All settings below require a Reload of the UI to take effect.\n",
					width = "full",
				},
				["Toggle"] = {
					order = 0.2,
					type = "execute",
					name = "Toggle Test Frame",
					desc = "Allows you to see the changes you make on a test frame to see what the Reskin will look like after a Reload of the UI (/rl).",
					func = function() MayronUI_ReskinUI:TestFrame(true) end,
				},
				headerColors = {
					order = 0.6,
					type = "header",
					name = "Colors",
				},
				["Background_Color"] = { 
					order = 1,
					type = "color",
					hasAlpha = true,
					name = "Background",
					desc = "Alter the background color.",
				},
				["Border_Color"] = { 
					order = 2,
					type = "color",
					hasAlpha = true,
					name = "Border",
					desc = "Alter the background color.",
				},
				sep1 = {
					order = 3.1,
					type = "description",
					name = "\n",
					width = "full",
				},
				headerClassColors = {
					order = 3.6,
					type = "header",
					name = "Class Colored Features",
				},
				["DropDown_ClassColor"] = { 
					order = 4, 
					type = "toggle",				
					name = "Drop Down Menus",
					desc = "Whether or not drop down menus should be class colored.",
				},
				["Tab_ClassColor"] = {
					order = 5, 
					type = "toggle",				
					name = "Tabs",
					desc = "Whether or not frame tabs should be class colored.",
				},
				sep2 = {
					order = 5.1,
					type = "description",
					name = "",
					width = "full",
				},
				["DropDown_Color"] = { 
					order = 5.5, 
					type = "color",
					hasAlpha = true,
					disabled = function() if (MayronUI.db.profile.ReskinUI.Appearance["DropDown_ClassColor"] == nil) then return true end end,
					name = "Drop Down Menus",
					desc = "Alter the color of all frame drop down menus.",
				},
				["Tab_Color"] = { 
					order = 6, 
					type = "color",
					hasAlpha = true,
					disabled = function() if (MayronUI.db.profile.ReskinUI.Appearance["Tab_ClassColor"] == nil) then return true end end,
					name = "Tabs",
					desc = "Alter the color of all frame tabs.",
				},
				sep3 = {
					order = 7.1,
					type = "description",
					name = "\n",
					width = "full",
				},
				headerTexture = {
					order = 7.6,
					type = "header",
					name = "Textures",
				},
				["Border_Type"] = { 
					order = 7,
					type = "select",
					dialogControl = "LSM30_Border",
					name = "Border Type",
					desc = "Alter the border style.",
					values = AceGUIWidgetLSMlists.border,
				},
				["Background_Type"] = { 
					order = 8,
					type = "select",
					dialogControl = "LSM30_Background",
					name = "Background Type",
					desc = "Alter the Background Texture type.",
					values = AceGUIWidgetLSMlists.background,
				},
				sep4 = {
					order = 9.1,
					type = "description",
					name = "",
					width = "full",
				},
				["Border_EdgeSize"] = { 
					order = 10,
					type = "range",
					name = "Edge Size",
					min = 1,
					max = 32,
					step = 1,
				},
				["Border_Inset"] = {
					order = 11,
					type = "range",
					name = "Inset",
					min = -30,
					max = 32,
					step = 1,
				},
			},
		},
		Miscellaneous = {
			type = "group",
			name = "Miscellaneous",
			desc = "Other uncategorised options.",
			order = 5,
			args = {
				Fonts = {
					order = 1,
					type = "toggle",
					name = "Change Quest Font",
					desc = "Changes the Blizzard Quest Font Colors.",
					get = function() 
						return not MayronUI.db.profile.ReskinUI["Font Colors"];
					end,
					set = function(info, value)
						if (value) then												
							MayronUI.db.profile.ReskinUI["Font Colors"] = nil;
						else
							MayronUI.db.profile.ReskinUI["Font Colors"] = true; 
						end
					end,
				},
			},
		},
		["Movable"] = {
			type = "toggle",
			name = "Movable Frames",
			desc = "Enable/Disable the ability to drag and reposition reskinned frames.\n\nRequires a Reload of the UI to take effect.",
			order = 4,
			get = function() 
				if (not MayronUI.db.profile.ReskinUI["Stop Movable"]) then 
					return true 
				end
				return false;
			end,
			set = function(info, value)
				if (value) then value = nil else value = true end
				MayronUI.db.profile.ReskinUI["Stop Movable"] = value;
			end,
		},
		["Save_Repositioning"] = {
			type = "toggle",
			name = "Save Repositioning",
			desc = "Enable/Disable the ability to save the repositioning of Frames after dragging.\n\nRequires a Reload of the UI to take effect.",
			order = 5,
			get = function() 
				if (not MayronUI.db.profile.ReskinUI["Stop Reposition"]) then 
					return true 
				end
				return false;
			end,
			set = function(info, value)
				if (value) then value = nil else value = true end
				MayronUI.db.profile.ReskinUI["Stop Reposition"] = value;
			end,
		},
	},
}
local function GetModules()
	local tbl = {
		type = "group",
		name = "Modules",
		childGroups = "tab",
		order = 3,
		args = {
			General = {
				type = "group",
				name = "General",
				desc = "Control the General settings of MUI.",
				childGroups = "select",
				order = 1,
				get = function(info)
					local name = select(3, unpack(info))
					if (not MayronUI.db.global.General[name]) then return true end
					return false;
				end,
				set = function(info, value)
					local name = select(3, unpack(info))
					if (not value) then value = true else value = nil end
					MayronUI.db.global.General[name] = value;
				end,
				args = {
					header1 = {
						order = 1,
						type = "header",
						name = "Auto Hide Frames",
					},
					["Objectives Tracker"] = {
						order = 2,
						type = "toggle",
						name = "Objectives Tracker",
						desc = "Auto Hide when entering instances or engaging an instance boss.",
					},
					["Inventory"] = {
						order = 3,
						type = "toggle",
						name = "Bags / Inventory",
						desc = "Auto Hide when entering combat",
					},
					sep0 = {
						order = 3.1,
						type = "description",
						name = "\n",
						width = "full",
					},
					header2 = {
						order = 3.5,
						type = "header",
						name = "Objectives Tracker",
					},
					sep1 = {
						order = 3.6,
						type = "description",
						name = "\n",
						width = "full",
					},
					["Connect"] = {
						order = 4,
						type = "toggle",
						name = "Connect to SideBars",
						hidden = function() if (MayronUI_SideBars:IsEnabled()) then return false else return true end end,
						get = function()
							if (MayronUI.db.global.Tracker["Point"]) then return false else return true end
						end,
						set = function(info, value)
							local mui = MayronUI.db.global.Tracker;
							if (value) then
								mui["Point"] = nil;
								mui["Relative Point"] = nil;
								mui["x"], mui["y"] = ObjectiveTrackerFrame:GetCenter();
							else								
								mui["Point"] = "TOPRIGHT";
								mui["Relative Point"] = "TOPRIGHT";
								mui["x"] = -40;
								mui["y"] = -234;
							end
							MayronUI:SetTrackerPosition();
						end,
					},
					["x"] = {
						order = 5,
						type = "range",
						name = "X Offset",
						min = -1000,
						max = 1000,
						step = 1,
						get = function()
							if (MayronUI.db.global.Tracker["Point"] or (not MayronUI_SideBars:IsEnabled()) ) then 
								return MayronUI.db.global.Tracker["x"];
							else 
								return MayronUI.db.global.Tracker["specialX"];
							end
						end,
						set = function(info, value)
							if ( MayronUI.db.global.Tracker["Point"] or (not MayronUI_SideBars:IsEnabled()) ) then 
								MayronUI.db.global.Tracker["x"] = value;
							else
								MayronUI.db.global.Tracker["specialX"] = value;
							end
							MayronUI:SetTrackerPosition();
						end,
					},
					["y"] = {
						order = 6,
						type = "range",
						name = "Y Offset",
						min = -1000,
						max = 1000,
						step = 1,
						get = function()
							if (MayronUI.db.global.Tracker["Point"] or (not MayronUI_SideBars:IsEnabled()) ) then 
								return MayronUI.db.global.Tracker["y"];
							else 
								return MayronUI.db.global.Tracker["specialY"];
							end
						end,
						set = function(info, value)
							if (MayronUI.db.global.Tracker["Point"] or (not MayronUI_SideBars:IsEnabled()) ) then 
								MayronUI.db.global.Tracker["y"] = value;
							else 
								MayronUI.db.global.Tracker["specialY"] = value;
							end
							MayronUI:SetTrackerPosition();
						end,
					},
					sep2 = {
						order = 6.1,
						type = "description",
						name = "\n",
						width = "full",
					},
					["Point"] = {
						order = 7,
						type = "select",
						name = "Point",
						hidden = function() 
							if (MayronUI.db.global.Tracker["Point"] or (not MayronUI_SideBars:IsEnabled()) ) then
								return false 
							else 
								return true 
							end 
						end,
						get = function()
							return MayronUI.db.global.Tracker["Point"];
						end,
						set = function(info, value)	
							MayronUI.db.global.Tracker["Point"] = value;
							MayronUI:SetTrackerPosition();
						end,
						values = {
							["TOPLEFT"] = "Top Left",
							["TOPRIGHT"] = "Top Right",
						},
					},
					["Relative Point"] = {
						order = 8,
						type = "select",
						name = "Relative Point",
						hidden = function() 
							if (MayronUI.db.global.Tracker["Relative Point"] or (not MayronUI_SideBars:IsEnabled()) ) then 
								return false 
							else 
								return true 
							end 
						end,
						get = function()
							return MayronUI.db.global.Tracker["Relative Point"];
						end,
						set = function(info, value)	
							MayronUI.db.global.Tracker["Relative Point"] = value;
							MayronUI:SetTrackerPosition();
						end,
						values = {
							["TOPLEFT"] = "Top Left",
							["TOPRIGHT"] = "Top Right",
							["BOTTOMLEFT"] = "Bottom Left",
							["BOTTOMRIGHT"] = "Bottom Right",
							["LEFT"] = "Left",
							["RIGHT"] = "Right",
							["CENTER"] = "Center",
						},
					},
					header3 = {
						order = 8.5,
						type = "header",
						name = "Master Color Theme",
					},
					sep3 = {
						order = 8.6,
						type = "description",
						name = "\nIf you do not like the Class Colored theme for the UI then you can pick another theme choice or use a custom Color. \n\nPicking a new theme will require a Reload of the UI to take effect! (/rl)\n",
						width = "full",
					},
					["Custom Color"] = {
						order = 9,
						type = "color",
						hidden = function() 
							if (MayronUI.db.profile.theme ~= "Custom") then 
								return true 
							end 
						end,
						name = "Custom Color",
						desc = "Choose a custom color to use as your theme color. Will require a Reload of the UI to take effect!",
						get = function(info)
							local loc = MayronUI.db.profile["Custom Color"];
							if (loc) then
								return loc.r, loc.g, loc.b;
							else
								return MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b;							
							end
						end,
						set = function(info, vr, vg, vb)
							MayronUI.db.profile["Custom Color"] = { r = vr, g = vg, b = vb, }
						end,
					},	
					["MasterColor"] = {
						order = 10,
						type = "select",
						name = "Themes",
						desc = "Pick custom if you would rather choose and use a custom color of your choice.",
						get = function()
							return MayronUI.db.profile.theme or select(2, UnitClass("player"));
						end,
						set = function(info, value)
							if (value == select(2, UnitClass("player"))) then value = nil end
							MayronUI.db.profile.theme = value;							
						end,
						values = {
							["DEATHKNIGHT"] = "Death Knight",
							["DRUID"] = "Druid",
							["HUNTER"] = "Hunter",
							["MAGE"] = "Mage",
							["MONK"] = "Monk",
							["PALADIN"] = "Paladin",
							["PRIEST"] = "Priest",
							["ROGUE"] = "Rogue",
							["SHAMAN"] = "Shaman",
							["WARLOCK"] = "Warlock",
							["WARRIOR"] = "Warrior",
							["Custom"] = "Custom",
						},
						style = "radio",
					},
					sep3 = {
						order = 10.1,
						type = "description",
						name = "\n",
						width = "full",
					},
					header4 = {
						order = 10.5,
						type = "header",
						name = "Miscellaneous",
					},
					Message = {
						order = 10.6,
						type = "description",
						name = "Leatrix Plus contains most of the General settings you will need to use.\n\nMake sure that you do not uninstall this AddOn!\n",
						width = "full",
					},
					["Toggle"] = {
						order = 11,
						type = "execute",
						name = "Leatrix Plus",
						func = function() 
							if (IsAddOnLoaded("Leatrix_Plus") and leatrix) then
								leatrix();
							else
								print("Missing AddOn: Leatrix Plus!");
							end
						end,
					},	
				},
			},
		},
	}
	local modules = {"BottomUI", "CastBars", "ChatBox", "MiniMap", "SideBars", "ReskinUI"}
	
	for i, mod in ipairs(modules) do
		if ( _G["MayronUI_"..mod]:IsEnabled() ) then
			tbl.args[mod] = {
				type = "group",
				name = mod,
				order = i + 1,
				args = ModOptions["MayronUI_"..mod.."_Options"],
			}
			tbl.args[mod].args.main = {
				type = "execute",
				order = 1,
				name = "Disable Module",
				desc = "Will require a Reload of the UI to take effect.",
				func = function(info)
					if (MayronUI.db.profile.modules[mod]) then
						tbl.args[mod].args.main.name = "Enable Module";
						print(mod.." Module will be Disabled after a Reload of the UI (/rl).")
						MayronUI.db.profile.modules[mod] = false;
					else
						tbl.args[mod].args.main.name = "Disable Module";
						print(mod.." Module will be Enabled after a Reload of the UI (/rl).")
						MayronUI.db.profile.modules[mod] = true;
					end
				end,
			}
			
			-- Fit in expansion tables to modules here:
			if (mod == "ReskinUI") then
				local loc = tbl.args[mod].args.BlizzardFrames.args; -- set location
				local frames = {"Achievement Frame", "Addon Frame", "Archaeology Frame", "Auction Frame", "Bags/Inventory", "Bank Frame", "Calendar", "Challenges Frame", "Character Frame", "Chat Config Frame", "Color Picker Frame", "Compact Raid Manager", "Dressing Room", "Drop Down List", "Encounter Journal", "Friends Frame", "Game Menu", "Game Tooltips", "Garrison UI", "Glyph Frame", "Guild Bank", "Guild Control", "Guild Frame", "Guild Invite Frame", "Help Frame", "Inspect Frame", "Interface Options", "Item Socketing Frame", "Item Upgrade Frame", "Key Binding Frame", "LFD Role Check Popup", "LFG Dungeon Ready", "Looking For Guild", "Loot Frame", "Macro Frame", "Mail Frame", "Merchant Frame", "Mount/Pet Journal", "Mirror Timers", "Pet Battle Frame", "PVE Frame", "PVP Frame", "PVP Ready Dialog", "Quest Choice", "Quest Frames", "Ready Check Frame", "Score Frame", "Spell Book", "Static Popup Frames", "System Options", "Tabard Frame", "Talent Frame", "Taxi Frame", "Time Manager Frame", "Trade Frame", "Trade Skill Frame", "Trainer Frame", "Transmog Frame", "Void Storage", "World Map"};

				for i, v in ipairs(frames) do
					loc[v] = {
						order = i,
						type = "toggle",
						name = v,
						get = function() 
							if (not MayronUI.db.profile.ReskinUI[v]) then return true end
						end,
						set = function(info, value)
							if (value) then MayronUI.db.profile.ReskinUI[v] = nil else MayronUI.db.profile.ReskinUI[v] = true end
						end,				
					}
				end
				loc = tbl.args[mod].args.AddonFrames.args;
				frames = {"Bagnon", "MayronUI", "Recount", "ColorPickerPlus", "MikScrollingBattleText"};
				for i, v in ipairs(frames) do
					loc[v] = {
						order = i,
						type = "toggle",
						name = v,
						get = function() 
							if (not MayronUI.db.profile.ReskinUI[v]) then return true end
						end,
						set = function(info, value)
							if (value) then MayronUI.db.profile.ReskinUI[v] = nil else MayronUI.db.profile.ReskinUI[v] = true end
						end,				
					}
				end
			elseif (mod == "BottomUI") then
				local loc = tbl.args[mod].args.Artwork.args;
				local defaults = {["Action Bars"] = {751, 40, 1}, ["Unit Frames"] = {102, 180, 0.8}, ["Data Fields"] = {751, 20, 1}, ["Unit Names"] = {232, 18, 0.8}, ["Portrait Gradients"] = {100, 20, 0.4}}
				for id, frame in ipairs({"Action Bars", "Unit Frames", "Data Fields", "Unit Names", "Portrait Gradients"}) do
					local width = ( (frame == "Unit Frames" and "Center Width") or "Width");
					loc[frame] = {				
						type = "group",
						name = frame,
						order = id,
						args = {
							[width] = {
								order = 1,
								type = "range",
								name = width,
								desc = "Default is "..defaults[frame][1],
								min = 50,
								max = 1200,
								step = 1,
							},
							["Height"] = {
								order = 2,
								type = "range",
								name = ((id == 1) and "Retract Height") or ("Height"),
								min = 10,
								max = ((frame == "Portrait Gradients") and 100) or 300,
								step = 1,
							},
							["Alpha"] = {
								order = 4,
								type = "range",
								name = "Alpha",
								desc = "Default is "..defaults[frame][3],
								min = 0,
								max = 1,
								step = 0.1,
							},
						},
					}
					if (frame == "Action Bars") then
						loc[frame].args["Height"].desc = "Default is "..defaults[frame][2].."\n\nRetract Height cannot be greater or equal to the Expand Height.";
					else
						loc[frame].args["Height"].desc = "Default is "..defaults[frame][2]
					end
					if (frame == "Action Bars") then
						loc[frame].args["Expand Height"] = {
							order = 3,
							type = "range",
							name = "Expand Height",
							desc = "Default is 75",
							min = 55,
							max = 335,
							step = 1,
						}						
					elseif (frame == "Data Fields") then
						loc[frame].args["Texture"] = {
							order = 3,
							type = "select",
							dialogControl = "LSM30_Background",
							name = "Texture",
							desc = "Alter the texture used for this frame.",
							values = AceGUIWidgetLSMlists.background,
						}
						loc[frame].args["Color"] = {
							order = 4,
							type = "color",
							name = "Color",
							desc = "Alter the color of the Texture used.",
						}						
					elseif (frame == "Unit Frames" or frame == "Unit Names") then
						loc[frame].args.sep1 = {
							order = 4.2,
							type = "description",
							name = "\n",
							width = "full",
						}
						loc[frame].args["Use Color"] = {
							order = 5,
							type = "toggle",
							name = "Use Background Color",
							get = function()
								return MayronUI.db.profile.BottomUI[frame]["Use Color"];
							end,
							set = function(info, value)
								if (not value) then value = nil end
								MayronUI.db.profile.BottomUI[frame]["Use Color"] = value;
								MayronUI_BottomUI:UpdateSettings(frame, "Color")
							end,
						}
						loc[frame].args["Color"] = {
							order = 6,
							type = "color",
							name = "Background Color",
							disabled = function() return (not MayronUI.db.profile.BottomUI[frame]["Use Color"]) end,
						}						
						if (frame == "Unit Names") then 
							loc[frame].args["Inner Offset"] = {
								order = 4.1,
								type = "range",
								name = "Inner Offset",
								desc = "Increasing this value will move the Unit Name Frames closer to the center of the UI.\n\nDefault is 23",
								min = 0,
								max = 100,
								step = 1,
							}
							loc[frame].args["Inner Offset"] = {
								order = 4.1,
								type = "range",
								name = "Inner Offset",
								desc = "Increasing this value will move the Unit Name Frames closer to the center of the UI.\n\nDefault is 23",
								min = 0,
								max = 100,
								step = 1,
							}								
							loc[frame].args["Target Class Colored"] = {
								order = 5,
								type = "toggle",
								name = "Target Class Colored",
								desc = "If disabled, this frame will never change color depending upon the target class.",
								get = function(info)
									if (MayronUI.db.profile.BottomUI["Unit Names"]["Target Class Colored"]) then return false else return true end
								end,
								set = function(info, value)
									if (not value) then value = true else value = nil end
									MayronUI.db.profile.BottomUI["Unit Names"]["Target Class Colored"] = value;
									MayronUI_BottomUI:UpdateSettings("Unit Names", "Color");
								end,
							}
						else
							loc[frame].args["Left Width"] = {
								order = 0.2,
								type = "range",
								name = "Left Width",
								desc = "Default is 324",
								min = 200,
								max = 600,
								step = 1,
							}
							loc[frame].args["Right Width"] = {
								order = 0.4,
								type = "range",
								name = "Right Width",
								desc = "Default is 324",
								min = 200,
								max = 600,
								step = 1,
							}
							loc[frame].args["Y Offset"] = {
								order = 4.1,
								type = "range",
								name = "Y Offset",
								desc = "Move the Unit Frame background art higher or lower.\n\nDefault is -110",
								min = -300,
								max = 300,
								step = 1,
							}
							loc[frame].args["Classic Mode"] = {
								order = 6,
								type = "toggle",
								name = "Classic Mode",
								desc = "Enabling this will maintain the Symmetric feel of the unit frames background frame by no longer hiding the right side background texture when the target frame no longer exists.",
								get = function()
									if (MayronUI.db.profile.BottomUI.ClassicMode) then return true end
								end,
								set = function(info, value)
									if (not value) then value = nil end
									MayronUI.db.profile.BottomUI.ClassicMode = value;
									MUI_Bottom:GetScript("OnEvent")(MUI_Bottom, "PLAYER_TARGET_CHANGED");
								end,
							}
						end
					elseif (frame == "Portrait Gradients") then 
						loc[frame].args["Width"] = nil
						loc[frame].args["Enable"] = {
							order = 1,
							type = "toggle",
							name = "Enable",
							desc = "Enable the Gradient over the Unit Frame Portraits.",
							get = function(info)
								if (MayronUI.db.profile.BottomUI["Portrait Gradients"]["Disable"]) then return false else return true end
							end,
							set = function(info, value)
								if (not value) then value = true else value = nil end
								MayronUI.db.profile.BottomUI["Portrait Gradients"]["Disable"] = value;
								if (value) then
									if (_G["MUI_targetGradient"]) then
										MUI_targetGradient:Hide()
									end
									if (_G["MUI_playerGradient"]) then
										MUI_playerGradient:Hide()
									end						
								else
									if (_G["MUI_targetGradient"] and _G["MUI_playerGradient"]) then
										MUI_targetGradient:Show()
										MUI_playerGradient:Show()
									else
										MayronUI_BottomUI:SetupPortraitGradients();
									end
								end
							end,
						}
						loc[frame].args["Target Class Colored"] = {
							order = 1,
							type = "toggle",
							name = "Target Class Colored",
							desc = "If disabled, this frame will never change color depending upon the target class.",
							get = function(info)
								if (MayronUI.db.profile.BottomUI["Portrait Gradients"]["Target Class Colored"]) then return false else return true end
							end,
							set = function(info, value)
								if (not value) then value = true else value = nil end
								MayronUI.db.profile.BottomUI["Portrait Gradients"]["Target Class Colored"] = value;
								if (not value) then
									if (UnitExists("target")) then
										local c = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[(select(2, UnitClass("target")))];
										if (c and c.r) then
											if (_G["MUI_targetGradient"]) then
												MUI_targetGradient.t:SetGradientAlpha("VERTICAL", c.r, c.g, c.b, 0, c.r, c.g, c.b, 0.4)
											end
										end
									end
								else
									if (UnitExists("target")) then
										if (_G["MUI_targetGradient"]) then
											MUI_targetGradient.t:SetGradientAlpha("VERTICAL", MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 0, MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 0.4)
										end
									end
								end
							end,
						}
						loc[frame].args.sep1 = {
							order = 2,
							type = "description",
							name = "\n",
							width = "full",
						}
						loc[frame].args["Height"].order = 3;
					end
				end
			elseif (mod == "ChatBox") then
				local loc = tbl.args[mod].args.ChatBoxes.args;
				local fValues = {
					["Character"] = "Character",
					["Bags"] = "Bags",
					["Friends"] = "Friends",
					["Guild"] = "Guild",
					["Help Menu"] = "Help Menu",
					["PVP"] = "PVP",
					["Spell Book"] = "Spell Book",
					["Talents"] = "Talents",
					["Achievements"] = "Achievements",
					["Glyphs"] = "Glyphs",
					["Calendar"] = "Calendar",
					["LFD"] = "LFD",
					["Raid"] = "Raid",
					["Encounter Journal"] = "Encounter Journal",
					["Mounts and Pets"] = "Mounts and Pets",
					["Macros"] = "Macros",
					["World Map"] = "World Map",
					["Reputation"] = "Reputation",
					["PVP Score"] = "PVP Score",
					["Currency"] = "Currency",
					[" "] = "None",
					["Switch"] = "Switch Layout",
				}
				for cbID, corner in ipairs({"Top Left", "Top Right", "Bottom Left", "Bottom Right"}) do
					local cb;									
					if (not MayronUI.db.profile.ChatBox[corner]) then loc[corner] = nil else
						cb = _G["MUI_ChatFrame"..cbID];
						cb.Name = "MUI_ChatFrame"..cbID;
						cbID = MayronUI.db.profile.ChatBox[corner];
						loc[corner] = {
							type = "group",
							name = corner,
							desc = "Controls the functionality of Chat Frame Buttons.",
							childGroups = "select",
							order = cbID,
							args = {		
								SideButton = {				
									type = "group",
									name = "Side Button",
									desc = "You can change the Chat Frame Art buttons to do something else instead.",
									order = 4,
									args = {
										func = {
											order = 1,
											type = "select",
											name = "Side Button",
											desc = "Choose which Function should be triggered when this button is clicked.",
											get = function()
												return MayronUI.db.profile.ChatFrame[corner].functions[10];
											end,
											set = function(info, value)
												MayronUI.db.profile.ChatFrame[corner].functions[10] = value;
												if (value == "Switch") then
													_G[cb.Name.."LayoutButton"]:SetText( " "..(MayronUI.db.profile.CurrentLayout:sub(1, 1):upper()) );
													_G[cb.Name.."LayoutButton"]:GetFontString():Show()
												else
													_G[cb.Name.."LayoutButton"]:GetFontString():SetText(value);
													_G[cb.Name.."LayoutButton"]:GetFontString():Hide()
												end													
											end,
											values = fValues,									
										},						
									},
								},
								Appearance = {				
									type = "group",
									name = "Appearance",
									order = 5,
									args = {
										header = {
											order = 0.5,
											type = "header",
											name = "Enabled Sub Frames",
										},
										SideButton = {
											order = 1,
											type = "toggle",
											name = "Side Button",
											get = function() 
												return not MayronUI.db.profile.ChatFrame[corner].SideButton;
											end,
											set = function(info, value)
												if (value) then												
													MayronUI.db.profile.ChatFrame[corner].SideButton = nil;
													_G[cb.Name.."LayoutButton"]:Show()
												else
													MayronUI.db.profile.ChatFrame[corner].SideButton = true; 
													_G[cb.Name.."LayoutButton"]:Hide()
												end
											end,
										},
										BackgroundWindow = {
											order = 2,
											type = "toggle",
											name = "Background Window",
											get = function() 
												return not MayronUI.db.profile.ChatFrame[corner].BackgroundWindow
											end,
											set = function(info, value)
												if (value) then												
													MayronUI.db.profile.ChatFrame[corner].BackgroundWindow = nil;
													cb.Window:Show()
												else
													MayronUI.db.profile.ChatFrame[corner].BackgroundWindow = true;
													cb.Window:Hide()
												end
											end,
										},
										TabBackground = {
											order = 3,
											type = "toggle",
											name = "Tab Background",
											get = function() 
												return not MayronUI.db.profile.ChatFrame[corner].TabBackground
											end,
											set = function(info, value)
												if (value) then												
													MayronUI.db.profile.ChatFrame[corner].TabBackground = nil;
													_G[cb.Name.."Tabs"]:Show()
												else
													MayronUI.db.profile.ChatFrame[corner].TabBackground = true;
													_G[cb.Name.."Tabs"]:Hide()
												end
											end,
										},
										sep1 = {
											order = 3.1,
											type = "description",
											name = "\n",
											width = "full",
										},
										header1 = {
											order = 3.5,
											type = "header",
											name = "Move Sub-Frames",
										},
										TabYOffset = {
											order = 4,
											type = "range",
											name = "Tab BG - Y Offset",
											desc = "Change the position of the Tab Sub-Frame vertically. Default is -12.",
											min = -262,
											max = -5,
											step = 1,
											get = function()
												return select(5, _G[cb.Name.."Tabs"]:GetPoint())												
											end,
											set = function(info, value) 
												local p, rf, rp, x, _ = _G[cb.Name.."Tabs"]:GetPoint()
												if ( value > -200 and value < -70) then
													if (corner:find("Right")) then x = 3 else x = -3 end
													_G[cb.Name.."Tabs"]:SetPoint(p, rf, rp, x, value)
												else
													_G[cb.Name.."Tabs"]:SetPoint(p, rf, rp, 0, value)
												end
												if (value == -12) then 
													MayronUI.db.profile.ChatFrame[corner].TabYOffset = nil;
												else
													MayronUI.db.profile.ChatFrame[corner].TabYOffset = value;
												end
											end,
										},
										WindowYOffset = {
											order = 4,
											type = "range",
											name = "Window BG - Y Offset",
											desc = "Change the position of the Window Sub-Frame vertically. Default is -2 for top frames and 12 for bottom frames.",
											min = -10,
											max = 50,
											step = 1,
											get = function()
												return select(5, cb.Window:GetPoint())											
											end,
											set = function(info, value) 
												local p, rf, rp, x, _ = cb.Window:GetPoint()												
												cb.Window:SetPoint(p, rf, rp, x, value)
												if ( (corner:find("Top") and value == -2) or (corner:find("Bottom") and value == 12) ) then 
													MayronUI.db.profile.ChatFrame[corner].WindowYOffset = nil;
												else
													MayronUI.db.profile.ChatFrame[corner].WindowYOffset = value;
												end
											end,
										},
										sep2 = {
											order = 4.1,
											type = "description",
											name = "\n",
											width = "full",
										},
										header2 = {
											order = 4.5,
											type = "header",
											name = "Resize Sub-Frames",
										},
										WindowResizeWidth = {
											order = 5,
											type = "range",
											name = "Window BG - Width",
											desc = "Change the width of the Window Sub-Frame. Default is 367.",
											min = 200,
											max = 500,
											step = 1,
											get = function()
												return cb.Window:GetWidth()											
											end,
											set = function(info, value) 										
												cb.Window:SetWidth(value)
												if (value == 367) then 
													MayronUI.db.profile.ChatFrame[corner].WindowWidth = nil;
												else
													MayronUI.db.profile.ChatFrame[corner].WindowWidth = value;
												end
											end,
										},
										WindowResizeHeight = {
											order = 6,
											type = "range",
											name = "Window BG - Height",
											desc = "Change the height of the Window Sub-Frame. Default is 248.",
											min = 200,
											max = 500,
											step = 1,
											get = function()
												return cb.Window:GetHeight()											
											end,
											set = function(info, value) 										
												cb.Window:SetHeight(value)
												if (value == 248) then 
													MayronUI.db.profile.ChatFrame[corner].WindowHeight = nil;
												else
													MayronUI.db.profile.ChatFrame[corner].WindowHeight = value;
												end
											end,
										},
									},
								},
							},
						} -- end of SideBar and Appearance sub tables.						
						for rowNum = 1, 3 do -- Add in button sub tables.
							local funcID = 0;							
							loc[corner].args["ButtonRow"..rowNum] = {				
								type = "group",
								name = "Button Row "..rowNum,
								desc = "You can change the Chat Frame Art buttons to do something else instead.",
								order = rowNum,
								args = {},
							}							
							if (rowNum > 1) then
								if (rowNum == 2) then funcID = 3 else funcID = 6 end
								loc[corner].args["ButtonRow"..rowNum].args.Keybinding = {
									order = 1,
									type = "select",
									name = "Modifier Key",
									desc = "Choose which Modifier Key should be held down while out of combat to show this row of buttons.",
									get = function()
										return MayronUI.db.profile.ChatFrame[corner].ModKeys[rowNum - 1];
									end,
									set = function(info, value)
										MayronUI.db.profile.ChatFrame[corner].ModKeys[rowNum - 1] = value;								
									end,
									values = {
										none = "none",
										S = "SHIFT",
										C = "CTRL",
										A = "ALT",
										SC = "SHIFT + CTRL",
										SA = "SHIFT + ALT",
										CA = "CTRL + ALT",
										SCA = "SHIFT + CTRL + ALT",
									},
									style = "radio",
								}
								loc[corner].args["ButtonRow"..rowNum].args.sep1 = {
									order = 2,
									type = "description",
									name = " ",
									width = "full",
								}
							end
							for id, b in ipairs({"Left", "Middle", "Right"}) do
								local bName = b.."Button";
								loc[corner].args["ButtonRow"..rowNum].args[bName] = {
									order = ((funcID > 2) and 4) or 1,
									type = "select",
									name = bName,
									desc = "Choose which Function should be triggered when this button is clicked.",
									get = function()
										return MayronUI.db.profile.ChatFrame[corner].functions[funcID + id]; -- this is 1, 4, 7
									end,
									set = function(info, value)
										MayronUI.db.profile.ChatFrame[corner].functions[funcID + id] = value;
										if (rowNum == 1) then 
											if (value == "Switch") then
												_G[cb.Name..bName]:GetFontString():SetText( " "..(MayronUI.db.profile.CurrentLayout:sub(1, 1):upper()) )
											else
												_G[cb.Name..bName]:GetFontString():SetText(value);	
											end
										end
									end,
									values = fValues,
								}
							end
						end
					end
				end
			elseif (mod == "CastBars") then
				local loc = tbl.args[mod].args.CastBars.args;
				local id = 0;
				for barType, subtbl in pairs(MayronUI.db.profile.CastBars) do -- subtbl is the settings for each bar
					if (not MayronUI.db.profile.CastBars[barType].Enabled) then loc[barType] = nil else
						id = id + 1;
						loc[barType] = { 
							type = "group",
							name = barType,
							desc = "Control the Appearence of the "..barType.." Cast bar.",
							childGroups = "select",
							order = id,
							get = function(info)
								local _, _, _, _, property = unpack(info)
								local value =  MayronUI.db.profile.CastBars[barType][property];
								local bar = _G["MUI_"..barType.."CastBar"];								
								if ( (not value) and (property ~= "AnchorSUF") ) then
									local s = {						
										["point"] = function() return ( select(1, bar:GetPoint()) ) end,
										["relativeFrame"] = function() 
											return select(2, bar:GetPoint()):GetName(); 
										end,
										["relativePoint"] = function() return ( select(3, bar:GetPoint()) ) end,
										["ofsx"] = function() return ( select(4, bar:GetPoint()) ) end,
										["ofsy"] = function() return ( select(5, bar:GetPoint()) ) end,
										["width"] = function() return bar:GetWidth() end,
										["height"] = function() return bar:GetHeight() end,
									}
									value = s[property]();									
								end								
								if (property == "ofsx" or property == "ofsy") then
									value = tostring(value)
								end								
								return value;
							end,
							set = function(info, value)								
								local _, _, _, _, property = unpack(info)
								local bar = _G["MUI_"..barType.."CastBar"];
								local loc = MayronUI.db.profile.CastBars[barType];								
								if (property == "ofsx" or property == "ofsy") then
									value = tonumber(value)								
								end
								local SUFframe = _G["SUFUnit"..barType:lower()];
								if (property == "AnchorSUF" and value and SUFframe) then 
									loc["width"] = (SUFframe:GetWidth() * SUFframe:GetScale());
									loc["height"] = (28 * SUFframe:GetScale());
								end							
								if (not loc.set and property ~= "AnchorSUF") then -- get the current properties for never been set before
									loc["point"], _, loc["relativePoint"], loc["ofsx"], loc["ofsy"] = bar:GetPoint();
									loc["relativeFrame"] = select(2, bar:GetPoint()):GetName()
									loc["width"], loc["height"] = bar:GetSize();
									loc.set = true;	
								end
								loc[property] = value;	-- add new property								
								MayronUI:PositionCastBars(barType)								
							end,
							args = {
								headerSize = {
									order = 0.5,
									type = "header",
									name = "Size",
								},
								AnchorSUF = {
									order = 1,
									type = "toggle",
									desc = "Allows you to anchor the Cast bar to the equivalent Shadowed Unit Frame (Player Cast Bar to the Player Unit Frame for example). This will automatically set the Width of the Cast bar to match the Width of the Unit Frame.",
									name = "Enable SUF Anchoring",
								},
								sep1 = {
									order = 2.1,
									type = "description",
									name = "",
									width = "full",
								},
								width = {
									order = 3,
									type = "range",
									name = "Width",
									min = 50,
									max = 500,
									step = 1,
									desc = "This setting can only be used if the Cast Bar is not anchored to a Shadowed Unit Frame.",
									--hidden = function() if (MayronUI.db.profile.CastBars[barType].AnchorSUF) then return true end end,
								},
								height = {
									order = 4,
									type = "range",
									name = "Height",
									min = 5,
									max = 200,
									step = 1,
								},								
								sep2 = {
									order = 4.1,
									type = "description",
									name = " ",
									width = "full",
								},
								headerPosition = {
									order = 4.5,
									type = "header",
									name = "Position",
								},								
								ofsx = {
									order = 7,
									type = "input",
									name = "X Offset",
								},
								ofsy = {
									order = 8,
									type = "input",
									name = "Y Offset",
								},
								panelSep2 = {
									order = 9,
									type = "description",
									name = "",
									width = "full",
								},
								point = {
									order = 10,
									type = "select",
									name = "Frame Point",
									hidden = function() if (MayronUI.db.profile.CastBars[barType].AnchorSUF) then return true end end,
									values = { 
										TOP = "Top",
										TOPLEFT = "Top Left",
										TOPRIGHT = "Top Right",
										BOTTOM = "Bottom",
										BOTTOMLEFT = "Bottom Left",
										BOTTOMRIGHT = "Bottom Right",
										LEFT = "Left",
										RIGHT = "Right",
										CENTER = "Center",
									},
									style = "dropdown",
								},
								relativeFrame = {
									order = 11,
									type = "input",
									name = "Relative Frame",
									hidden = function() if (MayronUI.db.profile.CastBars[barType].AnchorSUF) then return true end end,
								},
								relativePoint = {
									order = 12,
									type = "select",
									name = "Relative Point",
									hidden = function() if (MayronUI.db.profile.CastBars[barType].AnchorSUF) then return true end end,
									values = { 
										TOP = "Top",
										TOPLEFT = "Top Left",
										TOPRIGHT = "Top Right",
										BOTTOM = "Bottom",
										BOTTOMLEFT = "Bottom Left",
										BOTTOMRIGHT = "Bottom Right",
										LEFT = "Left",
										RIGHT = "Right",
										CENTER = "Center",
									},
									style = "dropdown",
								},
								sep3 = {
									order = 12.1,
									type = "description",
									name = " ",
									width = "full",
								},
							},
						}
					end
				end				
			end
		else
			tbl.args[mod] = {
				type = "group",
				name = mod,
				order = i + 1,
				args = {
					main = {
						type = "execute",
						order = 1,
						name = "Enable Module",
						desc = "Will require a Reload of the UI to take effect.",
						func = function(info)
							if (MayronUI.db.profile.modules[mod]) then
								tbl.args[mod].args.main.name = "Enable Module";
								print(mod.." Module will be Disabled after a Reload of the UI (/rl).")
								MayronUI.db.profile.modules[mod] = false;
							else
								tbl.args[mod].args.main.name = "Disable Module";
								print(mod.." Module will be Enabled after a Reload of the UI (/rl).")
								MayronUI.db.profile.modules[mod] = true;
							end
						end,
					},
				},
			}
		end
	end
	return tbl;
end
	
function MayronUI:OnInitialize() 
	
	local options = {
		type = "group",
		name = "MayronUI 4.0",
		childGroups = "tab",			
		args = {
			-- Main funcitons:
			Config = {
				order = 1,
				type = "execute",
				name = "Toggle MayronUI Config Menu",
				guiHidden = true,
				func = function()
					if (InCombatLockdown()) then
						print("Cannot access config menu while in combat.")
					else
						MayronUI.LoadGUI();
						LibStub("AceConfigDialog-3.0"):Open("MayronUI Config");
					end
				end,
			},
			Install = {
				order = 2,
				type = "execute",
				name = "Toggle Installer",
				guiHidden = true,
				func = function()
					if (MUI_Setup) then 
						MUI_Setup:Show();
					else
						MayronUI_SetupUI:Enable()
					end
				end,
			},
			Modules = {
				order = 3,
				type = "execute",
				name = "Load Options",
				cmdHidden = true,
				func = function()
					MayronUI.LoadGUI();
				end,
			},
		},
	}	
	
	function self.LoadGUI()		
		options.args.Modules = GetModules();
		collectgarbage("collect");
	end
	
	self.db = LibStub("AceDB-3.0"):New("MayronUIdb", defaults)
	options.args.Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	-- Setup MUI_COLOR:
	if (MayronUI.db.profile.theme) then
		if (MayronUI.db.profile.theme == "Custom") then
			if (MayronUI.db.profile["Custom Color"]) then
				MUI_COLOR = MayronUI.db.profile["Custom Color"];				
			end			
		else
			MUI_COLOR = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[MayronUI.db.profile.theme];
			MUI_PREFIX = "|c"..RAID_CLASS_COLORS[MayronUI.db.profile.theme].colorStr.."MayronUI:|r"; 
		end
	end
	
	-- Enable Modules:
	if (MayronUI.db.profile.install) then
		MayronUI_SetupUI:Enable();
	end	
	for k, v in pairs(self.db.profile.modules) do
		if (v) then _G["MayronUI_"..k]:Enable() end
	end
	
	if (not MayronUI_BottomUI:IsEnabled()) then
		hooksecurefunc(ShadowUF, "LoadUnits", function()
			if (not MayronUI_BottomUI:IsEnabled()) then -- have to double check incase enabled later on.
				local currentProfile = ShadowUF.db:GetCurrentProfile();
				local SUF = ShadowedUFDB["profiles"][currentProfile]["positions"]["targettarget"];
				SUF["anchorTo"] = "UIParent"; 	
				if (SUF["anchorTo"] == "MUI_Bottom") then 		
					SUF["anchorTo"] = "UIParent";
					SUF["relativePoint"] = "BOTTOM";
					SUF["y"] = "65";
				end
				ShadowUF.Layout:Reload();
			end
		end)
	end
	
	-- Setup Config Menu
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MayronUI Config", options, {"mui", "mayronui"})
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MayronUI Config", "MayronUI")
	
	local f = CreateFrame("Frame")
	f:RegisterEvent("PLAYER_REGEN_DISABLED");
	f:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT");
	f:RegisterEvent("PLAYER_ENTERING_WORLD");	
	f:SetScript("OnEvent", function(self, event)
		if (event == "PLAYER_REGEN_DISABLED") then
			if (not MayronUI.db.global.General["Inventory"] and ContainerFrame1:IsVisible()) then
				ToggleBackpack();
			end	
		elseif (not MayronUI.db.global.General["Objectives Tracker"]) then
			if (IsInInstance()) then
				if (not f.hiddenOT) then					
					if (ObjectiveTrackerBlocksFrame:IsVisible()) then
						ObjectiveTracker_Collapse();
						f.hiddenOT = true;
					end
				end
			else
				if (f.hiddenOT) then					
					if (not ObjectiveTrackerBlocksFrame:IsVisible()) then
						ObjectiveTracker_Expand();
						f.hiddenOT = nil;
					end
				end
			end
		end
	end)
	if ( MayronUI.db.profile.ReAnchor ) then		
		f:RegisterEvent("PLAYER_LOGIN")
		f:HookScript("OnEvent", function(self, event)	
			if ( event == "PLAYER_LOGIN" and IsAddOnLoaded("Recount") ) then
				Recount_MainWindow:ClearAllPoints();
				Recount_MainWindow:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -2, 2)
				Recount_MainWindow:SaveMainWindowPosition()
				if ( IsAddOnLoaded("Omen") ) then
					OmenAnchor:ClearAllPoints();
					OmenAnchor:SetPoint("BOTTOMRIGHT", Recount_MainWindow, "TOPRIGHT", 0, -6);
					Omen:SetAnchors();					
				end
				MayronUI.db.profile.ReAnchor = nil;
			end
		end)
	end
	
	hooksecurefunc("ObjectiveTracker_Update", function()
		ObjectiveTrackerFrame:SetMovable(true);
		ObjectiveTrackerFrame:SetUserPlaced(true);
		ObjectiveTrackerFrame:SetHeight(600);
		
		local mui = MayronUI.db.global.Tracker;		
		if ((MayronUI_SideBars:IsEnabled() and MUI_SideBars) and mui["Position Tracker"]) then
			ObjectiveTrackerFrame:ClearAllPoints();
			ObjectiveTrackerFrame:SetPoint("TOPRIGHT", "Minimap", "BOTTOMRIGHT", -(MUI_SideBars:GetWidth() + 40), -30);
			mui["x"], mui["specialY"] = ObjectiveTrackerFrame:GetCenter();	
			mui["specialX"] = -40;
			mui["Point"] = nil;
			MayronUI.db.profile.MiniMap["Positioned WF"] = nil; -- older out-dated variable;
			mui["Relative Point"] = nil;
			mui["Position Tracker"] = nil;
		end
		MayronUI:SetTrackerPosition();
	end)	
	print("Welcome back "..UnitName("player")..".\n");
end

-- Text for Buttons
function MUI_TextOnLeave(self)
	if ( MUI_COLOR.r == 0.9 and MUI_COLOR.g == 0.9 and MUI_COLOR.b == 0.9 ) then
		self:GetFontString():SetTextColor(1, 1, 1, 1);
	else
		self:GetFontString():SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 1);
	end
end
function MUI_TextOnEnter(self)	
	if ( MUI_COLOR.r == 0.9 and MUI_COLOR.g == 0.9 and MUI_COLOR.b == 0.9 ) then
		self:GetFontString():SetTextColor(0.2, 0.6, 1, 1);
	else
		self:GetFontString():SetTextColor(1, 1, 1, 1);
	end
end

-- Slash Commands:
--  ReloadUI:
SLASH_RELOADUI1 = "/rl";
SlashCmdList.RELOADUI = ReloadUI;
-- Display FrameStack:
SLASH_FRAMESTK1 = "/fs"
SlashCmdList.FRAMESTK = function() 
	LoadAddOn('Blizzard_DebugTools')
	FrameStackTooltip_Toggle()
end
-- Ready Check:
SLASH_READYCHECK1 = "/rc"
SlashCmdList.READYCHECK = function() 
	DoReadyCheck()
end

-- Align:
do
	local grid
	local boxSize = 32

	local function Grid_Show()
		if not grid then
			Grid_Create()
		elseif grid.boxSize ~= boxSize then
			grid:Hide()
			Grid_Create()
		else
			grid:Show()
		end
	end

	local function Grid_Hide()
		if grid then
			grid:Hide()
		end
	end

	local isAligning = false
	SLASH_TOGGLEGRID1 = "/align"
	SlashCmdList["TOGGLEGRID"] = function(arg)
		if isAligning then
			Grid_Hide()
			isAligning = false
		else
			boxSize = (math.ceil((tonumber(arg) or boxSize) / 32) * 32)
		if boxSize > 256 then boxSize = 256 end    
			Grid_Show()
			isAligning = true
		end
	end

	function Grid_Create() 
		grid = CreateFrame('Frame', nil, UIParent) 
		grid.boxSize = boxSize 
		grid:SetAllPoints(UIParent) 

		local size = 2 
		local width = GetScreenWidth()
		local ratio = width / GetScreenHeight()
		local height = GetScreenHeight() * ratio

		local wStep = width / boxSize
		local hStep = height / boxSize

		for i = 0, boxSize do 
			local tx = grid:CreateTexture(nil, 'BACKGROUND') 
			if i == boxSize / 2 then 
				tx:SetTexture(1, 0, 0, 0.5) 
			else 
				tx:SetTexture(0, 0, 0, 0.5) 
			end 
			tx:SetPoint("TOPLEFT", grid, "TOPLEFT", i*wStep - (size/2), 0) 
			tx:SetPoint('BOTTOMRIGHT', grid, 'BOTTOMLEFT', i*wStep + (size/2), 0) 
		end 
		height = GetScreenHeight()
		
		do
			local tx = grid:CreateTexture(nil, 'BACKGROUND') 
			tx:SetTexture(1, 0, 0, 0.5)
			tx:SetPoint("TOPLEFT", grid, "TOPLEFT", 0, -(height/2) + (size/2))
			tx:SetPoint('BOTTOMRIGHT', grid, 'TOPRIGHT', 0, -(height/2 + size/2))
		end
		
		for i = 1, math.floor((height/2)/hStep) do
			local tx = grid:CreateTexture(nil, 'BACKGROUND') 
			tx:SetTexture(0, 0, 0, 0.5)
			
			tx:SetPoint("TOPLEFT", grid, "TOPLEFT", 0, -(height/2+i*hStep) + (size/2))
			tx:SetPoint('BOTTOMRIGHT', grid, 'TOPRIGHT', 0, -(height/2+i*hStep + size/2))
			
			tx = grid:CreateTexture(nil, 'BACKGROUND') 
			tx:SetTexture(0, 0, 0, 0.5)
			
			tx:SetPoint("TOPLEFT", grid, "TOPLEFT", 0, -(height/2-i*hStep) + (size/2))
			tx:SetPoint('BOTTOMRIGHT', grid, 'TOPRIGHT', 0, -(height/2-i*hStep + size/2))
		end
	end
end