do
	local defaults = {};
	
	local function ResetPoints()
		if (#defaults > 0) then
			for id, bar in ipairs(defaults) do
				bar:ClearAllPoints();
				if (id == 1) then
					bar:SetPoint("TOP", UIParent, "CENTER", 0, -100);
				else
					bar:SetPoint("TOP", defaults[id - 1], "BOTTOM", 0, -20);
				end
				bar:SetSize(270, 28)
			end
		end
	end
	
	local function CheckTable(bar) -- remove bar if found.
		for id, item in ipairs(defaults) do
			if (item == bar) then
				table.remove(defaults, id);
			end	
		end
	end
	
	function MayronUI:PositionCastBars(individual)
		if (MayronUI_CastBars:IsEnabled()) then
			for barType, tbl in pairs(MayronUI.db.profile.CastBars) do
				if ( (individual == barType) or (not individual) ) then
					local loc = MayronUI.db.profile.CastBars[barType];	
					local bar = _G["MUI_"..barType.."CastBar"];
					if (loc.Enabled) then
						local SUFframe = _G["SUFUnit"..barType:lower()];
						if ( loc["AnchorSUF"] and IsAddOnLoaded("ShadowedUnitFrames") and SUFframe ) then -- bars can be anchored to SUF and not set
							local s = {						
								point = "TOPLEFT",
								relativeFrame = "SUFUnit"..barType:lower(),
								relativePoint = "TOPLEFT",
								ofsx = loc["ofsx"] or 0,
								ofsy = loc["ofsy"] or 0,
								width = loc["width"] or (SUFframe:GetWidth() * SUFframe:GetScale()),
								height = loc["height"] or (28 * SUFframe:GetScale()),
							}
							bar:ClearAllPoints();
							bar:SetPoint(s.point, s.relativeFrame, s.relativePoint, s.ofsx, s.ofsy);
							bar:SetSize(s.width, s.height);	
							_G[bar:GetName().."Latency"]:SetHeight(s.height);
							
							-- XML TopLevel bug fix:
							if (_G["SUFUnit"..barType:lower()]) then
								local suf = _G["SUFUnit"..barType:lower()];
								if (suf:GetFrameLevel() > bar:GetFrameLevel()) then
									bar:SetFrameLevel(suf:GetFrameLevel() + 50);
								end
							end							
							
							if (individual) then CheckTable(bar) end							
						elseif (loc.set) then -- GUI sets the saved variable settings, (no need to store all unchanged ones! GUI also sets loc.set to true.
							bar:ClearAllPoints();
							local f;
							if (_G[loc["relativeFrame"]]) then 
								f = _G[loc["relativeFrame"]];
							else 
								f = UIParent; 
								loc["relativeFrame"] = "UIParent";
							end
							bar:SetPoint(loc["point"], f, loc["relativePoint"], loc["ofsx"], loc["ofsy"]);
							
							bar:SetSize(loc["width"], loc["height"]);							
							_G[bar:GetName().."Latency"]:SetHeight(loc["height"]);
						else
							table.insert(defaults, bar)		
						end
					else -- bar has been disabled! 
						CheckTable(bar);
					end
				end
			end
			ResetPoints()
		end
	end
end

function MayronUI_BottomUI:OnEnable()

	local STOPPING_POINT;
	local STARTING_POINT;
	
	function MayronUI_BottomUI:SetupPortraitGradients()
		for id, str in ipairs({"player", "target"}) do
			if (_G["SUFUnit"..str]) then 
				local gradient = CreateFrame("Frame", "MUI_"..str.."Gradient", _G["SUFUnit"..str]);
				gradient:SetSize(100, 20);
				gradient:SetPoint("TOPLEFT", _G["SUFUnit"..str], "TOPLEFT", 1, -1);
				gradient:SetPoint("TOPRIGHT", _G["SUFUnit"..str], "TOPRIGHT", -1, -1);
				gradient.t = gradient:CreateTexture();
				gradient.t:SetAllPoints(gradient);
				gradient.t:SetTexture(1, 1, 1, 1);
				gradient.t:SetGradientAlpha("VERTICAL", MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 0, MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 0.4);
				gradient:SetFrameLevel(5);
				gradient:Show();
			end
		end
	end
	
	local function print(...)
		DEFAULT_CHAT_FRAME:AddMessage(string.join(" ", MUI_PREFIX, tostringall(...)));
	end

	local function RegisterEvents(self, ...)
		for i=1,select('#', ...) do
			self:RegisterEvent(select(i, ...));
		end
	end
	local Startup = true;
	
	-- default table:
	local C = {
		["Action Bars"] = {
			["Width"] = 751,
			["Height"] = 40,
			["Alpha"] = 1,
			["Special Height"] = 75,
		},
		["Unit Frames"] = {
			["Left Width"] = 324,
			["Right Width"] = 324,
			["Center Width"] = 102,
			["Height"] = 180,
			["Y Offset"] = -110,
			["Alpha"] = 0.8,
			["Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b},
		},
		["Data Fields"] = {
			["Width"] = 751,
			["Height"] = 20,
			["Alpha"] = 1,
			["Texture"] = "MUI_DataFields",
			["Color"] = {r = 0.4, g = 0.4, b = 0.4},
		},
		["Unit Names"] = {
			["Width"] = 232,
			["Height"] = 18,
			["Alpha"] = 0.8,
			["Inner Offset"] = 23,
			["Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b},
		},
		["Portrait Gradients"] = {				
			["Height"] = 20,
			["Alpha"] = 0.4,
		},
	}
	
	local muiBottom = CreateFrame("Frame", "MUI_Bottom", UIParent, "MUI_BottomTemplate");
	
	function MayronUI_BottomUI:UpdateSettings(arg1, arg2) -- ftype and property			
		local f = {
			["Action Bars"] = function()
				if (MayronUI.db.profile.BottomUI["Expand"]) then
					MUI_Bottom:SetPoint("TOP", UIParent, "BOTTOM", 0, C["Action Bars"]["Special Height"] + 20);
					MUI_BottomSwitchButton:SetText("Retract");
				else
					MUI_Bottom:SetPoint("TOP", UIParent, "BOTTOM", 0, C["Action Bars"]["Height"] + 20);
				end
				MUI_Bottom["ActionBars"]:SetWidth(C["Action Bars"]["Width"])
				MUI_Bottom["ActionBars"]:SetAlpha(C["Action Bars"]["Alpha"]);
			end,
			["Unit Frames"] = function()				
				for id, str in ipairs({"left", "right", "center"}) do	
					local alpha = 0;
					if ( (str == "left") or ( (MUI_Bottom[str]:GetAlpha()) > 0) ) then
						MUI_Bottom[str]:SetAlpha(C["Unit Frames"]["Alpha"]);
						alpha = C["Unit Frames"]["Alpha"];
					end
					if (MayronUI.db.profile.BottomUI["Unit Frames"]["Use Color"]) then
						MUI_Bottom[str]:SetVertexColor(C["Unit Frames"]["Color"].r, C["Unit Frames"]["Color"].g, C["Unit Frames"]["Color"].b, alpha)
					else
						MUI_Bottom[str]:SetVertexColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, alpha)
					end
				end
				MUI_Bottom["center"]:SetSize(C["Unit Frames"]["Center Width"], C["Unit Frames"]["Height"]);
				MUI_Bottom["left"]:SetSize(C["Unit Frames"]["Left Width"], 180);
				MUI_Bottom["right"]:SetSize(C["Unit Frames"]["Right Width"], 180);
				MUI_Bottom["center"]:SetPoint("BOTTOM", MUI_Bottom, "TOP", 0, C["Unit Frames"]["Y Offset"]);
			end,
			["Data Fields"] = function()
				MUI_Bottom["DataFields"]:SetSize(C["Data Fields"]["Width"], C["Data Fields"]["Height"]);
				MUI_Bottom["DataFields"]:SetAlpha(C["Data Fields"]["Alpha"]);
				MUI_Bottom["DataFields"]:SetTexture(LibStub("LibSharedMedia-3.0"):Fetch("background", C["Data Fields"]["Texture"]), size, flags);
				MUI_Bottom["DataFields"]:SetVertexColor(C["Data Fields"]["Color"].r, C["Data Fields"]["Color"].g, C["Data Fields"]["Color"].b)
			end,
			["Unit Names"] = function()
				for _, str in ipairs({"LeftName", "RightName"}) do					
					if (MayronUI.db.profile.BottomUI["Unit Names"]["Use Color"]) then
						MUI_Bottom[str]:SetVertexColor(C["Unit Names"]["Color"].r, C["Unit Names"]["Color"].g, C["Unit Names"]["Color"].b, C["Unit Names"]["Alpha"]);
					else
						MUI_Bottom[str]:SetVertexColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, C["Unit Names"]["Alpha"]);						
					end
					if ( str == "RightName" and not MayronUI.db.profile.BottomUI["Unit Names"]["Target Class Colored"] ) then
						if (UnitExists("target")) then
							local c = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[(select(2, UnitClass("target")))];
							MUI_BottomRightName:SetVertexColor(c.r, c.g, c.b);
						end
					end
					MUI_Bottom[str]:SetSize(C["Unit Names"]["Width"], C["Unit Names"]["Height"]);	
					local p, rf, rp, _, y = MUI_Bottom[str]:GetPoint();
					local x = C["Unit Names"]["Inner Offset"];
					if (str == "RightName") then
						x = -(x);
					end
					MUI_Bottom[str]:SetPoint(p, rf, rp, x, y);					
				end
			end,
			["Portrait Gradients"] = function()
				if (not MayronUI.db.profile.BottomUI["Portrait Gradients"]["Disable"]) then
					if (_G["MUI_targetGradient"] and _G["MUI_playerGradient"]) then
						MUI_targetGradient:SetHeight(C["Portrait Gradients"]["Height"]);
						MUI_playerGradient:SetHeight(C["Portrait Gradients"]["Height"]);
						MUI_playerGradient.t:SetGradientAlpha("VERTICAL", MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 0, MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, C["Portrait Gradients"]["Alpha"])
						if (MayronUI.db.profile.BottomUI["Portrait Gradients"]["Target Class Colored"]) then
							MUI_targetGradient.t:SetGradientAlpha("VERTICAL", MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 0, MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, C["Portrait Gradients"]["Alpha"])
						else
							MUI_targetGradient.t:SetGradientAlpha("VERTICAL", c.r, c.g, c.b, 0, c.r, c.g, c.b, C["Portrait Gradients"]["Alpha"]);
						end
					end
				end
			end,
		}
		local reset = true;
		local trigged = {}
		for ftype, properties in pairs(MayronUI.db.profile.BottomUI) do
			if ( C[ftype] and ( (not arg1) or (ftype == arg1) ) ) then
				for property, value in pairs(properties) do
					if ( C[ftype][property] and ( (not arg2) or (property == arg2) ) ) then
						reset = nil;
						-- Import Settings:
						if (type(value) == "table") then 
							C[ftype][property].r, C[ftype][property].g, C[ftype][property].b = value.r, value.g, value.b;
						else
							C[ftype][property] = value;
						end	
						-- Apply Settings:
						f[ftype]();
						trigged[ftype] = true;
					end
				end
			end			
		end
		if (not arg1 and not arg2) then -- on setup
			for ftype, _ in pairs(f) do
				if (not trigged[ftype]) then
					f[ftype]();
				end
			end
		end
		
		STOPPING_POINT = C["Action Bars"]["Special Height"] + 20;
		STARTING_POINT = C["Action Bars"]["Height"] + 20;
		
		-- RESET TABLE!
		if (reset and arg1 and arg2) then
			local default = {
				["Action Bars"] = {
					["Width"] = 751,
					["Height"] = 40,
					["Alpha"] = 1,
					["Special Height"] = 75,
				},
				["Unit Frames"] = {
					["Left Width"] = 324,
					["Right Width"] = 324,
					["Center Width"] = 102,
					["Height"] = 180,
					["Y Offset"] = -110,
					["Alpha"] = 0.8,
					["Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b},
				},
				["Data Fields"] = {
					["Width"] = 751,
					["Height"] = 20,
					["Alpha"] = 1,
					["Texture"] = "MUI_DataFields",
					["Color"] = {r = 0.4, g = 0.4, b = 0.4},
				},
				["Unit Names"] = {
					["Width"] = 232,
					["Height"] = 18,
					["Alpha"] = 0.8,
					["Inner Offset"] = 23,
					["Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b},
				},
				["Portrait Gradients"] = {				
					["Height"] = 20,
					["Alpha"] = 0.4,
				},
			}			
			C[arg1][arg2] = default[arg1][arg2];
			default = nil;
			f[arg1]();
		end		
	end	
	MayronUI_BottomUI:UpdateSettings()
	
	if ( IsAddOnLoaded("Bartender4") ) then	
		Bartender4:GetModule("ActionBars"):EnableBar(10);
		Bartender4:GetModule("ActionBars"):EnableBar(9);
		if ( MayronUI.db.profile.BottomUI["Expand"] ) then 
			Bartender4.Bar.barregistry["10"]:SetVisibilityOption("always", false);
			Bartender4.Bar.barregistry["9"]:SetVisibilityOption("always", false);
		else
			Bartender4.Bar.barregistry["10"]:SetVisibilityOption("always", true);
			Bartender4.Bar.barregistry["9"]:SetVisibilityOption("always", true);
		end
	end

	muiBottom.RegisterEvents = RegisterEvents;
	muiBottom:RegisterEvents("PLAYER_ENTERING_WORLD", "PLAYER_TARGET_CHANGED", "PLAYER_LOGOUT", "PLAYER_QUIT", "PLAYER_CAMPING", "PLAYER_REGEN_ENABLED", "PLAYER_REGEN_DISABLED", "MODIFIER_STATE_CHANGED", "PET_BATTLE_OPENING_START", "PET_BATTLE_CLOSE");
	muiBottom:SetScript("OnEvent", function(self, event, ...)	
		local currentProfile = nil	
		if ( event == "PLAYER_LOGOUT" or event == "PLAYER_CAMPING" or event == "PLAYER_QUIT"  ) then
			if ( ShadowUF and ShadowUF.db ) then
				local currentProfile = ShadowUF.db:GetCurrentProfile();
				if (ShadowedUFDB["profiles"][currentProfile]["positions"]["targettarget"]["anchorTo"] == "MUI_Bottom") then				
					ShadowedUFDB["profiles"][currentProfile]["positions"]["targettarget"]["anchorTo"] = "UIParent";
					ShadowedUFDB["profiles"][currentProfile]["positions"]["targettarget"]["relativePoint"] = "BOTTOM";
					ShadowedUFDB["profiles"][currentProfile]["positions"]["targettarget"]["y"] = "65";
				end
			end
		elseif ( event == "PLAYER_TARGET_CHANGED" or event == "PLAYER_ENTERING_WORLD" ) then
			if (Startup) then
				if ( IsAddOnLoaded("ShadowedUnitFrames") and ShadowUF.db ) then									
					-- Setup Gradients:
					if (not MayronUI.db.profile.BottomUI["Portrait Gradients"]["Disable"]) then
						MayronUI_BottomUI:SetupPortraitGradients();
						MayronUI_BottomUI:UpdateSettings("Portrait Gradients") 
					end
					local function AnchorSUFtoBottomUI()
						if (not MayronUI.db.profile.BottomUI.General["ShadowUF"]) then
							currentProfile = ShadowUF.db:GetCurrentProfile();
							if (_G["SUFUnittargettarget"]) then
								SUFUnittargettarget:ClearAllPoints();
								SUFUnittargettarget:SetPoint("BOTTOM", "MUI_Bottom", "TOP", 0, 4);
							end
							local SUF = ShadowedUFDB["profiles"][currentProfile]["positions"]["targettarget"];
							SUF["point"] = "BOTTOM"; SUF["anchorTo"] = "MUI_Bottom"; 
							SUF["relativePoint"] = "TOP"; SUF["x"] = 0; SUF["y"] = 4;
							if (_G["SUFUnitplayer"]) then SUFUnitplayer:SetParent(MUI_Bottom) end
							if (_G["SUFUnittarget"]) then SUFUnittarget:SetParent(MUI_Bottom) end
							if (_G["SUFUnittargettarget"]) then SUFUnittargettarget:SetParent(MUI_Bottom) end
							ShadowUF.Layout:Reload();
						end
					end						
					hooksecurefunc(ShadowUF, "ProfilesChanged", AnchorSUFtoBottomUI)					
					AnchorSUFtoBottomUI();
					
				end
				if ( IsAddOnLoaded("Grid") and (Grid and Grid.db) ) then
					local function AnchorGridtoBottomUI()					
						if (not MayronUI.db.profile.BottomUI.General["Grid"] and (Grid.db:GetCurrentProfile() == "MayronUIH")) then
							currentProfile = "MayronUIH";
							if (_G["GridLayoutFrame"]) then
								GridLayoutFrame:ClearAllPoints();
								GridLayoutFrame:SetPoint("BOTTOMRIGHT", MUI_Bottom, "TOPRIGHT", 26, 88);
								Grid:GetModule("GridLayout"):SavePosition();
							end
						end						
					end	
					hooksecurefunc(Grid, "OnProfileEnable", AnchorGridtoBottomUI)
					AnchorGridtoBottomUI();
				end
				
				BINDING_CATEGORY_MayronUI = "MayronUI";
				if (not MayronUI.db.global["ExtraViewDistance"]) then
					SetCVar("cameraDistanceMaxFactor", "4");-- extra long view	
				end
				-- Reposition Cast bars:
				MayronUI:PositionCastBars();				
				Startup = nil;
				-- Remove some taint:
				PVPReadyDialog:Hide();
				LoadAddOn("Blizzard_PetJournal");
				ToggleFrame(SpellBookFrame);
				ToggleFrame(SpellBookFrame);
				LoadAddOn("Blizzard_Collections");				
				ToggleFrame(CollectionsJournal);
				ToggleFrame(CollectionsJournal);
				collectgarbage("collect");				
			end
			if ( UnitExists("target") or MayronUI.db.profile.BottomUI.ClassicMode ) then			
				if (UnitExists("target")) then
					local c = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[(select(2, UnitClass("target")))];
					if (not c) then
						c = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)["WARRIOR"];
					end
					if (c and c.r) then
						if (_G["MUI_targetGradient"]) then
							if (MayronUI.db.profile.BottomUI["Portrait Gradients"]["Target Class Colored"]) then
								MUI_targetGradient.t:SetGradientAlpha("VERTICAL", MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 0, MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, C["Portrait Gradients"]["Alpha"])
							else
								MUI_targetGradient.t:SetGradientAlpha("VERTICAL", c.r, c.g, c.b, 0, c.r, c.g, c.b, C["Portrait Gradients"]["Alpha"]);
							end
						end				
						MUI_BottomRightName:Show();						
						if (MayronUI.db.profile.BottomUI["Unit Names"]["Target Class Colored"]) then
							local loc = MayronUI.db.profile.BottomUI["Unit Names"];
							if (loc["Use Color"] and loc["Color"] and loc["Color"].r) then
								MUI_BottomRightName:SetVertexColor(loc["Color"].r, loc["Color"].g, loc["Color"].b);
							else
								MUI_BottomRightName:SetVertexColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b);
							end
						else
							MUI_BottomRightName:SetVertexColor(c.r, c.g, c.b);
						end
					end
				else
					MUI_BottomRightName:Hide();
				end
				
				MUI_Bottom.left:SetTexture(MUI_MEDIA.."Double.tga");
				MUI_Bottom.center:SetAlpha(C["Unit Frames"]["Alpha"])
				MUI_Bottom.right:SetAlpha(C["Unit Frames"]["Alpha"])
				
			else
				MUI_BottomRightName:Hide();
				MUI_Bottom.left:SetTexture(MUI_MEDIA.."Single.tga");
				MUI_Bottom.center:SetAlpha(0)
				MUI_Bottom.right:SetAlpha(0)
				MUI_BottomRightName:SetVertexColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b);
			end
		elseif ( (event == "MODIFIER_STATE_CHANGED") and (not InCombatLockdown()) ) then
			MayronUI:OnStateChanged(...)
		elseif ( event == "PLAYER_REGEN_ENABLED" ) then
			MUI_BottomFadeOut:Stop();
			MUI_Bottom.blocker:Hide();			
		elseif ( event == "PLAYER_REGEN_DISABLED" ) then
			MUI_BottomSwitchButton:Hide();
			MUI_BottomGlow:SetAlpha(0);
			MUI_Bottom.blocker:Show();					
		elseif ( event == "PET_BATTLE_OPENING_START" ) then 
			muiBottom:Hide()
			if IsAddOnLoaded("SLDataText") then
				-- Hide
				local currentProfile = SLDataText.db:GetCurrentProfile();
				for _, str in ipairs({"Coords", "Reputation", "Mail", "Currency", "Latency", "FPS", "Gold", "Armor", "Friends", "Guild", "Memory", "ZoneText", "Bag", "Clock"}) do
					local btn = SLDataText[str];
					local value = SLDTDB["namespaces"][str]["profiles"][currentProfile];
					if (btn and value["enabled"] ~= false) then
						value["enabled"] = false;
						btn:Disable();
						MayronUI.db.profile["SLDT_"..str] = true;
					end
					btn:Refresh()
				end 
			end
			PetBattleFrame:SetFrameStrata("HIGH")
		elseif ( event == "PET_BATTLE_CLOSE" ) then 
			muiBottom:Show()
			if IsAddOnLoaded("SLDataText") then
				-- Show
				local currentProfile = SLDataText.db:GetCurrentProfile();
				for _, str in ipairs({"Coords", "Reputation", "Mail", "Currency", "Latency", "FPS", "Gold", "Armor", "Friends", "Guild", "Memory", "ZoneText", "Bag", "Clock"}) do
					local btn = SLDataText[str];
					local value = SLDTDB["namespaces"][str]["profiles"][currentProfile];
					if (btn and MayronUI.db.profile["SLDT_"..str]) then
						value["enabled"] = true;
						btn:Enable();
						MayronUI.db.profile["SLDT_"..str] = nil;
					end
					btn:Refresh();
				end
			end
		end
	end)
	
	do
		local function IsModKeyDown(strKey) -- "C" for example.	
			if (strKey == "S") then
				if (not IsControlKeyDown() and IsShiftKeyDown() and not IsAltKeyDown()) then return true end
			elseif (strKey == "C") then 
				if (IsControlKeyDown() and not IsShiftKeyDown() and not IsAltKeyDown()) then return true end
			elseif (strKey == "A") then
				if (not IsControlKeyDown() and not IsShiftKeyDown() and IsAltKeyDown()) then return true end
			elseif (strKey == "SC") then 
				if (IsControlKeyDown() and IsShiftKeyDown() and not IsAltKeyDown()) then return true end
			elseif (strKey == "SA") then 
				if (not IsControlKeyDown() and IsShiftKeyDown() and IsAltKeyDown()) then return true end
			elseif (strKey == "CA") then 
				if (IsControlKeyDown() and not IsShiftKeyDown() and IsAltKeyDown()) then return true end
			elseif (strKey == "SCA") then 
				if (IsControlKeyDown() and IsShiftKeyDown() and IsAltKeyDown()) then return true end
			else return false end
		end
		
		-- Expand / Retract Functions:
		function MayronUI:OnStateChanged()		
			if ( IsModKeyDown(MayronUI.db.profile.BottomUI.ModKey) ) then
				MUI_BottomFadeOut:Stop();
				MUI_BottomFadeIn:Play();
			else
				MUI_BottomFadeIn:Stop();
				MUI_BottomFadeOut:Play();
			end
		end
	end
	
	do		
		local down, alpha = nil;
		local point, relFrame, relPoint, x = nil;
		local group = muiBottom:CreateAnimationGroup()
		local ANIMATION_SPEED = MayronUI.db.profile.BottomUI["Animation Speed"] or 0.8;
			
		group.move = group:CreateAnimation()
		group.move:SetDuration(ANIMATION_SPEED / 100)
		group:SetLooping("REPEAT")	

		function MayronUI_BottomUI:SetAnimation(value)
			ANIMATION_SPEED = value or 0.8;
			group.move:SetDuration(ANIMATION_SPEED / 100)
		end
		
		function MayronUI_BottomUI:GetAnimation()
			return ANIMATION_SPEED;
		end
		
		group:SetScript("OnLoop", function(self)
			local y = select(5, MUI_Bottom:GetPoint("TOP"));
			if ( down ) then
				y = y - 1;
				MUI_Bottom:SetPoint(point, relFrame, relPoint, x, y);
				if ( IsAddOnLoaded("Bartender4") ) then 
					alpha = BT4Bar10:GetAlpha()
					if ( alpha > 0 ) then 
						BT4Bar10:SetConfigAlpha(alpha - 0.1)
						BT4Bar9:SetConfigAlpha(alpha - 0.1)
					end
				end
			else
				y = y + 1;
				MUI_Bottom:SetPoint(point, relFrame, relPoint, x, y);
				if ( IsAddOnLoaded("Bartender4") ) then 
					alpha = BT4Bar10:GetAlpha()
					if ( alpha < 1 ) then 
						BT4Bar10:SetConfigAlpha(alpha + 0.1)
						BT4Bar9:SetConfigAlpha(alpha + 0.1)
					end
				end
			end
			if ( y >= STOPPING_POINT or math.floor(y) == STARTING_POINT ) then 
				self:Stop()  
				if ( down and IsAddOnLoaded("Bartender4") ) then 
					-- Hide Bars (Retracted):
					MayronUI.db.profile.BottomUI["Expand"] = nil;
					if ( IsAddOnLoaded("Bartender4") ) then 
						Bartender4.Bar.barregistry["10"]:SetVisibilityOption("always", true);
						Bartender4.Bar.barregistry["9"]:SetVisibilityOption("always", true);
					end
				else
					-- Show Bars (Expanded):
					MayronUI.db.profile.BottomUI["Expand"] = true;
					if ( IsAddOnLoaded("Bartender4") ) then 
						Bartender4.Bar.barregistry["10"]:SetVisibilityOption("always", false);
						Bartender4.Bar.barregistry["9"]:SetVisibilityOption("always", false);
					end
				end
			return end
		end)
		
		function MayronUI_BottomUI:MoveButton_OnClick()
			if (InCombatLockdown()) then return end
			point, relFrame, relPoint, x = MUI_Bottom:GetPoint("TOP");
			if ( math.floor(select(5, MUI_Bottom:GetPoint("TOP"))) >= STOPPING_POINT ) then 
				down = true;
				MUI_BottomSwitchButton:SetText("Expand");
			else 
				down = nil ;
				MUI_BottomSwitchButton:SetText("Retract");
			end;
			MUI_BottomFadeIn:Stop();
			MUI_BottomFadeOut:Play();
			group:Play();
		end		
	end
end