MayronUI_SetupUI.Import = {}; -- table of AddOn install functions.
function MayronUI_SetupUI:OnEnable() 
	local UIScale;
	MUI_BOTTOM = UIParent;
	local function print(...)
		DEFAULT_CHAT_FRAME:AddMessage(string.join(" ", MUI_PREFIX, tostringall(...)));
	end

	local modules = {"BottomUI", "Chat Box", "Cast Bars", "Mini Map", "Side Bars", "ReskinUI"}	
	
	do
		local TEXT = { -- Both these tables should be in 1 main table called Content!
			"|cff00ccffWelcome!", " ",
			
			"I recommend reading each one of these helpful tips", 
			"as you may wish to customize your new UI and this", 
			"could be greatly beneficial to you.", " ",
			
			"|cff00ccffTop Tips:|r", " ",
			
 "|cff00ccff>> |rHold the Control key while out of combat to", 
			"show the Expand and Retract button to add more", 
			"action bars at the bottom of the UI.", " ",
			
 "|cff00ccff>> |r Hold the Shift or Control key while out of", 
			"combat to view up to six extra buttons attached",
			"to the chat box.", " ",
			
 "|cff00ccff>> |r You can set these modifier keys to a different", 
			"key or a combination of keys.", " ",
			
 "|cff00ccff>> |r You can modify what each button on the Chat", 
			"Art does in the MUI Config Menu.", " ",
		
 "|cff00ccff>> |r For a full list of MUI slash commands",
			"type: \"/mui\".", " ",
			
 "|cff00ccff>> |r You can re-install the UI at any time using:",
			"\"/mui install\".", " ",
			
 "|cff00ccff>> |r To access the MUI Config Menu type: ",
			"\"/mui config\"", " ",
			
 "|cff00ccff>> |r Left click on the Main chat art button to switch", 
			"layouts. This will switch a pre-defined group of",			
			"AddOn profiles to a different profile of your choice.", " ",			
			
 "|cff00ccff>> |r Right click on the Main chat art button to build", 
			"your own layouts.", " ",
			
"|cff00ccff>> |r You can enable more Chat Art for each corner of", 
			"your screen in the MUI Config Menu.", " ",
			
"|cff00ccff>> |r Please help to report all bugs and feedback to", 
			"the UI's home page!", " ",
			
			"Thank you for downloading and reading,",
			"Mayron <3",
		};

		local NUM_FRAMES = (#TEXT);
		local frames = {};
		
		function MayronUI_SetupUI:Setup_OnLoad()
			for i = 1, NUM_FRAMES do
				local frame = CreateFrame("Frame", nil, MUI_SetupTips);
				if i == 1 then
					frame:SetPoint("TOPLEFT", MUI_SetupTips, "TOPLEFT", 10, -10);
				else
					frame:SetPoint("TOP", frames[i - 1], "BOTTOM");
				end		
				frame:SetSize((MUI_SetupTips:GetWidth() - 10), 12); -- 12 is the height of each line:
				-- also need to change FauxScrollFrame_OnVerticalScroll to the same value!:
				
				frame.body = frame:CreateFontString(nil, "ARTWORK");
				frame.body:SetAllPoints(frame);
				frame.body:SetFont("Fonts\\FRIZQT__.ttf", 10)
				frame.body:SetText(TEXT[i]);
				frame.body:SetJustifyH("LEFT");
				frame.body:SetJustifyV("TOP");
				frames[i] = frame;
			end
			MUI_SetupTipsScrollFrame:Show()
		end
		
		function MayronUI_SetupUI:SetupScrollBar_Update()
			local self = MUI_SetupTipsScrollFrame;
			FauxScrollFrame_Update(self, NUM_FRAMES, 4, 8);
			local offset = FauxScrollFrame_GetOffset(self);
			for line = 1, NUM_FRAMES do
				local frame = frames[line];
				if (line < 22) then -- 13 is the number of lines to show at one time
					frame.body:SetText(TEXT[line + offset]);
					frame:Show();
				else
					frame.body:SetText(" ");
					frame:Hide();
				end
				
			end
		end
	end
	
	-- AddOn Scroll Bar:
	do	
		-- Default checked State:
		local ADDONS = {
			-- Alias, default enabled state, AddOn name
			{"Ara Broker - Tradeskills", true, "Ara_Broker_Tradeskills"}, 
			{"Aura Frames", true, "AuraFrames"}, 
			{"Bartender4", true, "Bartender4"},
			{"Bazooka", true, "Bazooka"}, 
			{"Grid", true, "Grid"},
			{"Masque", true, "Masque"}, 
			{"Mik Scrolling Battle Text", true, "MikScrollingBattleText"}, 			
			{"OmniCC", true, "OmniCC"}, 
			--{"Parrot", true, "Parrot"}, inactive project :(
			{"Raid Buff Status", true, "RaidBuffStatus"}, 
			{"Recount", true, "Recount"}, 
			{"Omen", true, "Omen"}, -- Depends on Recount!
			{"Shadowed Unit Frames", true, "ShadowedUnitFrames"},
			{"Simple Power Bar", true, "SimplePowerBar"}, 
			{"SLData Text", true, "SLDataText"}, 
			{"TidyPlates - ThreatPlates", true, "TidyPlates_ThreatPlates"},
			{"TipTac", true, "TipTac"},
			{"Totem Timers", true, "TotemTimers"},		
		}
		
		 -- Setup Check Buttons:
		function MayronUI_SetupUI:AddonScrollBar_OnLoad()
			for i = 1, 8 do
				local cb = CreateFrame("CheckButton", "MUI_Addon"..i, MUI_SetupConfirmBoxAddons, "UICheckButtonTemplate")
				if (i == 1) then
					cb:SetPoint("TOPLEFT", MUI_SetupConfirmBoxAddons, "TOPLEFT", 2, 0)
				else
					cb:SetPoint("TOPLEFT", "MUI_Addon"..(i-1), "BOTTOMLEFT", 0, 10)
				end
				
				-- Export settings from DB:
				for name, value in pairs(MayronUI.db.global.AddOns) do
					for id, tbl in ipairs(ADDONS) do
						if (name == tbl[3]) then
							tbl[2] = (not value); -- invert
						end
					end
				end
			
				cb:SetScript("OnClick", function(self)
					if (self:GetChecked()) then 
						ADDONS[self.id][2] = true;
					else
						ADDONS[self.id][2] = false;
					end
				end)
			end		
		end
		
		-- Update Check Button Values:
		-- true = installed before, false = needs to be installed
		function MayronUI_SetupUI:AddonScrollBar_Update() 
			local self = MUI_SetupConfirmBoxAddonsScrollFrame;
			FauxScrollFrame_Update(self, #ADDONS, 8, 50);
			local offset = FauxScrollFrame_GetOffset(self);
			for i = 1, 8 do
				local cbtxt = _G["MUI_Addon"..i.."Text"];
				local cb = _G["MUI_Addon"..i];
				
				-- Assign Values here:
				cb.id = i + offset;
				cb.alias = ADDONS[cb.id][1];
				cbtxt:SetText(cb.alias);				
				if ( not IsAddOnLoaded(ADDONS[cb.id][3]) ) then
					ADDONS[cb.id][2] = false;
					cb:Disable();
					_G[cb:GetName().."Text"]:SetFontObject("GameFontDisable");
					cb:EnableMouse(0)
					if ( ADDONS[cb.id][2] ) then
						cb:SetChecked(true)
					else
						cb:SetChecked(false)
					end
				else
					-- due to scrolling, properties must be re-applied:
					cb:Enable();
					_G[cb:GetName().."Text"]:SetFontObject("GameFontNormal");
					cb:EnableMouse(1);						
					if ( ADDONS[cb.id][2] ) then
						cb:SetChecked(true)
					else
						cb:SetChecked(false)
					end
				end
			end
		end
		
		function MayronUI_SetupUI:Install(self) -- Major Function to Install UI!
			if ( pcall(function() LoadAddOn("MayronUI_SetupData") end) ) then				
				-- Export Module values to db (Will Enable/Disable Module on ReloadUI):
				for i, str in ipairs(modules) do
					if ( _G["MUI_Module"..i.."Text"]:GetText() == str ) then
						str = str:gsub("%s+", "");
						MayronUI.db.profile.modules[str] = _G["MUI_Module"..i]:GetChecked();
					end
				end
				
				-- Chat Frame settings:
				FCF_SetLocked(ChatFrame1, 1);
				ChatFrame1:ClearAllPoints();
				if (MayronUI.db.profile.install) then
					ChatFrame1:SetHeight(222);
					ChatFrame1:SetWidth(375);
					FCF_SetWindowAlpha(ChatFrame1, 0);
					SetChatWindowSize(1, 13);					
					SetCVar("chatStyle", "classic");
				end
				if (MayronUI.db.profile.install or MayronUI.db.profile.ChatBox["Top Left"]) then	
					ChatFrame1:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 34, -55);
				elseif (MayronUI.db.profile.ChatBox["Bottom Left"]) then
					ChatFrame1:SetPoint('BOTTOMLEFT',UIParent, 'BOTTOMLEFT', 34, 30);
				end
				ChatFrame1:SetUserPlaced(true);
				FCF_SavePositionAndDimensions(ChatFrame1);
				
				-- if UIScale has been altered then reposition some AddOns:
				if (UIScale) then
					MayronUI.db.profile.MiniMap.ReAnchor = true;
					MayronUI.db.profile.ChatFrame.ReAnchor = true;
					MayronUI.db.profile.ChatFrame.ReAnchor_Other = true;
					MayronUI.db.profile.ReAnchor = true;
				end
				
				-- Export AddOn values to db:
				for id, tbl in ipairs(ADDONS) do
					local value = tbl[2];
					MayronUI.db.global.AddOns[tbl[3]] = (not value); -- the opposite (false to install)
				end	

				-- Run each AddOn Import function required:
				for name, value in pairs(MayronUI.db.global.AddOns) do				
					if ( not value and IsAddOnLoaded(name) ) then					
						if ( name ~= "TotemTimers" or (name == "TotemTimers" and UnitClass("player") == "Shaman") ) then
							MayronUI_SetupUI.Import[name]()
							MayronUI.db.global.AddOns[name] = true;
						end
					end
				end
				
				-- Finish up:				
				MayronUI.db.profile.install = false;				
				MayronUI.db.global.Tracker["Position Tracker"] = true;
				MayronUI.db.profile.CurrentLayout = nil;
				PlaySoundFile(MUI_MEDIA .. "Sound\\Install.ogg"); -- no longer works :<
				
				local function GetDBObject(addonStr) -- gets addon name
					local addon, dbstr, okay;
					if (_G[addonStr]) then
						addon = _G[addonStr];
						okay = true;
					else
						okay, addon = pcall(function() LibStub("AceAddon-3.0"):GetAddon(addonStr) end)
					end
					if (not okay) then return end
					if (addon and not addon.db) then
						for dbname, tbl in pairs(addon) do
							if (strfind(dbname, "db")) then					
								if (type(addon[dbname]) == "table") then						
									if (addon[dbname].profile) then
										dbstr = dbname;
									end
								end
							end
						end		
						return addon[dbstr];
					elseif (addon and addon.db) then 
						return addon.db;
					end
				end
				
				-- MayronUI profiles:
				for id, name in ipairs({"AuraFrames", "Bartender4", "Omen", "RaidBuffStatus", "Recount", "SLDataText"}) do
					if ( _G[name] ) then						
						local path = GetDBObject(name);
						if (path) then 
							if (path:GetCurrentProfile() ~= "MayronUI") then
								path:SetProfile("MayronUI");
							end
						end						
					end
				end
				
				-- Default Profiles:
				for id, name in ipairs({"Bazooka", "Grid", "Parrot", "ShadowUF", "SimplePowerBar", "TidyPlatesThreat"}) do
					if ( _G[name] ) then
						local path = GetDBObject(name);
						if (path) then 
							if (path:GetCurrentProfile() ~= "Default") then
								path:SetProfile("Default");
							end
						end
					end
				end
				if (IsAddOnLoaded("TidyPlates_ThreatPlates")) then
					TidyPlatesOptions.primary = "Threat Plates";
					TidyPlatesOptions.secondary = "Threat Plates";
				end
				
				ReloadUI();
			else
				print("Error occurred: Missing AddOn \"MayronUI - Setup Data\".")
			end
		end		
	end
	
	CreateFrame("Frame", "MUI_Setup", UIParent, "MUI_SetupTemplate");
	MUI_SetupVersionText:SetText("MayronUI Gen4\nVersion "..tostring(GetAddOnMetadata("MayronUI", "Version")))
	
	function MayronUI_SetupUI:UpdateUIScale(self)
		if (UIScale) then
			SetCVar("useUiScale", "1");
			SetCVar("uiscale", UIScale);
			self:Disable();
		end		
	end	
	MUI_SetupConfirmBoxUIScaleSlider:SetScript("OnValueChanged", function(self, value)
		MUI_SetupConfirmBoxTitle:SetText(string.format("%s %.2f", "Using UI Scale: ", tostring(value)));
		MUI_SetupConfirmBoxUpdateScale:Enable();
		UIScale = tostring(value);
	end)
	
	-- Module Check boxes:
	for i, str in ipairs(modules) do
		local cb = CreateFrame("CheckButton", "MUI_Module"..i, MUI_SetupConfirmBoxModules, "UICheckButtonTemplate");
		if (i == 1) then
			cb:SetPoint("TOPLEFT", MUI_SetupConfirmBoxModules, "TOPLEFT", 2, 0);
		elseif (i == 4) then
			cb:SetPoint("TOPLEFT", MUI_Module1, "TOPRIGHT", 80, 0);
		else
			cb:SetPoint("TOPLEFT", "MUI_Module"..(i-1), "BOTTOMLEFT", 0, 10);
		end
		_G[cb:GetName() .. "Text"]:SetText(str);
		
		-- default value:
		str = str:gsub("%s+", "");
		if (MayronUI.db.profile.install) then			
			cb:SetChecked(true);
		elseif (MayronUI.db.profile.modules[str]) then
			cb:SetChecked(true);
		else
			cb:SetChecked(false);
		end
	end
end