function MayronUI_ReskinUI:OnEnable()
	-- Constants:
	if (not MayronUI.db.profile.ReskinUI["Font Colors"]) then
		NORMAL_QUEST_DISPLAY = "|cffffffff%s|r"
		TRIVIAL_QUEST_DISPLAY = "|cffffffff%s (low level)|r"
	end
	
	--/run for str, value in pairs(MUI_CharacterAdvanceCheckButton) do print(str.." : "..tostring(value)) end

	local MOVABLE = 1
	local CREATE_FRAME = 2
	local DISABLE_BACKGROUND = 4
	local DISABLE_BORDER = 8
	local DISABLE_ARTWORK = 16
	local DISABLE_OVERLAY = 32
	local DISABLE_HIGHLIGHT = 64
	local SUB_FRAME = 128
	
	-- Saved Variable Settings will override this table:
	local C = {
		["Background_Color"] = {r = 0.2, g = 0.2, b = 0.2, a = 0.9}, -- DONE
		["Border_Color"] = {r = 0, g = 0, b = 0, a = 1},
		["DropDown_ClassColor"] = true,
		["Tab_ClassColor"] = true,
		["Tab_Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b, a = 1},
		["DropDown_Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b, a = 1},
		["Border_Type"] = "Skinner",
		["Background_Type"] = "MayronUI",
		["Border_EdgeSize"] = 1,
		["Border_Inset"] = 0,
	}
	-- import saved settings into table:
	for str, value in pairs(MayronUI.db.profile.ReskinUI.Appearance) do	
		if (type(value) == "table") then 
			C[str].r, C[str].g, C[str].b, C[str].a = unpack(value)
		else
			C[str] = value
		end
	end
	if (C["Tab_ClassColor"]) then
		C["Tab_Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b, a = 1}
	end
	if (C["DropDown_ClassColor"]) then
		C["DropDown_Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b, a = 1}
	end
	MayronUI.db.profile.ReskinUI.Appearance.Toggle = nil
	
	-----------------------------------
	--[[  Core Skinning Functions  ]]--
	-----------------------------------
	
	local function SkinArrows(tbl)
		for _, name in ipairs(tbl) do
			if (_G[name.."Button"]) then	
				local btn = _G[name.."Button"]
				btn:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
				btn:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
				btn:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Disabled")
			end
		end
	end

	local function ScanDropdowns(frame, value, keyword) -- Returns a table of frame names
		keyword = keyword or "DropDown"
		value = value or true
		local collected = {}
		frame = { frame:GetChildren() }
		for _, child in ipairs(frame) do
			local name = child:GetName()
			local isKeyWord, endPosition
			if (name) then
				isKeyWord, endPoint = name:find(keyword)		
				if ( isKeyWord and (endPoint == #name) ) then 
					collected[name] = value 
					SkinArrows({name})
				end
			end
			
			local _, subCollection = pcall(ScanDropdowns, child, value)
			if (subCollection and type(subCollection) == "table") then 
				for k, v in pairs(subCollection) do collected[k] = v end
			end
		end
		return collected
	end	
	
	local MovableFrames = {}
	local function RepositionFrame(self)
		local mui = MayronUI.db.global.SavePoint[self:GetName()]
		if ( mui and (not InCombatLockdown() or not self.protected) ) then
			self:ClearAllPoints()
			self:SetPoint(mui[1], mui[2], mui[3], mui[4], mui[5])
		end
	end
	
	if (not MayronUI.db.profile.ReskinUI["Stop Movable"]) then  
		hooksecurefunc("ShowUIPanel", function() 
			if (MayronUI.db.profile.ReskinUI["Stop Reposition"]) then return end
			for i, f in ipairs(MovableFrames) do
				if (f:IsVisible()) then
					RepositionFrame(f)
				end
			end
		end)		
		hooksecurefunc("HideUIPanel", function()
			if (MayronUI.db.profile.ReskinUI["Stop Reposition"]) then return end
			for i, f in ipairs(MovableFrames) do
				if (f:IsVisible()) then
					RepositionFrame(f)
				end
			end
		end)
	end
	
	local function MakeMovable(f, dontSave, frames)
		if (MayronUI.db.profile.ReskinUI["Stop Movable"]) then return end
		if (type(f) == "string") then
			f = _G[f]
		end
		if (not f.GetObjectType) then
			for _, v in ipairs(f) do
				MakeMovable(v)
			end
			return
		end
		f:SetMovable(true)
		f:EnableMouse(true)
		f:SetClampedToScreen(true)
		f:SetClampRectInsets(-10, 10, 10, -10)
		f:RegisterForDrag("LeftButton")
		table.insert(MovableFrames, f)
		if (not dontSave and not MayronUI.db.profile.ReskinUI["Stop Reposition"]) then
			if (frames) then
				for _, name in ipairs(frames) do
					_G[name]:HookScript("OnClick", function() RepositionFrame(f) end)
				end
			end
			f:HookScript("OnShow", RepositionFrame)
		end
		
		local function handlers(script)
			if ( script == "OnDragStart" ) then
				f:StartMoving()		
			elseif ( script == "OnDragStop" ) then
				f:StopMovingOrSizing()
				if (not dontSave and not MayronUI.db.profile.ReskinUI["Stop Reposition"]) then
					MayronUI.db.global.SavePoint[f:GetName()] = { f:GetPoint() }
				end
			end	
		end
		local function GetHookType(script)
			local oldHandler = f:GetScript(script)
			if ( oldHandler ) then
				f:SetScript(script, function(...)
					handlers(script)
					oldHandler(...)
				end)
			else
				f:SetScript(script, function() handlers(script) end)
			end
		end
		GetHookType("OnDragStart")
		GetHookType("OnDragStop")
	end	
	
	local function SkinTabs(item)
		if (type(item) == "table") then
			for _, str in ipairs(item) do
				SkinTabs(str)
			end
		return end
		
		local tab = _G[item]
		if (tab) then
			if (tab:GetObjectType() == "Button") then
				local newLvl = tab:GetParent():GetFrameLevel() - 10
				tab:SetFrameLevel( ((newLvl >= 0) and newLvl) or 0 )
			end
			tab:SetHighlightTexture("")		
			tab:HookScript("OnShow", function(self)
				_G[self:GetName().."Left"]:SetTexture(MUI_MEDIA.."Skin\\Tab-InActive.tga")
				_G[self:GetName().."Right"]:SetTexture(MUI_MEDIA.."Skin\\Tab-InActive.tga")
				_G[self:GetName().."Middle"]:SetTexture(MUI_MEDIA.."Skin\\Tab-InActive.tga")
				
				_G[self:GetName().."Left"]:SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)
				_G[self:GetName().."Right"]:SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)
				_G[self:GetName().."Middle"]:SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)
				
				_G[self:GetName().."LeftDisabled"]:SetTexture(MUI_MEDIA.."Skin\\Tab-Active.tga")
				_G[self:GetName().."RightDisabled"]:SetTexture(MUI_MEDIA.."Skin\\Tab-Active.tga")
				_G[self:GetName().."MiddleDisabled"]:SetTexture(MUI_MEDIA.."Skin\\Tab-Active.tga")
				
				_G[self:GetName().."LeftDisabled"]:SetVertexColor(C["Tab_Color"].r * 0.5, C["Tab_Color"].g * 0.5, C["Tab_Color"].b * 0.5, 0.8)
				_G[self:GetName().."RightDisabled"]:SetVertexColor(C["Tab_Color"].r * 0.5, C["Tab_Color"].g * 0.5, C["Tab_Color"].b * 0.5, 0.8)
				_G[self:GetName().."MiddleDisabled"]:SetVertexColor(C["Tab_Color"].r * 0.5, C["Tab_Color"].g * 0.5, C["Tab_Color"].b * 0.5, 0.8)				
			end)				
			tab:HookScript("OnEnter", function(self)
				_G[self:GetName().."Left"]:SetVertexColor(C["Tab_Color"].r * 1.4, C["Tab_Color"].g * 1.4, C["Tab_Color"].b * 1.4, 0.85)
				_G[self:GetName().."Right"]:SetVertexColor(C["Tab_Color"].r * 1.4, C["Tab_Color"].g * 1.4, C["Tab_Color"].b * 1.4, 0.85)
				_G[self:GetName().."Middle"]:SetVertexColor(C["Tab_Color"].r * 1.4, C["Tab_Color"].g * 1.4, C["Tab_Color"].b * 1.4, 0.85)	
			end)
			tab:HookScript("OnLeave", function(self)
				self:GetScript("OnShow")(self)					
			end)
		end
	end
	
	local function UpdateQualityColor(frame)
		if (frame.IconBorder and frame.new) then
			local r, g, b = frame.IconBorder:GetVertexColor()
			frame.new:SetVertexColor(r, g, b)
		end
	end	
	
	local function ChangeTextColor(fontobj, colour, ignore)
		local r, g, b
		if ( colour == "white" ) then
			r, g, b = 1, 1, 1
		elseif ( colour == "red" ) then
			r, g, b = RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b						
		elseif ( colour == "yellow" ) then
			r, g, b = YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b	
		elseif ( colour == "orange" ) then
			r, g, b = ORANGE_FONT_COLOR.r, ORANGE_FONT_COLOR.g, ORANGE_FONT_COLOR.b	
		elseif ( colour == "green" ) then
			r, g, b = GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b	
		elseif ( colour == "dim red" ) then
			r, g, b = DIM_RED_FONT_COLOR.r, DIM_RED_FONT_COLOR.g, DIM_RED_FONT_COLOR.b	
		elseif ( colour == "grey" ) then
			r, g, b = GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b	
		end
		fontobj:SetShadowColor(0, 0, 0, 0.6)
		fontobj:SetShadowOffset(1, -1)
		fontobj:SetTextColor(r, g, b, 1)
		if ( (fontobj.SetTextColor) and (not ignore) ) then
			fontobj.SetTextColor = function() end
		end
	end
	
	local function SkinButton(btn, hideOverlay, block)
		if (not btn or btn.reskinned) then return end		
		if (not btn:IsObjectType("Button")) then return end
		btn:SetNormalTexture("")
		btn:SetPushedTexture("")
		btn:SetHighlightTexture("")
		if (hideOverlay) then btn:DisableDrawLayer("OVERLAY") end
		btn:SetNormalFontObject("GameFontNormal")		
		local fobj = btn:GetFontString()
		if (fobj) then
			ChangeTextColor(fobj, "white")
		end
		btn:SetDisabledFontObject("GameFontDisable")
		btn:SetHighlightFontObject("GameFontHighlight")
		btn:SetBackdrop({
			bgFile = LibStub("LibSharedMedia-3.0"):Fetch("background", C["Background_Type"]),
			edgeFile = LibStub("LibSharedMedia-3.0"):Fetch("border", "Skinner"), tile = false, tileSize = 0, edgeSize = 1,
		})
		btn.oldBC = {0, 0, 0, 1}
		btn:SetBackdropColor(C["Background_Color"].r, C["Background_Color"].g, C["Background_Color"].b, C["Background_Color"].a)
		btn:SetBackdropBorderColor(0, 0, 0, 1)
		btn:SetFrameLevel(btn:GetFrameLevel() + 2)
		btn:HookScript("OnLeave", function(self)
			if (self.oldBC) then
				self:SetBackdropBorderColor(self.oldBC[1], self.oldBC[2], self.oldBC[3], self.oldBC[4])
			else
				self:SetBackdropBorderColor(0, 0, 0, 1)
			end
		end)
		btn:HookScript("OnEnter", function(self)
			if (not self:IsEnabled()) then return end
			self.oldBC = { self:GetBackdropBorderColor() }
			self:SetBackdropBorderColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, MUI_COLOR.a)
		end)
		if (not block) then
			btn:SetScript("OnMouseUp", function() end)
			btn:SetScript("OnMouseDown", function() end)
		end
		btn.reskinned = true
	end
	
	local SkinFrame 
	do
		local function HideTextures(frame, tbl) -- a table of layer/s
			if (not frame) then return end
			for layer = 1, (#tbl) do
				for r = 1, frame:GetNumRegions() do
					local region = select(r, frame:GetRegions())
					if not (region) then return end
					
					if (region:GetObjectType() == "Texture") then
						if (region:GetDrawLayer() == tbl[layer]) then
							region:SetTexture("")
							region:SetAlpha(0)
						end
					end
				end
			end
		end
		
		function SkinFrame(frame, value) -- frame and number value or layer string
			local layers = {}
			if ( type(value) == "string" ) then -- for single layer commands only.
				layers[1] = value
				HideTextures(frame, layers)
				return
			end	
			-- If value is a number:			
			if ( (bit.band(value, DISABLE_BACKGROUND) ~= 0)) then
				layers[#layers + 1] = "BACKGROUND"
			end
			if ( (bit.band(value, DISABLE_BORDER) ~= 0) ) then
				layers[#layers + 1] = "BORDER"
			end
			if ( (bit.band(value, DISABLE_ARTWORK) ~= 0) ) then
				layers[#layers + 1] = "ARTWORK"
			end
			if ( (bit.band(value, DISABLE_OVERLAY) ~= 0) ) then
				layers[#layers + 1] = "OVERLAY"
			end
			if ( (bit.band(value, DISABLE_HIGHLIGHT) ~= 0) ) then
				layers[#layers + 1] = "HIGHLIGHT"
			end			
			HideTextures(frame, layers)
		end
	end
	
	local function CreateBG(frame, left, right, top, bottom, widget, backdrop, subframe)	
		if (frame and frame.new) then return end
		
		left = (left or 0)
		right = (right or 0)
		top = (top or 0)
		bottom = (bottom or 0)
		
		if (not widget and not subframe) then
			left = left - (C["Border_EdgeSize"] - C["Border_Inset"])
			right = right + (C["Border_EdgeSize"] - C["Border_Inset"])
			top = top + (C["Border_EdgeSize"] - C["Border_Inset"])
			bottom = bottom - (C["Border_EdgeSize"] - C["Border_Inset"])
		end
		
		backdrop = backdrop or "MUI_ToolTipTemplate"
		frame = frame or CreateFrame("Frame")		
		widget = (widget and "Button") or "Frame" -- used for drop down menus
		
		if (frame:GetObjectType() == "Texture") then frame = frame:GetParent() end
		frame:SetBackdrop(nil)

		local new = CreateFrame(widget, nil, frame, backdrop)
		new:SetPoint("TOPLEFT", frame, "TOPLEFT", left, top)
		new:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", right, bottom)
		new:SetFrameLevel( ( (frame:GetFrameLevel() - 1) < 0 and 0) or (frame:GetFrameLevel() - 1) )

		-- Drop Down Menu (Button):
		if (new:GetObjectType() == "Button") then			
			new.background:SetVertexColor(C["DropDown_Color"].r, C["DropDown_Color"].g, C["DropDown_Color"].b, C["DropDown_Color"].a)		
			if ( (frame:GetName()) and (_G[frame:GetName().."Button"]) ) then
				local btn = _G[frame:GetName().."Button"]									
				_G[frame:GetName().."Text"]:SetParent(new)
				btn:SetFrameLevel(frame:GetFrameLevel() + 7)
				new:SetFrameLevel(frame:GetFrameLevel() + 4)
				new:RegisterForClicks("LeftButtonDown", "RightButtonDown")
				
				btn:HookScript("OnDisable", function()
					new.background:SetVertexColor(C["DropDown_Color"].r / 3, C["DropDown_Color"].g / 3, C["DropDown_Color"].b / 3, 0.5)
				end)	
				btn:HookScript("OnEnable", function()
					new.background:SetVertexColor(C["DropDown_Color"].r, C["DropDown_Color"].g, C["DropDown_Color"].b, 0.9)						
				end)
				new:SetScript("OnEnter", function()
					if not (btn:IsEnabled()) then return end
					new.background:SetVertexColor(C["DropDown_Color"].r * 1.5, C["DropDown_Color"].g * 1.5, C["DropDown_Color"].b * 1.5, 1)
				end)
				new:SetScript("OnLeave", function()
					if not (btn:IsEnabled()) then return end
					btn:GetScript("OnEnable")()
				end)
				btn:SetScript("OnShow", function(self)
					if (self:IsEnabled()) then
						self:GetScript("OnEnable")()
					else
						self:GetScript("OnDisable")()
					end
				end)	
				new:SetScript("OnClick", function()
					pcall(function() btn:Click() end) -- can be buggy
				end)
			else			
				frame:SetScript("OnEnter", function()
					new.background:SetVertexColor(C["DropDown_Color"].r * 1.8, C["DropDown_Color"].g * 1.8, C["DropDown_Color"].b * 1.8, 1)
				end)
				frame:SetScript("OnLeave", function()
					new.background:SetVertexColor(C["DropDown_Color"].r, C["DropDown_Color"].g, C["DropDown_Color"].b, 0.9)
				end)				
			end
		else
			new.background:SetTexture(LibStub("LibSharedMedia-3.0"):Fetch("background", C["Background_Type"]))
			new.background:SetVertexColor(C["Background_Color"].r, C["Background_Color"].g, C["Background_Color"].b, C["Background_Color"].a)
			
			if (subframe) then
				new:SetBackdrop({
					edgeFile = LibStub("LibSharedMedia-3.0"):Fetch("border", "Skinner"), tile = false, tileSize = 0, edgeSize = 1, 
				})
				new:SetBackdropBorderColor(0, 0, 0, 1)
			else
				new:SetBackdrop({
					edgeFile = LibStub("LibSharedMedia-3.0"):Fetch("border", C["Border_Type"]), tile = false, tileSize = 0, edgeSize = C["Border_EdgeSize"], 
				})
				new.background:SetPoint("TOPLEFT", (C["Border_EdgeSize"] - C["Border_Inset"]), -(C["Border_EdgeSize"] - C["Border_Inset"]))
				new.background:SetPoint("BOTTOMRIGHT", -(C["Border_EdgeSize"] - C["Border_Inset"]), (C["Border_EdgeSize"] - C["Border_Inset"]))
				new:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b, C["Border_Color"].a)
			end
		end
		frame.new = new
		return new
	end
	
	local function SkinIcons(item, distance, hideOverlay)
		if (type(item[1]) == "string") then -- the frame uses a parentKey styled name
			for id, str in ipairs(item) do
				if (_G[str]) then
					SkinIcons(_G[str], distance, hideOverlay)
				else					
					local located = strfind(str, ".")
					if (located) then 						
						local path
						for i = 1, #(str) do
							local obj = select(i, strsplit(".", str))
							if (not obj) then break end
							if (i == 1) then
								path = _G[obj]
							else
								path = path[obj]
							end
						end
						SkinIcons(path, distance, hideOverlay)
					end
				end
			end	
			return
		end	

		if (item:GetObjectType() == "Texture") then
			texture = item
			item = item:GetParent()
		else
			if (item.icon) then
				texture = item.icon
			elseif (_G[tostring(item:GetName()).."IconTexture"]) then
				texture = _G[item:GetName().."IconTexture"]						
			elseif (item:GetObjectType() == "CheckButton" or item:GetObjectType() == "Button") then
				texture = item:GetNormalTexture()
				if ( (not texture) and (item:GetObjectType() == "CheckButton")) then
					texture = item:GetCheckedTexture()
				end
				if not (texture) then
					texture = item:GetHighlightTexture()
				end				
			end
		end

		if not (texture) then return end
		if not (item) then return end

		-- so now the texture and item is found:		
		texture:SetTexCoord(0.1, 0.92, 0.08, 0.92)		
		if (hideOverlay) then
			item:DisableDrawLayer("OVERLAY") -- to remove quality border by Blizzard!
		end
		if (item.new) then return end
		item.new = item:CreateTexture(nil, "BACKGROUND")
		item.new:SetDrawLayer("BACKGROUND", 5)
		item.new:SetAllPoints(true)
		item.new:SetSize(50, 50)
		item.new:SetTexture("Interface\\AddOns\\MayronUI\\Media\\Skin\\MUI_Button.tga")
		item.new:SetVertexColor(0,0,0,1)
		texture:SetDrawLayer("ARTWORK", 7)
		
		local d = distance or 1.5
		item.new:SetPoint("TOPLEFT", texture, "TOPLEFT", -(d), d)
		item.new:SetPoint("BOTTOMRIGHT", texture, "BOTTOMRIGHT", d, -(d))		
		if (texture:GetNumPoints() == 2) then			
			local p1 = { texture:GetPoint(1) }
			local p2 = { texture:GetPoint(2) }
			texture:ClearAllPoints()
			if (p1[1] == "TOPLEFT" and p2[1] == "BOTTOMRIGHT") then
				-- make texture smaller:
				texture:SetPoint(p1[1], p1[2], p1[3], p1[4] + d, p1[5] - d) -- TOPLEFT
				texture:SetPoint(p2[1], p2[2], p2[3], p2[4] - d, p2[5] + d) -- BOTTOMRIGHT
			end
		end
		if (item.GetHighlightTexture and item:GetHighlightTexture() ) then
			item:SetHighlightTexture("Interface\\AddOns\\MayronUI\\Media\\Skin\\Highlight_Button.blp")
		end		
		if (item:IsObjectType("CheckButton")) then
			item:SetCheckedTexture("Interface\\AddOns\\MayronUI\\Media\\Skin\\Highlight_Button.blp")
		end	
		return item
	end
		
	local function StartSkinning(tbl, left, right, top, bottom, button) -- sub tables have to be in tbl.
		for k, v in pairs(tbl) do			
			if ( type(v) == "table" ) then
				local tbl, left, right, top, bottom, button
				for id, arg in pairs(v) do				
					if (type(arg) == "table") then
						tbl = arg
					elseif (id == 2) then
						left = arg
					elseif (id == 3) then
						right = arg
					elseif (id == 4) then
						top = arg
					elseif (id == 5) then
						bottom = arg
					elseif (id == 6) then
						button = arg
					end
				end
				StartSkinning(tbl, left, right, top, bottom, button)
			else
				local located
				if (_G[k]) then
					k = _G[k]
					located = true
				else
					located = strfind(k, ".")
					if (located) then 
						local path
						for i = 1, #(k) do
							local obj = select(i, strsplit(".", k))
							if (not obj) then break end
							if (i == 1) then
								path = _G[obj]
							else
								path = path[obj]
							end
						end
						k = path
					end
				end
				if (located) then
					-- Old style:
					--[[if ( bit.band(v, MOVABLE) ~= 0 ) then
						MakeMovable(k) 
					end]]
					if ( bit.band(v, 124) ~= 0 ) then		
						SkinFrame(k, v) -- frame and number value
					end
					if ( bit.band(v, CREATE_FRAME) ~= 0 ) then
						if ( bit.band(v, SUB_FRAME) ~= 0 ) then
							CreateBG(k, left, right, top, bottom, button, nil, true)
						else
							CreateBG(k, left, right, top, bottom, button)
						end
					end
				end
			end
		end
	end
	
	---------------------------------
	--[[    Frame Alterations    ]]--
	---------------------------------
	
	-- NON-BLIZZARD ADDONS:
	-- MayronUI:
	if ( not MayronUI.db.profile.ReskinUI["MayronUI"] ) then
		if ( MayronUI_ChatBox:IsEnabled() and MUI_LayoutTool_OnLoad ) then
			hooksecurefunc("MUI_LayoutTool_OnLoad", function(self)
				CreateBG(self)
				StartSkinning(ScanDropdowns(MUI_LayoutToolAddons, 18), 20, 180, 0, 6, true)
				StartSkinning({["MUI_LayoutToolLayoutDropDown"] = 18}, 20, 115, 0, 6, true)
				MUI_LayoutToolLayoutDropDownButton:GetScript("OnShow")(MUI_LayoutToolLayoutDropDownButton)
				SkinArrows({"MUI_LayoutToolLayoutDropDown"})
				SkinFrame(MUI_LayoutTool, 16)
				MUI_LayoutToolHeaderText:SetPoint("TOP", 0, -14)
				for i = 1, 6 do
					if not (_G["MUI_AceAddon"..i]) then break end
					local self = _G["MUI_AceAddon"..i.."DropDownButton"]
					self:GetScript("OnShow")(self)
				end
			end)
		end
	end
	
	-- Recount:
	if ( IsAddOnLoaded("Recount") and not MayronUI.db.profile.ReskinUI["Recount"] ) then
		CreateBG(Recount_MainWindow, nil, nil, -8)			
		CreateBG(Recount_ConfigWindow, nil, nil, -8)
		Recount_MainWindow:SetBackdrop(nil)
		Recount_ConfigWindow:SetBackdrop(nil)
	end
	
	-- Color Picker Plus:
	if ( IsAddOnLoaded("ColorPickerPlus") and not MayronUI.db.profile.ReskinUI["ColorPickerPlus"]) then
		local run = true
		ColorPickerFrame:SetScript("OnShow", function()
			if (run) then
				run = nil
				StartSkinning({["ColorPPBoxR"] = 6, ["ColorPPBoxG"] = 6, ["ColorPPBoxB"] = 6, ["ColorPPBoxH"] = 6, ["ColorPPBoxA"] = 6})
			end
		end)
	end
	
	-- BLIZZARD FRAMES:
	--> Game Menu Frame:
	if ( not MayronUI.db.profile.ReskinUI["Game Menu"] ) then
		StartSkinning({["GameMenuFrame"] = 30})
	end

	FloatingGarrisonFollowerTooltip:DisableDrawLayer("BACKGROUND")
	CreateBG(FloatingGarrisonFollowerTooltip)

	--> Character Frame:	
	if ( not MayronUI.db.profile.ReskinUI["Character Frame"] ) then
		StartSkinning({["CharacterFrame"] = 46, ["CharacterModelFrame"] = 44, ["CharacterFrameInsetRight"] = 12, ["CharacterFrameInset"] = 12, ["ReputationListScrollFrame"] = 4, ["CharacterStatsPane"] = 16, ["PaperDollTitlesPaneScrollBar"] = 16, ["PaperDollEquipmentManagerPaneScrollBar"] = 16, ["PaperDollSidebarTabs"] = 16, ["ReputationDetailFrame"] = 62, ["GearManagerDialogPopup"] = 6, ["GearManagerDialogPopupEditBox"] = 4, ["GearManagerDialogPopupScrollFrame"] = 4, ["TokenFramePopup"] = 46})
		CreateBG(CharacterModelFrame, -1, 2, 1, -2)
		CreateBG(GearManagerDialogPopupEditBox, -4, -20)
		PaperDollTitlesPane:EnableMouse(0)
		PaperDollEquipmentManagerPane:EnableMouse(0)
		PaperDollSidebarTabs:SetPoint("BOTTOMRIGHT", CharacterFrameInsetRight, "TOPRIGHT", 2, 4)
		TokenFrameContainerScrollBar:DisableDrawLayer("ARTWORK")
		TokenFramePopup:SetPoint("TOPLEFT", TokenFrame, "TOPRIGHT", 2, 0)

		for i = 1, 15 do 
			SkinIcons(_G["GearManagerDialogPopupButton"..i])
		end
		
		local function UpdateFlyout()
			EquipmentFlyoutFrameButtons:DisableDrawLayer("BACKGROUND")
			EquipmentFlyoutFrameButtons:DisableDrawLayer("ARTWORK")
			local buttons = EquipmentFlyoutFrame.buttons				
			for i = 1, #buttons do 
				local btn = _G["EquipmentFlyoutFrameButton"..i]
				if (not btn) then break end
				SkinIcons(btn, nil, true)
				if (EquipmentFlyoutFrame.buttons[i].count == 0) then
					btn.new:SetVertexColor(1, 1, 1)
				else
					local r, g, b = btn.IconBorder:GetVertexColor()
					btn.new:SetVertexColor(r, g, b)
				end
			end
		end
		CreateBG(EquipmentFlyoutFrameButtons, nil, 4, nil, nil)
		EquipmentFlyoutFrameButtons:SetScript("OnShow", UpdateFlyout)
	
		local point, relframe, relpoint, x, y = CharacterModelFrame:GetPoint()
		CharacterModelFrame:SetPoint(point, relframe, relpoint, x, y + 3)
		point, relframe, relpoint, x, y = nil
		
		local lvl
		hooksecurefunc("PaperDoll_InitStatCategories", function(attributes)
			for i = 1, #attributes do 
				local pane = _G["CharacterStatsPaneCategory"..i]
				pane.NameText:SetDrawLayer("ARTWORK")
				pane:DisableDrawLayer("BACKGROUND")
				lvl = lvl or pane:GetFrameLevel() + 20
				pane:SetFrameLevel(lvl)
				for n = 1, 16 do				
					local stat = _G["CharacterStatsPaneCategory"..i.."Stat"..n]
					if (not stat) then break end
					stat:SetFrameLevel(lvl + 5)
				end
			end
		end)
		
		for i = 1, #PAPERDOLL_SIDEBARS do
			StartSkinning({["PaperDollSidebarTab"..i] = 36}) -- EXPERIMENTAL
			local tab = _G["PaperDollSidebarTab"..i]
			tab.Highlight:SetTexture("")
			tab:SetHighlightTexture("")
			SkinButton(_G["PaperDollSidebarTab"..i])
			local icon = select(2,tab:GetRegions())
			if (i == 1) then 
				icon:SetTexCoord(0.2, 0.8, 0.2, 0.8) 
			end
			icon:ClearAllPoints()
			icon:SetPoint("TOPLEFT", tab, "TOPLEFT", 1, -1)
			icon:SetPoint("BOTTOMRIGHT", tab, "BOTTOMRIGHT", -1, 1)
		end
		
		hooksecurefunc("PaperDollTitlesPane_Update", function()
			for i = 1, 16 do
				_G["PaperDollTitlesPaneButton"..i]:DisableDrawLayer("BACKGROUND")
			end
		end)
		
		-- Tabs:
		SkinTabs({"CharacterFrameTab1", "CharacterFrameTab2", "CharacterFrameTab3", "CharacterFrameTab4"})	
		
		local slots = {"Head", "Neck", "Shoulder", "Back", "Chest", "Shirt", "Tabard", "Wrist", "Waist", "Legs", "Feet", "Hands", "Finger0", "Finger1", "Trinket0", "Trinket1", "MainHand", "SecondaryHand"}
		
		local function Hook_SetSideBars()
			local numOfButtons = GetNumEquipmentSets() + 1
			for i = 1, numOfButtons do
				if _G["PaperDollEquipmentManagerPaneButton"..i] then		
					_G["PaperDollEquipmentManagerPaneButton"..i]:DisableDrawLayer("BACKGROUND")
					_G["PaperDollEquipmentManagerPaneButton"..i].icon:SetTexCoord(0.1, 0.92, 0.08, 0.92)
				end
			end
			for id, slot in ipairs(slots) do
				_G["Character"..slot.."SlotPopoutButton"]:SetFrameStrata("HIGH");	
			end
		end
		
		hooksecurefunc("PaperDollFrame_SetSidebar", Hook_SetSideBars)
		hooksecurefunc("GearManagerDialogPopupOkay_OnClick", Hook_SetSideBars)

		for id, slot in ipairs(slots) do
			local frame = _G["Character"..slot.."Slot"]
			select(10, frame:GetRegions()):Hide()
			local sides = select(11, frame:GetRegions())
			if (sides) then sides:Hide() end
			local icon = SkinIcons(frame, nil, true)			
			frame.ilvlFrame = CreateBG();
			local f = frame.ilvlFrame
			f.background:SetTexture(MUI_MEDIA.."GlowEffect");			
			f:SetBackdrop(nil);
			f:SetParent(icon);
			f:SetWidth(55);	
			f.ilvl = f:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
			f.durability = f:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
			if (id < 9 or slot == "SecondaryHand") then				
				f.background:SetRotation(3 * (math.pi/2))
				f:SetPoint("TOPLEFT",  icon, "TOPRIGHT", 0, 12)
				f:SetPoint("BOTTOMLEFT",  icon, "BOTTOMRIGHT", 0, -12)
				f.ilvl:SetPoint("LEFT", f, "LEFT", 8, 0);
				f.durability:SetPoint("LEFT", f, "LEFT", 8, -6);
			else
				f.background:SetRotation(math.pi/2)
				f:SetPoint("TOPRIGHT",  icon, "TOPLEFT", 0, 12)
				f:SetPoint("BOTTOMRIGHT",  icon, "BOTTOMLEFT", 0, -12)
				f.ilvl:SetPoint("RIGHT", f, "RIGHT", -8, 0);
				f.durability:SetPoint("RIGHT", f, "RIGHT", -8, -6);
			end
			
			frame:HookScript("OnEnter", function()
				if (IsAltKeyDown()) then
					UpdateFlyout()
				end
			end)
		end
		
		-- check box:
		local cb = CreateFrame("CheckButton", "MUI_CharacterAdvanceCheckButton", PaperDollFrame, "UICheckButtonTemplate");
		local function ToggleCharacterAdvanced()	
			MayronUI.db.profile.ReskinUI["Disable Advanced Character"] = not cb:GetChecked()
			if (not MayronUI.db.profile.ReskinUI["Disable Advanced Character"]) then
				MayronUI.db.profile.ReskinUI["Disable Advanced Character"] = nil;
			end
			for id, slot in ipairs(slots) do
				local frame = _G["Character"..slot.."Slot"]				
				if ( not MayronUI.db.profile.ReskinUI["Disable Advanced Character"] ) then
					frame.ilvlFrame:Show()
				else
					frame.ilvlFrame:Hide()
				end				
			end
		end		

		cb:SetPoint("BOTTOMLEFT", CharacterModelFrame, "TOPLEFT", -6, 0)
		cb.text:SetText("Advanced")
		cb:SetScript("OnClick", ToggleCharacterAdvanced)
		if ( not MayronUI.db.profile.ReskinUI["Disable Advanced Character"] ) then
			cb:SetChecked(true)
		end
		ToggleCharacterAdvanced()
		
		-- Reputation bars:
		for i = 1, 15 do
			_G["ReputationBar"..i]:DisableDrawLayer("ARTWORK")				
			for r = 1, _G["ReputationBar"..i.."ReputationBar"]:GetNumRegions() do
				local region = select(r, _G["ReputationBar"..i.."ReputationBar"]:GetRegions())
				if region:GetObjectType() == "FontString" then
					region:SetDrawLayer("OVERLAY")
				end
			end	
			_G["ReputationBar"..i.."ReputationBar"]:DisableDrawLayer("ARTWORK")
			local bg = CreateBG(_G["ReputationBar"..i.."ReputationBar"], -1, 1, 1, -1, nil, nil, true)
			_G["ReputationBar"..i.."ReputationBar"]:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
		end
		ReputationDetailFrame:ClearAllPoints()
		ReputationDetailFrame:SetPoint("TOPLEFT", CharacterFrame, "TOPRIGHT", 2, 0)

		local function UpdateCharacterSlots()
			for id, slot in ipairs(slots) do					
				local slotName = (slot.."Slot")
				local quality
				if ( GetInventorySlotInfo(slotName) ) then
					quality = GetInventoryItemQuality("player", GetInventorySlotInfo(slotName))			
				end					
				local frame = _G["Character"..slotName]
				if (quality) then					
					frame.icon:SetAlpha(1)			
					local r, g, b = GetItemQualityColor(quality)
					frame.new:SetVertexColor(r, g, b)
					frame.ilvlFrame.background:SetVertexColor(r, g, b, 0.35)
					
					-- Text update:
					local slotID = GetInventorySlotInfo(slotName)
					local itemLink = GetInventoryItemLink("player", slotID)
					local _, _, _, ilvl = GetItemInfo(itemLink)
					
					local diff = GetAverageItemLevel() - ilvl;
					if (diff > 10) then
						if (diff > 20) then
							ChangeTextColor(frame.ilvlFrame.ilvl, "grey", true)
						else
							ChangeTextColor(frame.ilvlFrame.ilvl, "green", true)
						end						
					elseif (diff < -10) then
						if (diff < -20) then
							ChangeTextColor(frame.ilvlFrame.ilvl, "red", true)
						else
							ChangeTextColor(frame.ilvlFrame.ilvl, "orange", true)
						end	
					else
						ChangeTextColor(frame.ilvlFrame.ilvl, "yellow", true)
					end					
					frame.ilvlFrame.ilvl:SetText(ilvl)
					
					local durability, maximum = GetInventoryItemDurability(slotID);
					if (durability and maximum) then
						local perc = math.floor((durability / maximum) * 100);
						if (perc < 25) then
							ChangeTextColor(frame.ilvlFrame.durability, "red", true)
						elseif (perc < 50) then
							ChangeTextColor(frame.ilvlFrame.durability, "orange", true)
						elseif (perc < 75) then
							ChangeTextColor(frame.ilvlFrame.durability, "yellow", true)
						else
							ChangeTextColor(frame.ilvlFrame.durability, "green", true)
						end
						if (perc == 100) then
							frame.ilvlFrame.durability:SetText("");
							if (id < 9 or slot == "SecondaryHand") then
								frame.ilvlFrame.ilvl:SetPoint("LEFT", frame.ilvlFrame, "LEFT", 8, 0);
							else
								frame.ilvlFrame.ilvl:SetPoint("RIGHT", frame.ilvlFrame, "RIGHT", -8, 0);
							end							
						else
							frame.ilvlFrame.durability:SetText(perc.."%");
							if (id < 9 or slot == "SecondaryHand") then
								frame.ilvlFrame.ilvl:SetPoint("LEFT", frame.ilvlFrame, "LEFT", 8, 6);
							else
								frame.ilvlFrame.ilvl:SetPoint("RIGHT", frame.ilvlFrame, "RIGHT", -8, 6);
							end
						end
					else
						frame.ilvlFrame.durability:SetText("");
						if (id < 9 or slot == "SecondaryHand") then
							frame.ilvlFrame.ilvl:SetPoint("LEFT", frame.ilvlFrame, "LEFT", 8, 0);
						else
							frame.ilvlFrame.ilvl:SetPoint("RIGHT", frame.ilvlFrame, "RIGHT", -8, 0);
						end	
					end
				else
					frame.icon:SetAlpha(0)
					frame.new:SetVertexColor(0,0,0)
					frame.ilvlFrame.background:SetVertexColor(0, 0, 0, 0)
				end
			end
		end
		hooksecurefunc("CharacterFrame_Expand", function() CharacterFrame:SetWidth(610) end)
		hooksecurefunc("CharacterFrame_Collapse", function() CharacterFrame:SetWidth(405) end)
		
		local events = CreateFrame("Frame")
		events:RegisterEvent("UPDATE_INVENTORY_DURABILITY")		
		events:RegisterUnitEvent("UNIT_INVENTORY_CHANGED", "player") -- should be registered to MayronUI!
		events:SetScript("OnEvent", UpdateCharacterSlots)
		PaperDollFrame:HookScript("OnShow", UpdateCharacterSlots)
		hooksecurefunc("PaperDollFrame_SetArmor", UpdateCharacterSlots)
		
		-- Change Frame for Item Level feature:
		CharacterModelFrame:SetWidth(300)
		hooksecurefunc("PaperDollFrame_SetLevel", function() CharacterFrame:SetWidth(610) end)	
		
		CharacterMainHandSlot:ClearAllPoints()
		CharacterMainHandSlot:SetPoint("CENTER", CharacterModelFrame, "BOTTOM", -20, 0)
		CharacterFrameInset:SetPoint("BOTTOMRIGHT", CharacterModelFrame, "BOTTOMRIGHT", 50, -35)
		hooksecurefunc("CharacterFrameTab_OnClick", function(self)			
			if ( self:GetName() == "CharacterFrameTab1" ) then
				CharacterFrame:SetWidth(610)
				CharacterFrameInset:SetPoint("BOTTOMRIGHT", CharacterModelFrame, "BOTTOMRIGHT", 50, 0)
			else
				CharacterFrame:SetWidth(338)
				CharacterFrameInset:SetPoint("BOTTOMRIGHT", CharacterFrame, "BOTTOMLEFT", 332, 4)
			end		
		end)
	end
	
	--> Dressing Room:
	if ( not MayronUI.db.profile.ReskinUI["Dressing Room"] ) then
		StartSkinning({["DressUpFrame"] = 22}, 10, -30, -8, 70)
	end
	
	--> Help Frame:	
	if ( not MayronUI.db.profile.ReskinUI["Help Frame"] ) then
		StartSkinning({["HelpFrameLeftInset"] = 12, ["HelpFrameHeader"] = 12, ["HelpFrame"] = 14})
		HelpFrameMainInset:Hide()
		HelpFrameCloseButton:SetPoint("TOPRIGHT", HelpFrame, "TOPRIGHT", 4, 4)
		SkinIcons(HelpFrameCharacterStuckHearthstone)
		local function UpdateButtons()		
			for i = 1, 7 do
				if (i == 7) then i = 16 end
				local self = _G["HelpFrameButton"..i]
				if (_G["HelpFrameButton"..i.."Selected"]:IsShown()) then
					self:SetBackdropBorderColor(1,1,1)
					self.oldBC = {1, 1, 1}
				else
					self:SetBackdropBorderColor(0,0,0)
					self.oldBC = {0,0,0}
				end
			end		
		end		
		for i = 1, 6 do
			SkinFrame(_G["HelpFrameButton"..i], "ARTWORK")
			SkinButton(_G["HelpFrameButton"..i])
			_G["HelpFrameButton"..i.."Selected"]:SetAlpha(0)
			_G["HelpFrameButton"..i]:HookScript("OnClick", UpdateButtons)
		end
		SkinFrame(HelpFrameButton16, "ARTWORK")
		SkinButton(HelpFrameButton16)
		HelpFrameButton16Selected:SetAlpha(0)
		HelpFrameButton16:HookScript("OnClick", UpdateButtons)
	end
	
	--> Item Text Frame:
	if ( not MayronUI.db.profile.ReskinUI["Item Text"] ) then
		StartSkinning({["ItemTextFrame"] = 62})
		ItemTextFrameInset:Hide()
		ItemTextScrollFrame:DisableDrawLayer("BACKGROUND")
		ItemTextScrollFrame:DisableDrawLayer("ARTWORK")
		ChangeTextColor(ItemTextPageText, "white")
	end

	--> World Map:
	if ( not MayronUI.db.profile.ReskinUI["World Map"] ) then
		WorldMapFrame:DisableDrawLayer("BACKGROUND")
		WorldMapFrame:HookScript("OnShow", function()
			if ( WORLDMAP_SETTINGS.size == WORLDMAP_WINDOWED_SIZE ) then
				if (QuestScrollFrame.Contents:IsVisible()) then
					WorldMapFrame:SetWidth(1032)
					WorldMapFrame:SetMovable(true)
				end
			else
				WorldMapFrameTutorialButton:Hide()
				WorldMapFrame:EnableMouse(false)
				WorldMapFrame:SetMovable(false)
			end
		end)
	
		UIPanelWindows["WorldMapFrame"] = nil
		tinsert(UISpecialFrames, "WorldMapFrame")
		QuestMapFrame.DetailsFrame:EnableMouse(false)
		WorldMapFrameNavBarOverflowButton.arrow = WorldMapFrameNavBarOverflowButton:CreateFontString(nil, "ARTWORK")
		WorldMapFrameNavBarOverflowButton.arrow:SetFontObject("GameFontHighlightLarge")
		WorldMapFrameNavBarOverflowButton.arrow:SetText("<")
		WorldMapFrameNavBarOverflowButton.arrow:SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
		WorldMapFrameNavBarOverflowButton.arrow:SetAllPoints(WorldMapFrameNavBarOverflowButton)
		WorldMapFrameNavBarOverflowButton:SetScript("OnEnter", function()
			WorldMapFrameNavBarOverflowButton.arrow:SetTextColor(1,1,1)
		end)
		WorldMapFrameNavBarOverflowButton:SetScript("OnLeave", function()
			WorldMapFrameNavBarOverflowButton.arrow:SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
		end)
		WorldMapFrameNavBarOverflowButton:SetPushedTexture("")
		WorldMapFrameNavBarOverflowButton:SetNormalTexture("")
		WorldMapFrameNavBarOverflowButton:SetHighlightTexture("")
		SkinButton(WorldMapFrameNavBarOverflowButton)
		
		-- Alter Button Position to not clash with nav bar:
		QuestScrollFrame.ViewAll:SetWidth(180)
		QuestMapFrame.DetailsFrame.BackButton:SetPoint("TOPLEFT", QuestMapFrame.DetailsFrame, "TOPLEFT",  200, 30)
		QuestScrollFrame.ViewAll:SetPoint("TOPRIGHT", QuestMapFrame, "TOPRIGHT", 5, 29)
		
		CreateBG(WorldMapFrame.BorderFrame, -22, 40)
		WorldMapFrameCloseButton:ClearAllPoints()
		WorldMapFrameCloseButton:SetPoint("TOPRIGHT", WorldMapFrame.BorderFrame.new, "TOPRIGHT", 0, 0)		
		WorldMapFrame:HookScript("OnKeyDown", function(self, key)
			local binding = GetBindingFromClick(key)
			if ((binding == "OPENCHAT") or (binding == "OPENCHATSLASH")) then
				ChatEdit_ActivateChat(_G[SELECTED_CHAT_FRAME:GetName().."EditBox"])
			end
		end)
		
		hooksecurefunc("WorldMap_ToggleSizeUp", function()
			WorldMapFrame:EnableMouse(false)
			WorldMapFrame:EnableKeyboard(false)
			WorldMapFrame:DisableDrawLayer("BACKGROUND")				
		end)
		hooksecurefunc("WorldMap_ToggleSizeDown", function()
			WorldMapFrame:EnableMouse(true)			
		end)
		hooksecurefunc("QuestMapFrame_Open", function(value)
			WorldMapFrame:SetWidth(1032)
			for _, title in pairs(QuestScrollFrame.Contents.Titles) do 
				title:SetParent(QuestScrollFrame.Contents)
				title:SetWidth(300)
			end
		end)

		WorldMapFrameTutorialButton:DisableDrawLayer("BORDER")
		WorldMapFrameTutorialButton:SetPoint("TOPLEFT", WorldMapFrame.BorderFrame, "TOPLEFT", -36, 16)
		QuestMapFrame:DisableDrawLayer("OVERLAY")
		SkinFrame(QuestScrollFrame, 4)
		
		WorldMapFrame.BorderFrame.Inset:Hide()
		QuestMapFrame:EnableMouse(false)
		SkinFrame(WorldMapFrame.BorderFrame, 44)
		SkinFrame(WorldMapFrameNavBar, 12)
		SkinFrame(WorldMapFrameNavBarHomeButton, 48)
		SkinButton(WorldMapFrameNavBarHomeButton)
		SkinFrame(QuestMapFrame.DetailsFrame, 28)
		SkinFrame(QuestMapFrame.DetailsFrame.RewardsFrame, 8)
		
		select(2, MapQuestInfoRewardsFrame.MoneyFrame:GetRegions()):Hide()
		select(2, MapQuestInfoRewardsFrame.SkillPointFrame:GetRegions()):Hide()
		select(2, MapQuestInfoRewardsFrame.XPFrame:GetRegions()):Hide()		
		
		SkinIcons(select(1, MapQuestInfoRewardsFrame.MoneyFrame:GetRegions()), 1)
		SkinIcons(select(1, MapQuestInfoRewardsFrame.SkillPointFrame:GetRegions()), 1)
		SkinIcons(select(1, MapQuestInfoRewardsFrame.XPFrame:GetRegions()), 1)
		
		MapQuestInfoRewardsFrame.SkillPointFrame.CircleBackground:Hide()
		MapQuestInfoRewardsFrame.SkillPointFrame.CircleBackgroundGlow:Hide()
		MapQuestInfoRewardsFrame.SkillPointFrame.ValueText:SetPoint("CENTER", MapQuestInfoRewardsFrame.SkillPointFrame.Icon, "BOTTOMRIGHT", -6, 6)
		
		for i = 1, #(MapQuestInfoRewardsFrame.RewardButtons) do		
			local f = _G["MapQuestInfoRewardsFrameQuestInfoItem"..i]
			if (f) then
				select(2, f:GetRegions()):Hide()
				SkinIcons(select(1, f:GetRegions()), 1)
			end
		end
		
		QuestMapFrame.DetailsFrame.ShareButton:DisableDrawLayer("OVERLAY")
		WorldMapTitleButton:EnableMouse(false)		
		SkinArrows({"WorldMapLevelDropDown"})
		StartSkinning({["WorldMapLevelDropDown"] = 18}, 19, -16, 0, 6, true)
		WorldMapLevelDropDown:ClearAllPoints()
		WorldMapLevelDropDown:SetPoint("TOPLEFT", WorldMapPOIFrame, "TOPLEFT", -14, -4)

		-- Scroll Bars:
		SkinFrame(QuestScrollFrameScrollBar, 16)
		QuestMapDetailsScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")
			
		local TEXT_WIDTH = 300
		QuestScrollFrame:SetWidth(TEXT_WIDTH + 10)
		QuestScrollFrame:ClearAllPoints()
		QuestScrollFrame:SetPoint("TOPLEFT", WorldMapFrame.UIElementsFrame, "TOPRIGHT", 5, 0)
		QuestScrollFrameScrollBar:SetPoint("TOPLEFT", QuestScrollFrame, "TOPRIGHT", 16, -15)
		
		QuestMapDetailsScrollFrame:SetWidth(TEXT_WIDTH + 10)
		QuestMapDetailsScrollFrameScrollBar:SetPoint("TOPLEFT", QuestMapDetailsScrollFrame, "TOPRIGHT", 10, -14)
		
		hooksecurefunc("SelectQuestLogEntry", function()
			for _, name in ipairs({"MapQuestInfoRewardsFrameQuestInfoItem", "QuestInfoRewardsFrameQuestInfoItem"}) do	 
				for _, total in ipairs({GetNumQuestLogChoices(), GetNumQuestLogRewards()}) do	 
					for i = 1, total do
						local item = _G[name..i]
						if (not item) then break end
						select(2, item:GetRegions()):Hide()
						SkinIcons(select(1, item:GetRegions()), 1)
						local quality = (select(4, GetQuestLogChoiceInfo(i)))
						if (quality) then
							local r, g, b = GetItemQualityColor(quality)
							item.new:SetVertexColor(r, g, b)
						else
							item.new:SetVertexColor(0,0,0)
						end
					end
				end
			end
		end)
		
		hooksecurefunc("QuestMapFrame_ShowQuestDetails", function(questID)	
			local questInfo = QuestMapDetailsScrollFrame.Contents
			if (questInfo:GetNumRegions() < 5) then
				ChangeTextColor(select(1, questInfo:GetRegions()), "yellow") -- questName
				ChangeTextColor(select(2, questInfo:GetRegions()), "white") -- summary
				ChangeTextColor(select(3, questInfo:GetRegions()), "yellow") -- descriptionTitle
				ChangeTextColor(select(4, questInfo:GetRegions()), "white") -- content
				select(2, questInfo:GetRegions()):SetWidth(TEXT_WIDTH) -- summary
				select(4, questInfo:GetRegions()):SetWidth(TEXT_WIDTH) -- content	
				questInfo.reskinned = true
			elseif (questInfo.reskinned) then
				ChangeTextColor(select(5, questInfo:GetRegions()), "orange") -- changed to suggested players
				select(2, questInfo:GetRegions()):SetWidth(TEXT_WIDTH)
				select(5, questInfo:GetRegions()):SetWidth(TEXT_WIDTH)
			else
				ChangeTextColor(select(1, questInfo:GetRegions()), "yellow") -- quest name
				ChangeTextColor(select(2, questInfo:GetRegions()), "white") -- summary
				ChangeTextColor(select(3, questInfo:GetRegions()), "orange") -- suggested players
				ChangeTextColor(select(4, questInfo:GetRegions()), "yellow") -- description title
				ChangeTextColor(select(5, questInfo:GetRegions()), "white") -- content
				select(2, questInfo:GetRegions()):SetWidth(TEXT_WIDTH) -- summary
				select(5, questInfo:GetRegions()):SetWidth(TEXT_WIDTH) -- content	
			end	
					
			for i = 1, #QuestInfoObjectivesFrame.Objectives do
				local fs = _G["QuestInfoObjective"..i]
				if (not fs) then break end
				local text = fs:GetText()
				if (text and text ~= "") then					
					local current, total = text:match("(%d+)/(%d+)")
					if (current and total) then
						local percent = tonumber(current) / tonumber(total) * 100
						percent = math.floor(percent * 10) / 10
						if (percent < 40) then
							fs:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, 1)
						elseif (percent < 70 and percent >= 40) then
							fs:SetTextColor(ORANGE_FONT_COLOR.r, ORANGE_FONT_COLOR.g, ORANGE_FONT_COLOR.b, 1)
						elseif (percent < 100 and percent >= 70) then
							fs:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b, 1)
						elseif (percent == 100) then
							fs:SetTextColor(YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b, 1)
						end
						if (percent ~= 100 and percent ~= 0) then
							fs:SetText(text.." ("..percent.."%)")
						end
						fs:SetWidth(TEXT_WIDTH)
					else
						local complete = text:find("(Complete)")
						if (not complete) then
							fs:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, 1)
						else
							fs:SetTextColor(YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b, 1)
						end
					end
				end
			end
		end)

		hooksecurefunc("WorldMapNavBar_Update", function()			
			local IDs = {}
			local sort = {}
			for i = 2, 5 do
				local btn = _G["WorldMapFrameNavBarButton"..i]
				if (not btn) then break end
				SkinFrame(btn, 48)
				SkinButton(btn)
				_G["WorldMapFrameNavBarButton"..i].MenuArrowButton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
				_G["WorldMapFrameNavBarButton"..i].MenuArrowButton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
				_G["WorldMapFrameNavBarButton"..i].MenuArrowButton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Disabled")				
				local anchor
				local n = 1						
				IDs["WorldMapFrameNavBarButton"..i] = btn.id
				table.insert(sort, btn.id)
			end	
			-- sorting:				
			table.sort(sort)
			local anchor
			for i, value1 in ipairs(sort) do					
				for bName, value2 in pairs(IDs) do						
					if (value1 == value2) then							
						if (not anchor) then
							anchor = WorldMapFrameNavBarHomeButton
						end
						local frame = _G[bName]
						if (frame and not (frame == anchor)) then
							if (frame:IsVisible()) then
								frame:ClearAllPoints()
								frame:SetPoint("LEFT", anchor, "RIGHT", 2, 0)
								anchor = frame									
							end
						end
					end
				end
			end
			sort = {}
			IDs = {}
			WorldMapFrameNavBarOverlay:Hide()
		end)
		
		QuestScrollFrame.ViewAll:HookScript("OnClick", function()
			for _, title in pairs(QuestScrollFrame.Contents.Titles) do title:SetWidth(300) end
		end)
	end
	
	--> QuestLogPopupDetailFrame, QuestFrame, GossipFrame, QuestProgress, GreetingPanel GuildRegistrarFrame and PetitionFrame:
	if ( not MayronUI.db.profile.ReskinUI["Quest Frames"] ) then
	
		--> QuestLogPopupDetailFrame:		
		SkinFrame(QuestLogPopupDetailFrame, 44)
		CreateBG(QuestLogPopupDetailFrame)
		QuestLogPopupDetailFrameInset:Hide()
		SkinFrame(QuestLogPopupDetailFrameScrollFrame, 16)
		ChangeTextColor(QuestInfoRewardsFrame.SpellLearnText, "yellow") -- "The following spell will be cast on you:"
		
		hooksecurefunc("ObjectiveTrackerBlockHeader_OnClick", function(self, mouseButton)
			if (mouseButton == "LeftButton" and (not IsShiftKeyDown())) then
				if (QuestLogPopupDetailFrameScrollFrame.ScrollChild:IsVisible()) then
					for i = 1, #QuestInfoRewardsFrame.RewardButtons do
						local item = _G["QuestInfoRewardsFrameQuestInfoItem"..i]
						if (not item) then break end
						select(2, item:GetRegions()):Hide()
						SkinIcons(item, 1)
						local quality = (select(4, GetQuestLogChoiceInfo(i)))
						if (quality) then
							local r, g, b = GetItemQualityColor(quality)
							item.new:SetVertexColor(r, g, b)
						else
							item.new:SetVertexColor(0,0,0)
						end
					end
					local questInfo = QuestLogPopupDetailFrameScrollFrame.ScrollChild
					if (questInfo:GetNumRegions() < 5) then
						ChangeTextColor(select(1, questInfo:GetRegions()), "yellow") -- questName
						ChangeTextColor(select(2, questInfo:GetRegions()), "white") -- summary
						ChangeTextColor(select(3, questInfo:GetRegions()), "yellow") -- descriptionTitle
						ChangeTextColor(select(4, questInfo:GetRegions()), "white") -- content
						questInfo.reskinned = true
					elseif (questInfo.reskinned) then
						ChangeTextColor(select(5, questInfo:GetRegions()), "orange") -- changed to suggested players
					else
						ChangeTextColor(select(1, questInfo:GetRegions()), "yellow") -- quest name
						ChangeTextColor(select(2, questInfo:GetRegions()), "white") -- summary
						ChangeTextColor(select(3, questInfo:GetRegions()), "orange") -- suggested players
						ChangeTextColor(select(4, questInfo:GetRegions()), "yellow") -- description title
						ChangeTextColor(select(5, questInfo:GetRegions()), "white") -- content
					end
					for i = 1, #QuestInfoObjectivesFrame.Objectives do
						local fs = _G["QuestInfoObjective"..i]
						if (not fs) then break end
						local text = fs:GetText()
						if (text and text ~= "") then					
							local current, total = text:match("(%d+)/(%d+)")
							if (current and total) then
								local percent = tonumber(current) / tonumber(total) * 100
								percent = math.floor(percent * 10) / 10
								if (percent < 40) then
									fs:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, 1)
								elseif (percent < 70 and percent >= 40) then
									fs:SetTextColor(ORANGE_FONT_COLOR.r, ORANGE_FONT_COLOR.g, ORANGE_FONT_COLOR.b, 1)
								elseif (percent < 100 and percent >= 70) then
									fs:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b, 1)
								elseif (percent == 100) then
									fs:SetTextColor(YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b, 1)
								end
								if (percent ~= 100 and percent ~= 0) then
									fs:SetText(text.." ("..percent.."%)")
								end
								fs:SetWidth(280)
							else
								local complete = text:find("(Complete)")
								if (not complete) then
									fs:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, 1)
								else
									fs:SetTextColor(YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b, 1)
								end
							end
						end
					end
					ChangeTextColor(select(1, QuestInfoRewardsFrame:GetRegions()), "yellow") -- rewardsTitle
					ChangeTextColor(select(2, QuestInfoRewardsFrame:GetRegions()), "white") -- "You will receive:"
					ChangeTextColor(select(3, QuestInfoRewardsFrame:GetRegions()), "white") -- "You will also receive:"
					ChangeTextColor(select(1, QuestInfoXPFrame:GetRegions()), "white") -- "Experience:"
				end
			end
		end)
		
		--> QuestFrame:
		SkinFrame(QuestFrame, 44)
		CreateBG(QuestFrame)	
		
		QuestFrameInset:Hide()
		QuestFrameRewardPanel:DisableDrawLayer("BACKGROUND")
		QuestRewardScrollFrame:DisableDrawLayer("ARTWORK")
		QuestFrameDetailPanel:DisableDrawLayer("BACKGROUND")
		QuestDetailScrollFrame:DisableDrawLayer("ARTWORK")
		SkinFrame(QuestNPCModel, 36)
		SkinFrame(QuestNPCModelTextFrame, 36)
		CreateBG(QuestNPCModel, -1, 2, 1, -1)
		CreateBG(QuestNPCModelTextFrame, -1, 2, 18)
		
		-- Special Learn Spell quest icon:
		QuestInfoSpellObjectiveFrameNameFrame:Hide()
		ChangeTextColor(QuestInfoSpellObjectiveLearnLabel, "yellow")
		QuestInfoSpellObjectiveFrameSpellBorder:Hide()
		SkinIcons(QuestInfoSpellObjectiveFrameIconTexture, 1)
		
		hooksecurefunc("QuestFrame_ShowQuestPortrait", function(parentFrame)		
			QuestNPCModel:ClearAllPoints()
			if (parentFrame == WorldMapFrame) then
				QuestNPCModel:SetPoint("TOPLEFT", parentFrame.BorderFrame.new, "TOPRIGHT", 2, -1)
			else
				QuestNPCModel:SetPoint("TOPLEFT", parentFrame, "TOPRIGHT", 4, -1)
			end
		end)
		
		hooksecurefunc("ShowUIPanel", function(frame)
			if (frame == QuestFrame) then				
				 -- Hand in Quest
				if (QuestFrameRewardPanel:IsShown()) then				
					ChangeTextColor(select(1, QuestRewardScrollChildFrame:GetRegions()), "yellow") -- questTitle
					ChangeTextColor(select(2, QuestRewardScrollChildFrame:GetRegions()), "white") -- questDescription				
				elseif (QuestFrameProgressPanel:IsShown()) then -- Continue to reward frame
					ChangeTextColor(select(1, QuestProgressScrollChildFrame:GetRegions()), "yellow") -- questTitle
					ChangeTextColor(select(2, QuestProgressScrollChildFrame:GetRegions()), "white") -- summary
					ChangeTextColor(select(3, QuestProgressScrollChildFrame:GetRegions()), "yellow") -- "Required items:"
					for i = 1, 10 do
						local btn = _G["QuestProgressItem"..i]
						if (not btn) then break end
						btn.NameFrame:Hide()
						SkinIcons(btn.Icon)
					end					
				elseif (QuestDetailScrollChildFrame:IsVisible()) then  -- Accept in Quest
					ChangeTextColor(select(1, QuestDetailScrollChildFrame:GetRegions()), "yellow") -- questTitle
					ChangeTextColor(select(2, QuestDetailScrollChildFrame:GetRegions()), "white") -- questDescription
					ChangeTextColor(select(3, QuestDetailScrollChildFrame:GetRegions()), "yellow") -- "Quest Objectives"
					ChangeTextColor(select(4, QuestDetailScrollChildFrame:GetRegions()), "white") -- summary	
					local suggested = select(5, QuestDetailScrollChildFrame:GetRegions())
					if (suggested) then
						local num = suggested:GetText():match("[%d]")
						if (num == "2") then
							ChangeTextColor(suggested, "yellow")
						elseif (num == "3") then
							ChangeTextColor(suggested, "orange")
						else
							ChangeTextColor(suggested, "red")
						end
					end
				end
				
				select(2, QuestInfoRewardSpell:GetRegions()):Hide()
				select(4, QuestInfoRewardSpell:GetRegions()):Hide()
				SkinIcons(QuestInfoRewardSpell.Icon)
				
				ChangeTextColor(select(1, QuestInfoRewardsFrame:GetRegions()), "yellow") -- rewardsTitle
				ChangeTextColor(select(2, QuestInfoRewardsFrame:GetRegions()), "white") -- "You will receive:"
				ChangeTextColor(select(3, QuestInfoRewardsFrame:GetRegions()), "white") -- "You will also receive:"
				ChangeTextColor(select(1, QuestInfoXPFrame:GetRegions()), "white") -- "Experience:"

				-- GetNumQuestChoices
				for i = 1, GetNumQuestRewards() do
					local item = _G["QuestInfoRewardsFrameQuestInfoItem"..i]
					if (not item or not item:IsVisible()) then break end
					select(2, item:GetRegions()):Hide()
					SkinIcons(select(1, item:GetRegions()), 1)					
					local quality = (select(4, GetQuestItemInfo("choice", i)))
					if (quality) then
						local r, g, b = GetItemQualityColor(quality)
						item.new:SetVertexColor(r, g, b)
					else
						item.new:SetVertexColor(0,0,0)
					end
				end
			end
		end)
		
		--> GossipFrame
		SkinFrame(GossipFrame, 44)
		CreateBG(GossipFrame)
		GossipFrameInset:Hide()
		GossipGreetingScrollFrame:DisableDrawLayer("ARTWORK")
		ChangeTextColor(select(1, GossipGreetingScrollChildFrame:GetRegions()), "white")
		hooksecurefunc("GossipFrameUpdate", function()
			for i = 1, NUMGOSSIPBUTTONS do
				_G["GossipTitleButton" .. i]:GetFontString():SetTextColor(1,1,1)
			end
		end)
		
		--> QuestProgress:
		QuestProgressScrollFrame:DisableDrawLayer("ARTWORK")
		QuestFrameProgressPanel:DisableDrawLayer("BACKGROUND")
		
		--> GreetingPanel:
		QuestFrameGreetingPanel:DisableDrawLayer("BACKGROUND")
		QuestGreetingScrollFrame:DisableDrawLayer("ARTWORK")
		ChangeTextColor(GreetingText, "white")		
		ChangeTextColor(AvailableQuestsText, "yellow")
		ChangeTextColor(CurrentQuestsText, "yellow")		
		
		--> GuildRegistrarFrame:
		StartSkinning({["GuildRegistrarFrame"] = 62})		
		GuildRegistrarFrameInset:Hide()
		ChangeTextColor(AvailableServicesText, "yellow")
		ChangeTextColor(GuildRegistrarButton1:GetFontString(), "white")
		ChangeTextColor(GuildRegistrarButton2:GetFontString(), "white")	
		ChangeTextColor(GuildRegistrarPurchaseText, "white")
		GuildRegistrarFrameEditBox:DisableDrawLayer("BACKGROUND")
		CreateBG(GuildRegistrarFrameEditBox, -4, 2, -2, 2)
		
		--> PetitionFrame:
		StartSkinning({["PetitionFrame"] = 62})		
		PetitionFrameInset:Hide()
		ChangeTextColor(PetitionFrameCharterTitle, "yellow")
		ChangeTextColor(PetitionFrameCharterName, "white")
		ChangeTextColor(PetitionFrameMasterTitle, "yellow")
		ChangeTextColor(PetitionFrameMasterName, "white")
		ChangeTextColor(PetitionFrameMemberTitle, "yellow")
		for i = 1, 9 do
			local name = _G["PetitionFrameMemberName"..i]
			if (not name) then break end
			ChangeTextColor(name, "white")
		end
		ChangeTextColor(PetitionFrameInstructions, "white")		
	end
	
	--> LFGDungeonReadyDialog:
	if ( not MayronUI.db.profile.ReskinUI["LFG Dungeon Ready"] ) then
		hooksecurefunc("LFGDungeonReadyPopup_Update", function()
			StartSkinning({["LFGDungeonReadyDialog"] = 46, ["LFGDungeonReadyStatus"] = 14})
		end)
	end
	
	hooksecurefunc("UIParent_ManageFramePositions", function()
		if (VehicleSeatIndicator:IsVisible()) then
			VehicleSeatIndicator:ClearAllPoints()
			if (ObjectiveTrackerBlocksFrame:IsVisible()) then
				VehicleSeatIndicator:SetPoint("TOPRIGHT", ObjectiveTrackerBlocksFrame, "TOPLEFT", -30, 5)
			elseif (_G["MUI_SideBars"] and MUI_SideBars:IsVisible()) then
				VehicleSeatIndicator:SetPoint("TOPRIGHT", MUI_SideBars, "TOPLEFT", -5, 5)
			end
		end
		if (DurabilityFrame:IsVisible()) then			
			DurabilityFrame:ClearAllPoints()
			if (ObjectiveTrackerBlocksFrame:IsVisible()) then
				DurabilityFrame:SetPoint("TOPRIGHT", ObjectiveTrackerBlocksFrame, "TOPLEFT", -80, 5)
			elseif (_G["MUI_SideBars"] and MUI_SideBars:IsVisible()) then
				DurabilityFrame:SetPoint("TOPRIGHT", MUI_SideBars, "TOPLEFT", -5, 5)
			end
		end
		MayronUI:SetTrackerPosition()
	end)
	
	--> Color Picker Frame:
	if ( not MayronUI.db.profile.ReskinUI["Color Picker Frame"] ) then
		StartSkinning({["ColorPickerFrame"] = 14}, nil, nil, 4)
		ColorPickerFrameHeader:Hide()
		local m = CreateFrame('Frame', nil, ColorPickerFrame)
		m:SetPoint("TOPLEFT", ColorPickerFrame, "TOPLEFT")
		m:SetPoint("BOTTOMRIGHT", ColorPickerFrame, "TOPRIGHT", 0, -20)
		m:SetScript('OnMouseDown', function() ColorPickerFrame:StartMoving() end)
		m:SetScript('OnMouseUp', function() ColorPickerFrame:StopMovingOrSizing() end)
	end
	
	--> Addon Frame:
	if ( not MayronUI.db.profile.ReskinUI["Addon Frame"] ) then
		StartSkinning({["AddonList"] = 46})
		AddonListInset:Hide()
		AddonListScrollFrame:DisableDrawLayer("BACKGROUND")
		StartSkinning({["AddonCharacterDropDown"] = 18}, 20, 110, 0, 6, true)
		SkinArrows({"AddonCharacterDropDown"})
		AddonListEnableAllButton:DisableDrawLayer("BORDER")
		AddonListDisableAllButton:DisableDrawLayer("BORDER")
		AddonListOkayButton:DisableDrawLayer("BORDER")
		AddonListCancelButton:DisableDrawLayer("BORDER")
	end

	SkinButton(GhostFrame)
	for i = 1, 3 do
		select(i, GhostFrame:GetRegions()):Hide()
	end
		
	--> Score Frame:
	if ( not MayronUI.db.profile.ReskinUI["Score Frame"] ) then
		StartSkinning({["WorldStateScoreFrame"] = 46, ["WorldStateScoreWinnerFrame"] = 8, ["WorldStateScoreScrollFrame"] = 16, ["WorldStateScoreFrameInset"] = 12})
		hooksecurefunc("WorldStateScoreFrame_Update", function()
			if (GetBattlefieldWinner()) then
				WorldStateScoreFrameLabel:Hide()
			else
				WorldStateScoreFrameLabel:Show()
			end
		end)
		SkinTabs({"WorldStateScoreFrameTab1", "WorldStateScoreFrameTab2", "WorldStateScoreFrameTab3"})
	end
	
	--> Drop Down Lists:
	if ( not MayronUI.db.profile.ReskinUI["Drop Down Lists"] ) then
		StartSkinning({["DropDownList1Backdrop"] = 12, ["DropDownList1"] = 2, ["DropDownList2Backdrop"] = 12, ["DropDownList2"] = 2, ["DropDownList1MenuBackdrop"] = 12, ["DropDownList2MenuBackdrop"] = 14})
	end
	
	--> LFD Role Check Popup:
	if ( not MayronUI.db.profile.ReskinUI["LFD Role Check Popup"] ) then
		StartSkinning({["LFDRoleCheckPopup"] = 14})
	end
	
	--> PVP Ready Dialog:
	if ( not MayronUI.db.profile.ReskinUI["PVP Ready Dialog"] ) then
		StartSkinning({["PVPReadyDialog"] = 38})
	end
	
	--> ChatConfigFrame
	if ( not MayronUI.db.profile.ReskinUI["Chat Config Frame"] ) then
		StartSkinning({["ChatConfigFrame"] = 30})
		ChatConfigCategoryFrame:SetBackdrop(nil)
		ChatConfigBackgroundFrame:SetBackdrop(nil)
		ChatConfigChatSettingsLeft:SetBackdrop(nil)
		ChatConfigChatSettingsClassColorLegend:SetBackdrop(nil)
		ChatConfigChannelSettingsClassColorLegend:SetBackdrop(nil)
		ChatConfigChannelSettingsLeft:SetBackdrop(nil)
		ChatConfigChannelSettingsClassColorLegend:SetBackdrop(nil)
		ChatConfigOtherSettingsCombat:SetBackdrop(nil)
		ChatConfigOtherSettingsPVP:SetBackdrop(nil)
		ChatConfigOtherSettingsSystem:SetBackdrop(nil)
		ChatConfigOtherSettingsCreature:SetBackdrop(nil)
		
		hooksecurefunc("ChatConfig_UpdateCheckboxes", function(self)
			for i = 1, 17 do
				_G["ChatConfigChatSettingsLeftCheckBox"..i]:SetBackdrop(nil)
			end
			if (ChatConfigChannelSettingsLeft:IsVisible()) then
				for i = 1, 4 do
					if (not _G["ChatConfigChannelSettingsLeftCheckBox"..i]) then break end
					_G["ChatConfigChannelSettingsLeftCheckBox"..i]:SetBackdrop(nil)
				end
			end
			if (ChatConfigOtherSettings:IsVisible()) then
				for i = 1, 12 do
					_G["ChatConfigOtherSettingsCombatCheckBox"..i]:SetBackdrop(nil)
				end
				for i = 1, 3 do			
					_G["ChatConfigOtherSettingsPVPCheckBox"..i]:SetBackdrop(nil)
				end
				for i = 1, 8 do			
					_G["ChatConfigOtherSettingsSystemCheckBox"..i]:SetBackdrop(nil)
				end
				for i = 1, 6 do
					_G["ChatConfigOtherSettingsCreatureCheckBox"..i]:SetBackdrop(nil)			
				end
			end
		end)
	end
	
	--> Guild Invite Frame:
	if ( not MayronUI.db.profile.ReskinUI["Guild Invite Frame"] ) then
		StartSkinning({["GuildInviteFrame"] = 14})
		GuildInviteFrameTabardBorder:Hide()
		GuildInviteFrameTabardRing:Hide()
	end
	
	--> Compact Raid Manager:
	if ( not MayronUI.db.profile.ReskinUI["Compact Raid Manager"] ) then
		StartSkinning({["CompactRaidFrameManager"] = 16, ["CompactRaidFrameManagerDisplayFrame"] = 28})
		local new = CreateBG(CompactRaidFrameManager)
		new.background:SetTexture(nil)
		new.background = new:CreateTexture()
		new.background:SetPoint("TOPLEFT", new, "TOPLEFT", 1, -1)
		new.background:SetPoint("BOTTOMRIGHT", new, "BOTTOMRIGHT", -1, 1)
		new.background:SetTexture(0.1, 0.1, 0.1, 1)
		
		-- Toggle Button stuff:
		if ( not (MayronUI.db.profile.modules["ChatBox"] and MayronUI.db.profile.ChatBox["TOPLEFT"]) ) then
			CompactRaidFrameManager.toggleButton:SetNormalTexture("Interface\\MoneyFrame\\Arrow-Right-Down.png")
			CompactRaidFrameManager.toggleButton:GetNormalTexture():SetTexCoord(0, 1, 0, 1)
			
			local p, rt, rp, x, y = CompactRaidFrameManager.toggleButton:GetPoint()
			CompactRaidFrameManager.toggleButton:SetPoint(p, rt, rp, x + 12, y)
			
			local width, height = CompactRaidFrameManager.toggleButton:GetSize()
			CompactRaidFrameManager.toggleButton:SetSize(width + 4, height - 8)
			width, height = nil
			
			CompactRaidFrameManager.toggleButton:SetScript("OnEnter", function(self)
				self:GetNormalTexture():SetVertexColor(0.7, 0.7, 0.7)
			end)
			CompactRaidFrameManager.toggleButton:SetScript("OnLeave", function(self)
				self:GetNormalTexture():SetVertexColor(1, 1, 1)
			end)
			
			CompactRaidFrameManager.toggleButton:HookScript("OnClick", function(self)
				self = CompactRaidFrameManager
				if ( self.collapsed ) then
					self.toggleButton:SetNormalTexture("Interface\\MoneyFrame\\Arrow-Right-Down.png")
					self.toggleButton:GetNormalTexture():SetTexCoord(0, 1, 0, 1)
					self.toggleButton:SetPoint(p, rt, rp, x + 12, y)
				else
					self.toggleButton:SetNormalTexture("Interface\\MoneyFrame\\Arrow-Right-Down.png")
					self.toggleButton:GetNormalTexture():SetTexCoord(1, 0, 0, 1)
					self.toggleButton:SetPoint(p, rt, rp, x + 5, y)
				end
				PlaySoundFile(MUI_MEDIA .. "Sound\\Click.mp3")
				collectgarbage("collect")
			end)
		end		
		SkinArrows({"CompactUnitFrameProfilesProfileSelector", "CompactUnitFrameProfilesGeneralOptionsFrameSortByDropdown", "CompactUnitFrameProfilesGeneralOptionsFrameHealthTextDropdown", "CompactRaidFrameManagerDisplayFrameProfileSelector"})
		StartSkinning({
			["CompactUnitFrameProfilesProfileSelector"] = 18,
			{{["CompactUnitFrameProfilesGeneralOptionsFrameSortByDropdown"] = 18}, 20, -8, 0, 6, true},
			{{["CompactUnitFrameProfilesGeneralOptionsFrameHealthTextDropdown"] = 18}, 20, -8, 0, 6, true},
			{{["CompactRaidFrameManagerDisplayFrameProfileSelector"] = 18}, 20, -8, 0, 6, true},
		}, 19, -16, 0, 6, true)
	end
	
	--> Loot Frame
	if ( not MayronUI.db.profile.ReskinUI["Loot Frame"] ) then
		LOOTFRAME_NUMBUTTONS = 50
		StartSkinning({["LootFrame"] = 44})
		local bg = CreateBG(LootFrame)
		local titleText = select(19, LootFrame:GetRegions())
		titleText:SetPoint("TOPLEFT", LootFrame, "TOPLEFT", 8, -6)
		titleText:SetText("Loot:")
		LootFrameInset:Hide()
		for i = 1, LOOTFRAME_NUMBUTTONS do	
			local newBtn
			if (i > 4) then		
				newBtn = CreateFrame("Button", "LootButton"..i, LootFrame, "LootButtonTemplate", i)
			end	
			_G["LootButton"..i]:ClearAllPoints()
			if ((i % 10) == 0) then
				local n = i / 10
				_G["LootButton"..i]:SetPoint("TOPLEFT", _G["LootButton"..((i - (10 * n)) + 1)], "TOPRIGHT", 130, 0)
			else
				if (i == 1) then
					LootButton1:SetPoint("TOPLEFT", LootFrame, "TOPLEFT", 8, -30)
				else
					_G["LootButton"..i]:SetPoint("TOPLEFT", _G["LootButton"..(i-1)], "BOTTOMLEFT", 0, -4)
				end
			end
			_G["LootButton"..i.."NameFrame"]:Hide()
			SkinIcons(_G["LootButton"..i])
		end
		
		hooksecurefunc("LootFrame_UpdateButton", function(index)
			if (LootFrame.numLootItems >= 10) then
				local n = math.floor(LootFrame.numLootItems / 10) + 1
				LootFrame:SetHeight(35 + ( 41 * 9))
				LootFrame:SetWidth(165 * n)
			else
				LootFrame:SetHeight(35 + ( 41 * LootFrame.numLootItems))
				LootFrame:SetWidth(165)
			end	
			
			for slot = 1, LootFrame.numLootItems do
				local _, _, _, quality = GetLootSlotInfo(slot)		
				if (quality) then 
					local r, g, b = GetItemQualityColor(quality)
					_G["LootButton"..slot].new:SetVertexColor(r, g, b)
				end	
			end
		end)
		
		LootFrame:HookScript("OnDragStop", function(self)
			MayronUI.db.global.SavePoint["Loot Frame"] = { self:GetPoint() }
		end)
				
		hooksecurefunc("LootFrame_Show", function(self)			
			if (GetCVar("lootUnderMouse") == "0") then
				local mui = MayronUI.db.global.SavePoint["Loot Frame"]
				self:ClearAllPoints()
				if (mui) then			
					self:SetPoint(mui[1], mui[2], mui[3], mui[4], mui[5])				
				else
					self:SetPoint("BOTTOMRIGHT", UIParent, "CENTER", -150, 0)
				end
			end
		end)
	end
	
	--> Friends Frame:
	if ( not MayronUI.db.profile.ReskinUI["Friends Frame"] ) then
		StartSkinning({
			["FriendsFrame"] = 46, ["FriendsFrameInset"] = 12, ["FriendsFrameFriendsScrollFrame"] = 4, ["IgnoreListFrame"] = 4, ["PendingListFrame"] = 4, ["WhoFrameListInset"] = 12, ["ChannelFrameLeftInset"] = 12, ["ChannelFrameRightInset"] = 12, ["FriendsFrameStatusDropDown"] = 16, ["ChannelFrameDaughterFrame"] = 46, ["RecruitAFriendFrame"] = 46, ["FriendsFrameBattlenetFrame.BroadcastFrame"] = 14, ["WhoListScrollFrame"] = 4, ["ChannelRosterScrollFrame"] = 4, ["AddFriendFrame"] = 14,
			{{["RecruitAFriendNameEditBox"] = 6}, -4, nil, nil, -2},
			{{["RecruitAFriendNoteFrameScrollFrame"] = 2}, -4, nil, 4},
			{{["FriendsFrameBattlenetFrameScrollFrame"] = 6}, -4, nil, 4},
			{{["WhoFrameEditBoxInset"] = 14}, 3, -2},
			{{["WhoFrameDropDown"] = 18}, 16, -16, -2, 8, true},
			{{["AddFriendNameEditBox"] = 6}, -4},
		})
		SkinArrows({"WhoFrameDropDown"})		
		for i = 1, 4 do
			SkinFrame(_G["WhoFrameColumnHeader"..i], "BACKGROUND") 
		end
		for i = 1, MAX_CHANNEL_BUTTONS do
			SkinFrame(_G["ChannelButton"..i], "ARTWORK") 
		end
		SkinTabs({"FriendsFrameTab1", "FriendsFrameTab2", "FriendsFrameTab3", "FriendsFrameTab4"})
		for i = 1, 3 do
			_G["FriendsTabHeaderTab"..i.."LeftDisabled"]:SetTexture("")
			_G["FriendsTabHeaderTab"..i.."RightDisabled"]:SetTexture("")
			_G["FriendsTabHeaderTab"..i.."MiddleDisabled"]:SetTexture("")
			_G["FriendsTabHeaderTab"..i.."Left"]:SetTexture("")
			_G["FriendsTabHeaderTab"..i.."Right"]:SetTexture("")
			_G["FriendsTabHeaderTab"..i.."Middle"]:SetTexture("")
		end
		local new = CreateBG(FriendsFrameStatusDropDown, 16, -16, 0, 4, true)
		FriendsFrameStatusDropDownStatus:SetParent(new)
		RecruitAFriendNoteFrame:DisableDrawLayer("BACKGROUND")
		
		SkinArrows({"FriendsFrameStatusDropDown"})		
		SkinIcons(FriendsTabHeaderRecruitAFriendButtonIcon)
		
		new:RegisterEvent('GROUP_ROSTER_UPDATE')
		new:SetScript("OnEvent", function()
			if not IsInGroup() then return end
			if (RaidGroup1) then
				for i = 1, 8 do 
					_G["RaidGroup"..i]:DisableDrawLayer("BACKGROUND")					
				end
				for n = 1, (GetNumGroupMembers()) do
					StartSkinning({["RaidGroupButton"..n] = 18})
				end
			end
		end)
		
		StartSkinning({["RaidInfoFrame"] = 62})
		RaidInfoFrame:SetPoint("TOPLEFT",  RaidFrame, "TOPRIGHT", 2, 0)
		RaidInfoInstanceLabel:DisableDrawLayer("BACKGROUND")
		RaidInfoIDLabel:DisableDrawLayer("BACKGROUND")
		RaidInfoScrollFrameScrollBar:DisableDrawLayer("ARTWORK")
	end
	
	-- Ready Frame
	if ( not MayronUI.db.profile.ReskinUI["Ready Check Frame"] ) then
		StartSkinning({["ReadyCheckFrame"] = 14})
		select(2, ReadyCheckListenerFrame:GetRegions()):Hide()
		local new = CreateFrame("Frame", nil, ReadyCheckFrame, "MUI_ToolTipTemplate")
		new:SetSize(54, 54)
		new:SetPoint("CENTER", ReadyCheckFrame, "TOPLEFT", 22, -22)
		ReadyCheckPortrait:SetTexCoord(0.15, 0.85, 0.15, 0.85)
		ReadyCheckPortrait:ClearAllPoints()
		ReadyCheckPortrait:SetPoint("TOPLEFT", new, "TOPLEFT", 1, -1)
		ReadyCheckPortrait:SetPoint("BOTTOMRIGHT", new, "BOTTOMRIGHT", -1, 1)
		hooksecurefunc("ShowReadyCheck", function(initiator) 
			ReadyCheckFrame.initiator = initiator
			if ( initiator ) then				
				if ( UnitIsUnit("player", initiator) ) then
					ReadyCheckFrame:Hide()
				else
					ReadyCheckFrame:Show()
				end
			end
		end)
	end

	-- Static Popup:
	if ( not MayronUI.db.profile.ReskinUI["Static Popup Frames"] ) then
		for i = 1, STATICPOPUP_NUMDIALOGS do
			StartSkinning({
				["StaticPopup"..i.."MoneyInputFrameGold"] = 6, ["StaticPopup"..i.."ItemFrame"] = 16,				
				 -- sub tables:
				{{["StaticPopup"..i.."MoneyInputFrameSilver"] = 6}, -4, -8},
				{{["StaticPopup"..i.."MoneyInputFrameCopper"] = 6}, -4, -8},
				{{["StaticPopup"..i] = 14}},
			}, -4)
			SkinIcons({"StaticPopup"..i.."ItemFrame"})
		end
	end
	
	StartSkinning({["StackSplitFrame"] = 6})	
	
	--> Pet Battle Frame:
	if ( not MayronUI.db.profile.ReskinUI["Pet Battle Frame"] ) then
		StartSkinning({
			["PetBattleFrame.BottomFrame"] = 12,
			{{["PetBattleFrame"] = 4}},
			{{["PetBattleFrame.BottomFrame.FlowFrame"] = 12}},
			{{["PetBattleFrame.BottomFrame.MicroButtonFrame"] = 12}},
			{{["PetBattleFrame.BottomFrame.xpBar"] = 2}, -1, 1, 1, -1},
		})
		SkinIcons({"PetBattleFrame.BottomFrame.SwitchPetButton.Icon", "PetBattleFrame.BottomFrame.CatchButton.Icon", "PetBattleFrame.BottomFrame.ForfeitButton.Icon"})
		
		PetBattleFrame.TopVersusText:Hide()
		PetBattleFrame.WeatherFrame:ClearAllPoints()
		PetBattleFrame.WeatherFrame:SetPoint("TOP", UIParent, "TOP", 0, -20)
		
		-- Bottom Pet Frame:
		local bottom = PetBattleFrame.BottomFrame
		CreateBG(bottom, 20, -20, -20)
		bottom.xpBar:ClearAllPoints()
		bottom.xpBar:SetPoint("BOTTOMLEFT", bottom.new, "TOPLEFT", 1, 2)
		bottom.xpBar:SetPoint("BOTTOMRIGHT", bottom.new, "TOPRIGHT", -1, 2)
		bottom.xpBar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
		bottom.xpBar:DisableDrawLayer("OVERLAY")		
		bottom.TurnTimer:DisableDrawLayer("OVERLAY")
		local p, rt, rp, x, y = bottom.MicroButtonFrame:GetPoint()
		bottom.MicroButtonFrame:SetPoint(p, rt, rp, x-5, y-4)
		p, rt, rp, x, y	= nil		
		hooksecurefunc("PetBattleFrame_Display", function()
			bottom.TurnTimer.SkipButton:ClearAllPoints()
			bottom.TurnTimer.SkipButton:SetPoint("BOTTOM", bottom.xpBar, "TOP", 0, 5)
		end)

		-- Ally Frame:
		for id, str in ipairs({"ActiveAlly", "ActiveEnemy"}) do		
			local f = PetBattleFrame[str]			
			f:ClearAllPoints()	
			f.HealthBarFrame:SetAlpha(0)
			local levelUnderlay = CreateBG()
			f.LevelUnderlay:ClearAllPoints()
			f.LevelUnderlay:SetAlpha(0)
			f.Level:SetParent(levelUnderlay)			
			levelUnderlay:SetPoint("TOPLEFT", f.LevelUnderlay, "TOPLEFT", -1, 1)
			levelUnderlay:SetPoint("BOTTOMRIGHT", f.LevelUnderlay, "BOTTOMRIGHT", 1, -1)
			levelUnderlay:SetParent(f)
			levelUnderlay:SetFrameLevel(10)
			local healthBar = CreateBG()
			healthBar:SetPoint("TOPLEFT", f.HealthBarBG, "TOPLEFT", 4, -4)
			healthBar:SetPoint("BOTTOMRIGHT", f.HealthBarBG, "BOTTOMRIGHT", -4, 4)
			healthBar:SetParent(f)
			f.ActualHealthBar:SetParent(healthBar)	
			f.ActualHealthBar:SetTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
			f.ActualHealthBar:SetVertexColor(0.2, 0.8, 0.2)	
			f.HealthText:SetParent(healthBar)
			f.HealthText:SetFontObject("GameFontNormal")
			f.HealthBarBG:SetAlpha(0)
			f.PetType:DisableDrawLayer("BACKGROUND")
			f.Name:SetFontObject("GameFontNormal")				
			f.BorderFlash:Hide()
			
			if (id == 1) then -- player
				CreateBG(f.HealthBarFrame, -3, 1, -2, 2)
				f:SetPoint("BOTTOMRIGHT", bottom, "TOP", -100, 150)
				f.LevelUnderlay:SetPoint("CENTER", f, "BOTTOMLEFT", 7, 12)
				f.Name:SetPoint("TOPLEFT", f.Icon, "TOPRIGHT", 10, -4)
				PetBattleFrame.AllyDebuffFrame:ClearAllPoints()
				PetBattleFrame.AllyDebuffFrame:SetPoint("TOPLEFT", f, "BOTTOMLEFT", -19, -6)
				PetBattleFrame.AllyBuffFrame:SetPoint("TOPLEFT", f, "TOPLEFT", -18, 44)				
				PetBattleFrame.AllyPadBuffFrame:SetPoint("TOPRIGHT", f, "TOPRIGHT", 16, 44)
			else -- enemy
				CreateBG(f.HealthBarFrame, -1, 3, -2, 2)
				f:SetPoint("BOTTOMLEFT", bottom, "TOP", 100, 150)
				f.LevelUnderlay:SetPoint("CENTER", f, "BOTTOMRIGHT", -7, 12)
				f.Name:SetPoint("TOPRIGHT", f.Icon, "TOPLEFT", -10, -4)
				PetBattleFrame.EnemyDebuffFrame:ClearAllPoints()
				PetBattleFrame.EnemyDebuffFrame:SetPoint("TOPRIGHT", f, "BOTTOMRIGHT", 19, -6)
				PetBattleFrame.EnemyBuffFrame:SetPoint("TOPRIGHT", f, "TOPRIGHT", 18, 44)
				PetBattleFrame.EnemyPadBuffFrame:SetPoint("TOPLEFT", f, "TOPLEFT", -16, 44)
			end
		end
		
		-- Other Frames:
		PetBattleFrame.Ally2:ClearAllPoints()
		PetBattleFrame.Ally2:SetPoint("TOPRIGHT", PetBattleFrame.ActiveAlly, "TOPLEFT", -10, -4)
		PetBattleFrame.Enemy2:ClearAllPoints()
		PetBattleFrame.Enemy2:SetPoint("TOPLEFT", PetBattleFrame.ActiveEnemy, "TOPRIGHT", 10, -4)
		
		hooksecurefunc("PetBattleFrame_UpdateSpeedIndicators", function() -- For altering Portrait Border	
			for id, str in ipairs({"ActiveAlly", "ActiveEnemy"}) do
				local f = PetBattleFrame[str]
				if (not f.portrait) then
					f.portrait = CreateBG()
					f.portrait:ClearAllPoints()					
					f.portrait:SetPoint("TOPLEFT", f.Icon, "TOPLEFT", -1, 1)
					f.portrait:SetPoint("BOTTOMRIGHT", f.Icon, "BOTTOMRIGHT", 1, -1)
					f.portrait:SetParent(f)					
					f.Icon:SetParent(f.portrait)
					f.SpeedIcon:SetParent(f.portrait)
					f.SpeedUnderlay:SetParent(f.portrait)					
				end
				if (id == 1) then 
					f.Icon:SetTexCoord(0.1, 0.92, 0.1, 0.92)					
				else
					f.Icon:SetTexCoord(0.92, 0.1, 0.1, 0.92)
				end
				f.Border:Hide()
				f.Border2:Hide()
			end
		end)
		
		--HealthBarFrame
		local finished
		hooksecurefunc("PetBattleFrame_UpdateActionBarLayout", function()
			if ( finished ) then return end
			for i = 1, NUM_BATTLE_PET_ABILITIES do
				local actionButton = PetBattleFrame.BottomFrame.abilityButtons[i]
				if ( actionButton ) then
					SkinIcons(actionButton.Icon)
					actionButton:SetCheckedTexture("")
				else break end			
			end
			finished = true
		end)
	end
	
	--> Loot Frames for Rolling:
	StartSkinning({["BonusRollMoneyWonFrame"] = 6, ["BonusRollFrame"] = 6})
	
	hooksecurefunc("GroupLootFrame_OpenNewFrame", function()
		for i = 1, 10 do
			if (not _G["GroupLootFrame"..i]) then break end
			if (not _G["GroupLootFrame"..i].reskinned) then
				StartSkinning({["GroupLootFrame"..i] = 14})
				_G["GroupLootFrame"..i].reskinned = true
			end
		end
	end)
	
	--> Taxi Frame:
	if ( not MayronUI.db.profile.ReskinUI["Taxi Frame"] ) then	
		StartSkinning({["TaxiFrame"] = 40})
		TaxiFrame.TitleBg:Hide()		
		TaxiFrame.CloseButton:ClearAllPoints()
		TaxiFrame.CloseButton:SetPoint("TOPRIGHT", TaxiRouteMap, "TOPRIGHT", 2, 2)
		TaxiFrame.TitleText:ClearAllPoints() 
		TaxiFrame.TitleText:SetPoint("TOP", TaxiRouteMap, "TOP", 0, -4)
		CreateBG(TaxiRouteMap)
		TaxiFrame.Bg:Hide()
		TaxiFrame.TopTileStreaks:Hide()
	end
	
	--> Trade Frame:
	if ( not MayronUI.db.profile.ReskinUI["Trade Frame"] ) then
		StartSkinning({["TradeFrame"] = 46, ["TradeRecipientItemsInset"] = 12, ["TradePlayerItemsInset"] = 12, ["TradePlayerEnchantInset"] = 12, ["TradePlayerInputMoneyInset"] = 12, ["TradeRecipientMoneyInset"] = 8, ["TradeRecipientEnchantInset"] = 12})
		for i = 1, 7 do
			local pla = _G["TradePlayerItem"..i.."ItemButton"]
			local rec = _G["TradeRecipientItem"..i.."ItemButton"]
			SkinIcons(pla, 1)
			SkinIcons(rec, 1)
			CreateBG(_G["TradeRecipientItem"..i])
			CreateBG(_G["TradePlayerItem"..i])
			_G["TradePlayerItem"..i.."NameFrame"]:Hide()
			_G["TradeRecipientItem"..i.."NameFrame"]:Hide()
		end
		for id, str in ipairs({"Gold", "Silver", "Copper", "TradeRecipientMoneyBg"}) do
			local f = ( (id < 4) and "TradePlayerInputMoneyFrame") or ""
			_G[f..str]:DisableDrawLayer("BACKGROUND")
			TradeRecipientMoneyBg:DisableDrawLayer("BACKGROUND")
		end
		CreateBG(TradePlayerInputMoneyFrameGold, -4)
		TradePlayerInputMoneyFrame:SetPoint("TOPLEFT", TradeFrame, "TOPLEFT", 18, -61)
		CreateBG(TradePlayerInputMoneyFrameSilver, -4, -8)
		CreateBG(TradePlayerInputMoneyFrameCopper, -4, -8)

		select(4, TradePlayerItem7:GetRegions()):Hide()
		select(4, TradeRecipientItem7:GetRegions()):Hide()
		TradeFrameInset:Hide()
		
		hooksecurefunc("TradeFrame_UpdatePlayerItem", function(id)
			local pla = _G["TradePlayerItem"..id.."ItemButton"]
			if (pla and pla.new) then
				local link = GetTradePlayerItemLink(id)
				if (link) then	
					local quality = select(3, GetItemInfo(link))
					if (quality) then
						local r, g, b = GetItemQualityColor(quality)
						pla.new:SetVertexColor(r, g, b)
					else
						pla.new:SetVertexColor(0,0,0)
					end
				else
					pla.new:SetVertexColor(0,0,0)
				end	
			end
		end)
		hooksecurefunc("TradeFrame_UpdateTargetItem", function(id)
			local rec = _G["TradeRecipientItem"..id.."ItemButton"]
			if (rec and rec.new) then
				local link = GetTradeTargetItemLink(id)
				if (link) then		
					local quality = select(3, GetItemInfo(link))
					if (quality) then
						local r, g, b = GetItemQualityColor(quality)
						rec.new:SetVertexColor(r, g, b)
					else
						rec.new:SetVertexColor(0,0,0)
					end
				else
					rec.new:SetVertexColor(0,0,0)
				end	
			end
		end)
	end
	
	--> Mail Frame:
	if ( not MayronUI.db.profile.ReskinUI["Mail Frame"] ) then
		StartSkinning({["MailFrame"] = 46, ["InboxFrame"] = 4, ["MailFrameInset"] = 12, ["OpenMailFrame"] = 62, ["OpenMailFrameInset"] = 12, ["OpenMailScrollFrame"] = 36, ["SendMailFrame"] = 4, ["SendMailScrollFrame"] = 20, ["SendMailMoneyInset"] = 12, ["SendMailMoneyBg"] = 4})
		for i = 1, 7 do
			SkinFrame(_G["MailItem"..i], 4)
			CreateBG(_G["MailItem"..i], -6, 14, 0, 2)
			SkinIcons(_G["MailItem"..i.."ButtonIcon"])
			SkinIcons(_G["SendMailAttachment"..i])
			-- Remove Blizzard annoying stuff:
			select(1, _G["SendMailAttachment"..i]:GetRegions()):Hide()
			_G["SendMailAttachment"..i]:SetHighlightTexture("")
		end
		
		SendMailBodyEditBox:SetFontObject(GameFontHighlight)
		SkinIcons(OpenMailLetterButton)
		SendMailFrame:EnableMouse(false)
		CreateBG(SendMailScrollFrame, 6, nil, 2, -4)
		ChangeTextColor(OpenMailInvoicePurchaser, "white")
		ChangeTextColor(OpenMailInvoiceItemLabel, "white")
		ChangeTextColor(OpenMailBodyText, "white")
		ChangeTextColor(OpenMailInvoiceAmountReceived, "white")
		OpenMailArithmeticLine:Hide()
		StartSkinning({["SendMailNameEditBox"] = 6, ["SendMailSubjectEditBox"] = 6}, -4, nil, -1, 1)
		StartSkinning({["SendMailMoneySilver"] = 6, ["SendMailMoneyCopper"] = 6}, -4, -10, 0, 0)
		StartSkinning({["SendMailMoneyGold"] = 6}, -4, 0, 0, 0)
		for i = 1, 16 do
			SkinIcons(_G["OpenMailAttachmentButton"..i], 1)
		end
		for i = 1, 2 do
			SkinTabs({"MailFrameTab"..i})
		end

		hooksecurefunc("OpenMail_Update", function()
			-- Inbox:
			for mailID = 1, (GetInboxNumItems()) do
				local btn = _G["MailItem"..mailID.."Button"]
				if (not btn) then break end
				if (not btn.new) then break end
				local link = GetInboxItemLink(mailID)
				if (link) then		
					local quality = select(3, GetItemInfo(link))
					if (quality) then
						local r, g, b = GetItemQualityColor(quality)
						btn.new:SetVertexColor(r, g, b)
					else
						btn.new:SetVertexColor(0,0,0)
					end
				else
					btn.new:SetVertexColor(0,0,0)
				end				
				-- Open Mail:
				if (InboxFrame.openMailID ) then				
					for attachmentID = 1, InboxFrame.openMailID do
						if (not btn:GetChecked()) then break end
						local attachBtn = _G["OpenMailAttachmentButton"..attachmentID]
						if (attachBtn and attachBtn.new) then														
							local link = GetInboxItemLink(mailID, attachmentID)							
							if (link) then						
								local quality = select(3, GetItemInfo(link))
								if (quality) then									
									local r, g, b = GetItemQualityColor(quality)
									attachBtn.new:SetVertexColor(r, g, b)
								else									
									attachBtn.new:SetVertexColor(0,0,0)
								end
							else
								attachBtn.new:SetVertexColor(0,0,0)
							end
						end
					end
				end
			end		
		end)
		
		-- Send Mail:		
		hooksecurefunc("SendMailFrame_Update", function()
			for i = 1, ATTACHMENTS_MAX_SEND do
				local btn = _G["SendMailAttachment"..i]
				if (not btn) then break end
				if (not btn.new) then break end				
				local link = GetSendMailItemLink(i)
				if (link) then	
					local iconTexture = select(6, btn:GetRegions())
					iconTexture:SetTexCoord(0.1, 0.92, 0.08, 0.92)
					iconTexture:ClearAllPoints()
					iconTexture:SetPoint("TOPLEFT", btn, "TOPLEFT", 1.5, -1.5)
					iconTexture:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT", -1.5, 1.5)
					local quality = select(3, GetItemInfo(link))
					if (quality) then		
						local r, g, b = GetItemQualityColor(quality)
						btn.new:SetVertexColor(r, g, b)
					else
						btn.new:SetVertexColor(0,0,0)
					end
				else
					btn.new:SetVertexColor(0,0,0)
				end
			end
		end)
	end
	
	--> PVE Frame:
	if ( not MayronUI.db.profile.ReskinUI["PVE Frame"] ) then
		StartSkinning({["PVEFrame"] = 62, ["PVEFrameLeftInset"] = 12, ["LFDParentFrameInset"] = 12, ["LFDQueueFrameRandomScrollFrameScrollBar"] = 20, ["LFGListApplicationViewerScrollFrameScrollBar"] = 20, ["LFDParentFrame"] = 12, ["LFGListSearchPanelScrollFrameScrollBar"] = 20})
		PVEFrame.shadows:Hide()
		-- LFDQueueFrame:
		LFDQueueFrame:DisableDrawLayer("BORDER")
		LFDQueueFrameFindGroupButton:DisableDrawLayer("BORDER")
		LFDQueueFrameSpecificListScrollFrame:DisableDrawLayer("BACKGROUND")
		LFDQueueFrameRandomScrollFrameChildFrameMoneyRewardNameFrame:Hide()
		SkinIcons(LFDQueueFrameRandomScrollFrameChildFrameMoneyReward.Icon, 1)
		LFGListFrame.ApplicationViewer.InfoBackground:SetTexCoord(0.1, 0.9, 0.1, 0.9)
		LFGListFrame.ApplicationViewer.InfoBackground:SetBlendMode("ADD")
		LFGListFrame.ApplicationViewer.InfoBackground:SetSize(330, 80)
		LFGListFrame.ApplicationViewer.InfoBackground:SetPoint("TOPLEFT", LFGListFrame.ApplicationViewer, "TOPLEFT", -2, -25)	
		
		local banner = CreateBG()
		banner:SetPoint("TOPLEFT", LFGListFrame.ApplicationViewer.InfoBackground, "TOPLEFT", -1, 1)
		banner:SetPoint("BOTTOMRIGHT", LFGListFrame.ApplicationViewer.InfoBackground, "BOTTOMRIGHT", 1, -1)
		banner:SetParent(LFGListFrame.ApplicationViewer)
		banner:SetFrameLevel(LFGListFrame.ApplicationViewer:GetFrameLevel() - 2)
		banner.background:SetVertexColor(MUI_COLOR.r * 0.5, MUI_COLOR.g * 0.5, MUI_COLOR.b * 0.5, 0.4)
		
		-- ScenarioQueueFrame:
		ScenarioQueueFrame.Bg:Hide()
		ScenarioFinderFrame:DisableDrawLayer("BORDER")
		ScenarioFinderFrameInset:Hide()
		ScenarioQueueFrameFindGroupButton:DisableDrawLayer("BORDER")
		ScenarioQueueFrameRandomScrollFrameChildFrameMoneyRewardNameFrame:Hide()
		SkinIcons(ScenarioQueueFrameRandomScrollFrameChildFrameMoneyReward.Icon, 1)
		ScenarioQueueFrameSpecificScrollFrame:DisableDrawLayer("BACKGROUND")
		
		-- RaidFinderQueueFrame:
		RaidFinderFrameBottomInset:Hide()
		RaidFinderQueueFrame:DisableDrawLayer("BORDER")
		SkinFrame(RaidFinderFrame, 12)
		RaidFinderFrameRoleInset:Hide()
		RaidFinderFrameFindRaidButton:DisableDrawLayer("BORDER")
		RaidFinderQueueFrameScrollFrameChildFrameMoneyRewardNameFrame:Hide()
		SkinIcons(RaidFinderQueueFrameScrollFrameChildFrameMoneyReward.Icon, 1)
		
		-- LFGListFrame:
		LFGListFrame.CategorySelection.Inset:Hide()
		LFGListFrame.CategorySelection.StartGroupButton:DisableDrawLayer("BORDER")
		LFGListFrame.CategorySelection.FindGroupButton:DisableDrawLayer("BORDER")
		LFGListFrame.EntryCreation.Inset:Hide()
		LFGListFrame.EntryCreation.CancelButton:DisableDrawLayer("BORDER")		
		LFGListFrame.EntryCreation.ListGroupButton:DisableDrawLayer("BORDER")
		LFGListFrame.SearchPanel.AutoCompleteFrame:DisableDrawLayer("OVERLAY")
		LFGListFrame.SearchPanel.ResultsInset:Hide()
		LFGListFrame.SearchPanel.BackButton:DisableDrawLayer("BORDER")
		LFGListFrame.SearchPanel.SignUpButton:DisableDrawLayer("BORDER")		
		LFGListFrame.ApplicationViewer.Inset:Hide()
		LFGListFrame.ApplicationViewer.NameColumnHeader:DisableDrawLayer("BACKGROUND")
		LFGListFrame.ApplicationViewer.RoleColumnHeader:DisableDrawLayer("BACKGROUND")
		LFGListFrame.ApplicationViewer.ItemLevelColumnHeader:DisableDrawLayer("BACKGROUND")
		LFGListFrame.ApplicationViewer.RemoveEntryButton:DisableDrawLayer("BORDER")
		LFGListFrame.ApplicationViewer.EditButton:DisableDrawLayer("BORDER")
		
		StartSkinning({["LFGListFrame.SearchPanel.SearchBox"] = 6}, -6, 0)
		StartSkinning({["LFGListFrame.EntryCreation.Name"] = 6}, -6, -50)
		StartSkinning({["LFGListEntryCreationDescription"] = 6}, -6, -16, 6, -6)
		StartSkinning({["LFGListFrame.EntryCreation.ItemLevel.EditBox"] = 6}, -6, 6)
		StartSkinning({["LFGListFrame.EntryCreation.VoiceChat.EditBox"] = 6}, -6, 6)
		
		-- DropDowns:
		StartSkinning(ScanDropdowns(LFDQueueFrame, 18), 20, -16, 0, 6, true)
		StartSkinning(ScanDropdowns(ScenarioQueueFrame, 18), 20, -16, 0, 6, true)
		StartSkinning(ScanDropdowns(RaidFinderQueueFrame, 18), 20, -16, 0, 6, true)
		StartSkinning(ScanDropdowns(LFGListFrame.EntryCreation, 18), 20, -16, 0, 6, true)
		
		-- LFGListApplicationDialog:
		hooksecurefunc("LFGListApplicationDialog_Show", function()
			StartSkinning({["LFGListFrame.SearchPanel.SearchBox"] = 14})			
		end)

		hooksecurefunc("LFDQueueFrameRandom_UpdateFrame", function()
			for i = 1, LFD_MAX_REWARDS do
				local button = _G["LFDQueueFrameRandomScrollFrameChildFrameItem"..i]
				if (button) then
					local icon = _G["LFDQueueFrameRandomScrollFrameChildFrameItem"..i.."IconTexture"]
					local nameFrame = _G["LFDQueueFrameRandomScrollFrameChildFrameItem"..i.."NameFrame"]
					local ShortageBorder = _G["LFDQueueFrameRandomScrollFrameChildFrameItem"..i.."ShortageBorder"]
					SkinIcons(icon, 1)
					nameFrame:Hide()
					ShortageBorder:SetAlpha(0)
					if (ShortageBorder:IsVisible()) then
						button.new:SetVertexColor(1.0, 1.0, 0.6)
					end
				end
			end
		end)
		
		hooksecurefunc("ScenarioQueueFrameRandom_UpdateFrame", function()
			for i = 1, 2 do
				local button = _G["ScenarioQueueFrameRandomScrollFrameChildFrameItem"..i]
				if (button) then
					local icon = _G["ScenarioQueueFrameRandomScrollFrameChildFrameItem"..i.."IconTexture"]
					local nameFrame = _G["ScenarioQueueFrameRandomScrollFrameChildFrameItem"..i.."NameFrame"]
					local ShortageBorder = _G["ScenarioQueueFrameRandomScrollFrameChildFrameItem"..i.."ShortageBorder"]
					SkinIcons(icon, 1)
					nameFrame:Hide()
					ShortageBorder:SetAlpha(0)
					if (ShortageBorder:IsVisible()) then
						button.new:SetVertexColor(1.0, 1.0, 0.6)
					end
				end
			end
		end)
		hooksecurefunc("RaidFinderQueueFrameRewards_UpdateFrame", function()
			for i = 1, LFD_MAX_REWARDS do
				local button = _G["RaidFinderQueueFrameScrollFrameChildFrameItem"..i]
				if (button) then
					local icon = _G["RaidFinderQueueFrameScrollFrameChildFrameItem"..i.."IconTexture"]
					local nameFrame = _G["RaidFinderQueueFrameScrollFrameChildFrameItem"..i.."NameFrame"]
					local ShortageBorder = _G["RaidFinderQueueFrameScrollFrameChildFrameItem"..i.."ShortageBorder"]					
					SkinIcons(icon, 1)
					nameFrame:Hide()
					ShortageBorder:SetAlpha(0)
					if (ShortageBorder:IsVisible()) then
						button.new:SetVertexColor(1.0, 1.0, 0.6)
					end
				end
			end
		end)
		
		hooksecurefunc("GroupFinderFrame_SelectGroupButton", function(index)
			if (not LFGListFrame.CategorySelection.altered and LFGListFrame.CategorySelection:IsVisible()) then
				for i = 2, 5 do
					local f = LFGListFrame.CategorySelection.CategoryButtons
					if (not f[i]) then break end
					f[i]:SetFrameLevel(f[i]:GetFrameLevel() + 10)
					LFGListFrame.CategorySelection.altered = true
				end
			end
			for i = 1, 4 do				
				local f = _G["GroupFinderFrameGroupButton"..i]
				if (i == index) then
					f:SetBackdropBorderColor(1, 1, 1, 1)
					f.oldBC = {1, 1, 1, 1}
				else
					f:SetBackdropBorderColor(0, 0, 0, 1)
					f.oldBC = {0, 0, 0, 1}	
				end
			end
		end)
		
		for i = 1, 4 do
			local btn = _G["GroupFinderFrameGroupButton"..i]
			local icon = _G["GroupFinderFrameGroupButton"..i.."Icon"]
			SkinFrame(btn, 4)
			_G["GroupFinderFrameGroupButton"..i.."Ring"]:Hide()
			local frame = SkinIcons(icon, 1)
			icon:SetTexCoord(0.15, 0.87, 0.15, 0.87)
			icon:SetSize(52, 52)
			icon:ClearAllPoints()
			icon:SetPoint("TOPLEFT", btn, "TOPLEFT", 4, -4)
			frame.new:SetParent(frame)
			SkinButton(btn)
		end
		
		SkinTabs({"PVEFrameTab1", "PVEFrameTab2", "PVEFrameTab3"})
		SkinArrows({"LFDQueueFrameTypeDropDown", "ScenarioQueueFrameTypeDropDown", "RaidFinderQueueFrameSelectionDropDown", "FlexRaidFrameSelectionDropDown"})
		
		local point, relTo, relPoint, x, y = LFDQueueFrameRoleButtonTank:GetPoint()
		LFDQueueFrameRoleButtonTank:SetPoint(point, relTo, relPoint, x-20, y)
		PVEFrame.shadows:DisableDrawLayer("OVERLAY")
		point, relTo, relPoint, x, y = RaidFinderQueueFrameRoleButtonTank:GetPoint()
		RaidFinderQueueFrameRoleButtonTank:SetPoint(point, relTo, relPoint, x-20, y)
		LFDQueueFrameRoleButtonTank:DisableDrawLayer("BACKGROUND")
	end
	
	--> Mirror Timer:
	if ( not MayronUI.db.profile.ReskinUI["Mirror Timers"] ) then
		for i = 1, 3 do
			local timer = _G["MirrorTimer"..i]
			local text = _G["MirrorTimer"..i.."Text"]
			local bar = _G["MirrorTimer"..i.."StatusBar"]
			SkinFrame(timer, 36)
			bar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
			bar:SetAllPoints(timer)
			CreateBG(timer, -1, 1, 1, -1)
			timer:SetHeight(20)
			text:ClearAllPoints()			
			text:SetPoint("CENTER", timer, "CENTER", 0, 0)
			if (i ~= 1) then
				timer:SetPoint("TOP", _G["MirrorTimer"..(i - 1)], "BOTTOM", 0, -6)
			end
		end
	end
	
	--> Merchant Frame:
	if ( not MayronUI.db.profile.ReskinUI["Merchant Frame"] ) then
		StartSkinning({["MerchantFrame"] = 44, ["MerchantFrameInset"] = 14, ["MerchantMoneyBg"] = 4, ["MerchantMoneyInset"] = 12, ["MerchantPrevPageButton"] = 4, ["MerchantNextPageButton"] = 4})
		StartSkinning({["MerchantFrameLootFilter"] = 18}, 19, -16, 0, 6, true)
		SkinArrows({"MerchantFrameLootFilter"})
		CreateBG(MerchantFrame, nil, nil, nil, -1)
		for i = 1, 12 do
			SkinIcons(_G["MerchantItem"..i.."ItemButton"])
			SkinFrame(_G["MerchantItem"..i], 4)
			for icon = 1, 3 do
				SkinIcons(_G["MerchantItem"..i.."AltCurrencyFrameItem"..icon.."Texture"], 1, true)
			end
		end
		SkinFrame(MerchantBuyBackItem, 4)
		CreateBG(MerchantBuyBackItem, -4, -4, 4, -4)		
		SkinIcons(MerchantBuyBackItemItemButton)
		SkinTabs({"MerchantFrameTab1", "MerchantFrameTab2"})
		local icon = MerchantRepairItemButton:GetRegions()
		icon:SetTexture("Interface\\Icons\\INV_Hammer_20")
		SkinIcons(icon)
		icon = MerchantRepairAllButton:GetRegions()
		SkinIcons(icon)
		icon:SetTexCoord(0.31375, 0.53, 0.06, 0.52)
		icon = MerchantGuildBankRepairButton:GetRegions()
		SkinIcons(icon)
		icon:SetTexCoord(0.595, 0.8075, 0.05, 0.52)
		MerchantExtraCurrencyBg:SetAlpha(0)
		MerchantExtraCurrencyInset:SetAlpha(0)
		
		hooksecurefunc("MerchantFrame_UpdateMerchantInfo", function()
			for i = 1, MERCHANT_ITEMS_PER_PAGE do
				local btn = _G["MerchantItem"..i.."ItemButton"]
				local name = _G["MerchantItem"..i.."Name"]
				if ( btn.link ) then
					local quality = select(3, GetItemInfo(btn.link))
					if (quality) then
						local r, g, b = GetItemQualityColor(quality)
						btn.new:SetVertexColor(r, g, b)
					else
						btn.new:SetVertexColor(0,0,0)
					end
				else
					btn.new:SetVertexColor(0,0,0)
				end
			end
			-- Buy back button:
			local link = GetBuybackItemLink(GetNumBuybackItems())
			if ( link ) then
				local quality = select(3, GetItemInfo(link))
				if (quality) then
					local r, g, b = GetItemQualityColor(quality)
					MerchantBuyBackItemItemButton.new:SetVertexColor(r, g, b)
				else
					MerchantBuyBackItemItemButton.new:SetVertexColor(0,0,0)
				end
			else
				MerchantBuyBackItemItemButton.new:SetVertexColor(0,0,0)
			end
		end)
			
		hooksecurefunc("MerchantFrame_UpdateBuybackInfo", function()
			for i = 1, BUYBACK_ITEMS_PER_PAGE do
				local link = GetBuybackItemLink(i)			
				if ( link ) then
					local quality = select(3, GetItemInfo(link))
					if (quality) then
						local r, g, b = GetItemQualityColor(quality)
						_G["MerchantItem"..i.."ItemButton"].new:SetVertexColor(r, g, b)
					else
						_G["MerchantItem"..i.."ItemButton"].new:SetVertexColor(0,0,0)
					end
				else
					_G["MerchantItem"..i.."ItemButton"].new:SetVertexColor(0,0,0)
				end
			end
		end)
	end
	
	--> TabardFrame:
	if ( not MayronUI.db.profile.ReskinUI["Tabard Frame"] ) then
		TabardFrameEmblemTopRight:SetPoint("TOPRIGHT", TabardFrame, "TOPRIGHT", 0, -40)
		TabardFrameEmblemTopRight:SetDrawLayer("ARTWORK")
		TabardFrameEmblemBottomRight:SetDrawLayer("ARTWORK")
		TabardFrameEmblemTopLeft:SetDrawLayer("ARTWORK")
		TabardFrameEmblemBottomLeft:SetDrawLayer("ARTWORK")
		StartSkinning({["TabardFrame"] = 46})
		for i = 1, 5 do
			local btn = _G["TabardFrameCustomization"..i]
			if (not btn) then break end			
			SkinFrame(btn, 4)
			CreateBG(btn, nil, nil, -1, -5)
		end
		
		TabardFrameCustomizationBorder:Hide()
		TabardFrameInset:Hide()
		TabardFrameCostFrame:SetBackdrop(nil)
		TabardFrame:SetWidth(520)
		TabardModel:SetSize(500, 400)
		TabardFrameMoneyBg:DisableDrawLayer("BACKGROUND")
		TabardFrameMoneyInset:Hide()
	end
	
	--> Bank Frame:
	if ( not MayronUI.db.profile.ReskinUI["Bank Frame"] ) then	
		StartSkinning({["BankFrame"] = 62, ["BankFrameMoneyFrameInset"] = 12, ["BankFrameMoneyFrameBorder"] = 4, ["BankItemSearchBox"] = 4})
		for i = 1, 28 do
			SkinIcons(_G["BankFrameItem"..i], nil, true)
		end
		for i = 1, 7 do
			SkinIcons(BankSlotsFrame["Bag"..i], nil, true)
		end
		
		if ( not (IsAddOnLoaded("Bagnon") and Bagnon:IsFrameEnabled("bank") and not MayronUI.db.profile.ReskinUI["Bagnon"]) ) then
			CreateBG(BankItemSearchBox, -4, nil, -1)
			BankSlotsFrame:DisableDrawLayer("BORDER")		
			SkinTabs({"BankFrameTab1", "BankFrameTab2"})
			
			hooksecurefunc("BankFrameItemButton_Update", function(button)			
				local inventoryID			
				if (button.isBag) then		
					inventoryID = BankButtonIDToInvSlotID( (button:GetID()) , button.isBag)				
				elseif (not ReagentBankFrame:IsShown()) then
					inventoryID = BankButtonIDToInvSlotID((button:GetID()), false)
				else
					inventoryID = ReagentBankButtonIDToInvSlotID((button:GetID()))
				end
				if (not inventoryID) then return end			
				local link = GetInventoryItemLink("player", inventoryID)
				if (link) then
					local quality = select(3, GetItemInfo(link))
					if (quality >= 0) then
						local r, g, b = GetItemQualityColor(quality)							
						button.new:SetVertexColor(r,g,b)
					else
						button.new:SetVertexColor(0,0,0)
					end
				else
					button.new:SetVertexColor(0,0,0)
				end
			end)
			
			hooksecurefunc("ReagentBankFrameItemButton_OnLoad", function(self)			
				local name = self:GetName()			
				if (name and name:find("98")) then
					SkinFrame(ReagentBankFrame, 28)
					for i = 1, 98 do
						local item = _G["ReagentBankFrameItem"..i]
						if (not item) then break end
						SkinIcons(item, nil, true)
						item.new:SetVertexColor(0,0,0)
					end
				end
			end)
		end
	end
	if ( not MayronUI.db.profile.ReskinUI["Bagnon"] ) then
		local isBagnon = IsAddOnLoaded("Bagnon")
		if (isBagnon) then
			bInvent = Bagnon:IsFrameEnabled("inventory")
			bBank = Bagnon:IsFrameEnabled("bank")
		end
		if (isBagnon and bInvent and not bBank) then
			hooksecurefunc(Bagnon.ItemSlot, "Create", function()	
				local id = Bagnon.ItemSlot:GetNextID()
				local item = _G["BagnonItemSlot"..(id - 1)]				
				if (not item) then return end
				SkinIcons(item)
				_G[item:GetName().."IconQuestTexture"]:SetDrawLayer("OVERLAY")
				item.IconBorder:SetAlpha(0)
			end)					
		elseif ( MayronUI.db.profile.ReskinUI["Bags/Inventory"] ) then	
			for i = 1, NUM_CONTAINER_FRAMES do
				for item = 1, MAX_CONTAINER_ITEMS do
					local item = _G["ContainerFrame"..i.."Item"..item]		
					SkinIcons(item)
					_G[item:GetName().."IconQuestTexture"]:SetDrawLayer("OVERLAY")
					item.IconBorder:SetAlpha(0)
				end
			end
		end
	end
	
	
	--> Bag/Inventory:	
	if ( not MayronUI.db.profile.ReskinUI["Bags/Inventory"] ) then
		local isBagnon = IsAddOnLoaded("Bagnon") and not MayronUI.db.profile.ReskinUI["Bagnon"]
		local bInvent, bBank
		if (isBagnon) then
			bInvent = Bagnon:IsFrameEnabled("inventory")
			bBank = Bagnon:IsFrameEnabled("bank")
		end
		
		for i = 1, NUM_CONTAINER_FRAMES do
			local bag = _G["ContainerFrame"..i]
			if ( bag and not (isBagnon and bInvent and bBank) ) then				
				CreateBG(bag, 1, -1, -2, 2)
				SkinFrame(bag, 48)
				select(1, bag:GetRegions()):SetAlpha(0)
				local title = _G["ContainerFrame"..i.."Name"]
				title:ClearAllPoints()
				title:SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
				if ( i == 1) then				
					title:SetPoint("TOPLEFT", _G["ContainerFrame"..i], "TOPLEFT", -14, -6)
				else
					title:SetPoint("TOPLEFT", _G["ContainerFrame"..i], "TOPLEFT", 2, -6)
				end				
				
				-- New WoD Button:	
				_G["ContainerFrame"..i.."PortraitButton"]:SetScript("OnEnter", function() 
					title:SetTextColor(1,1,1)
				end)
				_G["ContainerFrame"..i.."PortraitButton"]:SetScript("OnLeave", function() 
					title:SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
				end)
				_G["ContainerFrame"..i.."PortraitButton"]:SetAllPoints(select(6, bag:GetRegions()))
				_G["ContainerFrame"..i.."CloseButton"]:ClearAllPoints()
				_G["ContainerFrame"..i.."CloseButton"]:SetPoint("TOPRIGHT", bag, "TOPRIGHT", 2, 2)
			end
			for item = 1, MAX_CONTAINER_ITEMS do
				local item = _G["ContainerFrame"..i.."Item"..item]		
				SkinIcons(item)
				_G[item:GetName().."IconQuestTexture"]:SetDrawLayer("OVERLAY")
				item.IconBorder:SetAlpha(0)
			end
		end
		
		if ( (not isBagnon) or (isBagnon and not bBank) ) then
			StartSkinning({["BagItemSearchBox"] = 4})
			BackpackTokenFrame:DisableDrawLayer("BACKGROUND")		
			CreateBG(BagItemSearchBox, -4, nil, -1)				
		
			local function RepositionBags()
				local repTracker = {}
				local repFrames = {}
				for index = 0, NUM_CONTAINER_FRAMES do
					local frame = _G["ContainerFrame"..(index + 1)]					
					if (frame and frame:IsVisible()) then
						frame:SetFrameLevel(80)
						local name = frame:GetName()
						if (not frame:GetPoint()) then return end
						local rfName = select(2, frame:GetPoint()):GetName()
						if (rfName == "UIParent") then
							if (not repTracker[name]) then
								table.insert(repFrames, name)
								repTracker[name] = true
							end
						end
						for id, name in ipairs(repFrames) do
							local frame = _G[name]
							local rf, rp, x, y
							if (id == 1) then
								rf, rp, y = UIParent, "BOTTOMRIGHT", 2
							else
								rf, rp, y = repFrames[id - 1], "BOTTOMLEFT", 0
							end
							frame:ClearAllPoints()
							frame:SetPoint("BOTTOMRIGHT", rf, rp, -2, y)
						end
					end
				end
			end
		
			hooksecurefunc("UpdateContainerFrameAnchors", function()
				if (ContainerFrame1:IsShown()) then
					RepositionBags()
				end
			end)
			
			hooksecurefunc("ToggleBag", function(self)
				MainMenuBarBackpackButton:Hide()
				if (ContainerFrame1Name:GetText() ~= "Backpack") then
					ContainerFrame1Name:SetPoint("TOPLEFT", ContainerFrame1, "TOPLEFT", 2, -6)					
				else
					ContainerFrame1Name:SetPoint("TOPLEFT", ContainerFrame1, "TOPLEFT", -14, -6)
				end				
				RepositionBags()
			end)

			ContainerFrame1CloseButton:SetScript("OnClick", ToggleBackpack)
			
			hooksecurefunc("ContainerFrame_Update", function(self)
				BagItemSearchBox:ClearAllPoints()				
				BagItemSearchBox:SetPoint("TOPLEFT", ContainerFrame1PortraitButton, "TOPLEFT", 34, -22)
				for i = 1, self.size do
					local button = _G[self:GetName().."Item"..i]			
					local quality = select(4, GetContainerItemInfo(self:GetID(), button:GetID()))
					if (quality) then
						UpdateQualityColor(button)
					elseif (button.new) then
						button.new:SetVertexColor(0,0,0)
					end
				end
			end)
			
			-- Toggle Bag Icons:
			local ToggleBag = CreateFrame("Button", "BagItemToggleBagButtons", ContainerFrame1)
			ToggleBag:SetSize(24, 22)
			ToggleBag:SetPoint("TOPRIGHT", BagItemAutoSortButton, "TOPLEFT", -2, -2)
			ToggleBag:SetHighlightTexture("Interface\\AddOns\\MayronUI\\Media\\Skin\\Highlight_Button.blp")
			ToggleBag:SetNormalTexture("Interface\\ICONS\\INV_MISC_BAG_09.png")
			ToggleBag:SetScript("OnClick", function()
				if (MainMenuBarBackpackButton:IsVisible()) then
					MainMenuBarBackpackButton:Hide()
					for i = 0, 3 do 
						_G["CharacterBag"..i.."Slot"]:Hide()
					end
				else
					MainMenuBarBackpackButton:Show()
					for i = 0, 3 do 
						_G["CharacterBag"..i.."Slot"]:Show()
					end
				end
			end)
			ToggleBag:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT", -28, 0)
				GameTooltip:SetText("Show Bag Buttons")
				GameTooltip:Show()
			end)
			ToggleBag:SetScript("OnLeave", function()
				GameTooltip:Hide()
			end)
			SkinIcons(ToggleBag, 1)
			ToggleBag:GetNormalTexture():SetTexCoord(0.2, 0.8, 0.2, 0.8)
			
			MainMenuBarBackpackButton:SetParent(ContainerFrame1)			
			CharacterBag0Slot:SetParent(MainMenuBarBackpackButton)			
			CharacterBag1Slot:SetParent(CharacterBag0Slot)
			CharacterBag2Slot:SetParent(CharacterBag1Slot)
			CharacterBag3Slot:SetParent(CharacterBag2Slot)			
			MainMenuBarBackpackButton:Hide()			
			
			CharacterBag3Slot:SetScript("OnShow", function(self)				
				local anchor
				for i = 1, 5 do
					local bag = _G["ContainerFrame"..i]
					if (bag and bag.size and bag:IsVisible()) then
						anchor = bag
					end
				end
				self:SetPoint("TOPLEFT", anchor, "TOPLEFT", 3, 32)
			end)
		end
	end
	
	--> Time Manager Frame:
	if ( not MayronUI.db.profile.ReskinUI["Time Manager Frame"] ) then	
		StartSkinning({["TimeManagerFrame"] = 46, ["TimeManagerFrameInset"] = 12})
		SkinIcons(TimeManagerStopwatchCheck)
		TimeManagerAlarmMessageEditBox:DisableDrawLayer("BACKGROUND")
		StartSkinning(ScanDropdowns(TimeManagerFrame, 18), 20, -8, 0, 6, true)
		CreateBG(TimeManagerAlarmMessageEditBox, -4, nil, 1)
	end
	
	--> Interface Options Frame:
	if ( not MayronUI.db.profile.ReskinUI["Interface Options"] ) then
		StartSkinning({["InterfaceOptionsFrameTab1"] = 12, ["InterfaceOptionsFrameTab2"] = 12, ["InterfaceOptionsFrame"] = 31, ["InterfaceOptionsFramePanelContainer"] = 8, ["InterfaceOptionsFrameCategories"] = 4, ["InterfaceOptionsFrameAddOns"] = 4})
		StartSkinning(ScanDropdowns(InterfaceOptionsFramePanelContainer, 18), 20, -8, 0, 6, true)
		StartSkinning({["InterfaceOptionsSocialPanelChatStyle"] = 18, ["InterfaceOptionsSocialPanelTimestamps"] = 18, ["InterfaceOptionsSocialPanelWhisperMode"] = 18, ["InterfaceOptionsSocialPanelBnWhisperMode"] = 18, ["InterfaceOptionsSocialPanelConversationMode"] = 18, ["InterfaceOptionsObjectivesPanelQuestSorting"] = 18, ["CompactUnitFrameProfilesProfileSelector"] = 18, ["CompactUnitFrameProfilesGeneralOptionsFrameSortByDropdown"] = 18, ["CompactUnitFrameProfilesGeneralOptionsFrameHealthTextDropdown"] = 18}, 20, -8, 0, 6, true)
		SkinArrows({"InterfaceOptionsSocialPanelChatStyle", "InterfaceOptionsSocialPanelTimestamps", "InterfaceOptionsSocialPanelWhisperMode", "InterfaceOptionsSocialPanelBnWhisperMode", "InterfaceOptionsSocialPanelConversationMode", "InterfaceOptionsObjectivesPanelQuestSorting", "CompactUnitFrameProfilesProfileSelector", "CompactUnitFrameProfilesGeneralOptionsFrameSortByDropdown", "CompactUnitFrameProfilesGeneralOptionsFrameHealthTextDropdown"})
		
	end
	
	--> System Options:
	if ( not MayronUI.db.profile.ReskinUI["System Options"] ) then
		StartSkinning({["VideoOptionsFrame"] = 30, ["VideoOptionsFramePanelContainer"] = 8, ["VideoOptionsFrameCategoryFrame"] = 4, ["AudioOptionsSoundPanelPlayback"] = 10, ["AudioOptionsSoundPanelHardware"] = 10, ["AudioOptionsSoundPanelVolume"] = 10, ["AudioOptionsVoicePanelTalking"] = 10, ["AudioOptionsVoicePanelBinding"] = 10, ["AudioOptionsVoicePanelListening"] = 10})
		StartSkinning(ScanDropdowns(VideoOptionsFramePanelContainer, 18), 20, -8, 0, 6, true)
		StartSkinning({["Advanced_MultisampleAlphaTest"] = 18}, 20, -8, 0, 6, true)
		SkinArrows({"Advanced_MultisampleAlphaTest"})
		select(4, InterfaceOptionsLanguagesPanelLocaleDropDown:GetRegions()):SetParent(InterfaceOptionsLanguagesPanelLocaleDropDown.new)
		select(4, InterfaceOptionsLanguagesPanelAudioLocaleDropDown:GetRegions()):SetParent(InterfaceOptionsLanguagesPanelAudioLocaleDropDown.new)
		RaidButton:DisableDrawLayer("BACKGROUND")
		GraphicsButton:DisableDrawLayer("BACKGROUND")
		Display_:SetBackdrop(nil)
		Graphics_:SetBackdrop(nil)
		RaidGraphics_:SetBackdrop(nil)
	end
	
	-- Game Tooltips:
	if ( not MayronUI.db.profile.ReskinUI["Game Tooltips"] ) then		
		local backdrop = {
			bgFile = "Interface\\AddOns\\MayronUI\\Media\\ToolTip.tga",
			edgeFile = "Interface\\AddOns\\MayronUI\\Media\Borders\\Skinner.tga",
			edgeSize = 1,
		}
		
		local tooltips = {"GameTooltip", "ItemRefTooltip", "ShoppingTooltip1", "ShoppingTooltip2", "ShoppingTooltip3", "WorldMapTooltip", "ChatMenu", "EmoteMenu", "LanguageMenu", "VoiceMacroMenu"}

		for i = 1, #tooltips do
			local tip = _G[tooltips[i]]
			if (tip) then
				CreateBG(tip)
				tip.GetBackdrop = function() return backdrop end
				tip.GetBackdropColor = function() return 0.2, 0.2, 0.2, 0.9 end -- alter this as well to change frame
				tip.GetBackdropBorderColor = function() return 0, 0, 0, 1 end
			end
		end
		GameTooltipStatusBar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
		GameTooltipText:SetShadowColor(0, 0, 0, 0.6)
		GameTooltipText:SetShadowOffset(1, -1)
		GameTooltipHeaderText:SetShadowColor(0, 0, 0, 0.6)
		GameTooltipHeaderText:SetShadowOffset(1, -1)
		GameTooltipTextSmall:SetShadowColor(0, 0, 0, 0.6)
		GameTooltipTextSmall:SetShadowOffset(1, -1)
	end
	
	--> Spell Book Frame:
	if ( not MayronUI.db.profile.ReskinUI["Spell Book"] ) then
		StartSkinning({["SpellBookFrame"] = 44, ["SpellBookFrameInset"] = 8, ["SpellBookCoreAbilitiesFrame"] = 0, ["SpellBookPageNavigationFrame"] = 0, ["SpellBookProfessionFrame"] = 0})		
		
		SkinTabs({"SpellBookFrameTabButton1", "SpellBookFrameTabButton2", "SpellBookFrameTabButton3", "SpellBookFrameTabButton4"})
		CreateBG(SpellBookFrame, nil, nil, nil, -1)
		SpellBookFrame.protected = true
		for i = 1, 4 do
			if (i < 3) then
				ChangeTextColor(_G["PrimaryProfession"..i].missingText, "white")
			end
			ChangeTextColor(_G["SecondaryProfession"..i].missingText, "white")
		end		
		ChangeTextColor(SpellBookPageText, "white")
		ChangeTextColor(SpellBookCoreAbilitiesFrame.SpecName, "white")
		SpellBookFrameTutorialButton:DisableDrawLayer("BORDER")
		SpellBookFrameTutorialButton:SetPoint("TOPLEFT", SpellBookFrame, "TOPLEFT", -18, 18)
		
		local function AlterFont()
			for i, prof in ipairs({"Primary", "Primary", "Secondary", "Secondary", "Secondary", "Secondary"}) do
				local buttonNames = {"SpellButtonTop", "SpellButtonBottom"}
				if (i > 2) then 
					i = i - 2
					buttonNames = {"SpellButtonLeft", "SpellButtonRight"}
				end
				ChangeTextColor(_G[prof.."Profession"..i.."ProfessionName"], "yellow", true)
				ChangeTextColor(_G[prof.."Profession"..i.."Missing"], "yellow", true)
				for _, str in ipairs(buttonNames) do
					_G[prof.."Profession"..i..str.."NameFrame"]:Hide()
					if (not _G[prof.."Profession"..i..str].new) then
						SkinIcons(_G[prof.."Profession"..i..str.."IconTexture"], 1)
					else
						_G[prof.."Profession"..i..str.."IconTexture"]:SetTexCoord(0.1, 0.92, 0.08, 0.92)
						_G[prof.."Profession"..i..str]:SetHighlightTexture("Interface\\AddOns\\MayronUI\\Media\\Skin\\Highlight_Button.blp")
					end
				end
			end
			for i = 1, SPELLS_PER_PAGE do
				ChangeTextColor(_G["SpellButton"..i.."SpellName"], "white", true)
				ChangeTextColor(_G["SpellButton"..i.."SubSpellName"], "yellow", true)
				ChangeTextColor(_G["SpellButton"..i.."RequiredLevelString"], "yellow", true)
				_G["SpellButton"..i.."TextBackground"]:SetTexture("")
				_G["SpellButton"..i.."TextBackground2"]:SetTexture("")
				_G["SpellButton"..i.."Background"]:SetTexture("")
				_G["SpellButton"..i.."UnlearnedSlotFrame"]:SetTexture("")
				_G["SpellButton"..i.."SlotFrame"]:SetTexture("")
				if (not _G["SpellButton"..i].new) then
					SkinIcons(_G["SpellButton"..i], 1)
				else
					_G["SpellButton"..i.."IconTexture"]:SetTexCoord(0.1, 0.92, 0.08, 0.92)
					_G["SpellButton"..i]:SetHighlightTexture("Interface\\AddOns\\MayronUI\\Media\\Skin\\Highlight_Button.blp")
				end
			end
		end 
		
		hooksecurefunc("SpellBook_GetCoreAbilitySpecTab", function()
			local function repositionTabs()
				for i = 1, #SpellBookCoreAbilitiesFrame.SpecTabs do
					local tab = SpellBookCoreAbilitiesFrame.SpecTabs[i]
					if (i ~= 1) then						
						tab:ClearAllPoints()
						tab:SetPoint("TOPLEFT", SpellBookCoreAbilitiesFrame.SpecTabs[i-1], "BOTTOMLEFT", 0, -8)
					else
						tab:ClearAllPoints()
						tab:SetPoint("TOPLEFT", SpellBookFrame, "TOPRIGHT", 2, 0)
					end					
				end	
			end
			for i = 1, #SpellBookCoreAbilitiesFrame.SpecTabs do
				local tab = SpellBookCoreAbilitiesFrame.SpecTabs[i]
				if (not tab.new) then
					tab:HookScript("OnClick", repositionTabs)
					select(1, tab:GetRegions()):Hide()
					SkinIcons(tab, 1)
				end						
			end
			repositionTabs()
		end)
		local hook = true
		function MayronUI_ReskinUI.ToggleSpellBook()
			AlterFont()
				select(4, SpellBookSkillLineTab1:GetRegions()):SetTexCoord(0.15, 0.85, 0.15, 0.85)
				local function repositionTabs()				
					for i = 1, 5 do
						local tab = _G["SpellBookSkillLineTab"..i]
						if (not tab) then break end
						if (i ~= 1) then						
							tab:ClearAllPoints()
							tab:SetPoint("TOPLEFT", _G["SpellBookSkillLineTab"..(i-1)], "BOTTOMLEFT", 0, -8)
						else
							tab:ClearAllPoints()
							tab:SetPoint("TOPLEFT", SpellBookFrame, "TOPRIGHT", 2, 0)
						end					
					end					
				end			
				if (hook) then
					SpellBookPrevPageButton:HookScript("OnClick", repositionTabs)
					SpellBookNextPageButton:HookScript("OnClick", repositionTabs)
					hook = nil
				end
				for i = 1, 5 do
					local tab = _G["SpellBookSkillLineTab"..i]
					if (not tab) then break end
					if (not tab.new) then
						tab:HookScript("OnClick", repositionTabs)
						select(1, tab:GetRegions()):Hide()
						SkinIcons(tab, 1)
					end
				end
				repositionTabs()
			end
		hooksecurefunc("ToggleSpellBook", MayronUI_ReskinUI.ToggleSpellBook)		
		hooksecurefunc("SpellButton_UpdateButton", AlterFont)
		
		hooksecurefunc("SpellBook_UpdateCoreAbilitiesTab", function()
			for i = 1, #(SpellBookCoreAbilitiesFrame.Abilities) do
				SkinIcons(SpellBookCoreAbilitiesFrame.Abilities[i].iconTexture, 1)
				ChangeTextColor(SpellBookCoreAbilitiesFrame.Abilities[i].Name, "white")
				ChangeTextColor(SpellBookCoreAbilitiesFrame.Abilities[i].InfoText, "white")
				ChangeTextColor(SpellBookCoreAbilitiesFrame.Abilities[i].RequiredLevel, "yellow")
				SpellBookCoreAbilitiesFrame.Abilities[i].ActiveTexture:Hide()
				SpellBookCoreAbilitiesFrame.Abilities[i].EmptySlot:Hide()
				SpellBookCoreAbilitiesFrame.Abilities[i]:SetHighlightTexture("Interface\\AddOns\\MayronUI\\Media\\Skin\\Highlight_Button.blp")
				SpellBookCoreAbilitiesFrame.Abilities[i]:SetParent(SpellBookCoreAbilitiesFrame)
			end
		end)
	end

	-- Blizzard AddOns:
	local ef = CreateFrame("Frame")
	ef:RegisterEvent("ADDON_LOADED") -- Should be registered to MayronUI!
	ef:SetScript("OnEvent", function(self, event, name)
			--print(name)
			-- MikScrollingBattleText:
			if (name == "MSBTOptions") then
				if ( IsAddOnLoaded("MikScrollingBattleText") and not MayronUI.db.profile.ReskinUI["MikScrollingBattleText"] ) then
					hooksecurefunc(MSBTOptions["Main"], "ShowMainFrame", function()
						StartSkinning({["MSBTMainOptionsFrame"] = 54})						
					end)
				end		
				hooksecurefunc(MSBTOptions["Main"], "ShowMainFrame", function()
					MakeMovable(MSBTMainOptionsFrame)					
				end)				
			elseif ( name == "Blizzard_GarrisonUI" ) then
				MakeMovable({"GarrisonCapacitiveDisplayFrame", "GarrisonLandingPage", "GarrisonMissionFrame", "GarrisonBuildingFrame", "GarrisonRecruitSelectFrame", "GarrisonRecruiterFrame"})
				if (MayronUI.db.profile.ReskinUI["Garrison UI"]) then return end
				StartSkinning({["GarrisonLandingPage"] = 14, ["GarrisonLandingPage.FollowerList"] = 8, ["GarrisonCapacitiveDisplayFrame"] = 47, ["GarrisonCapacitiveDisplayFrame.CapacitiveDisplay"] = 16, ["GarrisonLandingPageReportList"] = 4,  ["GarrisonLandingPageReportListListScrollFrameScrollBar"] = 4,  ["GarrisonLandingPage.FollowerList.SearchBox"] = 4, ["GarrisonLandingPageReport.InProgress"] = 80, ["GarrisonLandingPageReport.Available"] = 80})
				SkinButton(GarrisonLandingPageReport.InProgress)
				SkinButton(GarrisonLandingPageReport.Available)				
				GarrisonLandingPageReport.Available:HookScript("OnClick", function(self) self:SetHeight(30) end)
				GarrisonLandingPage.CloseButton:SetPoint("TOPRIGHT", GarrisonLandingPage, "TOPRIGHT", 0, 0)
				CreateBG(GarrisonLandingPage.FollowerList.SearchBox, -4)
				GarrisonLandingPageTab1:SetPoint("TOPLEFT", GarrisonLandingPage, "BOTTOMLEFT", 0, 0)								
				StartSkinning({["GarrisonRecruiterFramePickThreatDropDown"] = 18}, 20, -10, 0, 6, true)
				SkinArrows({"GarrisonRecruiterFramePickThreatDropDown"})
				local function GarrisonLandingPageTab_OnClick(self)
					local enabledTab
					local id = tonumber(self:GetName():match("%a+(%d+)"))
					if (id == 1) then						
						enabledTab = GarrisonLandingPageTab2
					else
						enabledTab = GarrisonLandingPageTab1
					end					
					enabledTab:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-Active.tga")
					enabledTab:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)
					self:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-InActive.tga")
					self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.5, C["Tab_Color"].g * 0.5, C["Tab_Color"].b * 0.5, 0.8)
					
					self:GetNormalTexture():SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
					self:GetNormalTexture():SetPoint("BOTTOM", self, "BOTTOM", 0, -10)	
					enabledTab:GetNormalTexture():SetPoint("TOPLEFT", enabledTab, "TOPLEFT", 0, 3)
					enabledTab:GetNormalTexture():SetPoint("BOTTOM", enabledTab, "BOTTOM", 0, -35)
				end
				local function GarrisonTab_OnEnter(self)
					self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 1.4, C["Tab_Color"].g * 1.4, C["Tab_Color"].b * 1.4, 0.85)
				end
				local function GarrisonTab_OnLeave(self)
					self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)
				end
				
				for n = 1, 2 do
					local self = _G["GarrisonLandingPageTab"..n]
					self:DisableDrawLayer("BACKGROUND")
					self:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-Active.tga")
					self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)					
					self:GetNormalTexture():SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
					self:GetNormalTexture():SetPoint("BOTTOM", self, "BOTTOM", 0, -30)
					self:HookScript("OnClick", GarrisonLandingPageTab_OnClick)			
					self:HookScript("OnEnter", GarrisonTab_OnEnter)
					self:HookScript("OnLeave", GarrisonTab_OnLeave)
				end
				GarrisonLandingPageTab2:Click()
				GarrisonLandingPageTab1:Click()
				
				GarrisonCapacitiveDisplayFrameInset:Hide()
				GarrisonCapacitiveDisplayFrame.StartWorkOrderButton:DisableDrawLayer("BORDER")
				SkinIcons(GarrisonCapacitiveDisplayFrame.CapacitiveDisplay.ShipmentIconFrame.Icon)
				
				hooksecurefunc("GarrisonCapacitiveDisplayFrame_Update", function(self)
					GarrisonLandingPageReport.InProgress:SetHeight(30)
					for i = 1, C_Garrison.GetNumShipmentReagents() do
						local reagent = self.CapacitiveDisplay.Reagents[i]
						if (not reagent.reskinned) then
							reagent.NameFrame:Hide()
							SkinIcons(reagent.Icon)
							reagent.reskinned = true
						end
					end
				end)
				hooksecurefunc("GarrisonLandingPageReport_OnUpdate", function() 
					GarrisonLandingPageReport.InProgress:SetHeight(30)
				end)
				
				for _, name in ipairs({"GarrisonLandingPage", "GarrisonRecruitSelectFrame", "GarrisonMissionFrameFollowers"}) do
					for i = 1, 20 do
						local btn = _G[name .."ListScrollFrameButton"..i]
						if (not btn) then break end						
						SkinFrame(btn, 32)
						btn.Selection:ClearAllPoints()
						btn.Selection:SetAllPoints(btn)
						btn.PortraitFrame.Level:ClearAllPoints()
						btn.PortraitFrame.Level:SetPoint("TOPRIGHT", btn, "TOPRIGHT", -4, -2)
						btn.PortraitFrame.LevelBorder:SetAlpha(0)
						btn.PortraitFrame.PortraitRing:Hide()					
						btn.PortraitFrame.Portrait.item = SkinIcons(btn.PortraitFrame.Portrait)
						btn.PortraitFrame.PortraitRingQuality:SetAlpha(0)
						btn.PortraitFrame.Portrait:SetSize(44, 44)
						btn.PortraitFrame.Portrait:SetPoint("TOPLEFT", btn, "TOPLEFT", 5, -5)
						btn.PortraitFrame.Portrait:SetTexCoord(0.15, 0.85, 0.15, 0.85)	
						select(1, btn:GetRegions()):Hide() -- background
						SkinButton(btn)
						btn.BusyFrame:SetPoint("TOPLEFT", btn, "TOPLEFT", 1, -1)
						btn.BusyFrame:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT", -1, 1)
					end	
				end	

				for i = 1, 10 do
					local btn = _G["GarrisonLandingPageReportListListScrollFrameButton"..i]
					select(1, btn:GetRegions()):Hide()
					btn:SetHeight(44)
					SkinButton(btn)					
					if (i ~= 1) then
						btn:SetPoint("TOPLEFT", _G["GarrisonLandingPageReportListListScrollFrameButton"..(i-1)], "BOTTOMLEFT", 0, -2)
					end
					if (btn.Reward1) then SkinIcons(btn.Reward1.Icon) end
					if (btn.Reward2) then SkinIcons(btn.Reward2.Icon) end
				end			
				
				hooksecurefunc("GarrisonFollowerList_Update", function(self)					
					local name = self:GetName()
					if (name == "GarrisonMissionFrame") then name = name .. "Followers" end
					for i = 1, 20 do
						local btn = _G[name.."ListScrollFrameButton"..i]
						if (not btn) then return end
						btn.PortraitFrame.PortraitRingCover:Hide()
						btn.PortraitFrame.PortraitRingCover:SetAlpha(0)
						local r, g, b = btn.PortraitFrame.PortraitRingQuality:GetVertexColor()						
						if (r and g and b) then
							btn.PortraitFrame.Portrait.item.new:SetVertexColor(r, g, b, 1)
						end
					end
				end)				
				
				-- Garrison Mission Frame:
				StartSkinning({["GarrisonMissionFrame"] = 44, ["GarrisonMissionFrameMissionsListScrollFrameScrollBar"] = 20})
				CreateBG(GarrisonMissionFrame, nil, nil, nil, -3)
				GarrisonMissionFrameMissions.MaterialFrame:DisableDrawLayer("BACKGROUND")
				CreateBG(GarrisonMissionFrameMissions.MaterialFrame, 8, -8, -8, 8)
				GarrisonMissionFrameMissions:DisableDrawLayer("BORDER")		
				GarrisonMissionFrameMissionsTab1:DisableDrawLayer("BORDER")
				GarrisonMissionFrameMissionsTab2:DisableDrawLayer("BORDER")				
				for i = 1, 8 do
					local btn = _G["GarrisonMissionFrameMissionsListScrollFrameButton"..i]
					btn.HighlightT:Hide()
					btn.HighlightB:Hide()
					btn.HighlightTL:Hide()
					btn.HighlightTR:Hide()
					btn.HighlightBL:Hide()
					btn.HighlightBR:Hide()
					btn.Highlight:Hide()
					btn.IconBG:Hide()
									
					for n = 1, 12 do
						select(n, btn:GetRegions()):Hide()
					end
					SkinButton(btn)
				end				
				hooksecurefunc("GarrisonMissionList_Update", function()			
					for i = 1, 8 do
						local btn = _G["GarrisonMissionFrameMissionsListScrollFrameButton"..i]
						local playerlvl = UnitLevel("player")						
						local lvl = tonumber(btn.Level:GetText())
						if (lvl) then 
							if (playerlvl < lvl) then
								if (lvl > playerlvl + 3) then
									if (lvl > playerlvl + 5) then
										ChangeTextColor(btn.Level, "red", true)
									else
										ChangeTextColor(btn.Level, "orange", true)
									end
									
								else
									ChangeTextColor(btn.Level, "yellow", true)
								end
							else
								if (lvl < playerlvl - 3) then
									if (lvl < playerlvl - 5) then
										ChangeTextColor(btn.Level, "grey", true)
									else
										ChangeTextColor(btn.Level, "green", true)
									end
								else
									ChangeTextColor(btn.Level, "yellow", true)
								end
							end
						end
						for _, item in ipairs(btn.Rewards) do
							SkinIcons(item.Icon)
						end
					end
				end)				
				
				GarrisonMissionFrame.MissionTab.MissionPage.Stage.LocBack:SetPoint("TOPLEFT", GarrisonMissionFrame.MissionTab.MissionPage, "TOPLEFT", 0, 0)
				GarrisonMissionFrame.MissionTab.MissionPage.Stage.LocBack:SetPoint("TOPRIGHT", GarrisonMissionFrame.MissionTab.MissionPage, "TOPRIGHT", 0, 0)
				GarrisonMissionFrame.MissionTab.MissionPage.Stage.LocBack:SetAlpha(0.4)
				ChangeTextColor(GarrisonMissionFrame.MissionTab.MissionPage.Stage.Level, "yellow")
				GarrisonMissionFrame.MissionTab.MissionPage.Stage.Level:ClearAllPoints()
				GarrisonMissionFrame.MissionTab.MissionPage.Stage.Level:SetPoint("TOPLEFT", GarrisonMissionFrame.MissionTab.MissionPage, "TOPLEFT", 10, -10)
				GarrisonMissionFrame.MissionTab.MissionPage:DisableDrawLayer("ARTWORK")
				StartSkinning({["GarrisonMissionFrame.MissionTab.MissionPage.Stage"] = 32, ["GarrisonMissionFrame.MissionTab.MissionPage"] = 12, 
				["GarrisonMissionFrameFollowersListScrollFrameScrollBar"] = 20})
				select(4, GarrisonMissionFrame.MissionTab.MissionPage.Stage:GetRegions()):Hide()
				GarrisonMissionFrameFollowers.SearchBox:DisableDrawLayer("BACKGROUND")
				CreateBG(GarrisonMissionFrameFollowers.SearchBox, -4)
				GarrisonMissionFrameFollowers:DisableDrawLayer("BORDER")
				GarrisonMissionFrameFollowers.MaterialFrame:DisableDrawLayer("BACKGROUND")
				CreateBG(GarrisonMissionFrameFollowers.MaterialFrame, 8, -8, -8, 8)				
				CreateBG(GarrisonMissionFrame.MissionTab.MissionPage, -1, 1, 1, -12)
				GarrisonMissionFrame.MissionTab.MissionPage.RewardsFrame:DisableDrawLayer("BACKGROUND")
				GarrisonMissionFrame.MissionTab.MissionPage.RewardsFrame:DisableDrawLayer("BORDER")
				CreateBG(GarrisonMissionFrame.MissionTab.MissionPage.RewardsFrame, 0, 0, 4)
				GarrisonMissionFrame.FollowerTab:DisableDrawLayer("BORDER")
				GarrisonMissionFrameTab1:DisableDrawLayer("BACKGROUND")
				GarrisonMissionFrameTab2:DisableDrawLayer("BACKGROUND")
				
				hooksecurefunc("GarrisonMissionFrame_SelectTab", function(clickedTab)
					for n = 1, 2 do
						local self = _G["GarrisonMissionFrameTab"..n]
						if ( n ~= clickedTab ) then
							self:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-Active.tga")
							self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)
							self:GetNormalTexture():SetPoint("BOTTOM", self, "BOTTOM", 0, -30)
							self:GetNormalTexture():SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
						else -- selected:
							self:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-InActive.tga")
							self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.5, C["Tab_Color"].g * 0.5, C["Tab_Color"].b * 0.5, 0.8)
							self:GetNormalTexture():SetPoint("BOTTOM", self, "BOTTOM", 0, -10)
							self:GetNormalTexture():SetPoint("TOPLEFT", self, "TOPLEFT", 0, -2)
						end
					end					
					for i = 1, 11 do
						local btn = _G["GarrisonMissionFrameFollowersListScrollFrameButton"..i]
						btn.PortraitFrame.PortraitRingCover:Hide()
						btn.PortraitFrame.PortraitRingCover:SetAlpha(0)						
					end	
				end)
				GarrisonMissionFrame_SelectTab(2)	
				GarrisonMissionFrame_SelectTab(1)	
				GarrisonMissionFrameTab1:HookScript("OnEnter", GarrisonTab_OnEnter)		
				GarrisonMissionFrameTab1:HookScript("OnLeave", GarrisonTab_OnLeave)	
				GarrisonMissionFrameTab2:HookScript("OnEnter", GarrisonTab_OnEnter)		
				GarrisonMissionFrameTab2:HookScript("OnLeave", GarrisonTab_OnLeave)	
							
				SkinFrame(GarrisonMissionFrame.MissionComplete, 28)
				CreateBG(GarrisonMissionFrame.MissionComplete, -2, 2, 2, -10)
				SkinFrame(GarrisonMissionFrame.MissionComplete.Stage.MissionInfo, 48)
				SkinFrame(GarrisonMissionFrameMissions.CompleteDialog.BorderFrame, 12)
				GarrisonMissionFrameMissions.CompleteDialog.BorderFrame.Stage:DisableDrawLayer("OVERLAY")
				GarrisonMissionFrameMissions.CompleteDialog.BorderFrame.Stage:SetAlpha(0.8)
				CreateBG(GarrisonMissionFrameMissions.CompleteDialog.BorderFrame, nil, nil, -2, 2)
				
				hooksecurefunc("GarrisonMissionComplete_Initialize", function()
					ChangeTextColor(select(11, GarrisonMissionFrame.MissionComplete.BonusRewards:GetRegions()), "yellow") -- fontstring
					GarrisonMissionFrame.MissionComplete.BonusRewards.Saturated:Hide()
					SkinFrame(GarrisonMissionFrame.MissionComplete.BonusRewards, 12)					
				end)
				
				--> GarrisonBuildingFrame:
				StartSkinning({["GarrisonBuildingFrame"] = 46})
				GarrisonBuildingFrameTutorialButton:DisableDrawLayer("BORDER")
				GarrisonBuildingFrameTutorialButton:SetPoint("TOPLEFT", GarrisonBuildingFrame, "TOPLEFT", -18, 18)
				GarrisonBuildingFrame.BuildingList:DisableDrawLayer("BORDER")
				GarrisonBuildingFrame.TownHallBox:DisableDrawLayer("BORDER")
				GarrisonBuildingFrame.InfoBox:DisableDrawLayer("BORDER")
				GarrisonBuildingFrame.BuildingList.MaterialFrame:DisableDrawLayer("BACKGROUND")
				CreateBG(GarrisonBuildingFrame.BuildingList)
				CreateBG(GarrisonBuildingFrame.InfoBox)
				CreateBG(GarrisonBuildingFrame.TownHallBox)
				for i = 1, 3 do
					local tab = GarrisonBuildingFrame.BuildingList["Tab"..i]
					SkinFrame(tab, 80)
					SkinButton(tab, nil, true)
					tab:SetSize(80, 30)
					select(1, tab:GetRegions()):SetPoint("TOP", tab, "TOP", 0, 10)
					if (i == 1) then
						local _, rf = tab:GetPoint()
						tab:SetPoint("BOTTOMLEFT", rf, "TOPLEFT", 0, 2)
					else
						tab:SetPoint("LEFT", GarrisonBuildingFrame.BuildingList["Tab"..(i-1)], "RIGHT", 2, 0)
					end
				end
				hooksecurefunc("GarrisonBuildingList_SelectTab", function(tab)					
					local list = GarrisonBuildingFrame.BuildingList
					if (tab.categoryID == 2) then
						list.Tab1:SetBackdropBorderColor(1, 1, 1)
						list.Tab1.oldBC = { 1, 1, 1 }
						list.Tab3:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b)
						list.Tab3.oldBC = { C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b }
						list.Tab2:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b)
						list.Tab2.oldBC = { C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b }
					elseif (tab.categoryID == 1) then
						list.Tab2:SetBackdropBorderColor(1, 1, 1)
						list.Tab2.oldBC = { 1, 1, 1 }
						list.Tab3:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b)
						list.Tab3.oldBC = { C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b }
						list.Tab1:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b)
						list.Tab1.oldBC = { C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b }						
					else
						list.Tab3:SetBackdropBorderColor(1, 1, 1)
						list.Tab3.oldBC = { 1, 1, 1 }
						list.Tab1:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b)
						list.Tab1.oldBC = { C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b }
						list.Tab2:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b)
						list.Tab2.oldBC = { C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b }
					end					
					for i = 1, #tab.buildings do
						local btn = list.Buttons[i] 
						if (btn and not btn.reskinned) then							
							SkinIcons(btn.Icon)
							btn.Icon:SetPoint("LEFT", btn, "LEFT", 4, 0)
							btn.SelectedBG:SetAlpha(0)
							btn.BG:Hide()
							btn:SetHighlightTexture("")
							SkinButton(btn, nil, true)
							if (i ~= 1) then
								btn:SetPoint("TOP", list.Buttons[i-1], "BOTTOM", 0, -2)
							end
						end
					end
				end)
				hooksecurefunc("GarrisonBuildingListButton_Select", function(self)				
					local list = GarrisonBuildingFrame.BuildingList				
					for i = 1, #list.Buttons do
						if (list.Buttons[i]) then
							local btn = list.Buttons[i]
							if (btn.SelectedBG:IsShown()) then
								btn.oldBC = { 1, 1, 1 }
								btn:SetBackdropBorderColor(1, 1, 1)
							else								
								btn:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b)
								btn.oldBC = { C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b }
							end										
						end
					end
				end)
				
				--> GarrisonRecruiterFrame:			
				GarrisonRecruiterFrameInset:Hide()
				StartSkinning({["GarrisonRecruiterFrame"] = 62})
				
				--> GarrisonRecruitSelectFrame:
				StartSkinning({["GarrisonRecruitSelectFrame"] = 46})
				GarrisonRecruitSelectFrame.FollowerList:DisableDrawLayer("BORDER")
				GarrisonRecruitSelectFrame.FollowerList.SearchBox:DisableDrawLayer("BACKGROUND")
				CreateBG(GarrisonRecruitSelectFrame.FollowerList.SearchBox, -4)				
				GarrisonRecruitSelectFrameListScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")			
				GarrisonRecruitSelectFrameListScrollFrameScrollBar:DisableDrawLayer("ARTWORK")
				GarrisonRecruitSelectFrame.FollowerSelection:DisableDrawLayer("BORDER")
				CreateBG(GarrisonRecruitSelectFrame.FollowerSelection)
				
				--GarrisonRecruitSelectFrameListScrollFrameButton11		
				
			elseif ( name == "Blizzard_LookingForGuildUI" ) then 
				MakeMovable(LookingForGuildFrame)
				if (MayronUI.db.profile.ReskinUI["Looking For Guild"]) then return end
				SkinFrame(LookingForGuildFrame, 60)				
				CreateBG(LookingForGuildFrame)
				LookingForGuildFrameInset:Hide()
				LookingForGuildCommentInputFrame:DisableDrawLayer("BACKGROUND")
				CreateBG(LookingForGuildCommentInputFrame)
				LookingForGuildRequestButton:DisableDrawLayer("BORDER")
				LookingForGuildBrowseButton:DisableDrawLayer("BORDER")
				LookingForGuildBrowseFrameContainerScrollBar:DisableDrawLayer("ARTWORK")
				LookingForGuildInterestFrame:DisableDrawLayer("BACKGROUND")
				LookingForGuildCommentFrame:DisableDrawLayer("BACKGROUND")
				LookingForGuildRolesFrame:DisableDrawLayer("BACKGROUND")
				LookingForGuildAvailabilityFrame:DisableDrawLayer("BACKGROUND")
				for i = 1, 3 do
					_G["LookingForGuildFrameTab"..i]:DisableDrawLayer("BACKGROUND")
				end
				GuildFinderRequestMembershipFrame:SetBackdrop(nil)
				CreateBG(GuildFinderRequestMembershipFrame)
				GuildFinderRequestMembershipFrameInputFrame:DisableDrawLayer("BACKGROUND")
				CreateBG(GuildFinderRequestMembershipFrameInputFrame)
				
				hooksecurefunc("RequestRecruitingGuildsList", function()
					local buttons = LookingForGuildBrowseFrameContainer.buttons
					local numButtons = #buttons				
					 for i = 1, numButtons do
						local btn = buttons[i]
						SkinButton(btn)
						btn.selectedTex:SetAlpha(0)
						-- reposition:
						btn:SetHeight(80)
						if (i ~= 1) then
							btn:SetPoint("TOPLEFT", buttons[i-1], "BOTTOMLEFT", 0, -2)
						end
					 end
				end)
				
				hooksecurefunc("LookingForGuild_Update", function()
					local buttons = LookingForGuildBrowseFrameContainer.buttons
					local numButtons = #buttons				
					 for i = 1, numButtons do
						local btn = buttons[i]
						if (btn.selectedTex:IsShown()) then
							btn:SetBackdropBorderColor(1,1,1)
							btn.oldBC = {1,1,1}
						else
							btn:SetBackdropBorderColor(0,0,0)
							btn.oldBC = {0,0,0}
						end
						btn:SetHeight(80)
					 end
				end) 
				
				for i = 1, 10 do
					_G["LookingForGuildAppsFrameContainerButton"..i]:SetBackdrop(nil)
				end
   
		elseif ( name == "Blizzard_VoidStorageUI" ) then
			MakeMovable(VoidStorageFrame)
			if (MayronUI.db.profile.ReskinUI["Void Storage"]) then return end
			if (IsAddOnLoaded("Bagnon") and Bagnon:IsFrameEnabled("vault")) then return end
			StartSkinning({["VoidStorageFrame"] = 14, ["VoidStorageBorderFrame"] = 44, ["VoidStorageStorageFrame"] = 28, ["VoidStorageDepositFrame"] = 12, ["VoidStorageWithdrawFrame"] = 12, ["VoidStorageCostFrame"] =  12})
			VoidItemSearchBox:DisableDrawLayer("BACKGROUND")
			CreateBG(VoidItemSearchBox, -4, nil, 1)
			for i = 1, 80 do
				SkinIcons(_G["VoidStorageStorageButton"..i], 1, true)
				_G["VoidStorageStorageButton"..i.."Bg"]:Hide()
			end
			for i = 1, 9 do
				SkinIcons(_G["VoidStorageDepositButton"..i], 1, true)
				_G["VoidStorageDepositButton"..i.."Bg"]:Hide()
			end
			for i = 1, 9 do
				SkinIcons(_G["VoidStorageWithdrawButton"..i], 1, true)
				_G["VoidStorageWithdrawButton"..i.."Bg"]:Hide()
			end				
						
			local function UpdateSlots(deposit)
				if (deposit) then
					for i = 1, 9 do
						if (select(1, GetVoidTransferDepositInfo(i))) then
							UpdateQualityColor(_G["VoidStorageDepositButton"..i])
						else
							_G["VoidStorageDepositButton"..i].new:SetVertexColor(0,0,0)
						end
					end
				else
					for i = 1, 80 do
						local tab = VoidStorageFrame.Page1:GetChecked()
						if (tab) then tab = 1 else tab = 2 end
						if (select(1, GetVoidItemInfo(tab, i))) then
							UpdateQualityColor(_G["VoidStorageStorageButton"..i])							
						else
							_G["VoidStorageStorageButton"..i].new:SetVertexColor(0,0,0)
						end			
					end
					for i = 1, 9 do						
						if (select(1, GetVoidTransferWithdrawalInfo(i))) then
							UpdateQualityColor(_G["VoidStorageWithdrawButton"..i])
						else
							_G["VoidStorageWithdrawButton"..i].new:SetVertexColor(0,0,0)
						end				
					end
				end
			end			
			VoidStorageTransferButton:HookScript("OnClick", function() UpdateSlots(true) end)	
			VoidStorageFrame:HookScript("OnEvent", function(self, event, ...)
				if (event == "VOID_TRANSFER_DONE") then
					UpdateSlots()
				elseif (event == "VOID_STORAGE_DEPOSIT_UPDATE") then
					UpdateSlots(true)
				end
			end)
			hooksecurefunc("VoidStorageFrame_UpdateTabs", UpdateSlots)
			hooksecurefunc("VoidStorage_ItemsUpdate", UpdateSlots)			
			
			VoidStorageFrame.Page1:HookScript("OnClick", function() UpdateSlots() end)
			select(1, VoidStorageFrame.Page1:GetRegions()):Hide()
			SkinIcons(VoidStorageFrame.Page1, 1)
			VoidStorageFrame.Page1:ClearAllPoints()
			VoidStorageFrame.Page1:SetPoint("TOPLEFT", VoidStorageFrame, "TOPRIGHT", 3, 0)
			VoidStorageFrame.Page2:HookScript("OnClick", function() UpdateSlots() end)
			select(1, VoidStorageFrame.Page2:GetRegions()):Hide()
			SkinIcons(VoidStorageFrame.Page2, 1)
			VoidStorageFrame.Page2:ClearAllPoints()
			VoidStorageFrame.Page2:SetPoint("TOPLEFT", VoidStorageFrame.Page1, "BOTTOMLEFT", 0, -8)
		elseif ( name == "Blizzard_ItemAlterationUI" ) then
			MakeMovable(TransmogrifyFrame)
			if (MayronUI.db.profile.ReskinUI["Transmog Frame"]) then return end
			StartSkinning({["TransmogrifyFrame"] = 46, ["TransmogrifyArtFrame"] = 44, ["TransmogrifyFrameButtonFrame"] = 24, ["TransmogrifyModelFrame"] = 4, ["TransmogrifyApplyButton"] = 8})			
			local slots = {"Head", "Shoulder", "Chest", "Waist", "Legs", "Feet", "Wrist", "Hands", "Back", "MainHand", "SecondaryHand"}
			for _, v in ipairs(slots) do
				_G["TransmogrifyFrame"..v.."SlotGrabber"]:Hide()
			end			
			CreateBG(TransmogrifyConfirmationPopup)
			TransmogrifyConfirmationPopup:DisableDrawLayer("BORDER")
			TransmogrifyConfirmationPopup:DisableDrawLayer("BACKGROUND")
			select(9, TransmogrifyConfirmationPopupItemFrame1:GetRegions()):Hide()
			SkinIcons(TransmogrifyConfirmationPopupItemFrame1.icon, 1)
			
		elseif ( name == "Blizzard_GuildBankUI" ) then
			MakeMovable(GuildBankFrame)
			if (MayronUI.db.profile.ReskinUI["Guild Bank"]) then return end
			StartSkinning({["GuildBankFrame"] = 46, ["GuildBankEmblemFrame"] = 28, ["GuildBankMoneyFrameBackground"] = 4, ["GuildBankTransactionsScrollFrame"] = 16, ["GuildBankInfoScrollFrame"] = 16})
			StartSkinning({["GuildItemSearchBox"] = 6}, -6, 2)
			SkinTabs({"GuildBankFrameTab1", "GuildBankFrameTab2", "GuildBankFrameTab3", "GuildBankFrameTab4"})

			for column = 1, NUM_GUILDBANK_COLUMNS do
				_G["GuildBankColumn"..column]:DisableDrawLayer("BACKGROUND")
				for button = 1, NUM_SLOTS_PER_GUILDBANK_GROUP do
					SkinIcons(_G["GuildBankColumn"..column.."Button"..button])
					_G["GuildBankColumn"..column.."Button"..button].IconBorder:SetAlpha(0)
				end
			end
			
			for i = 1, 6 do
				local tab = _G["GuildBankTab"..i]
				if (not tab) then break end
				SkinFrame(tab, 4)
				SkinIcons(_G["GuildBankTab"..i.."Button"], 1)
				tab:ClearAllPoints()
				if (i == 1) then					
					tab:SetPoint("TOPLEFT", GuildBankFrame, "TOPRIGHT", 0, 8)
				else
					tab:SetPoint("TOPLEFT", _G["GuildBankTab"..(i - 1)], "BOTTOMLEFT", 0, 10)
				end					
			end
			
			local function UpdateSlots()
				local tab = GetCurrentGuildBankTab()
				local id = 0
				for c = 1, NUM_GUILDBANK_COLUMNS do
					for s = 1, NUM_SLOTS_PER_GUILDBANK_GROUP do
						id = id + 1
						local link  = GetGuildBankItemLink(tab, id)
						local slot = _G["GuildBankColumn"..c.."Button"..s]
						if (link) then	
							local quality = select(3, GetItemInfo(link))
							if (quality) then
								local r, g, b = GetItemQualityColor(quality)
								slot.new:SetVertexColor(r, g, b)
							else
								slot.new:SetVertexColor(0,0,0)
							end
						else
							slot.new:SetVertexColor(0,0,0)
						end	
					end
				end
			end
			hooksecurefunc("GuildBankFrame_Update", UpdateSlots)
			
		elseif ( name == "Blizzard_TalentUI" ) then
			MakeMovable(PlayerTalentFrame)
			if (MayronUI.db.profile.ReskinUI["Talent Frame"]) then return end
			SkinTabs({"PlayerTalentFrameTab1", "PlayerTalentFrameTab2", "PlayerTalentFrameTab3", "PlayerTalentFrameTab4"})			
			StartSkinning({["PlayerTalentFrame"] = 44, ["PlayerTalentFrameInset"] = 12, ["PlayerTalentFrameTalents"] = 12})
			CreateBG(PlayerTalentFrame, nil, nil, nil, -1)			
			PlayerTalentFrameTalentsTutorialButton:DisableDrawLayer("BORDER")
			PlayerTalentFrameTalentsTutorialButton:SetPoint("TOPLEFT", PlayerTalentFrame, "TOPLEFT", -18, 18)
			PlayerTalentFrameSpecializationTutorialButton:DisableDrawLayer("BORDER")
			PlayerTalentFrameSpecializationTutorialButton:SetPoint("TOPLEFT", PlayerTalentFrame, "TOPLEFT", -18, 18)
			PlayerTalentFrameTalentsLearnButton:DisableDrawLayer("BORDER")			
			SkinIcons({"PlayerSpecTab1", "PlayerSpecTab2"}, 1)
			select(1, PlayerSpecTab1:GetRegions()):Hide()
			select(1, PlayerSpecTab2:GetRegions()):Hide()			
			PlayerTalentFrameSpecializationLearnButton:DisableDrawLayer("BORDER")
			SkinFrame(PlayerTalentFrameSpecialization, 24)
			select(7, PlayerTalentFrameSpecialization:GetChildren()):DisableDrawLayer("OVERLAY")
			for i = 1, 5 do
				select(i, PlayerTalentFrameSpecializationSpellScrollFrameScrollChild:GetRegions()):Hide()
			end
			select(13, PlayerTalentFrameSpecializationSpellScrollFrameScrollChild:GetRegions()):Hide()
			PlayerTalentFrameSpecializationSpellScrollFrameScrollChildRing:Hide()
			SkinIcons(PlayerTalentFrameSpecializationSpellScrollFrameScrollChildSpecIcon, 1)
			PlayerTalentFrameSpecializationSpellScrollFrameScrollChildSpecIcon:SetTexCoord(0.15, 0.85, 0.15, 0.85)
			
			hooksecurefunc("PlayerTalentFrame_UpdateSpecFrame", function(self, index)
				if (PanelTemplates_GetSelectedTab(PlayerTalentFrame) ~= SPECIALIZATION_TAB) then return end
				local isPet = (self.isPet and "Pet") or ""
				local btn = _G["PlayerTalentFrame"..isPet.."SpecializationSpellScrollFrameScrollChild"]
				if (btn) then
					for i = 1, 10 do
						local ability = btn["abilityButton"..i]
						if (not ability) then break end
						ability.ring:Hide()
						ability.subText:SetTextColor(YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b)
						SkinIcons(ability.icon, 1)
						ability.icon:SetTexCoord(0.15, 0.85, 0.15, 0.85)
					end
				end
			end)
			
			for i = 1, 7 do
				SkinFrame(_G["PlayerTalentFrameTalentsTalentRow"..i], 28)
				for t = 1, 3 do
					local talent = _G["PlayerTalentFrameTalentsTalentRow"..i.."Talent"..t]
					talent.Slot:Hide()
					talent.icon:SetDrawLayer("ARTWORK")
					SkinIcons(talent, 1, true)
					talent.learnSelection:SetAlpha(0)
					talent.knownSelection:SetAlpha(0)
					SkinButton(talent)
					talent:SetWidth(talent:GetWidth() - 10)
					talent.icon:ClearAllPoints()
					talent.icon:SetPoint("TOPLEFT", talent, "TOPLEFT", 3, -3)
					talent.name:SetWidth(120)		
					talent:HookScript("OnClick", function(self)
						local name = self:GetName()	
						local subname, talentNum = name:match("(%a+%d+%a+)(%d+)")
						for i = 1, 3 do
							if (tonumber(talentNum) == i) then
								if (self.learnSelection:IsShown() and self.oldBC) then
									self.oldBC = { 0.1, 1, 0.1, 1 }
									self:SetBackdropBorderColor(0.1, 1, 0.1, 1)
								elseif (self.knownSelection:IsShown() ) then
									self.oldBC = { MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b }
								else
									self.oldBC = { 0, 0, 0 }
								end
							elseif (_G[subname..i].knownSelection:IsShown() ) then
								_G[subname..i].oldBC = { MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b }
								_G[subname..i]:SetBackdropBorderColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
							else
								_G[subname..i].oldBC = { 0, 0, 0 }
								_G[subname..i]:SetBackdropBorderColor(0, 0, 0)
							end
						end
					end)
				end
			end
			
			hooksecurefunc("PlayerTalentFrame_Update", function()
				PlayerSpecTab1:ClearAllPoints()
				PlayerSpecTab1:SetPoint("TOPLEFT", PlayerTalentFrame, "TOPRIGHT", 3, 0)
				PlayerSpecTab2:ClearAllPoints()
				PlayerSpecTab2:SetPoint("TOPLEFT", PlayerSpecTab1, "BOTTOMLEFT", 0, -8)
				for i = 1, 7 do
					for t = 1, 3 do
						local talent = _G["PlayerTalentFrameTalentsTalentRow"..i.."Talent"..t]
						if ( talent.knownSelection:IsShown() ) then
							talent:SetBackdropBorderColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 1)	
							talent:SetBackdropColor(MUI_COLOR.r * 0.6, MUI_COLOR.g * 0.6, MUI_COLOR.b * 0.6)							
						else
							talent:SetBackdropBorderColor(0, 0, 0, 1)
							talent:SetBackdropColor(C["Background_Color"].r, C["Background_Color"].g, C["Background_Color"].b)
						end
						if ( talent.learnSelection:IsShown() ) then
							talent:SetBackdropBorderColor(0.1, 1, 0.1, 1)
							talent:SetBackdropColor(C["Background_Color"].r, C["Background_Color"].g, C["Background_Color"].b)
						end
					end
				end
			end)
			
			-- spec buttons:			
			hooksecurefunc("PlayerTalentFrame_Refresh", function(self)				
				local selectedTab = PanelTemplates_GetSelectedTab(PlayerTalentFrame)
				if (selectedTab == SPECIALIZATION_TAB) then				
					-- Player Talents:
					local numSpecs = GetNumSpecializations(false, false)
					local specName = PlayerTalentFrameSpecializationSpellScrollFrameScrollChildSpecName:GetText()
					for i = 1, numSpecs do
						local btn = _G["PlayerTalentFrameSpecializationSpecButton"..i]
						local text = btn.specName:GetText()
						if (specName == text) then
							btn:SetBackdropBorderColor(1, 1, 1)
							btn.oldBC = { 1, 1, 1 }							
						else
							btn:SetBackdropBorderColor(0, 0, 0)
							btn.oldBC = { 0, 0, 0 }
						end
					end	
				elseif (selectedTab == PET_SPECIALIZATION_TAB) then	
					-- Pet Talents:
					local numSpecs = GetNumSpecializations(false, true)
					local specName = PlayerTalentFramePetSpecializationSpellScrollFrameScrollChildSpecName:GetText()
					for i = 1, numSpecs do
						local btn = _G["PlayerTalentFramePetSpecializationSpecButton"..i]
						local text = btn.specName:GetText()
						if (specName == text) then
							btn:SetBackdropBorderColor(1, 1, 1)
							btn.oldBC = { 1, 1, 1 }
						else
							btn:SetBackdropBorderColor(0, 0, 0)
							btn.oldBC = { 0, 0, 0 }
						end
					end
				end
			end)
			
			for _, str in ipairs({"_", "Pet"}) do
				local Pet = ((str == "Pet") and str) or ""
				for i = 1, 4 do	
					local btn = _G["PlayerTalentFrame"..Pet.."SpecializationSpecButton"..i]
					local name = "PlayerTalentFrame"..Pet.."SpecializationSpecButton"..i
					if (not btn) then return end
					btn:HookScript("OnClick", function(self)
						local subname, id = name:match("(%a+)(%d+)")
						for i = 1, 3 do
							if (tonumber(id) == i) then							
								self.oldBC = { 1, 1, 1 }
								self:SetBackdropBorderColor(1, 1, 1)
							else
								_G[subname..i].oldBC = { 0, 0, 0 }
								_G[subname..i]:SetBackdropBorderColor(0, 0, 0)
							end
						end				
					end)
					select(1, btn:GetRegions()):Hide()		
					_G[name.."Ring"]:SetTexture("")
					_G[name.."Selection"]:SetTexture("")
					btn.selectedTex:SetAlpha(0)
					btn:SetHighlightTexture("")
					SkinIcons(_G[name.."SpecIcon"])
					_G[name.."SpecIcon"]:SetTexCoord(0.15, 0.87, 0.13, 0.87)
					local h, w = _G[name.."SpecIcon"]:GetSize()
					_G[name.."SpecIcon"]:SetSize(h - 14, w - 14)
					_G[name.."SpecIcon"]:ClearAllPoints()
					_G[name.."SpecIcon"]:SetPoint("TOPLEFT", btn, "TOPLEFT", 4, -4)
					SkinButton(btn)
				end
			end
			
			-- Hunter's Pet Frame:
			PlayerTalentFramePetSpecializationLearnButton:DisableDrawLayer("BORDER")			
			PlayerTalentFramePetSpecialization:DisableDrawLayer("BORDER")
			PlayerTalentFramePetSpecialization:DisableDrawLayer("ARTWORK")
			for i = 1, 6 do
				select(i, PlayerTalentFramePetSpecializationSpellScrollFrameScrollChild:GetRegions()):Hide()
			end
			select(13, PlayerTalentFramePetSpecializationSpellScrollFrameScrollChild:GetRegions()):Hide()
			PlayerTalentFramePetSpecializationSpellScrollFrameScrollChildRing:Hide()
			SkinIcons(PlayerTalentFramePetSpecializationSpellScrollFrameScrollChildSpecIcon, 1)
			PlayerTalentFramePetSpecializationSpellScrollFrameScrollChildSpecIcon:SetTexCoord(0.15, 0.85, 0.15, 0.85)
			PlayerTalentFramePetSpecializationTutorialButton:DisableDrawLayer("BORDER")
			PlayerTalentFramePetSpecializationTutorialButton:SetPoint("TOPLEFT", PlayerTalentFrame, "TOPLEFT", -18, 18)
			select(7, PlayerTalentFramePetSpecialization:GetChildren()):Hide() -- Side Art
			
		elseif ( name == "Blizzard_GlyphUI" ) then
			if (MayronUI.db.profile.ReskinUI["Glyph Frame"]) then return end
			SkinFrame(GlyphFrameSideInset, 12)
			GlyphFrameScrollFrameScrollBar:DisableDrawLayer("ARTWORK")
			for i = 1, 10 do
				local f = _G["GlyphFrameScrollFrameButton"..i]
				SkinFrame(f, 20)
				SkinIcons(_G[f:GetName().."Icon"])
				SkinButton(f)
			end
			CreateBG(GlyphFrameSideInset, nil, nil, nil, 1, nil, nil, true)
			local new = CreateBG(GlyphFrame, nil, nil, nil, -2, nil, nil, true)
			GlyphFrame:DisableDrawLayer("BACKGROUND")
			
			GlyphFrame.specRing:SetTexture("")
			SkinFrame(GlyphFrameHeader1, 12)
			SkinFrame(GlyphFrameHeader2, 12)				
			GlyphFrameSearchBox:DisableDrawLayer("BACKGROUND")
			
			CreateBG(GlyphFrameSearchBox, -6, 2, nil, nil, nil, nil, true)
			StartSkinning({["GlyphFrameFilterDropDown"] = 18}, 16, -16, 0, 4, true)			
			GlyphFrameFilterDropDownButton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
			GlyphFrameFilterDropDownButton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
			GlyphFrame.specIcon:SetAlpha(0)
			
			local SpecIcon = CreateFrame("Frame", nil, new)
			SpecIcon:SetPoint("CENTER", GlyphFrame.specIcon, "CENTER")
			SpecIcon:SetSize(112, 112)
			SpecIcon.Icon = SpecIcon:CreateTexture()
			SpecIcon.Icon:SetAllPoints(SpecIcon)		
			
			SpecIcon.Ring = SpecIcon:CreateTexture(nil, "OVERLAY")
			SpecIcon.Ring:SetPoint("CENTER", SpecIcon, "CENTER")
			SpecIcon.Ring:SetTexture("Interface\\TalentFrame\\talent-main")	
			SpecIcon.Ring:SetTexCoord(0.00390625, 0.49218750, 0.00195313, 0.24609375)
			SpecIcon.Ring:SetSize(156, 156)
			
			hooksecurefunc("GlyphFrame_Update", function(self)
				local spec = GetSpecialization(false, false, PlayerTalentFrame.talentGroup)
				local curSpec = GetSpecialization()				
				if (spec) then
					local _, _, _, icon = GetSpecializationInfo(spec, false, self.isPet)
					SpecIcon.Icon:SetTexture(icon)
					SetPortraitToTexture(SpecIcon.Icon, icon)
					if (spec == curSpec) then
						SetDesaturation(SpecIcon.Ring, false)
						SetDesaturation(SpecIcon.Icon, false)
					else
						SetDesaturation(SpecIcon.Ring, true)
						SetDesaturation(SpecIcon.Icon, true)
					end
				end
			end)
		elseif ( name == "Blizzard_MacroUI" ) then
			MakeMovable(MacroFrame)
			if (MayronUI.db.profile.ReskinUI["Macro Frame"]) then return end		
			StartSkinning({["MacroFrame"] = 62})			
			StartSkinning({["MacroButtonScrollFrame"] = 18}, nil, nil, 2, -2)
			SkinIcons(MacroFrameSelectedMacroButtonIcon, 1)
			SkinFrame(MacroFrameInset, 12)
			SkinFrame(MacroFrameTextBackground, 12)
			CreateBG(MacroFrameTextBackground, 4, nil, -2, 4)
			
			for i = 1, MAX_ACCOUNT_MACROS do
				SkinIcons(_G["MacroButton"..i.."Icon"], 1)			
			end
			for i = 1, NUM_MACRO_ICONS_SHOWN do
				SkinIcons(_G["MacroPopupButton"..i.."Icon"], 1)
			end
			
			MacroPopupFrame:DisableDrawLayer("BACKGROUND")
			for i = 1, 6 do
				select(i, MacroFrameTab1:GetRegions()):Hide()
				select(i, MacroFrameTab2:GetRegions()):Hide()
				select(i, MacroFrameTab1:GetRegions()).Show = function() end
				select(i, MacroFrameTab2:GetRegions()).Show = function() end
			end
			for i = 1, 5 do
				select(i, MacroPopupFrame:GetRegions()):Hide()
			end
			MacroPopupScrollFrame:GetRegions():Hide()
			select(2, MacroPopupScrollFrame:GetRegions()):Hide()
			CreateBG(MacroPopupFrame)
			MacroPopupFrame:SetPoint("TOPLEFT", MacroFrame, "TOPRIGHT", 4, 0)			
			MacroPopupEditBox:DisableDrawLayer("BACKGROUND")
			CreateBG(MacroPopupEditBox, -4)
		elseif ( name == "Blizzard_BindingUI") then
			MakeMovable(KeyBindingFrame)
			if (MayronUI.db.profile.ReskinUI["Key Binding Frame"]) then return end	
			SkinFrame(KeyBindingFrame, 20)			
			CreateBG(KeyBindingFrame)
			KeyBindingFrame.bindingsContainer:SetBackdrop(nil)
			KeyBindingFrameCategoryList:DisableDrawLayer("BACKGROUND")
			SkinFrame(KeyBindingFrameScrollFrame, 12)
			SkinFrame(KeyBindingFrame.header, 12)
		elseif ( name == "Blizzard_Calendar" ) then
			MakeMovable(CalendarFrame)			
			if (MayronUI.db.profile.ReskinUI["Calendar"]) then return end
			StartSkinning({["CalendarViewHolidayFrame"] = 12, ["CalendarViewHolidayTitleFrame"] = 16, ["CalendarCreateEventTitleFrame"] = 16, ["CalendarViewHolidayCloseButton"] = 4, ["CalendarViewEventFrame"] = 14, ["CalendarViewEventTitleFrame"] = 16, ["CalendarViewEventCloseButton"] = 4, ["CalendarCreateEventFrame"] = 30, ["CalendarCreateEventInviteListSection"] = 8, ["CalendarTexturePickerFrame"] = 28, ["CalendarTexturePickerTitleFrame"] = 16, ["CalendarCreateEventCloseButton"] = 4, ["CalendarViewRaidFrame"] = 14, ["CalendarViewRaidCloseButton"] = 4, ["CalendarViewRaidTitleFrame"] = 16, ["CalendarFilterFrame"] = 4})
			StartSkinning(ScanDropdowns(CalendarCreateEventFrame, 18), 20, -12, 0, 6, true)
			StartSkinning({["CalendarFrame"] = 62}, nil, nil, nil, -10)
			for i = 1, 42 do
				_G["CalendarDayButton"..i]:DisableDrawLayer("BACKGROUND")
				CreateBG(_G["CalendarDayButton"..i])
			end
			CalendarCreateEventFrame:SetPoint("TOPLEFT", CalendarFrame, "TOPRIGHT", 4, 0)
			CalendarViewEventInviteListSection:DisableDrawLayer("BORDER")
			CalendarClassTotalsButton:DisableDrawLayer("BACKGROUND")
			CalendarTexturePickerScrollBar:DisableDrawLayer("ARTWORK")
			CalendarCreateEventInviteEdit:DisableDrawLayer("BACKGROUND")
			CalendarCreateEventTitleEdit:DisableDrawLayer("BACKGROUND")
			CalendarViewEventInviteListScrollFrameScrollBar:DisableDrawLayer("ARTWORK")
			for i = 1, 11 do
				_G["CalendarClassButton"..i]:DisableDrawLayer("BACKGROUND")
			end
			CreateBG(CalendarCreateEventInviteList)
			CreateBG(CalendarCreateEventDescriptionContainer)
			CreateBG(CalendarCreateEventTitleEdit, -4)
			CreateBG(CalendarTexturePickerFrame, nil, nil, 4)
			CreateBG(CalendarCreateEventInviteEdit, -4, -8)
			CreateBG(CalendarViewEventInviteList)
			CreateBG(CalendarContextMenu)
			CreateBG(CalendarFilterFrame, 20, 6, 0, 0, true)
			CreateBG(CalendarViewEventDescriptionContainer)
			CreateBG(CalendarViewHolidayFrame)
			
			SkinArrows({"CalendarFilter"})
			CalendarViewHolidayFrame:ClearAllPoints()
			CalendarViewHolidayFrame:SetPoint("TOPLEFT", CalendarFrame, "TOPRIGHT", 3, 0)			
			select(6, CalendarCreateEventRaidInviteButton:GetRegions()):Hide()	
			CalendarContextMenu:SetBackdrop(nil)
			CalendarCreateEventCreateButtonBorder:Hide()
			CalendarCreateEventMassInviteButtonBorder:Hide()			
			CalendarTexturePickerAcceptButtonBorder:Hide()
			CalendarTexturePickerCancelButtonBorder:Hide()
			CalendarCloseButton:ClearAllPoints()
			CalendarCloseButton:SetPoint("TOPRIGHT", CalendarFrame, "TOPRIGHT", 4, 4)
			CalendarViewEventFrame:SetPoint("TOPLEFT", CalendarFrame, "TOPRIGHT", 2, 0)	
			
		elseif ( name == "Blizzard_GuildUI" ) then
			MakeMovable(GuildFrame)
			if (MayronUI.db.profile.ReskinUI["Guild Frame"]) then return end
			StartSkinning({
				["GuildFrame"] = 46, ["GuildFrameBottomInset"] = 14, ["GuildNewsFiltersFrame"] = 14, ["GuildRewardsFrame"] = 4, ["GuildInfoFrameInfo"] = 20, ["GuildTextEditFrame"] = 14, ["GuildMemberDetailFrame"] = 62, ["GuildMemberOfficerNoteBackground"] = 14, ["GuildLogFrame"] = 14, ["GuildLogContainer"] = 12, ["GuildLogScrollFrameScrollBar"] = 4,
				{{["GuildMemberRankDropdown"] = 18}, 20, -8, 0, 4, true},
				{{["GuildRosterViewDropdown"] = 18}, 30, -10, 0, 5, true},
				{{["GuildMemberNoteBackground"] = 18}, 0, 0, -2, 0},
			})
			SkinTabs({"GuildFrameTab1", "GuildFrameTab2", "GuildFrameTab3", "GuildFrameTab4", "GuildFrameTab5"})
			SkinArrows({"GuildMemberRankDropdown", "GuildRosterViewDropdown"})
			GuildInfoDetailsFrameScrollBar:DisableDrawLayer("BACKGROUND")
			GuildRecruitmentListGuildButton:DisableDrawLayer("BORDER")
			GuildLogFrame:SetPoint("TOPLEFT", GuildFrame, "TOPRIGHT", 2, 0)
			for i = 1, 3 do
				_G["GuildInfoFrameTab"..i.."LeftDisabled"]:SetTexture("")
				_G["GuildInfoFrameTab"..i.."RightDisabled"]:SetTexture("")
				_G["GuildInfoFrameTab"..i.."MiddleDisabled"]:SetTexture("")
				_G["GuildInfoFrameTab"..i.."Left"]:SetTexture("")
				_G["GuildInfoFrameTab"..i.."Right"]:SetTexture("")
				_G["GuildInfoFrameTab"..i.."Middle"]:SetTexture("")
			end			
			GuildFactionBar.progress:SetTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
			GuildFactionBar.progress:SetVertexColor(0.07, 0.78, 0.17)		
			GuildMemberDetailFrame:ClearAllPoints()
			GuildMemberDetailFrame:SetPoint("TOPLEFT", GuildFrame, "TOPRIGHT", 2, 0)
			GuildTextEditFrame:ClearAllPoints()
			GuildTextEditFrame:SetPoint("TOPLEFT", GuildFrame, "TOPRIGHT", 2, 0)			
			GuildFrameTabardBackground:Hide()
			GuildFrameTabardBorder:Hide()
			GuildFrameTabardEmblem:SetSize(GuildFrameTabardEmblem:GetWidth() + 15, GuildFrameTabardEmblem:GetHeight() + 15)
			GuildFactionBar:DisableDrawLayer("BORDER")
			GuildFrameInset:Hide()
			GuildFrame.TitleMouseover:EnableMouse(false)
			GuildRosterContainer:EnableMouse(0)			
			
			-- Perks:			
			for _, btn in pairs(GuildPerksContainer.buttons) do
				SkinIcons(btn.icon, 1)
				for i = 1, 4 do
					select(i, btn:GetRegions()):SetAlpha(0)
				end
			end		
			GuildAllPerksFrame:DisableDrawLayer("BACKGROUND") 				
			GuildPerksContainerScrollBar:DisableDrawLayer("BACKGROUND")
			GuildRosterContainerScrollBar:DisableDrawLayer("ARTWORK")
			
			GuildPointFrame:SetScript("OnMouseUp", function()
				if (not _G["AchievementFrame"] or (not AchievementFrame:IsShown())) then
					if (not IsAddOnLoaded("Blizzard_AchievementUI")) then
						LoadAddOn("Blizzard_AchievementUI")
					end
					AchievementFrame_ToggleAchievementFrame()
					AchievementFrameTab_OnClick(2)
				end
			end)
			
			for i = 1, 4 do
				_G["GuildRosterColumnButton"..i]:DisableDrawLayer("BACKGROUND")
				CreateBG(_G["GuildRosterColumnButton"..i], 0, -1, 0, 1)
			end
			GuildNewsFrame:DisableDrawLayer("BACKGROUND") 
			for i = 1, 18 do
				_G["GuildNewsContainerButton"..i]:DisableDrawLayer("BACKGROUND")
			end
			GuildNewsFiltersFrame:SetPoint("TOPLEFT", GuildFrame, "TOPRIGHT", 2, 0)
			GuildNewsContainerScrollBar:DisableDrawLayer("BACKGROUND")
			GuildRewardsContainerScrollBar:DisableDrawLayer("ARTWORK")
			for id, btn in pairs(GuildRewardsContainer.buttons) do
				SkinIcons(btn, 1)
				select(7, btn:GetRegions()):Hide()
				btn:SetHighlightTexture("")
				btn:SetNormalTexture("")
				btn.disabledBG:SetVertexColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
			end	
			GuildTextEditContainer:SetBackdrop(nil)
			CreateBG(GuildTextEditContainer)
			GuildTextEditScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")
			GuildAddMemberButton:DisableDrawLayer("BORDER")
			GuildControlButton:DisableDrawLayer("BORDER")
			GuildRecruitmentInviteButton:DisableDrawLayer("BORDER")
			GuildRecruitmentDeclineButton:DisableDrawLayer("BORDER")			
			hooksecurefunc("GuildRoster_Update", function()
				if (GuildMemberRankDropdown:IsShown()) then GuildMemberDetailRankText:Hide() else GuildMemberDetailRankText:Show() end
			end)			
		elseif ( name == "Blizzard_TradeSkillUI" ) then
			MakeMovable(TradeSkillFrame)
			if (MayronUI.db.profile.ReskinUI["Trade Skill Frame"]) then return end
			StartSkinning({["TradeSkillFrame"] = 62, ["TradeSkillFrameInset"] = 12, ["TradeSkillDetailScrollFrame"] = 4, ["TradeSkillListScrollFrame"] = 4, ["TradeSkillCreateAllButton"] = 8, ["TradeSkillCreateButton"] = 8, ["TradeSkillCancelButton"] = 8, ["TradeSkillExpandButtonFrame"] = 4})
			TradeSkillFrameSearchBox:DisableDrawLayer("BACKGROUND")	
			CreateBG(TradeSkillFrameSearchBox, -4)
			TradeSkillInputBox:DisableDrawLayer("BACKGROUND")
			CreateBG(TradeSkillInputBox, -4, -4, 0, 0)				
			TradeSkillRankFrameBorder:Hide()
			TradeSkillRankFrameBackground:Hide()			
			TradeSkillRankFrame:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
			CreateBG(TradeSkillRankFrame, -1, 1, 1, -1)
			for i = 1, MAX_TRADE_SKILL_REAGENTS do
				_G["TradeSkillReagent"..i.."NameFrame"]:SetAlpha(0)
				SkinIcons(_G["TradeSkillReagent"..i.."IconTexture"], 1)
				_G["TradeSkillReagent"..i.."IconTexture"]:SetSize(36, 36)				
			end
			CreateBG(TradeSkillDetailScrollFrame, nil, nil, 2, -2)
			local p, r, rp, x, y = TradeSkillReagent1:GetPoint()
			TradeSkillReagent1:SetPoint(p, r, rp, x+4, y)
			p, r, rp, x, y = nil
			TradeSkillDetailHeaderLeft:Hide()
			select(6, TradeSkillDetailScrollChildFrame:GetRegions()):Hide()
			
			hooksecurefunc("TradeSkillFrame_SetSelection", function()
				SkinIcons(TradeSkillSkillIcon, 1)		
				local index = GetTradeSkillSelectionIndex()
				local link = GetTradeSkillItemLink(index)
				if (link) then
					local quality = select(3, GetItemInfo(link))			
					if (quality) then 
						local r, g, b = GetItemQualityColor(quality)
						TradeSkillSkillIcon.new:SetVertexColor(r, g, b)
					end
				end
				for i = 1, (GetTradeSkillNumReagents(index)) do
					local link = GetTradeSkillReagentItemLink( index, i)
					if (link) then
						local quality = select(3, GetItemInfo(link))			
						if (quality) then 
							local r, g, b = GetItemQualityColor(quality)
							_G["TradeSkillReagent"..i].new:SetVertexColor(r, g, b)
						end
					end
				end
			end)
		elseif ( name == "Blizzard_EncounterJournal" ) then
			MakeMovable(EncounterJournal)
			if (MayronUI.db.profile.ReskinUI["Encounter Journal"]) then return end
			StartSkinning({["EncounterJournal"] = 46, ["EncounterJournalInset"] = 12, ["EncounterJournalInstanceSelect"] = 4, ["EncounterJournalNavBar"] = 12, ["EncounterJournalNavBarOverlay"] = 32, ["EncounterJournalSearchBox"] = 132, ["EncounterJournalNavBarHomeButton"] = 48, ["EncounterJournalEncounterFrameInfoModelFrame"] = 36, ["EncounterJournalEncounterFrameInfoLootTab"] = 80, ["EncounterJournalEncounterFrameInfoBossTab"] = 80, ["EncounterJournalEncounterFrameInfoModelTab"] = 80})

			-- Search Results Box:
			SkinFrame(EncounterJournalSearchResults, 12)
			CreateBG(EncounterJournalSearchResults)
			SkinFrame(EncounterJournalSearchResultsScrollFrameScrollBar, 20)
			for i = 1, 9 do
				local btn = _G["EncounterJournalSearchResultsScrollFrameButton"..i]
				if (btn) then
					SkinButton(btn)
					_G["EncounterJournalSearchResultsScrollFrameButton"..i.."IconFrame"]:Hide()
					SkinIcons(btn.icon, 1)
					btn:SetHighlightTexture("")
				end
			end
			hooksecurefunc("EncounterJournal_GetSearchDisplay", function()
				for i = 1, 10 do
					local btn = _G["EncounterJournalSearchResultsScrollFrameButton"..i]
					if (btn) then						
						btn.icon:SetTexCoord(0.2, 0.8, 0.2, 0.8)
					end
				end
			end)
			for i = 1, 5 do
				_G["EncounterJournalSearchBoxSearchButton"..i]:SetNormalTexture("")
				_G["EncounterJournalSearchBoxSearchButton"..i]:SetPushedTexture("")				
			end
			EncounterJournalSearchBoxShowALL:DisableDrawLayer("ARTWORK")
			EncounterJournalSearchBoxSearchButton1BottomBorder:SetAlpha(0)
			EncounterJournalSearchBoxSearchButton1LeftBorder:SetAlpha(0)
			EncounterJournalSearchBoxSearchButton1RightBorder:SetAlpha(0)
			EncounterJournalSearchBoxSearchButton1BotRightCorner:SetAlpha(0)
			EncounterJournalSearchBoxSearchButton1BotLeftCorner:SetAlpha(0)
			CreateBG(EncounterJournalSearchBoxSearchButton1, -4, 5, 1, nil, nil, nil, true)
			EncounterJournalSearchBoxSearchButton1.new:SetPoint("BOTTOMRIGHT", EncounterJournalSearchBoxShowALL, "BOTTOMRIGHT", 5, 0)
			EncounterJournalSearchBox:HookScript("OnTextChanged", function()
				for i = 1, 5 do
					local btn = _G["EncounterJournalSearchBoxSearchButton"..i]
					if (btn and btn:IsVisible()) then
						_G["EncounterJournalSearchBoxSearchButton"..i.."IconFrame"]:Hide()
						SkinIcons(btn.icon, 1)
						btn.icon:SetTexCoord(0.2, 0.8, 0.2, 0.8)
						btn:SetHighlightTexture("Interface\\EncounterJournal\\UI-EncounterJournalTextures")
					end
				end
			end)
			
			-- Encounter Instance buttons:
			local function Reskin_ListInstances()
				for i = 1, 100 do	
					local str
					if ( i == 1) then
						str = "EncounterJournalInstanceSelectScrollFrameScrollChildInstanceButton"
					else
						str = "EncounterJournalInstanceSelectScrollFrameinstance"
					end
					if (not _G[str..i]) then break end
					local f = _G[str..i]
					if (not f.reskinned) then			
						f:DisableDrawLayer("ARTWORK")
						SkinButton(f)
						f = _G[f:GetName().."bgImage"]					
						f:SetDrawLayer("BORDER")
						f:SetPoint("TOPLEFT", 1, -1)
						f:SetPoint("BOTTOMRIGHT", 5, 1)
						f:SetTexCoord(0.1, 0.68359375, 0.1, 0.6421875)
						f:SetAlpha(0.6)
					end
				end
			end
			Reskin_ListInstances()
			hooksecurefunc("EncounterJournal_ListInstances", Reskin_ListInstances)
			
			EncounterJournalInstanceSelectTier:SetFontObject(GameFontNormalLarge)
			ChangeTextColor(EncounterJournalInstanceSelectTier, "white")			
			
			EncounterJournalInstanceSelectTier:SetPoint("TOPLEFT", 16, -24)
			EncounterJournalEncounterFrameInfoModelFrameImageTile:SetFontObject(GameFontNormalLarge)
			ChangeTextColor(EncounterJournalEncounterFrameInfoModelFrameImageTile, "white")
			EncounterJournalEncounterFrameInfoModelFrameImageTile:SetPoint("BOTTOM", 0, 10)
			
			CreateBG(EncounterJournalSearchBox, -4, nil, 2, nil, nil, nil, true)
			EncounterJournalInstanceSelectScrollDownButton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
			EncounterJournalInstanceSelectScrollDownButton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")		
			EncounterJournalInstanceSelectDungeonTabMid:Hide()
			EncounterJournalInstanceSelectDungeonTabLeft:Hide()
			EncounterJournalInstanceSelectDungeonTabRight:Hide()
			EncounterJournalInstanceSelectRaidTabMid:Hide()
			EncounterJournalInstanceSelectRaidTabLeft:Hide()
			EncounterJournalInstanceSelectRaidTabRight:Hide()
			EncounterJournalInstanceSelectDungeonTab:DisableDrawLayer("OVERLAY")
			EncounterJournalInstanceSelectRaidTab:DisableDrawLayer("OVERLAY")
			
			EncounterJournalEncounterFrameInfoBossesScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")	
			EncounterJournalInstanceSelectScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")
			EncounterJournalEncounterFrameInfoDetailsScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")
			SkinButton(EncounterJournalNavBarHomeButton)
			
			for i = 8, 10 do
				select(i, EncounterJournalInstanceSelectDungeonTab:GetRegions()):SetAlpha(0)
				select(i, EncounterJournalInstanceSelectRaidTab:GetRegions()):SetAlpha(0)
			end

			SkinButton(EncounterJournalEncounterFrameInfoDifficulty)
			SkinButton(EncounterJournalEncounterFrameInfoResetButton)
			SkinButton(EncounterJournalEncounterFrameInfoLootScrollFrameFilterToggle)
			EncounterJournalEncounterFrameInfoLootScrollFrameFilterToggleUpLeft:SetAlpha(0)
			EncounterJournalEncounterFrameInfoLootScrollFrameFilterToggleUpRIGHT:SetAlpha(0)
			EncounterJournalEncounterFrameInfoLootScrollFrameFilterToggleDownLeft:SetAlpha(0)
			EncounterJournalEncounterFrameInfoLootScrollFrameFilterToggleDownRIGHT:SetAlpha(0)
			select(5, EncounterJournalEncounterFrameInfoLootScrollFrameFilterToggle:GetRegions()):Hide()
			select(6, EncounterJournalEncounterFrameInfoLootScrollFrameFilterToggle:GetRegions()):Hide()
			EncounterJournalNavBarHomeButton:GetFontString():ClearAllPoints()
			EncounterJournalNavBarHomeButton:GetFontString():SetPoint("LEFT", EncounterJournalNavBarHomeButton, "LEFT", 8,0)
			EncounterJournalEncounterFrameInfoDifficultyUpLeft:SetAlpha(0)
			EncounterJournalEncounterFrameInfoDifficultyUpRIGHT:SetAlpha(0)
			EncounterJournalEncounterFrameInfoDifficultyDownLeft:SetAlpha(0)
			EncounterJournalEncounterFrameInfoDifficultyDownRIGHT:SetAlpha(0)
			select(5, EncounterJournalEncounterFrameInfoDifficulty:GetRegions()):Hide()
			select(6, EncounterJournalEncounterFrameInfoDifficulty:GetRegions()):Hide()		
			EncounterJournalEncounterFrameInfo:DisableDrawLayer("BACKGROUND")
			EncounterJournalEncounterFrame.instance:DisableDrawLayer("ARTWORK")
			SkinFrame(EncounterJournalEncounterFrameInfoInstanceButton, 20)
			EncounterJournalEncounterFrameInfoInstanceButton:SetHighlightTexture("")
			EncounterJournalEncounterFrameInfoInstanceTitle:SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
			EncounterJournalEncounterFrameInfoInstanceTitle:SetPoint("TOPLEFT", EncounterJournalEncounterFrameInfo, "TOPLEFT", 24, -20)
			EncounterJournalEncounterFrameInfoInstanceButton:ClearAllPoints()
			EncounterJournalEncounterFrameInfoInstanceButton:SetAllPoints(EncounterJournalEncounterFrameInfoInstanceTitle)
			EncounterJournalEncounterFrameInfoInstanceButton:SetScript("OnEnter", function()
				EncounterJournalEncounterFrameInfoInstanceTitle:SetTextColor(1,1,1)
			end)
			EncounterJournalEncounterFrameInfoInstanceButton:SetScript("OnLeave", function()
				EncounterJournalEncounterFrameInfoInstanceTitle:SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
			end)
			EncounterJournalNavBarOverflowButton.arrow = EncounterJournalNavBarOverflowButton:CreateFontString(nil, "ARTWORK")
			EncounterJournalNavBarOverflowButton.arrow:SetFontObject("GameFontHighlightLarge")
			EncounterJournalNavBarOverflowButton.arrow:SetText("<")
			EncounterJournalNavBarOverflowButton.arrow:SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
			EncounterJournalNavBarOverflowButton.arrow:SetAllPoints(EncounterJournalNavBarOverflowButton)
			EncounterJournalNavBarOverflowButton:SetScript("OnEnter", function()
				EncounterJournalNavBarOverflowButton.arrow:SetTextColor(1,1,1)
			end)
			EncounterJournalNavBarOverflowButton:SetScript("OnLeave", function()
				EncounterJournalNavBarOverflowButton.arrow:SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b)
			end)
			EncounterJournalNavBarOverflowButton:SetPushedTexture("")
			EncounterJournalNavBarOverflowButton:SetNormalTexture("")
			EncounterJournalNavBarOverflowButton:SetHighlightTexture("")
			SkinButton(EncounterJournalNavBarOverflowButton)
			EncounterJournalEncounterFrameInstanceFrameLoreScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")
			EncounterJournalEncounterFrameInfoLootScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")
			
			local function MoveInfoFrame()		
				if (EncounterJournalEncounterFrame.instance.loreBG:IsVisible()) then
					EncounterJournalEncounterFrameInfoDifficulty:ClearAllPoints()
					EncounterJournalEncounterFrameInfoDifficulty:SetPoint("BOTTOMRIGHT", EncounterJournalEncounterFrame.instance.loreBG, "BOTTOMRIGHT", -15, 2)
				else
					EncounterJournalEncounterFrameInfoDifficulty:ClearAllPoints()
					EncounterJournalEncounterFrameInfoDifficulty:SetPoint("TOPRIGHT", EncounterJournalEncounterFrameInfo, "TOPRIGHT", -19, -13)
				end
			end
			EncounterJournalEncounterFrameInfoLootTab:HookScript("OnClick", MoveInfoFrame)
			
			local function UpdateEncounterJournalMenus()
				MoveInfoFrame()	
				if (_G["EncounterJournalNavBarButton2"] and EncounterJournalNavBarButton2:IsVisible()) then
					EncounterJournalNavBarButton2:ClearAllPoints()
					EncounterJournalNavBarButton2:SetPoint("LEFT", EncounterJournalNavBarHomeButton, "RIGHT", 4, 0)
					for i = 1, 10 do
						if (_G["EncounterJournalBossButton"..i]) then
							SkinButton(_G["EncounterJournalBossButton"..i])	
							_G["EncounterJournalBossButton"..i.."Creature"]:SetTexCoord(0, 1, 0, 0.9)	
							_G["EncounterJournalBossButton"..i.."Creature"]:SetPoint("TOPLEFT", _G["EncounterJournalBossButton"..i], "TOPLEFT", 0, 11)
						end
					end
					if (not EncounterJournalNavBarButton2.reskinned) then
						
						SkinButton(EncounterJournalNavBarButton2, true)
						EncounterJournalNavBarButton2:GetFontString():ClearAllPoints()
						EncounterJournalNavBarButton2:GetFontString():SetPoint("LEFT", EncounterJournalNavBarButton2, "LEFT", 8,0)
						EncounterJournalNavBarButton2.MenuArrowButton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
						EncounterJournalNavBarButton2.MenuArrowButton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
						EncounterJournalNavBarButton2.MenuArrowButton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Disabled")
					end
				end				
				if (_G["EncounterJournalNavBarButton3"] and EncounterJournalNavBarButton3:IsVisible()) then					
					EncounterJournalNavBarButton3:ClearAllPoints()
					if (_G["EncounterJournalNavBarButton2"] and EncounterJournalNavBarButton2:IsVisible()) then  
						EncounterJournalNavBarButton3:SetPoint("LEFT", EncounterJournalNavBarButton2, "RIGHT", 4, 0)
					else
						EncounterJournalNavBarButton3:SetPoint("LEFT", EncounterJournalNavBarHomeButton, "RIGHT", 4, 0)
					end
					if (not EncounterJournalNavBarButton3.reskinned) then
						SkinButton(EncounterJournalNavBarButton3, true)
						EncounterJournalNavBarButton3:GetFontString():ClearAllPoints()
						EncounterJournalNavBarButton3:GetFontString():SetPoint("LEFT", EncounterJournalNavBarButton3, "LEFT", 8,0)
						EncounterJournalNavBarButton3.MenuArrowButton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
						EncounterJournalNavBarButton3.MenuArrowButton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
						EncounterJournalNavBarButton3.MenuArrowButton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Disabled")
					end
				end
			end
			hooksecurefunc("EncounterJournal_DisplayInstance", UpdateEncounterJournalMenus)			
			hooksecurefunc("EncounterJournal_DisplayEncounter", UpdateEncounterJournalMenus)			
			hooksecurefunc("NavBar_Reset", UpdateEncounterJournalMenus)
				
			for i = 1, 20 do
				if not (_G["EncounterJournalEncounterFrameInfoLootScrollFrameButton"..i]) then break end
				SkinFrame(_G["EncounterJournalEncounterFrameInfoLootScrollFrameButton"..i], 8)
				ChangeTextColor(_G["EncounterJournalEncounterFrameInfoLootScrollFrameButton"..i.."Slot"], "white")
				ChangeTextColor(_G["EncounterJournalEncounterFrameInfoLootScrollFrameButton"..i.."ArmorClass"], "white")
				ChangeTextColor(_G["EncounterJournalEncounterFrameInfoLootScrollFrameButton"..i.."Boss"], "yellow")
				SkinIcons(_G["EncounterJournalEncounterFrameInfoLootScrollFrameButton"..i.."Icon"], 1)
			end
			
			local tabs = {"Overview", "Loot", "Boss", "Model"}
			for id, str in ipairs(tabs) do
				local tab = _G["EncounterJournalEncounterFrameInfo"..str.."Tab"]
				local icon = select(5, tab:GetRegions()) -- Select / Unselect
				icon:ClearAllPoints()
				icon:SetPoint("CENTER", tab, "CENTER", 0, 0)				
				tab:SetSize(45, 45)
				if (not tab) then break end
				SkinButton(tab)
				tab:ClearAllPoints()
				if ( id == 1) then					
					tab:SetPoint("TOPLEFT", EncounterJournal, "TOPRIGHT", 3, 0)
				else
					tab:SetPoint("TOPLEFT", _G["EncounterJournalEncounterFrameInfo"..tabs[id- 1].."Tab"], "BOTTOMLEFT", 0, -6)
				end
			end
			tabs = nil
			
			EncounterJournalEncounterFrame.instance.loreBG:SetTexCoord(0.2, 0.7, 0.08, 0.5)	
			EncounterJournalEncounterFrame.instance.loreBG:ClearAllPoints()
			EncounterJournalEncounterFrame.instance.loreBG:SetPoint("TOPLEFT", EncounterJournalEncounterFrame.instance, "TOPLEFT", 0, 2)
			EncounterJournal.encounter.instance.loreScroll:ClearAllPoints()
			EncounterJournal.encounter.instance.loreScroll:SetPoint("TOPLEFT", EncounterJournalEncounterFrame.instance.loreBG, "BOTTOMLEFT", 0, -2)
			EncounterJournal.encounter.instance.loreScroll:SetPoint("BOTTOMRIGHT", EncounterJournalEncounterFrame.instance, "BOTTOMRIGHT", -24, 2)
			EncounterJournal.encounter.instance.loreScroll.child.lore:SetTextColor(1,1,1)
			EncounterJournal.encounter.instance.mapButton:DisableDrawLayer("BACKGROUND")
			ChangeTextColor(EncounterJournal.encounter.instance.title, "white")
			
			local bg = CreateFrame("Frame", nil, EncounterJournal.encounter.instance, "MUI_ToolTipTemplate")	
			bg:SetPoint("TOPLEFT", EncounterJournal.encounter.instance, "TOPLEFT", -1, 3)
			bg:SetPoint("BOTTOMRIGHT", EncounterJournal.encounter.instance, "TOPRIGHT", 1, -50)
			bg:SetFrameStrata("MEDIUM")
			bg.background:SetVertexColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 0.6)				
			EncounterJournal.encounter.instance.title:ClearAllPoints()
			EncounterJournal.encounter.instance.title:SetPoint("CENTER", bg, "CENTER", 0, -8)
			EncounterJournal.encounter.instance.title:SetParent(bg)				
			
			CreateBG(EncounterJournal.encounter.instance, -1, 1, 3, -2)
			EncounterJournal.encounter.instance.mapButton:ClearAllPoints()
			EncounterJournal.encounter.instance.mapButton:SetPoint("BOTTOMLEFT", EncounterJournalEncounterFrame.instance.loreBG, "BOTTOMLEFT")
			EncounterJournalEncounterFrameInfoOverviewScrollFrameScrollBar:DisableDrawLayer("BACKGROUND")
			
			hooksecurefunc("EncounterJournal_SetBullets", function(object, description, hideBullets)
				local parent = object:GetParent()					
				if (parent.Bullets) then
					parent:DisableDrawLayer("BACKGROUND")
					for i = 1, #parent.Bullets do
						parent.Bullets[i].Text:SetTextColor(1,1,1)
					end
				end
			end)
			hooksecurefunc("EncounterJournal_ShowCreatures", function(object, description, hideBullets)
				local creatures = EncounterJournal.encounter.info.creatureButtons
				for i = 1, #creatures do
					if (i == 1) then
						creatures[i]:ClearAllPoints()
						creatures[i]:SetPoint("TOPLEFT", EncounterJournalEncounterFrameInfoModelFrame, "TOPLEFT", -2, 4)
					end
					creatures[i]:SetHighlightTexture("")
					creatures[i]:SetPushedTexture("")
					creatures[i]:SetNormalTexture("")
				end
			end)
			
			hooksecurefunc("EncounterJournal_ToggleHeaders", function()
				local index = 1
				local name = "EncounterJournalInfoHeader"..index
				local header = _G[name]
				
				for i = 1, 3 do
					SkinFrame(_G["EncounterJournalOverviewInfoHeader"..i.."HeaderButton"], 68)
				end
				EncounterJournalEncounterFrameInfoOverviewScrollFrameScrollChildHeader:Hide()
				EncounterJournalEncounterFrameInfoOverviewScrollFrameScrollChild.overviewDescription.Text:SetTextColor(1, 1, 1)
				EncounterJournalEncounterFrameInfoOverviewScrollFrameScrollChildLoreDescription:SetTextColor(1, 1, 1)
				
				while (header) do
					if (not header.reskinned) then
						header.reskinned = true
						EncounterJournal.encounter.info.detailsScroll.child.description:SetTextColor(1, 1, 1)
						header.description:SetTextColor(1, 1, 1)
						header.description:SetShadowOffset(1, -1)
						header.button.title:SetTextColor(1, 1, 1)
						header.button.title.SetTextColor = function() end
						header.button.expandedIcon:SetTextColor(1, 1, 1)
						header.button.expandedIcon.SetTextColor = function() end
						header.descriptionBG:SetAlpha(0)
						header.descriptionBGBottom:SetAlpha(0)
						header.button.abilityIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
						_G[name.."HeaderButtonELeftUp"]:SetAlpha(0)
						_G[name.."HeaderButtonERightUp"]:SetAlpha(0)
						_G[name.."HeaderButtonEMidUp"]:SetAlpha(0)
						_G[name.."HeaderButtonCLeftUp"]:SetAlpha(0)
						_G[name.."HeaderButtonCRightUp"]:SetAlpha(0)
						_G[name.."HeaderButtonCMidUp"]:SetAlpha(0)
						_G[name.."HeaderButtonELeftDown"]:SetAlpha(0)
						_G[name.."HeaderButtonERightDown"]:SetAlpha(0)
						_G[name.."HeaderButtonEMidDown"]:SetAlpha(0)
						_G[name.."HeaderButtonCLeftDown"]:SetAlpha(0)
						_G[name.."HeaderButtonCRightDown"]:SetAlpha(0)
						_G[name.."HeaderButtonCMidDown"]:SetAlpha(0)
						_G[name.."HeaderButtonHighlightLeft"]:Hide()
						_G[name.."HeaderButtonHighlightMid"]:Hide()
						_G[name.."HeaderButtonHighlightRight"]:Hide()
						header.button.bg = header.button:CreateTexture(nil, "BACKGROUND")
						header.button.bg:SetPoint("TOPLEFT", header.button.abilityIcon, -1, 1)
						header.button.bg:SetPoint("BOTTOMRIGHT", header.button.abilityIcon, 1, -1)
						header.button.bg:SetVertexColor(0, 0, 0)
					end

					if header.button.abilityIcon:IsShown() then
						header.button.bg:Show()
					else
						header.button.bg:Hide()
					end					
					index = index + 1
					name = "EncounterJournalInfoHeader"..index
					header = _G[name]					
				end
			end)				
		elseif ( name == "Blizzard_ArchaeologyUI" ) then
			if (MayronUI.db.profile.ReskinUI["Archaeology Frame"]) then return end
			StartSkinning({["ArchaeologyFrame"] = 47, ["ArchaeologyFrameInset"] = 12})	
			ChangeTextColor(ArchaeologyFrameSummaryPageTitle, "white")
			ChangeTextColor(ArchaeologyFrameArtifactPageHistoryTitle, "white")
			ChangeTextColor(ArchaeologyFrameArtifactPageHistoryScrollChildText, "white")
			ChangeTextColor(ArchaeologyFrameHelpPageTitle, "white")
			ChangeTextColor(ArchaeologyFrameHelpPageDigTitle, "white")
			ChangeTextColor(ArchaeologyFrameHelpPageHelpScrollHelpText, "white")
			ChangeTextColor(ArchaeologyFrameCompletedPage:GetRegions(), "white")
			ChangeTextColor(ArchaeologyFrameCompletedPageTitle, "white")
			ChangeTextColor(ArchaeologyFrameCompletedPageTitleTop, "white")
			ChangeTextColor(ArchaeologyFrameCompletedPageTitleMid, "white")
			ChangeTextColor(ArchaeologyFrameCompletedPagePageText, "white")
			ChangeTextColor(ArchaeologyFrameSummaryPage.pageText, "white")
			ArcheologyDigsiteProgressBar:ClearAllPoints()
			
			-- ArchaeologyFrameArtifactPage:
			SkinIcons(ArchaeologyFrameArtifactPage.icon, 1)
			ArchaeologyFrameRankBar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")	
			ArchaeologyFrameArtifactPageSolveFrameStatusBar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")	
			StartSkinning({["ArchaeologyFrameRankBar"] = 18, ["ArchaeologyFrameArtifactPageSolveFrameStatusBar"] = 6}, -1, 1, 1, -1)
			select(3, ArchaeologyFrameRankBar:GetRegions()):Hide() -- StatusBar Background			
			if (_G["MUI_Bottom"]) then
				ArcheologyDigsiteProgressBar:SetPoint("BOTTOM", MUI_Bottom, "TOP", 0, 200)
			else
				ArcheologyDigsiteProgressBar:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 300)
			end			
			StartSkinning({["ArchaeologyFrameRaceFilter"] = 18}, 16, -14, 0, 6, true)
			SkinArrows({"ArchaeologyFrameRaceFilter"})	
			for i = 1, ARCHAEOLOGY_MAX_RACES do
				local btn = _G["ArchaeologyFrameSummaryPageRace"..i]
				if (not btn) then break end
				btn:GetRegions():SetTextColor(1, 1, 1)
			end
			for i = 1, ARCHAEOLOGY_MAX_COMPLETED_SHOWN do
				local btn = _G["ArchaeologyFrameCompletedPageArtifact"..i]
				if (not btn) then break end
				SkinIcons(btn.icon, 1)
				ChangeTextColor(btn.artifactName, "yellow")
				ChangeTextColor(btn.artifactSubText, "white")
				btn.border:Hide()
				_G["ArchaeologyFrameCompletedPageArtifact"..i.."Bg"]:Hide()
			end
		elseif ( name == "Blizzard_PVPUI" ) then
			if (MayronUI.db.profile.ReskinUI["PVP Frame"]) then return end
			SkinFrame(HonorFrame.RoleInset, 12)
			SkinFrame(HonorFrame.BonusFrame, 12)
			HonorFrame.Inset:Hide()
			HonorFrame.BonusFrame.ShadowOverlay:Hide()
			HonorFrameSoloQueueButton:DisableDrawLayer("BORDER")
			HonorFrameGroupQueueButton:DisableDrawLayer("BORDER")
			
			StartSkinning({["HonorFrameTypeDropDown"] = 18}, 20, -12, 0, 6, true)
			SkinArrows({"HonorFrameTypeDropDown"})
			HonorFrameTypeDropDown.new:SetScript("OnClick", function() end) -- prevents blizzard taint
			
			HonorFrameSpecificFrameScrollBar:DisableDrawLayer("ARTWORK")
			
			for i = 1, 9 do
				local btn = _G["HonorFrameSpecificFrameButton"..i]
				if (not btn) then break end
				select(1, btn:GetRegions()):Hide()
				btn.Border:Hide()
				SkinIcons(btn.Icon, 1)
				SkinButton(btn)
				btn:SetSize(300, 38)
				if (i ~= 1) then
					btn:SetPoint("TOPLEFT", _G["HonorFrameSpecificFrameButton"..(i-1)], "BOTTOMLEFT", 0, -2)
				else
					btn:SetPoint("TOPLEFT", HonorFrameSpecificFrameScrollChild, "TOPLEFT", 2, -1)
				end
			end
			
			for id, str in ipairs({"RandomBGButton", "Arena1Button", "Arena2Button"}) do
				SkinButton(HonorFrame.BonusFrame[str])
				HonorFrame.BonusFrame[str]:SetID(id)
				HonorFrame.BonusFrame[str].SelectedTexture:SetAlpha(0)
				HonorFrame.BonusFrame[str]:SetWidth(320)
			end	
			HonorFrame.BonusFrame.Arena2Button:ClearAllPoints()
			HonorFrame.BonusFrame.Arena2Button:SetPoint("BOTTOMLEFT", HonorFrame.BonusFrame, "BOTTOMLEFT", 0, 20)
			hooksecurefunc("HonorFrameBonusFrame_SelectButton", function(self)
				if (self.SelectedTexture:IsShown()) then
					self:SetBackdropBorderColor(1, 1, 1)
					self.oldBC = { 1, 1, 1 }
					for _, str in ipairs({"RandomBGButton", "Arena1Button", "Arena2Button"}) do
						local id = HonorFrame.BonusFrame[str]:GetID()
						if (self:GetID() ~= id) then
							HonorFrame.BonusFrame[str]:SetBackdropBorderColor(0, 0, 0)
							HonorFrame.BonusFrame[str].oldBC = { 0, 0, 0 }
						end
					end
				end			
			end)
			for i = 1, 4 do
				local btn = _G["PVPQueueFrameCategoryButton"..i]
				if (not btn) then break end
				select(1, btn:GetRegions()):Hide()
				select(2, btn:GetRegions()):Hide()
				select(4, btn:GetRegions()):Hide()
				SkinIcons(btn.Icon, 1)
				SkinButton(btn)
				btn.Icon:SetTexCoord(0.15, 0.87, 0.15, 0.87)
				btn.Icon:SetSize(52, 52)
				btn.Icon:ClearAllPoints()
				btn.Icon:SetPoint("TOPLEFT", btn, "TOPLEFT", 4, -4)
				if (i == 1) then
					btn:SetBackdropBorderColor(1, 1, 1)
					btn.oldBC = { 1, 1, 1 }
				end
				btn:HookScript("OnClick", function(self)
					self:SetBackdropBorderColor(1, 1, 1)
					self.oldBC = { 1, 1, 1 }
					local selectedID = self:GetID()
					for id = 1, 4 do
						local btn = _G["PVPQueueFrameCategoryButton"..id]
						if (not btn) then break end 
						if (id ~= selectedID) then
							btn:SetBackdropBorderColor(0, 0, 0)
							btn.oldBC = { 0, 0, 0 }
						end
					end
				end)
			end
			
			-- ConquestFrame:
			ConquestFrame.Inset:Hide()
			ConquestFrame.ShadowOverlay:Hide()
			ConquestFrame.NoSeason:SetFrameStrata("HIGH")
			SkinFrame(ConquestFrame, 12)
			ConquestJoinButton:DisableDrawLayer("BORDER")		   
			for id, str in ipairs({"RatedBG", "Arena2v2", "Arena3v3", "Arena5v5"}) do
				SkinButton(ConquestFrame[str])
				ConquestFrame[str]:SetID(id)
				ConquestFrame[str].SelectedTexture:SetAlpha(0)
				ConquestFrame[str]:SetSize(320, 50)
			end	
			ConquestFrame.Arena5v5:ClearAllPoints()
			ConquestFrame.Arena5v5:SetPoint("BOTTOMLEFT", ConquestFrame, "BOTTOMLEFT", 8, 50)
			ConquestFrame.Arena3v3:ClearAllPoints()
			ConquestFrame.Arena3v3:SetPoint("BOTTOM", ConquestFrame.Arena5v5, "TOP", 0, 3)
			ConquestFrame.Arena2v2:ClearAllPoints()
			ConquestFrame.Arena2v2:SetPoint("BOTTOM", ConquestFrame.Arena3v3, "TOP", 0, 3)
			ConquestFrame.RatedBG:SetPoint("TOP", ConquestFrame.RatedBGHeader, "BOTTOM", -4, 15)
			
			hooksecurefunc("ConquestFrame_SelectButton", function(self)
				self:SetBackdropBorderColor(1, 1, 1)
				self.oldBC = { 1, 1, 1 }
				local selectedID = self:GetID()
				for id, str in ipairs({"RatedBG", "Arena2v2", "Arena3v3", "Arena5v5"}) do					
					if (not ConquestFrame[str]) then break end 
					if (id ~= selectedID) then
						ConquestFrame[str]:SetBackdropBorderColor(0, 0, 0)
						ConquestFrame[str].oldBC = { 0, 0, 0 }
					end
				end
			end)
			
			-- War Games:
			WarGamesFrame.RightInset:Hide()
			WarGameStartButton:DisableDrawLayer("BORDER")
			SkinFrame(WarGamesFrameInfoScrollFrameScrollBar, 20)
			WarGamesFrameScrollFrameScrollBar:DisableDrawLayer("ARTWORK")
			WarGamesFrame.HorizontalBar:Hide()
			WarGamesFrame:DisableDrawLayer("BACKGROUND")			
			for i = 1, 12 do
				local btn = _G["WarGamesFrameScrollFrameButton"..i]
				if (not btn) then break end
				btn.Entry.Border:Hide()
				btn.Entry.HighlightTexture:Hide()
				btn.Entry.Bg:Hide()
				SkinIcons(btn.Entry.Icon, 1)
				SkinButton(btn.Entry)
				btn.Entry:SetSize(300, 38)
			end

		elseif ( name == "Blizzard_AchievementUI" ) then
			MakeMovable(AchievementFrame)
			if (MayronUI.db.profile.ReskinUI["Achievement Frame"]) then return end
			StartSkinning({["AchievementFrame"] = 56, ["AchievementFrameHeader"] = 12, ["AchievementFrameCategories"] = 8, ["AchievementFrameSummary"] = 4, ["AchievementFrameStatsContainerScrollBar"] = 20, ["AchievementFrameAchievementsContainerScrollBar"] = 20, ["AchievementFrameCategoriesContainerScrollBar"] = 20, {{["AchievementFrameFilterDropDown"] = 18}, -6, nil, 0, 6, true}})
			
			-- AchievementFrame StatusBars:
			local function SkinStatusBar(bar)
				StartSkinning({[bar:GetName()] = 166}, -1, 1, 1, -1)				
				bar:SetHeight(22)			
				bar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")	
				bar:SetStatusBarColor(0.3, 0.7, 0.3)
				if (bar.text) then
					local p, rf, rp, x = bar.text:GetPoint()
					bar.text:SetPoint(p, rf, rp, x, 0)
				end
				if (bar.label) then
					local p, rf, rp, x = bar.label:GetPoint()
					bar.label:SetPoint(p, rf, rp, x, 0)
				end
			end
			SkinStatusBar(AchievementFrameSummaryCategoriesStatusBar)
			for i = 1, 12 do
				local bar = _G["AchievementFrameSummaryCategoriesCategory"..i]
				if (not bar) then break end
				SkinStatusBar(bar)
				_G["AchievementFrameSummaryCategoriesCategory"..i.."ButtonHighlight"]:SetAlpha(0)				
			end			
			if (_G["AchievementFrameSummaryCategoriesStatusBar"]) then
				local p, rf, rp, x = AchievementFrameSummaryCategoriesStatusBarTitle:GetPoint()				
				AchievementFrameSummaryCategoriesStatusBarTitle:SetPoint(p, rf, rp, x, 0)
				p, rf, rp, x = AchievementFrameSummaryCategoriesStatusBarText:GetPoint()
				AchievementFrameSummaryCategoriesStatusBarText:SetPoint(p, rf, rp, x, 0)
				p, rf, rp, x = nil
			end
			
			AchievementFrameHeaderTitle:ClearAllPoints()
			AchievementFrameHeaderTitle:SetPoint("TOPRIGHT", AchievementFrame, "TOP", 80, 0)
			AchievementFrameHeaderPoints:ClearAllPoints()
			AchievementFrameHeaderPoints:SetPoint("TOPLEFT", AchievementFrame, "TOP", 80, 0)
			AchievementFrameCloseButton:ClearAllPoints()
			
			-- tabs:
			hooksecurefunc("AchievementFrame_UpdateTabs", function(clickedTab)	
				for n = 1, 3 do
					local self = _G["AchievementFrameTab"..n]
					if ( n ~= clickedTab ) then
						self:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-Active.tga")
						self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)
						self:GetNormalTexture():SetPoint("BOTTOM", self, "BOTTOM", 0, -30)
						self:GetNormalTexture():SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
						self.text:SetPoint("CENTER", 0, 0)
						self.selected = nil
					else -- selected:
						self:Enable()
						self:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-InActive.tga")
						self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.5, C["Tab_Color"].g * 0.5, C["Tab_Color"].b * 0.5, 0.8)
						self:GetNormalTexture():SetPoint("BOTTOM", self, "BOTTOM", 0, -10)
						self:GetNormalTexture():SetPoint("TOPLEFT", self, "TOPLEFT", 0, -2)
						self.text:SetPoint("CENTER", 0, -4)
						self.selected = true
						self:Disable()
					end
				end	
			end)
			
			hooksecurefunc("AchievementFrame_SetTabs", function()
				for i = 1, 3 do
					local tab = _G["AchievementFrameTab"..i]					
					tab:DisableDrawLayer("BACKGROUND")	
					local newLvl = tab:GetParent():GetFrameLevel() - 10
					tab:SetFrameLevel( ((newLvl >= 0) and newLvl) or 0 )					
					tab:HookScript("OnEnter", function(self)
						self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 1.4, C["Tab_Color"].g * 1.4, C["Tab_Color"].b * 1.4, 0.85)
					end)
					tab:HookScript("OnLeave", function(self)
						if ( self.selected ) then
							self:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-InActive.tga")
							self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.5, C["Tab_Color"].g * 0.5, C["Tab_Color"].b * 0.5, 0.8)	
						else
							self:SetNormalTexture(MUI_MEDIA.."Skin\\Tab-Active.tga")
							self:GetNormalTexture():SetVertexColor(C["Tab_Color"].r * 0.8, C["Tab_Color"].g * 0.8, C["Tab_Color"].b * 0.8, 0.85)
						end
					end)				
				end	
			end)
			
			local new = CreateBG(AchievementFrame, nil, nil, 14, -1)
			AchievementFrameCloseButton:SetPoint("TOPRIGHT", new, "TOPRIGHT", 4, 4)
			AchievementFrameHeader:EnableMouse(false)
			AchievementFrameCategoriesContainer:EnableMouse(false)			
			SkinArrows({"AchievementFrameFilterDropDown"})			
			AchievementFrameAchievementsBackground:SetAlpha(0)
			CreateBG(AchievementFrameStats, nil, -4)
			CreateBG(AchievementFrameAchievementsBackground)			
			select(3, AchievementFrameAchievements:GetRegions()):Hide()
			AchievementFrameStatsBG:Hide()
			select(2, AchievementFrameAchievements:GetChildren()):Hide()
			for i = 1, 17 do select(i, AchievementFrame:GetRegions()):Hide() end
			AchievementFrameSummary:GetChildren():Hide()
			AchievementFrameSummaryAchievementsHeaderHeader:Hide()
			AchievementFrameSummaryCategoriesHeaderTexture:Hide()
			select(3, AchievementFrameStats:GetChildren()):Hide()
			
			for i = 1, 20 do
				_G["AchievementFrameStatsContainerButton"..i]:DisableDrawLayer("OVERLAY")
				_G["AchievementFrameStatsContainerButton"..i.."HeaderLeft"]:SetAlpha(0)
				_G["AchievementFrameStatsContainerButton"..i.."BG"]:SetDrawLayer("OVERLAY")
				_G["AchievementFrameStatsContainerButton"..i.."HeaderMiddle"]:SetAlpha(0)
				_G["AchievementFrameStatsContainerButton"..i.."HeaderRight"]:SetAlpha(0)
			end
			select(5, AchievementFrameStatsContainerButton1:GetRegions()):SetDrawLayer("BORDER")
			
			hooksecurefunc("AchievementObjectives_DisplayCriteria", function(F, id)
				for i = 1, ( GetAchievementNumCriteria(id) ) do				
					local item = (_G["AchievementFrameMeta"..i] and _G["AchievementFrameMeta"..i].label) or _G["AchievementFrameCriteria"..i.."Name"]
					local value = (item and item:GetTextColor()) or 1
					if (value == 0) then
						item:SetTextColor(0.1, 0.9, 0.1)
					end
				end
			end)			
			hooksecurefunc("AchievementButton_DisplayAchievement", function(btn, c, a)
				for i = 1, 7 do
					local f = _G["AchievementFrameAchievementsContainerButton"..i]
					if (f.reskinned) then
						f:GetScript("OnLeave")(f)
						f:DisableDrawLayer("ARTWORK")						
						for i = 1, 7 do
							select(i, f:GetRegions()):SetDrawLayer("ARTWORK")
						end
						_G["AchievementFrameAchievementsContainerButton"..i.."Highlight"]:DisableDrawLayer("OVERLAY")						
						_G["AchievementFrameAchievementsContainerButton"..i.."IconOverlay"]:Hide()
						_G["AchievementFrameAchievementsContainerButton"..i.."Glow"]:Hide()
						ChangeTextColor(select(15, f:GetRegions()), "white")
						if (i > 1) then
							f:SetPoint("TOPLEFT", _G["AchievementFrameAchievementsContainerButton"..(i-1)], "BOTTOMLEFT", 0, -4)							
						end						
						local complete = select(4, GetAchievementInfo(c, a))
						if ( complete ) then
							if ( f.accountWide ) then
								f.label:SetTextColor(1, 1, 0.6)
							end
						end
					else
						SkinIcons(_G["AchievementFrameAchievementsContainerButton"..i.."IconTexture"])
						SkinButton(f)
					end
				end
			end)
			AchievementFrameAchievementsObjectives:SetParent(AchievementFrameAchievementsContainer)
			hooksecurefunc("AchievementFrameSummary_UpdateAchievements", function()
				for i = 1, 4 do					
					local f = _G["AchievementFrameSummaryAchievement"..i]
					SkinIcons(_G["AchievementFrameSummaryAchievement"..i.."IconTexture"])
					_G["AchievementFrameSummaryAchievement"..i.."IconOverlay"]:Hide()
					_G["AchievementFrameSummaryAchievement"..i.."Highlight"]:SetAlpha(0)
					SkinButton(f)
					_G["AchievementFrameSummaryAchievement"..i.."Background"]:Hide()
					ChangeTextColor(_G["AchievementFrameSummaryAchievement"..i.."Description"], "white")
					f:DisableDrawLayer("ARTWORK")
					if (i > 1) then
						f:SetPoint("TOPLEFT", _G["AchievementFrameSummaryAchievement"..(i-1)], "BOTTOMLEFT", 0, -2)
					end
					if (i == 4) then
						AchievementFrameSummaryCategories:SetPoint("TOPLEFT", AchievementFrameSummaryAchievements, "BOTTOMLEFT", 0, -8)
					end
					f:GetScript("OnLeave")(f)
				end
			end)
			local function Hook_AchievementFrameCategories_Update()
				for i = 1, 20 do
					local f = _G["AchievementFrameCategoriesContainerButton"..i]
					SkinButton(f)
					f:SetHeight(20)
					if (i > 1) then
						f:SetPoint("TOP", _G["AchievementFrameCategoriesContainerButton"..(i-1)], "BOTTOM", 0, -2)
					end
					f.background:SetTexture(0, 0, 0, 0)
					f:GetScript("OnLeave")(f)
				end
			end
			
			hooksecurefunc("AchievementFrameCategories_Update", function()
				Hook_AchievementFrameCategories_Update()
				Hook_AchievementFrameCategories_Update = function() end
			end)
			
			--> AchievementFrameComparison:
			SkinFrame(AchievementFrameComparison, 20)
			AchievementFrameComparisonHeader:DisableDrawLayer("ARTWORK")
			SkinFrame(AchievementFrameComparisonContainerScrollBar, 20)
			SkinFrame(AchievementFrameComparisonSummaryFriend, 12)
			SkinFrame(AchievementFrameComparisonSummaryPlayer, 12)
			select(5, AchievementFrameComparison:GetChildren()):Hide()
			AchievementFrameComparisonHeaderPortrait:Hide()
			AchievementFrameComparisonHeaderPortraitBg:Hide()
			SkinFrame(AchievementFrameComparisonStatsContainerScrollBar, 20)			
			SkinStatusBar(AchievementFrameComparisonSummaryPlayerStatusBar)
			SkinStatusBar(AchievementFrameComparisonSummaryFriendStatusBar)
			for i = 1, 20 do
				local btn = _G["AchievementFrameComparisonStatsContainerButton"..i]
				btn:DisableDrawLayer("OVERLAY")
				_G["AchievementFrameComparisonStatsContainerButton"..i.."HeaderLeft"]:SetAlpha(0)
				_G["AchievementFrameComparisonStatsContainerButton"..i.."BG"]:SetDrawLayer("OVERLAY")
				_G["AchievementFrameComparisonStatsContainerButton"..i.."HeaderMiddle"]:SetAlpha(0)
				_G["AchievementFrameComparisonStatsContainerButton"..i.."HeaderRight"]:SetAlpha(0)
				btn.left2:SetAlpha(0)
				btn.middle2:SetAlpha(0)
				btn.right2:SetAlpha(0)
			end
			
			for _, str in ipairs({"Player", "Friend"}) do
				for i = 1, 10 do
					local btn = _G["AchievementFrameComparisonContainerButton"..i..str]
					if (not btn) then break end
					btn["background"]:Hide()
					btn["glow"]:Hide()
					btn["titleBar"]:Hide()
					btn.icon.frame:Hide()
					SkinIcons(btn.icon.texture, 1)
					CreateBG(btn)
					if (str == "Player") then
						ChangeTextColor(btn.description, "white")
					end
					if (i ~= 1) then						
						btn = _G["AchievementFrameComparisonContainerButton"..i]
						btn:SetPoint("TOPLEFT", _G["AchievementFrameComparisonContainerButton"..(i-1)], "BOTTOMLEFT", 0, -2)
					end
				end
			end
		elseif ( name == "Blizzard_AuctionUI" ) then
			MakeMovable(AuctionFrame)
			if (MayronUI.db.profile.ReskinUI["Auction Frame"]) then return end
			StartSkinning({
				["AuctionFrame"] = 20,
				{{["BrowseName"] = 4, ["BrowseMinLevel"] = 4, ["BrowseMaxLevel"] = 4, ["BrowseBidPriceGold"] = 6, ["BidBidPriceGold"] = 6, ["StartPriceGold"] = 6, ["BuyoutPriceGold"] = 6}, -10, nil, 0, 0},	
				{{["BrowseBidPriceSilver"] = 6, ["BrowseBidPriceCopper"] = 6, ["BidBidPriceSilver"] = 6, ["BidBidPriceCopper"] = 6, ["StartPriceSilver"] = 6, ["StartPriceCopper"] = 6, ["BuyoutPriceSilver"] = 6, ["BuyoutPriceCopper"] = 6}, -6, -10, 0, 0},
				{{["BidQualitySort"] = 6, ["BidLevelSort"] = 6, ["BidDurationSort"] = 6, ["BidBuyoutSort"] = 6, ["BidStatusSort"] = 6, ["BidBidSort"] = 6, ["AuctionsQualitySort"] = 6, ["AuctionsDurationSort"] = 6, ["AuctionsHighBidderSort"] = 6, ["AuctionsBidSort"] = 6, ["BrowseQualitySort"] = 6, ["BrowseLevelSort"] = 6, ["BrowseDurationSort"] = 6, ["BrowseHighBidderSort"] = 6, ["BrowseCurrentBidSort"] = 6}, nil, -1},
				{{["BrowseDropDown"] = 18}, 19, 110, 0, 6, true},				
				{{["PriceDropDown"] = 18, ["DurationDropDown"] = 18}, 20, -16, -2, 6, true},
			})
			local new = CreateBG(AuctionFrame)
			AuctionFrameTab1:ClearAllPoints()
			AuctionFrameTab1:SetPoint("TOPLEFT", new, "BOTTOMLEFT", 20, 2)		
			for i = 1, 15 do
				StartSkinning({["AuctionFilterButton"..i] = 18}, nil, nil, nil, -1)
			end
			CreateBG(BrowseName, -5, -2, 1, -1)
			CreateBG(BrowseMinLevel, -4, -2, 1, -1)
			CreateBG(BrowseMaxLevel, -4, -2, 1, -1)
			
			AuctionFrameCloseButton:ClearAllPoints()
			AuctionFrameCloseButton:SetPoint("TOPRIGHT", AuctionFrame, "TOPRIGHT", 4, 4)
			AuctionsScrollFrame:DisableDrawLayer("ARTWORK")
			BidScrollFrame:DisableDrawLayer("ARTWORK")
			BrowseFilterScrollFrame:DisableDrawLayer("ARTWORK")
			BrowseScrollFrame:DisableDrawLayer("ARTWORK")
			
			for i = 1, 8 do
				SkinIcons(_G["BrowseButton"..i.."Item"], nil, true)
				_G["BrowseButton"..i.."ItemCount"]:SetDrawLayer("ARTWORK")
				SkinFrame(_G["BrowseButton"..i], 4)	
			end	
			for i = 1, NUM_BIDS_TO_DISPLAY do
				SkinIcons(_G["BidButton"..i.."Item"], nil, true)
				_G["BidButton"..i.."ItemCount"]:SetDrawLayer("ARTWORK")
				SkinFrame(_G["BidButton"..i], 4)
			end	
			for i = 1, NUM_AUCTIONS_TO_DISPLAY do
				SkinIcons(_G["AuctionsButton"..i.."Item"], nil, true)
				_G["AuctionsButton"..i.."ItemCount"]:SetDrawLayer("ARTWORK")
				SkinFrame(_G["AuctionsButton"..i], 4)
			end
			
			hooksecurefunc("AuctionFrameBrowse_Update", function()
				for i = 1, 8 do
					if (not _G["BrowseButton"..i.."Name"]) then break end
					local r, g, b = _G["BrowseButton"..i.."Name"]:GetTextColor()
					_G["BrowseButton"..i.."Item"].new:SetVertexColor(r, g, b)		
				end			
			end)
			
			hooksecurefunc("AuctionFrameBid_Update", function()
				for i = 1, 8 do
					if (not _G["BidButton"..i.."Name"]) then break end
					local r, g, b = _G["BidButton"..i.."Name"]:GetTextColor()
					_G["BidButton"..i.."Item"].new:SetVertexColor(r, g, b)		
				end			
			end)
			
			hooksecurefunc("AuctionFrameAuctions_Update", function()
				for i = 1, 8 do
					if (not _G["AuctionsButton"..i.."Name"]) then break end
					local r, g, b = _G["AuctionsButton"..i.."Name"]:GetTextColor()
					_G["AuctionsButton"..i.."Item"].new:SetVertexColor(r, g, b)		
				end			
			end)
			
			SkinTabs({"AuctionFrameTab1", "AuctionFrameTab2", "AuctionFrameTab3"})
			SkinArrows({"PriceDropDown", "DurationDropDown", "BrowseDropDown"})
			select(2, AuctionsItemButton:GetRegions()):Hide()
			SkinIcons(AuctionsItemButton)
			
			hooksecurefunc("AuctionsFrameAuctions_ValidateAuction", function()
				if ( not GetAuctionSellItemInfo() ) then return end
				select(6, AuctionsItemButton:GetRegions()):SetTexCoord(0.1, 0.92, 0.08, 0.92)
				SkinFrame(AuctionsStackSizeEntry, 4)
				CreateBG(AuctionsStackSizeEntry, -7, -7)
				SkinFrame(AuctionsNumStacksEntry, 4)
				CreateBG(AuctionsNumStacksEntry, -7, -7)
			end)		
		elseif ( name == "Blizzard_TrainerUI" ) then
			MakeMovable(ClassTrainerFrame)
			if (MayronUI.db.profile.ReskinUI["Trainer Frame"]) then return end
			StartSkinning({["ClassTrainerFrame"] = 62, ["ClassTrainerFrameInset"] = 14, ["ClassTrainerFrameBottomInset"] = 14, ["ClassTrainerStatusBar"] = 16, ["ClassTrainerScrollFrameScrollBar"] = 16})
			ClassTrainerScrollFrame:EnableMouse(false)
			ClassTrainerTrainButton:DisableDrawLayer("BORDER")
			StartSkinning(ScanDropdowns(ClassTrainerFrame, 18), 20, -12, 0, 6, true)
			ClassTrainerStatusBar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
			CreateBG(ClassTrainerStatusBar, -1, 1, 1, -1)
			for i = 0, 7 do					
				local btn 
				if (i == 0) then btn = _G["ClassTrainerFrameSkillStepButton"] else btn = _G["ClassTrainerScrollFrameButton"..i] end
				if (not btn or btn.new) then break end
				btn:DisableDrawLayer("ARTWORK")
				SkinIcons(btn, 1)
				btn:SetHighlightTexture("")			
			end
			hooksecurefunc("ClassTrainerFrame_Update", function()
				for i = 0, 7 do					
					local btn 
					if (i == 0) then btn = _G["ClassTrainerFrameSkillStepButton"] else btn = _G["ClassTrainerScrollFrameButton"..i] end
					if (not btn) then break end
					select(1, btn:GetRegions()):SetDrawLayer("OVERLAY")			
				end
			end)
		elseif ( name == "Blizzard_Collections" ) then
			MakeMovable(CollectionsJournal)
			if (MayronUI.db.profile.ReskinUI["Mount/Pet Journal"]) then return end
			StartSkinning({
				["CollectionsJournal"] = 46, ["MountJournalListScrollFrameScrollBar"] = 20, ["PetJournalLeftInset"] = 12, ["PetJournalRightInset"] = 12, ["PetJournalListScrollFrameScrollBar"] = 20, ["MountJournalMountButton"] = 8, ["MountJournal.MountDisplay"] = 6, ["MountJournal.MountCount"] = 142, ["PetJournal.PetCount"] = 142, ["PetJournalSummonButton"] = 8, ["PetJournalFindBattle"] = 8, ["PetJournalLoadoutBorder"] = 48, ["HeirloomsJournal.iconsFrame"] = 60,
				{{["MountJournalSearchBox"] = 134}, -4},
				{{["PetJournalSearchBox"] = 134}, -4},				
			})
			MountJournal.RightInset:Hide()
			MountJournal.LeftInset:Hide()
			SkinIcons(MountJournal.MountDisplay.InfoButton.Icon, 1)
			SkinIcons(MountJournalSummonRandomFavoriteButton, 1)
			
			PetJournalPetCardInset:Hide()
			PetJournalPetCard:DisableDrawLayer("BACKGROUND")
			PetJournalAchievementStatus:GetRegions():Hide()						
			
			for i = 1, 4 do
				SkinTabs("CollectionsJournalTab"..i)
			end			
			
			SkinIcons(PetJournalHealPetButton)
			SkinIcons(PetJournalPetCardPetInfoIcon)
			PetJournalPetCardPetInfoQualityBorder:SetAlpha(0)
			PetJournalTutorialButton:DisableDrawLayer("BORDER")
			PetJournalTutorialButton:SetPoint("TOPLEFT", PetJournal, "TOPLEFT", -16, 18)
			
			for i = 1, 6 do
				local btn = _G["PetJournalPetCardSpell"..i]
				SkinIcons(btn, 1)
				btn:HookScript("OnEnter", function()
					btn.new:SetVertexColor(1,1,1,1)
				end)
				btn:HookScript("OnLeave", function()
					btn.new:SetVertexColor(0,0,0,1)
				end)
			end
			
			for tag, str in ipairs({"MountJournalListScrollFrameButton", "PetJournalListScrollFrameButton"}) do
				for i = 1, 12 do				
					local btn = _G[str..i]
					select(1, btn:GetRegions()):Hide()				
					btn.selectedTexture:SetTexture("")				
					SkinIcons(btn.icon)	
					SkinButton(btn)
					btn:SetHeight(40)
					if (btn.factionIcon) then
						btn.factionIcon:ClearAllPoints()
						btn.factionIcon:SetPoint("TOPRIGHT", btn, "TOPRIGHT", -1, -1)
						btn.factionIcon:SetPoint("BOTTOMRIGHT", btn, "BOTTOMLEFT", 0, 1)
					end
					if (i > 1) then
						btn:SetPoint("TOPLEFT", _G["MountJournalListScrollFrameButton"..(i-1)], "BOTTOMLEFT", 0, -2)
					end
					local drag
					if (tag == 1) then drag = "DragButton" else 
						drag = "dragButton"
						PetJournalListScrollFrameButton12.iconBorder.Show = function() end -- fix blizzard stupid
						btn.petTypeIcon:ClearAllPoints()
						btn.petTypeIcon:SetPoint("TOPRIGHT", btn, "TOPRIGHT", -1, -1)
						btn.petTypeIcon:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT", -1, 1)
						btn.name:Show()
					end
					btn[drag]:SetHighlightTexture("")
					btn[drag]:HookScript("OnEnter", function()
						btn.new:SetVertexColor(1,1,1,1)
					end)
					btn[drag]:HookScript("OnLeave", function()
						btn.new:SetVertexColor(0,0,0,1)
					end)
				end
			end
			
			hooksecurefunc("PetJournal_UpdatePetLoadOut", function()
				for i = 1, 3 do	
					local btn = _G["PetJournalLoadoutPet"..i]
					local helpBG = _G["PetJournalLoadoutPet"..i.."HelpFrameHelpPlate"]
					_G["PetJournalLoadoutPet"..i.."ModelFrameCardButton"]:EnableMouse(false)
					btn.MenuRegion:EnableMouse(false)
					if (helpBG and helpBG:IsVisible()) then
						helpBG:ClearAllPoints()
						helpBG:SetPoint("TOPLEFT", btn, "TOPLEFT", 1, -1)
						helpBG:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT", -1, 1)
						helpBG:SetAlpha(0.8)
					end
				end
			end)
			
			for i = 1, 3 do	
				local btn = _G["PetJournalLoadoutPet"..i]
				local helpBG = _G["PetJournalLoadoutPet"..i.."HelpFrameHelpPlate"]
				local setBtn = _G["PetJournalLoadoutPet"..i.."SetButton"]
				SkinIcons(btn.icon, 1)				
				setBtn:SetFrameLevel(btn["dragButton"]:GetFrameLevel() + 4)
				setBtn:ClearAllPoints()
				setBtn:SetPoint("TOPLEFT", btn.iconBorder, "TOPLEFT", -2, 2)
				setBtn:SetPoint("BOTTOMRIGHT", btn.iconBorder, "BOTTOMRIGHT", 2, -2)
				
				btn.level:SetFontObject(GameFontNormal)
				btn.level:SetTextColor(1.0, 1.0, 0)
				btn.iconBorder:SetAlpha(0)
				SkinButton(btn)
				_G["PetJournalLoadoutPet"..i.."BG"]:Hide()
				btn:EnableMouse(0)
				btn["dragButton"]:SetHighlightTexture("")
				btn["dragButton"]:HookScript("OnEnter", function()
					btn.new:SetVertexColor(1,1,1,1)
				end)
				btn["dragButton"]:HookScript("OnLeave", function()
					btn.new:SetVertexColor(0,0,0,1)
				end)
			end
			hooksecurefunc("GetPetTypeTexture", function()
				for i = 1, 12 do
					local btn = _G["PetJournalListScrollFrameButton"..i]
					btn.iconBorder:Hide()
					btn.name:SetTextColor(btn.iconBorder:GetVertexColor())
				end
			end)
		
			-- Toy Box:
			SkinFrame(ToyBox.iconsFrame, 60)
			ToyBox.searchBox:DisableDrawLayer("BACKGROUND")
			CreateBG(ToyBox.searchBox, -4, nil, nil, nil, nil, nil, true)
			ToyBox.progressBar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
			SkinFrame(ToyBox.progressBar, 36)
			ToyBox.progressBar:SetPoint("TOP", (select(2,ToyBox.progressBar:GetPoint())), "TOP", 0, -34)
			CreateBG(ToyBox.progressBar, -1, 1, 1, -1, nil, nil, true)
			ToyBox.progressBar:SetHeight(20)
			
			-- Heirlooms:
			HeirloomsJournal.progressBar:SetStatusBarTexture(MUI_MEDIA.."StatusBar\\MUI_StatusBar")
			SkinFrame(HeirloomsJournal.progressBar, 36)
			HeirloomsJournal.progressBar:SetPoint("TOP", (select(2,HeirloomsJournal.progressBar:GetPoint())), "TOP", 0, -34)
			CreateBG(HeirloomsJournal.progressBar, -1, 1, 1, -1, nil, nil, true)
			HeirloomsJournal.progressBar:SetHeight(20)
			StartSkinning({["HeirloomsJournalClassDropDown"] = 18}, 19, -16, 0, 6, true)
			SkinArrows({"HeirloomsJournalClassDropDown"})			
			StartSkinning({["HeirloomsJournalSearchBox"] = 134}, -6, 0)			
			
			hooksecurefunc(HeirloomsJournal, "LayoutCurrentPage", function()
				for id, f in ipairs(HeirloomsJournal.heirloomHeaderFrames) do
					ChangeTextColor(f.text, "yellow")
				end
			end)
		elseif ( name == "Blizzard_DeathRecap" ) then
			if (MayronUI.db.profile.ReskinUI["Death Recap"]) then return end
			SkinFrame(DeathRecapFrame, 44)
			CreateBG(DeathRecapFrame)			
		elseif ( (name == "Blizzard_GuildControlUI") and (not MayronUI.db.profile.ReskinUI["Guild Control"]) ) then
			MakeMovable(GuildControlUI)
			if (MayronUI.db.profile.ReskinUI["Guild Control"]) then return end
			StartSkinning({["GuildControlUI"] = 14})
			GuildControlUIHbar:Hide()
			GuildControlUIRankBankFrameInset:DisableDrawLayer("BORDER")
			GuildControlUIRankBankFrameInsetScrollFrame:DisableDrawLayer("ARTWORK")
			
			local function UpdateRankButtons()
				for i = 1, (GuildControlGetNumRanks()) do
					local btn = _G["GuildControlUIRankOrderFrameRank"..i.."NameEditBox"]
					if (not btn) then break end
					if (not btn.new) then
						btn:DisableDrawLayer("BACKGROUND")
						CreateBG(btn, -6, nil, -2, 3)
					end
				end				
			end
			local timeDiff = 0
			local btnName = ""
			local updateFrame = CreateFrame("Frame");
			local function NewRank_OnUpdate(self, elapsed)
				timeDiff = timeDiff + elapsed
				while ( (timeDiff > 0.2) ) do
					if (_G[btnName]) then
						local btn = _G[btnName]
						if (not btn.new) then
							btn:DisableDrawLayer("BACKGROUND")
							CreateBG(btn, -6, nil, -2, 3)
						end
						updateFrame:SetScript("OnUpdate", nil) -- and here..
						break
					end
					timeDiff = 0
				end
			end

			hooksecurefunc("GuildControlUI_RankOrder_Update", UpdateRankButtons)
			hooksecurefunc("GuildControlAddRank", function()
				local newID = GuildControlGetNumRanks() + 1
				btnName = "GuildControlUIRankOrderFrameRank"..newID.."NameEditBox"
				if (not _G[btnName]) then
					updateFrame:SetScript("OnUpdate", NewRank_OnUpdate) -- here as well? What's even going on?!
				end
			end)

			SkinArrows({"GuildControlUINavigationDropDown", "GuildControlUIRankBankFrameRankDropDown", "GuildControlUIRankSettingsFrameRankDropDown"})
			StartSkinning({["GuildControlUINavigationDropDown"] = 18, ["GuildControlUIRankBankFrameRankDropDown"] = 18, ["GuildControlUIRankSettingsFrameRankDropDown"] = 18}, 19, -16, 0, 6, true)
		
		elseif ( (name == "Blizzard_InspectUI") and (not MayronUI.db.profile.ReskinUI["Inspect Frame"]) ) then
			MakeMovable(InspectFrame)
			if (MayronUI.db.profile.ReskinUI["Inspect Frame"]) then return end
			StartSkinning({["InspectFrame"] = 46, ["InspectModelFrame"] = 44, ["InspectFrameInset"] = 12, ["InspectPVPFrame"] = 12, ["InspectTalentFrame"] = 12})
			SkinTabs({"InspectFrameTab1", "InspectFrameTab2", "InspectFrameTab3", "InspectFrameTab4"})
			CreateBG(InspectModelFrame, -1, 2, 1, -2)
			Specialization:EnableMouse(false)
			InspectGuildFrame:DisableDrawLayer("BACKGROUND")
			
			local slots = {"Head", "Neck", "Shoulder", "Shirt", "Chest", "Waist", "Legs", "Feet", "Wrist",	"Hands", "Finger0", "Finger1", "Trinket0", "Trinket1", "Back", "MainHand", "SecondaryHand", "Tabard"}
			
			for i, slot in next, slots do
				local icon = _G["Inspect"..slot.."Slot"]
				SkinIcons(icon)
				icon.IconBorder:SetAlpha(0)
				_G["Inspect"..slot.."SlotFrame"]:Hide()				
				if (slot == "MainHand") then
					local sideTexture = select(10, InspectMainHandSlot:GetRegions())
					if (sideTexture) then
						sideTexture:Hide()
					end
				end
				
			end

			local inspect = CreateFrame("Frame")
			inspect:RegisterUnitEvent("UNIT_INVENTORY_CHANGED", "target")
			inspect:RegisterEvent("INSPECT_READY")	
			local notLoaded = {}
			local timeDiff = 0
			
			local function Inspect_OnUpdate(self, elapsed)
				timeDiff = timeDiff + elapsed
				while ( (timeDiff > 0.2) ) do
					for slotName, id in pairs(notLoaded) do
						if (notLoaded[slotName]) then
							local quality = select(3, GetItemInfo(id))				
							if (quality) then 
								local r, g, b = GetItemQualityColor(quality)
								_G[slotName].new:SetVertexColor(r, g, b)
								notLoaded[slotName] = nil
							end
						end
					end
					local finished = true
					for slotName, id in pairs(notLoaded) do -- to find any value in a hash table.
						finished = nil
						break
					end
					if (finished) then
						inspect:SetScript("OnUpdate", nil)							
					end
					timeDiff = 0
				end
			end			
			
			local function UpdateCharacterSlots()	
				for i, slot in next, slots do						
					local slotName = (slot.."Slot")
					local quality
					if ( GetInventorySlotInfo(slotName) ) then
						quality = GetInventoryItemQuality("target", GetInventorySlotInfo(slotName))
					end			
					slotName = _G["Inspect"..slotName]
					local bg = slotName.new
					if quality then
						local r, g, b, _ = GetItemQualityColor(quality)
						slotName.icon:SetAlpha(1)
						bg:SetVertexColor(r, g, b, 0.8)
					else
						local itemID = GetInventoryItemID("target", GetInventorySlotInfo(slot.."Slot"))
						if (itemID) then
							slotName.icon:SetAlpha(1)
							notLoaded[(slotName:GetName())] = itemID
							inspect:SetScript("OnUpdate", function(self, elapsed) Inspect_OnUpdate(self, elapsed) end)
						else
							slotName.icon:SetAlpha(0)
							bg:SetVertexColor(0,0,0)
						end
					end
				end
			end
			inspect:SetScript("OnEvent", UpdateCharacterSlots)				
			inspect:SetScript("OnUpdate", nil)
			
			InspectModelFrame:SetHeight(InspectModelFrame:GetHeight() - 14)
			local point, relframe, relpoint, x, y = InspectModelFrame:GetPoint()
			InspectModelFrame:SetPoint(point, relframe, relpoint, x, y + 4)
			point, relframe, relpoint, x, y = nil
		
		elseif ( name == "Blizzard_ItemSocketingUI" ) then 
			MakeMovable(ItemSocketingFrame)
			if (MayronUI.db.profile.ReskinUI["Item Socketing Frame"]) then return end
			StartSkinning({["ItemSocketingFrame"] = 62, ["ItemSocketingFrameInset"] = 12, ["ItemSocketingScrollFrame"] = 16})
			for i = 1, 3 do
				_G["ItemSocketingSocket"..i]:DisableDrawLayer("BACKGROUND") 
			end
			CreateBG(ItemSocketingScrollFrame, nil, -4, nil, -4)
		elseif ( name == "Blizzard_ItemUpgradeUI" ) then
			MakeMovable(ItemUpgradeFrame)
			if (MayronUI.db.profile.ReskinUI["Item Upgrade Frame"]) then return end
			StartSkinning({["ItemUpgradeFrame"] = 46, ["ItemUpgradeFrameMoneyFrame"] = 4, ["ItemUpgradeFrameUpgradeButton"] = 8})
			ChangeTextColor(ItemUpgradeFrame.MissingDescription, "white")
			ItemUpgradeFrame.ItemButton.Grabber:Hide()
			ItemUpgradeFrame.ItemButton.TextGrabber:Hide()
			ItemUpgradeFrame.ButtonFrame:DisableDrawLayer("BORDER")			
		elseif ( name == "Blizzard_ChallengesUI" ) then
			if (MayronUI.db.profile.ReskinUI["Challenges Frame"]) then return end
			StartSkinning({["ChallengesFrameDetails"] = 12, ["ChallengesFrameInset"] = 12})
		elseif ( name == "Blizzard_QuestChoice" ) then
			MakeMovable("QuestChoiceFrame", true)
			if (MayronUI.db.profile.ReskinUI["Quest Choice"]) then return end
			QuestChoiceFrame:DisableDrawLayer("BACKGROUND")
			QuestChoiceFrame:DisableDrawLayer("BORDER")
			CreateBG(QuestChoiceFrame)			
			QuestChoiceFrame.QuestionFrameLeft:Hide()
			QuestChoiceFrame.QuestionFrameMiddle:Hide()
			QuestChoiceFrame.QuestionFrameRight:Hide()
			ChangeTextColor(QuestChoiceFrame.Option1.OptionText, "white")
			ChangeTextColor(QuestChoiceFrame.Option2.OptionText, "white")			
			QuestChoiceFrame.CloseButton:SetPoint("TOPRIGHT", QuestChoiceFrame, "TOPRIGHT", 4, 4)	
			QuestChoiceFrame:SetHeight(540)		
			for i, v in pairs(QuestChoiceFrameOption1.Rewards.Currencies) do
				if (type(i) == "string" and i:find("Currency")) then
					SkinIcons(QuestChoiceFrameOption1.Rewards.Currencies[i].Icon)
				end
			end			
			for i, v in pairs(QuestChoiceFrameOption2.Rewards.Currencies) do
				if (type(i) == "string" and i:find("Currency")) then
					SkinIcons(QuestChoiceFrameOption2.Rewards.Currencies[i].Icon)
				end
			end
			local new = CreateBG()
			new:SetPoint("TOPLEFT", QuestChoiceFrame.QuestionFrameLeft, "TOPLEFT", 30, 0)
			new:SetPoint("BOTTOMRIGHT", QuestChoiceFrame.QuestionFrameRight, "BOTTOMRIGHT", -34, 0)
			new:SetParent(QuestChoiceFrame)
			QuestChoiceFrame.QuestionText:ClearAllPoints()
			QuestChoiceFrame.QuestionText:SetPoint("CENTER", new, "CENTER")
			QuestChoiceFrame.QuestionText:SetParent(new)
		elseif ( name == "Bagnon_GuildBank" ) then -- End of Blizzard AddOns
			if ( not MayronUI.db.profile.ReskinUI["Bagnon"] ) then
				if (IsAddOnLoaded("Bagnon") and Bagnon:IsFrameEnabled("guild")) then				
					hooksecurefunc(Bagnon.GuildItemSlot, "Create", function()					
						local id = Bagnon.GuildItemSlot.nextID
						local item = _G["BagnonGuildItemSlot"..id]				
						if (not item) then return end
						SkinIcons(item)
						_G[item:GetName().."IconQuestTexture"]:SetDrawLayer("OVERLAY")
						item.IconBorder:SetAlpha(0)
					end)
					hooksecurefunc(Bagnon.GuildTab, "New", function(_, _, id)
						if (not _G["BagnonGuildTab"..id]) then return end
						SkinIcons(_G["BagnonGuildTab"..id], 1)
					end)
				end
			end
		elseif ( name == "Bagnon_VoidStorage" ) then
			if ( not MayronUI.db.profile.ReskinUI["Bagnon"] ) then
				if (IsAddOnLoaded("Bagnon") and Bagnon:IsFrameEnabled("vault")) then				
					hooksecurefunc(Bagnon.VaultSlot, "Create", function()					
						local id = Bagnon.VaultSlot.nextID
						local item = _G["BagnonVaultItemSlot"..id]				
						if (not item) then return end
						SkinIcons(item)
						_G[item:GetName().."IconQuestTexture"]:SetDrawLayer("OVERLAY")
						item.IconBorder:SetAlpha(0)
					end)				
				end
			end
		end
	end)	
	
	-- Make Movable Frames:
	if (not MayronUI.db.profile.ReskinUI["Stop Movable"]) then 
		MakeMovable({"QuestLogPopupDetailFrame", "QuestFrame", "GossipFrame", "DurabilityFrame", "VehicleSeatIndicator", "FriendsFrame", "MailFrame", "SpellBookFrame", "PetitionFrame", "BankFrame", "TimeManagerFrame",  "GearManagerDialogPopup", "LFGDungeonReadyStatus", "AddFriendFrame", "ChannelFrameDaughterFrame", "RecruitAFriendFrame", "VideoOptionsFrame", "LFGDungeonReadyDialog", "AddonList", "WorldStateScoreFrame", "LFDRoleCheckPopup", "PVPReadyDialog", "ChatConfigFrame", "GuildInviteFrame", "LootFrame", "ReadyCheckFrame", "BonusRollMoneyWonFrame", "BonusRollFrame", "TradeFrame", "OpenMailFrame", "TabardFrame", "GuildRegistrarFrame", "ItemTextFrame", "DressUpFrame", "GameMenuFrame", "TaxiFrame", "HelpFrame", "PVEFrame", "MerchantFrame"})
		MakeMovable("CharacterFrame", false, {"CharacterFrameExpandButton", "CharacterFrameTab1", "CharacterFrameTab2", "CharacterFrameTab3", "CharacterFrameTab4"})
		MakeMovable(PetBattleFrame["ActiveAlly"])
		MakeMovable(PetBattleFrame["ActiveEnemy"])
		MakeMovable(FriendsFrameBattlenetFrame.BroadcastFrame)
		MakeMovable("WorldMapFrame", true)
	end
	
	function MayronUI_ReskinUI:TestFrame(toggle)
		local loc = MayronUI.db.profile.ReskinUI.Appearance
		if ( loc.Toggle and toggle ) then 
			loc.Toggle = nil
			if (_G["MUI_ReskinTestFrame"]) then MUI_ReskinTestFrame:Hide() end
		elseif ( toggle ) then
			loc.Toggle = true
			if (not _G["MUI_ReskinTestFrame"]) then
				CreateFrame("Frame", "MUI_ReskinTestFrame", UIParent, "MUI_ToolTipTemplate")
				MUI_ReskinTestFrame:SetFrameStrata("TOOLTIP")
				MakeMovable(MUI_ReskinTestFrame)
			end
			MUI_ReskinTestFrame:Show()
		end
		
		-- Update settings:
		C = {
			["Background_Color"] = {r = 0.2, g = 0.2, b = 0.2, a = 0.9}, -- DONE
			["Border_Color"] = {r = 0, g = 0, b = 0, a = 1},
			["DropDown_ClassColor"] = true,
			["Tab_ClassColor"] = true,
			["Tab_Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b, a = MUI_COLOR.a},
			["DropDown_Color"] = {r = MUI_COLOR.r, g = MUI_COLOR.g, b = MUI_COLOR.b, a = MUI_COLOR.a},
			["Border_Type"] = "Skinner",
			["Background_Type"] = "MayronUI",
			["Border_EdgeSize"] = 1,
			["Border_Inset"] = 0,
		}
		for str, value in pairs(MayronUI.db.profile.ReskinUI.Appearance) do	
			if (type(value) == "table") then 
				C[str].r, C[str].g, C[str].b, C[str].a = unpack(value)
			else
				C[str] = value
			end
		end
		MUI_ReskinTestFrame.background:SetTexture(LibStub("LibSharedMedia-3.0"):Fetch("background", C["Background_Type"]))
		MUI_ReskinTestFrame.background:SetVertexColor(C["Background_Color"].r, C["Background_Color"].g, C["Background_Color"].b, C["Background_Color"].a)
		MUI_ReskinTestFrame:SetBackdrop({
			edgeFile = LibStub("LibSharedMedia-3.0"):Fetch("border", C["Border_Type"]), tile = false, tileSize = 0, edgeSize = C["Border_EdgeSize"], 
		})		
		MUI_ReskinTestFrame:SetBackdropBorderColor(C["Border_Color"].r, C["Border_Color"].g, C["Border_Color"].b, C["Border_Color"].a)
		
		MUI_ReskinTestFrame:SetSize(200 + (C["Border_EdgeSize"] - C["Border_Inset"]) * 2, 200 + (C["Border_EdgeSize"] - C["Border_Inset"]) * 2)		
		MUI_ReskinTestFrame.background:SetPoint("TOPLEFT", (C["Border_EdgeSize"] - C["Border_Inset"]), -(C["Border_EdgeSize"] - C["Border_Inset"]))
		MUI_ReskinTestFrame.background:SetPoint("BOTTOMRIGHT", -(C["Border_EdgeSize"] - C["Border_Inset"]), (C["Border_EdgeSize"] - C["Border_Inset"]))
	end
end